#ifndef LFORTRAN_ASR_H
#define LFORTRAN_ASR_H

// Generated by grammar/asdl_cpp.py

#include <libasr/alloc.h>
#include <libasr/location.h>
#include <libasr/colors.h>
#include <libasr/containers.h>
#include <libasr/exception.h>
#include <libasr/asr_scopes.h>
#include <libasr/string_utils.h>


namespace LCompilers::ASR {

enum asrType
{
    unit, symbol, stmt, expr, ttype, attribute, tbind, case_stmt, type_stmt, require_instantiation
};

struct asr_t
{
    asrType type;
    Location loc;
};


template <class T, class U>
inline bool is_a(const U &x)
{
    return T::class_type == x.type;
}

// Cast one level down

template <class T, class U>
static inline T* down_cast(const U *f)
{
    LCOMPILERS_ASSERT(f != nullptr);
    LCOMPILERS_ASSERT(is_a<T>(*f));
    return (T*)f;
}

// Cast two levels down

template <class T>
static inline T* down_cast2(const asr_t *f)
{
    typedef typename T::parent_type ptype;
    ptype *t = down_cast<ptype>(f);
    return down_cast<T>(t);
}

/******************************************************************************/
// Forward declarations

struct unit_t; // Sum
struct symbol_t; // Sum
struct stmt_t; // Sum
struct expr_t; // Sum
struct ttype_t; // Sum
enum cast_kindType // Simple Sum
{ // Types
    RealToInteger, IntegerToReal, LogicalToReal, RealToReal, IntegerToInteger, RealToComplex, IntegerToComplex, IntegerToLogical, RealToLogical, CharacterToLogical, CharacterToInteger, CharacterToList, ComplexToLogical, ComplexToComplex, ComplexToReal, ComplexToInteger, LogicalToInteger, RealToCharacter, IntegerToCharacter, LogicalToCharacter, UnsignedIntegerToInteger, UnsignedIntegerToUnsignedInteger, UnsignedIntegerToReal, UnsignedIntegerToLogical, IntegerToUnsignedInteger, RealToUnsignedInteger, CPtrToUnsignedInteger, UnsignedIntegerToCPtr, IntegerToSymbolicExpression, ListToArray
};
enum storage_typeType // Simple Sum
{ // Types
    Default, Save, Parameter
};
enum accessType // Simple Sum
{ // Types
    Public, Private
};
enum intentType // Simple Sum
{ // Types
    Local, In, Out, InOut, ReturnVar, Unspecified
};
enum deftypeType // Simple Sum
{ // Types
    Implementation, Interface
};
enum presenceType // Simple Sum
{ // Types
    Required, Optional
};
enum abiType // Simple Sum
{ // Types
    Source, LFortranModule, GFortranModule, BindC, BindPython, BindJS, Interactive, Intrinsic
};
struct dimension_t; // Product
struct alloc_arg_t; // Product
struct attribute_t; // Sum
struct attribute_arg_t; // Product
struct call_arg_t; // Product
struct reduction_expr_t; // Product
struct tbind_t; // Sum
struct array_index_t; // Product
struct do_loop_head_t; // Product
struct case_stmt_t; // Sum
struct type_stmt_t; // Sum
enum enumtypeType // Simple Sum
{ // Types
    IntegerConsecutiveFromZero, IntegerUnique, IntegerNotUnique, NonInteger
};
struct require_instantiation_t; // Sum
enum array_physical_typeType // Simple Sum
{ // Types
    DescriptorArray, PointerToDataArray, UnboundedPointerToDataArray, FixedSizeArray, CharacterArraySinglePointer, NumPyArray, ISODescriptorArray, SIMDArray
};
enum binopType // Simple Sum
{ // Types
    Add, Sub, Mul, Div, Pow, BitAnd, BitOr, BitXor, BitLShift, BitRShift
};
enum reduction_opType // Simple Sum
{ // Types
    ReduceAdd, ReduceSub, ReduceMul, ReduceMIN, ReduceMAX
};
enum logicalbinopType // Simple Sum
{ // Types
    And, Or, Xor, NEqv, Eqv
};
enum cmpopType // Simple Sum
{ // Types
    Eq, NotEq, Lt, LtE, Gt, GtE
};
enum integerbozType // Simple Sum
{ // Types
    Binary, Hex, Octal
};
enum arrayboundType // Simple Sum
{ // Types
    LBound, UBound
};
enum arraystorageType // Simple Sum
{ // Types
    RowMajor, ColMajor
};
enum string_format_kindType // Simple Sum
{ // Types
    FormatFortran, FormatC, FormatPythonPercent, FormatPythonFString, FormatPythonFormat
};


/******************************************************************************/
// Products declarations

struct dimension_t // Product
{
    Location loc;
    expr_t* m_start;
    expr_t* m_length;
};
struct alloc_arg_t // Product
{
    Location loc;
    expr_t* m_a;
    dimension_t* m_dims; size_t n_dims; // Sequence
    expr_t* m_len_expr;
    ttype_t* m_type;
};
struct attribute_arg_t // Product
{
    Location loc;
    char* m_arg;
};
struct call_arg_t // Product
{
    Location loc;
    expr_t* m_value;
};
struct reduction_expr_t // Product
{
    Location loc;
    reduction_opType m_op;
    expr_t* m_arg;
};
struct array_index_t // Product
{
    Location loc;
    expr_t* m_left;
    expr_t* m_right;
    expr_t* m_step;
};
struct do_loop_head_t // Product
{
    Location loc;
    expr_t* m_v;
    expr_t* m_start;
    expr_t* m_end;
    expr_t* m_increment;
};


/******************************************************************************/
// Sums declarations

enum unitType // Types
{
    TranslationUnit
};

struct unit_t // Sum
{
    const static asrType class_type = asrType::unit;
    asr_t base;
    unitType type;
};

    struct TranslationUnit_t // Constructor
    {
        const static unitType class_type = unitType::TranslationUnit;
        typedef unit_t parent_type;
        unit_t base;
        SymbolTable* m_symtab;
        asr_t** m_items; size_t n_items; // Sequence
    };
    static inline asr_t* make_TranslationUnit_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, asr_t** a_items, size_t n_items) {
        TranslationUnit_t *n;
        n = al.make_new<TranslationUnit_t>();
        n->base.type = unitType::TranslationUnit;
        n->base.base.type = asrType::unit;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_items = a_items;
        n->n_items = n_items;
        return (asr_t*)n;
    }



enum symbolType // Types
{
    Program, Module, Function, GenericProcedure, CustomOperator, ExternalSymbol, Struct, Enum, UnionType, Variable, Class, ClassProcedure, AssociateBlock, Block, Requirement, Template
};

struct symbol_t // Sum
{
    const static asrType class_type = asrType::symbol;
    asr_t base;
    symbolType type;
};

    struct Program_t // Constructor
    {
        const static symbolType class_type = symbolType::Program;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_dependencies; size_t n_dependencies; // Sequence
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_Program_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_dependencies, size_t n_dependencies, stmt_t** a_body, size_t n_body) {
        Program_t *n;
        n = al.make_new<Program_t>();
        n->base.type = symbolType::Program;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }

    struct Module_t // Constructor
    {
        const static symbolType class_type = symbolType::Module;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_dependencies; size_t n_dependencies; // Sequence
        bool m_loaded_from_mod;
        bool m_intrinsic;
    };
    static inline asr_t* make_Module_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_dependencies, size_t n_dependencies, bool a_loaded_from_mod, bool a_intrinsic) {
        Module_t *n;
        n = al.make_new<Module_t>();
        n->base.type = symbolType::Module;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_loaded_from_mod = a_loaded_from_mod;
        n->m_intrinsic = a_intrinsic;
        return (asr_t*)n;
    }

    struct Function_t // Constructor
    {
        const static symbolType class_type = symbolType::Function;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        ttype_t* m_function_signature;
        char** m_dependencies; size_t n_dependencies; // Sequence
        expr_t** m_args; size_t n_args; // Sequence
        stmt_t** m_body; size_t n_body; // Sequence
        expr_t* m_return_var;
        accessType m_access;
        bool m_deterministic;
        bool m_side_effect_free;
        char* m_module_file;
    };
    static inline asr_t* make_Function_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, ttype_t* a_function_signature, char** a_dependencies, size_t n_dependencies, expr_t** a_args, size_t n_args, stmt_t** a_body, size_t n_body, expr_t* a_return_var, accessType a_access, bool a_deterministic, bool a_side_effect_free, char* a_module_file) {
        Function_t *n;
        n = al.make_new<Function_t>();
        n->base.type = symbolType::Function;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_function_signature = a_function_signature;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_return_var = a_return_var;
        n->m_access = a_access;
        n->m_deterministic = a_deterministic;
        n->m_side_effect_free = a_side_effect_free;
        n->m_module_file = a_module_file;
        return (asr_t*)n;
    }

    struct GenericProcedure_t // Constructor
    {
        const static symbolType class_type = symbolType::GenericProcedure;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_parent_symtab;
        char* m_name;
        symbol_t** m_procs; size_t n_procs; // Sequence
        accessType m_access;
    };
    static inline asr_t* make_GenericProcedure_t(Allocator &al, const Location &a_loc, SymbolTable* a_parent_symtab, char* a_name, symbol_t** a_procs, size_t n_procs, accessType a_access) {
        GenericProcedure_t *n;
        n = al.make_new<GenericProcedure_t>();
        n->base.type = symbolType::GenericProcedure;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_parent_symtab = a_parent_symtab;
        n->m_name = a_name;
        n->m_procs = a_procs;
        n->n_procs = n_procs;
        n->m_access = a_access;
        return (asr_t*)n;
    }

    struct CustomOperator_t // Constructor
    {
        const static symbolType class_type = symbolType::CustomOperator;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_parent_symtab;
        char* m_name;
        symbol_t** m_procs; size_t n_procs; // Sequence
        accessType m_access;
    };
    static inline asr_t* make_CustomOperator_t(Allocator &al, const Location &a_loc, SymbolTable* a_parent_symtab, char* a_name, symbol_t** a_procs, size_t n_procs, accessType a_access) {
        CustomOperator_t *n;
        n = al.make_new<CustomOperator_t>();
        n->base.type = symbolType::CustomOperator;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_parent_symtab = a_parent_symtab;
        n->m_name = a_name;
        n->m_procs = a_procs;
        n->n_procs = n_procs;
        n->m_access = a_access;
        return (asr_t*)n;
    }

    struct ExternalSymbol_t // Constructor
    {
        const static symbolType class_type = symbolType::ExternalSymbol;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_parent_symtab;
        char* m_name;
        symbol_t* m_external;
        char* m_module_name;
        char** m_scope_names; size_t n_scope_names; // Sequence
        char* m_original_name;
        accessType m_access;
    };
    static inline asr_t* make_ExternalSymbol_t(Allocator &al, const Location &a_loc, SymbolTable* a_parent_symtab, char* a_name, symbol_t* a_external, char* a_module_name, char** a_scope_names, size_t n_scope_names, char* a_original_name, accessType a_access) {
        ExternalSymbol_t *n;
        n = al.make_new<ExternalSymbol_t>();
        n->base.type = symbolType::ExternalSymbol;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_parent_symtab = a_parent_symtab;
        n->m_name = a_name;
        n->m_external = a_external;
        n->m_module_name = a_module_name;
        n->m_scope_names = a_scope_names;
        n->n_scope_names = n_scope_names;
        n->m_original_name = a_original_name;
        n->m_access = a_access;
        return (asr_t*)n;
    }

    struct Struct_t // Constructor
    {
        const static symbolType class_type = symbolType::Struct;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_dependencies; size_t n_dependencies; // Sequence
        char** m_members; size_t n_members; // Sequence
        abiType m_abi;
        accessType m_access;
        bool m_is_packed;
        bool m_is_abstract;
        call_arg_t* m_initializers; size_t n_initializers; // Sequence
        expr_t* m_alignment;
        symbol_t* m_parent;
    };
    static inline asr_t* make_Struct_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_dependencies, size_t n_dependencies, char** a_members, size_t n_members, abiType a_abi, accessType a_access, bool a_is_packed, bool a_is_abstract, call_arg_t* a_initializers, size_t n_initializers, expr_t* a_alignment, symbol_t* a_parent) {
        Struct_t *n;
        n = al.make_new<Struct_t>();
        n->base.type = symbolType::Struct;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_members = a_members;
        n->n_members = n_members;
        n->m_abi = a_abi;
        n->m_access = a_access;
        n->m_is_packed = a_is_packed;
        n->m_is_abstract = a_is_abstract;
        n->m_initializers = a_initializers;
        n->n_initializers = n_initializers;
        n->m_alignment = a_alignment;
        n->m_parent = a_parent;
        return (asr_t*)n;
    }

    struct Enum_t // Constructor
    {
        const static symbolType class_type = symbolType::Enum;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_dependencies; size_t n_dependencies; // Sequence
        char** m_members; size_t n_members; // Sequence
        abiType m_abi;
        accessType m_access;
        enumtypeType m_enum_value_type;
        ttype_t* m_type;
        symbol_t* m_parent;
    };
    static inline asr_t* make_Enum_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_dependencies, size_t n_dependencies, char** a_members, size_t n_members, abiType a_abi, accessType a_access, enumtypeType a_enum_value_type, ttype_t* a_type, symbol_t* a_parent) {
        Enum_t *n;
        n = al.make_new<Enum_t>();
        n->base.type = symbolType::Enum;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_members = a_members;
        n->n_members = n_members;
        n->m_abi = a_abi;
        n->m_access = a_access;
        n->m_enum_value_type = a_enum_value_type;
        n->m_type = a_type;
        n->m_parent = a_parent;
        return (asr_t*)n;
    }

    struct UnionType_t // Constructor
    {
        const static symbolType class_type = symbolType::UnionType;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_dependencies; size_t n_dependencies; // Sequence
        char** m_members; size_t n_members; // Sequence
        abiType m_abi;
        accessType m_access;
        call_arg_t* m_initializers; size_t n_initializers; // Sequence
        symbol_t* m_parent;
    };
    static inline asr_t* make_UnionType_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_dependencies, size_t n_dependencies, char** a_members, size_t n_members, abiType a_abi, accessType a_access, call_arg_t* a_initializers, size_t n_initializers, symbol_t* a_parent) {
        UnionType_t *n;
        n = al.make_new<UnionType_t>();
        n->base.type = symbolType::UnionType;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_members = a_members;
        n->n_members = n_members;
        n->m_abi = a_abi;
        n->m_access = a_access;
        n->m_initializers = a_initializers;
        n->n_initializers = n_initializers;
        n->m_parent = a_parent;
        return (asr_t*)n;
    }

    struct Variable_t // Constructor
    {
        const static symbolType class_type = symbolType::Variable;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_parent_symtab;
        char* m_name;
        char** m_dependencies; size_t n_dependencies; // Sequence
        intentType m_intent;
        expr_t* m_symbolic_value;
        expr_t* m_value;
        storage_typeType m_storage;
        ttype_t* m_type;
        symbol_t* m_type_declaration;
        abiType m_abi;
        accessType m_access;
        presenceType m_presence;
        bool m_value_attr;
    };
    static inline asr_t* make_Variable_t(Allocator &al, const Location &a_loc, SymbolTable* a_parent_symtab, char* a_name, char** a_dependencies, size_t n_dependencies, intentType a_intent, expr_t* a_symbolic_value, expr_t* a_value, storage_typeType a_storage, ttype_t* a_type, symbol_t* a_type_declaration, abiType a_abi, accessType a_access, presenceType a_presence, bool a_value_attr) {
        Variable_t *n;
        n = al.make_new<Variable_t>();
        n->base.type = symbolType::Variable;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_parent_symtab = a_parent_symtab;
        n->m_name = a_name;
        n->m_dependencies = a_dependencies;
        n->n_dependencies = n_dependencies;
        n->m_intent = a_intent;
        n->m_symbolic_value = a_symbolic_value;
        n->m_value = a_value;
        n->m_storage = a_storage;
        n->m_type = a_type;
        n->m_type_declaration = a_type_declaration;
        n->m_abi = a_abi;
        n->m_access = a_access;
        n->m_presence = a_presence;
        n->m_value_attr = a_value_attr;
        return (asr_t*)n;
    }

    struct Class_t // Constructor
    {
        const static symbolType class_type = symbolType::Class;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        abiType m_abi;
        accessType m_access;
    };
    static inline asr_t* make_Class_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, abiType a_abi, accessType a_access) {
        Class_t *n;
        n = al.make_new<Class_t>();
        n->base.type = symbolType::Class;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_abi = a_abi;
        n->m_access = a_access;
        return (asr_t*)n;
    }

    struct ClassProcedure_t // Constructor
    {
        const static symbolType class_type = symbolType::ClassProcedure;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_parent_symtab;
        char* m_name;
        char* m_self_argument;
        char* m_proc_name;
        symbol_t* m_proc;
        abiType m_abi;
        bool m_is_deferred;
        bool m_is_nopass;
    };
    static inline asr_t* make_ClassProcedure_t(Allocator &al, const Location &a_loc, SymbolTable* a_parent_symtab, char* a_name, char* a_self_argument, char* a_proc_name, symbol_t* a_proc, abiType a_abi, bool a_is_deferred, bool a_is_nopass) {
        ClassProcedure_t *n;
        n = al.make_new<ClassProcedure_t>();
        n->base.type = symbolType::ClassProcedure;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_parent_symtab = a_parent_symtab;
        n->m_name = a_name;
        n->m_self_argument = a_self_argument;
        n->m_proc_name = a_proc_name;
        n->m_proc = a_proc;
        n->m_abi = a_abi;
        n->m_is_deferred = a_is_deferred;
        n->m_is_nopass = a_is_nopass;
        return (asr_t*)n;
    }

    struct AssociateBlock_t // Constructor
    {
        const static symbolType class_type = symbolType::AssociateBlock;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_AssociateBlock_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, stmt_t** a_body, size_t n_body) {
        AssociateBlock_t *n;
        n = al.make_new<AssociateBlock_t>();
        n->base.type = symbolType::AssociateBlock;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }

    struct Block_t // Constructor
    {
        const static symbolType class_type = symbolType::Block;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_Block_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, stmt_t** a_body, size_t n_body) {
        Block_t *n;
        n = al.make_new<Block_t>();
        n->base.type = symbolType::Block;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }

    struct Requirement_t // Constructor
    {
        const static symbolType class_type = symbolType::Requirement;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_args; size_t n_args; // Sequence
        require_instantiation_t** m_requires; size_t n_requires; // Sequence
    };
    static inline asr_t* make_Requirement_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_args, size_t n_args, require_instantiation_t** a_requires, size_t n_requires) {
        Requirement_t *n;
        n = al.make_new<Requirement_t>();
        n->base.type = symbolType::Requirement;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_requires = a_requires;
        n->n_requires = n_requires;
        return (asr_t*)n;
    }

    struct Template_t // Constructor
    {
        const static symbolType class_type = symbolType::Template;
        typedef symbol_t parent_type;
        symbol_t base;
        SymbolTable* m_symtab;
        char* m_name;
        char** m_args; size_t n_args; // Sequence
        require_instantiation_t** m_requires; size_t n_requires; // Sequence
    };
    static inline asr_t* make_Template_t(Allocator &al, const Location &a_loc, SymbolTable* a_symtab, char* a_name, char** a_args, size_t n_args, require_instantiation_t** a_requires, size_t n_requires) {
        Template_t *n;
        n = al.make_new<Template_t>();
        n->base.type = symbolType::Template;
        n->base.base.type = asrType::symbol;
        n->base.base.loc = a_loc;
        n->m_symtab = a_symtab;
        a_symtab->asr_owner = (asr_t*)n;
        n->m_name = a_name;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_requires = a_requires;
        n->n_requires = n_requires;
        return (asr_t*)n;
    }



enum stmtType // Types
{
    Allocate, ReAlloc, Assign, Assignment, Associate, Cycle, ExplicitDeallocate, ImplicitDeallocate, DoConcurrentLoop, DoLoop, ErrorStop, Exit, ForAllSingle, GoTo, GoToTarget, If, IfArithmetic, Print, FileOpen, FileClose, FileRead, FileBackspace, FileRewind, FileInquire, FileWrite, Return, Select, Stop, Assert, SubroutineCall, IntrinsicImpureSubroutine, Where, WhileLoop, Nullify, Flush, ListAppend, AssociateBlockCall, SelectType, CPtrToPointer, BlockCall, SetInsert, SetRemove, ListInsert, ListRemove, ListClear, DictInsert, Expr
};

struct stmt_t // Sum
{
    const static asrType class_type = asrType::stmt;
    asr_t base;
    stmtType type;
};

    struct Allocate_t // Constructor
    {
        const static stmtType class_type = stmtType::Allocate;
        typedef stmt_t parent_type;
        stmt_t base;
        alloc_arg_t* m_args; size_t n_args; // Sequence
        expr_t* m_stat;
        expr_t* m_errmsg;
        expr_t* m_source;
    };
    static inline asr_t* make_Allocate_t(Allocator &al, const Location &a_loc, alloc_arg_t* a_args, size_t n_args, expr_t* a_stat, expr_t* a_errmsg, expr_t* a_source) {
        Allocate_t *n;
        n = al.make_new<Allocate_t>();
        n->base.type = stmtType::Allocate;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_stat = a_stat;
        n->m_errmsg = a_errmsg;
        n->m_source = a_source;
        return (asr_t*)n;
    }

    struct ReAlloc_t // Constructor
    {
        const static stmtType class_type = stmtType::ReAlloc;
        typedef stmt_t parent_type;
        stmt_t base;
        alloc_arg_t* m_args; size_t n_args; // Sequence
    };
    static inline asr_t* make_ReAlloc_t(Allocator &al, const Location &a_loc, alloc_arg_t* a_args, size_t n_args) {
        ReAlloc_t *n;
        n = al.make_new<ReAlloc_t>();
        n->base.type = stmtType::ReAlloc;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_args = a_args;
        n->n_args = n_args;
        return (asr_t*)n;
    }

    struct Assign_t // Constructor
    {
        const static stmtType class_type = stmtType::Assign;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        char* m_variable;
    };
    static inline asr_t* make_Assign_t(Allocator &al, const Location &a_loc, int64_t a_label, char* a_variable) {
        Assign_t *n;
        n = al.make_new<Assign_t>();
        n->base.type = stmtType::Assign;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_variable = a_variable;
        return (asr_t*)n;
    }

    struct Assignment_t // Constructor
    {
        const static stmtType class_type = stmtType::Assignment;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_target;
        expr_t* m_value;
        stmt_t* m_overloaded;
    };
    static inline asr_t* make_Assignment_t(Allocator &al, const Location &a_loc, expr_t* a_target, expr_t* a_value, stmt_t* a_overloaded) {
        Assignment_t *n;
        n = al.make_new<Assignment_t>();
        n->base.type = stmtType::Assignment;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_target = a_target;
        n->m_value = a_value;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct Associate_t // Constructor
    {
        const static stmtType class_type = stmtType::Associate;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_target;
        expr_t* m_value;
    };
    static inline asr_t* make_Associate_t(Allocator &al, const Location &a_loc, expr_t* a_target, expr_t* a_value) {
        Associate_t *n;
        n = al.make_new<Associate_t>();
        n->base.type = stmtType::Associate;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_target = a_target;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct Cycle_t // Constructor
    {
        const static stmtType class_type = stmtType::Cycle;
        typedef stmt_t parent_type;
        stmt_t base;
        char* m_stmt_name;
    };
    static inline asr_t* make_Cycle_t(Allocator &al, const Location &a_loc, char* a_stmt_name) {
        Cycle_t *n;
        n = al.make_new<Cycle_t>();
        n->base.type = stmtType::Cycle;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_stmt_name = a_stmt_name;
        return (asr_t*)n;
    }

    struct ExplicitDeallocate_t // Constructor
    {
        const static stmtType class_type = stmtType::ExplicitDeallocate;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t** m_vars; size_t n_vars; // Sequence
    };
    static inline asr_t* make_ExplicitDeallocate_t(Allocator &al, const Location &a_loc, expr_t** a_vars, size_t n_vars) {
        ExplicitDeallocate_t *n;
        n = al.make_new<ExplicitDeallocate_t>();
        n->base.type = stmtType::ExplicitDeallocate;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_vars = a_vars;
        n->n_vars = n_vars;
        return (asr_t*)n;
    }

    struct ImplicitDeallocate_t // Constructor
    {
        const static stmtType class_type = stmtType::ImplicitDeallocate;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t** m_vars; size_t n_vars; // Sequence
    };
    static inline asr_t* make_ImplicitDeallocate_t(Allocator &al, const Location &a_loc, expr_t** a_vars, size_t n_vars) {
        ImplicitDeallocate_t *n;
        n = al.make_new<ImplicitDeallocate_t>();
        n->base.type = stmtType::ImplicitDeallocate;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_vars = a_vars;
        n->n_vars = n_vars;
        return (asr_t*)n;
    }

    struct DoConcurrentLoop_t // Constructor
    {
        const static stmtType class_type = stmtType::DoConcurrentLoop;
        typedef stmt_t parent_type;
        stmt_t base;
        do_loop_head_t m_head;
        expr_t** m_shared; size_t n_shared; // Sequence
        expr_t** m_local; size_t n_local; // Sequence
        reduction_expr_t* m_reduction; size_t n_reduction; // Sequence
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_DoConcurrentLoop_t(Allocator &al, const Location &a_loc, do_loop_head_t a_head, expr_t** a_shared, size_t n_shared, expr_t** a_local, size_t n_local, reduction_expr_t* a_reduction, size_t n_reduction, stmt_t** a_body, size_t n_body) {
        DoConcurrentLoop_t *n;
        n = al.make_new<DoConcurrentLoop_t>();
        n->base.type = stmtType::DoConcurrentLoop;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_head = a_head;
        n->m_shared = a_shared;
        n->n_shared = n_shared;
        n->m_local = a_local;
        n->n_local = n_local;
        n->m_reduction = a_reduction;
        n->n_reduction = n_reduction;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }

    struct DoLoop_t // Constructor
    {
        const static stmtType class_type = stmtType::DoLoop;
        typedef stmt_t parent_type;
        stmt_t base;
        char* m_name;
        do_loop_head_t m_head;
        stmt_t** m_body; size_t n_body; // Sequence
        stmt_t** m_orelse; size_t n_orelse; // Sequence
    };
    static inline asr_t* make_DoLoop_t(Allocator &al, const Location &a_loc, char* a_name, do_loop_head_t a_head, stmt_t** a_body, size_t n_body, stmt_t** a_orelse, size_t n_orelse) {
        DoLoop_t *n;
        n = al.make_new<DoLoop_t>();
        n->base.type = stmtType::DoLoop;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_name = a_name;
        n->m_head = a_head;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_orelse = a_orelse;
        n->n_orelse = n_orelse;
        return (asr_t*)n;
    }

    struct ErrorStop_t // Constructor
    {
        const static stmtType class_type = stmtType::ErrorStop;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_code;
    };
    static inline asr_t* make_ErrorStop_t(Allocator &al, const Location &a_loc, expr_t* a_code) {
        ErrorStop_t *n;
        n = al.make_new<ErrorStop_t>();
        n->base.type = stmtType::ErrorStop;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_code = a_code;
        return (asr_t*)n;
    }

    struct Exit_t // Constructor
    {
        const static stmtType class_type = stmtType::Exit;
        typedef stmt_t parent_type;
        stmt_t base;
        char* m_stmt_name;
    };
    static inline asr_t* make_Exit_t(Allocator &al, const Location &a_loc, char* a_stmt_name) {
        Exit_t *n;
        n = al.make_new<Exit_t>();
        n->base.type = stmtType::Exit;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_stmt_name = a_stmt_name;
        return (asr_t*)n;
    }

    struct ForAllSingle_t // Constructor
    {
        const static stmtType class_type = stmtType::ForAllSingle;
        typedef stmt_t parent_type;
        stmt_t base;
        do_loop_head_t m_head;
        stmt_t* m_assign_stmt;
    };
    static inline asr_t* make_ForAllSingle_t(Allocator &al, const Location &a_loc, do_loop_head_t a_head, stmt_t* a_assign_stmt) {
        ForAllSingle_t *n;
        n = al.make_new<ForAllSingle_t>();
        n->base.type = stmtType::ForAllSingle;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_head = a_head;
        n->m_assign_stmt = a_assign_stmt;
        return (asr_t*)n;
    }

    struct GoTo_t // Constructor
    {
        const static stmtType class_type = stmtType::GoTo;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_target_id;
        char* m_name;
    };
    static inline asr_t* make_GoTo_t(Allocator &al, const Location &a_loc, int64_t a_target_id, char* a_name) {
        GoTo_t *n;
        n = al.make_new<GoTo_t>();
        n->base.type = stmtType::GoTo;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_target_id = a_target_id;
        n->m_name = a_name;
        return (asr_t*)n;
    }

    struct GoToTarget_t // Constructor
    {
        const static stmtType class_type = stmtType::GoToTarget;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_id;
        char* m_name;
    };
    static inline asr_t* make_GoToTarget_t(Allocator &al, const Location &a_loc, int64_t a_id, char* a_name) {
        GoToTarget_t *n;
        n = al.make_new<GoToTarget_t>();
        n->base.type = stmtType::GoToTarget;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_id = a_id;
        n->m_name = a_name;
        return (asr_t*)n;
    }

    struct If_t // Constructor
    {
        const static stmtType class_type = stmtType::If;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_test;
        stmt_t** m_body; size_t n_body; // Sequence
        stmt_t** m_orelse; size_t n_orelse; // Sequence
    };
    static inline asr_t* make_If_t(Allocator &al, const Location &a_loc, expr_t* a_test, stmt_t** a_body, size_t n_body, stmt_t** a_orelse, size_t n_orelse) {
        If_t *n;
        n = al.make_new<If_t>();
        n->base.type = stmtType::If;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_orelse = a_orelse;
        n->n_orelse = n_orelse;
        return (asr_t*)n;
    }

    struct IfArithmetic_t // Constructor
    {
        const static stmtType class_type = stmtType::IfArithmetic;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_test;
        int64_t m_lt_label;
        int64_t m_eq_label;
        int64_t m_gt_label;
    };
    static inline asr_t* make_IfArithmetic_t(Allocator &al, const Location &a_loc, expr_t* a_test, int64_t a_lt_label, int64_t a_eq_label, int64_t a_gt_label) {
        IfArithmetic_t *n;
        n = al.make_new<IfArithmetic_t>();
        n->base.type = stmtType::IfArithmetic;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->m_lt_label = a_lt_label;
        n->m_eq_label = a_eq_label;
        n->m_gt_label = a_gt_label;
        return (asr_t*)n;
    }

    struct Print_t // Constructor
    {
        const static stmtType class_type = stmtType::Print;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t** m_values; size_t n_values; // Sequence
        expr_t* m_separator;
        expr_t* m_end;
    };
    static inline asr_t* make_Print_t(Allocator &al, const Location &a_loc, expr_t** a_values, size_t n_values, expr_t* a_separator, expr_t* a_end) {
        Print_t *n;
        n = al.make_new<Print_t>();
        n->base.type = stmtType::Print;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_values = a_values;
        n->n_values = n_values;
        n->m_separator = a_separator;
        n->m_end = a_end;
        return (asr_t*)n;
    }

    struct FileOpen_t // Constructor
    {
        const static stmtType class_type = stmtType::FileOpen;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_newunit;
        expr_t* m_filename;
        expr_t* m_status;
        expr_t* m_form;
    };
    static inline asr_t* make_FileOpen_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_newunit, expr_t* a_filename, expr_t* a_status, expr_t* a_form) {
        FileOpen_t *n;
        n = al.make_new<FileOpen_t>();
        n->base.type = stmtType::FileOpen;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_newunit = a_newunit;
        n->m_filename = a_filename;
        n->m_status = a_status;
        n->m_form = a_form;
        return (asr_t*)n;
    }

    struct FileClose_t // Constructor
    {
        const static stmtType class_type = stmtType::FileClose;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_iostat;
        expr_t* m_iomsg;
        expr_t* m_err;
        expr_t* m_status;
    };
    static inline asr_t* make_FileClose_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_iostat, expr_t* a_iomsg, expr_t* a_err, expr_t* a_status) {
        FileClose_t *n;
        n = al.make_new<FileClose_t>();
        n->base.type = stmtType::FileClose;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_iostat = a_iostat;
        n->m_iomsg = a_iomsg;
        n->m_err = a_err;
        n->m_status = a_status;
        return (asr_t*)n;
    }

    struct FileRead_t // Constructor
    {
        const static stmtType class_type = stmtType::FileRead;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_fmt;
        expr_t* m_iomsg;
        expr_t* m_iostat;
        expr_t* m_size;
        expr_t* m_id;
        expr_t** m_values; size_t n_values; // Sequence
        stmt_t* m_overloaded;
    };
    static inline asr_t* make_FileRead_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_fmt, expr_t* a_iomsg, expr_t* a_iostat, expr_t* a_size, expr_t* a_id, expr_t** a_values, size_t n_values, stmt_t* a_overloaded) {
        FileRead_t *n;
        n = al.make_new<FileRead_t>();
        n->base.type = stmtType::FileRead;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_fmt = a_fmt;
        n->m_iomsg = a_iomsg;
        n->m_iostat = a_iostat;
        n->m_size = a_size;
        n->m_id = a_id;
        n->m_values = a_values;
        n->n_values = n_values;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct FileBackspace_t // Constructor
    {
        const static stmtType class_type = stmtType::FileBackspace;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_iostat;
        expr_t* m_err;
    };
    static inline asr_t* make_FileBackspace_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_iostat, expr_t* a_err) {
        FileBackspace_t *n;
        n = al.make_new<FileBackspace_t>();
        n->base.type = stmtType::FileBackspace;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_iostat = a_iostat;
        n->m_err = a_err;
        return (asr_t*)n;
    }

    struct FileRewind_t // Constructor
    {
        const static stmtType class_type = stmtType::FileRewind;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_iostat;
        expr_t* m_err;
    };
    static inline asr_t* make_FileRewind_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_iostat, expr_t* a_err) {
        FileRewind_t *n;
        n = al.make_new<FileRewind_t>();
        n->base.type = stmtType::FileRewind;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_iostat = a_iostat;
        n->m_err = a_err;
        return (asr_t*)n;
    }

    struct FileInquire_t // Constructor
    {
        const static stmtType class_type = stmtType::FileInquire;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_file;
        expr_t* m_iostat;
        expr_t* m_err;
        expr_t* m_exist;
        expr_t* m_opened;
        expr_t* m_number;
        expr_t* m_named;
        expr_t* m_name;
        expr_t* m_access;
        expr_t* m_sequential;
        expr_t* m_direct;
        expr_t* m_form;
        expr_t* m_formatted;
        expr_t* m_unformatted;
        expr_t* m_recl;
        expr_t* m_nextrec;
        expr_t* m_blank;
        expr_t* m_position;
        expr_t* m_action;
        expr_t* m_read;
        expr_t* m_write;
        expr_t* m_readwrite;
        expr_t* m_delim;
        expr_t* m_pad;
        expr_t* m_flen;
        expr_t* m_blocksize;
        expr_t* m_convert;
        expr_t* m_carriagecontrol;
        expr_t* m_iolength;
    };
    static inline asr_t* make_FileInquire_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_file, expr_t* a_iostat, expr_t* a_err, expr_t* a_exist, expr_t* a_opened, expr_t* a_number, expr_t* a_named, expr_t* a_name, expr_t* a_access, expr_t* a_sequential, expr_t* a_direct, expr_t* a_form, expr_t* a_formatted, expr_t* a_unformatted, expr_t* a_recl, expr_t* a_nextrec, expr_t* a_blank, expr_t* a_position, expr_t* a_action, expr_t* a_read, expr_t* a_write, expr_t* a_readwrite, expr_t* a_delim, expr_t* a_pad, expr_t* a_flen, expr_t* a_blocksize, expr_t* a_convert, expr_t* a_carriagecontrol, expr_t* a_iolength) {
        FileInquire_t *n;
        n = al.make_new<FileInquire_t>();
        n->base.type = stmtType::FileInquire;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_file = a_file;
        n->m_iostat = a_iostat;
        n->m_err = a_err;
        n->m_exist = a_exist;
        n->m_opened = a_opened;
        n->m_number = a_number;
        n->m_named = a_named;
        n->m_name = a_name;
        n->m_access = a_access;
        n->m_sequential = a_sequential;
        n->m_direct = a_direct;
        n->m_form = a_form;
        n->m_formatted = a_formatted;
        n->m_unformatted = a_unformatted;
        n->m_recl = a_recl;
        n->m_nextrec = a_nextrec;
        n->m_blank = a_blank;
        n->m_position = a_position;
        n->m_action = a_action;
        n->m_read = a_read;
        n->m_write = a_write;
        n->m_readwrite = a_readwrite;
        n->m_delim = a_delim;
        n->m_pad = a_pad;
        n->m_flen = a_flen;
        n->m_blocksize = a_blocksize;
        n->m_convert = a_convert;
        n->m_carriagecontrol = a_carriagecontrol;
        n->m_iolength = a_iolength;
        return (asr_t*)n;
    }

    struct FileWrite_t // Constructor
    {
        const static stmtType class_type = stmtType::FileWrite;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_iomsg;
        expr_t* m_iostat;
        expr_t* m_id;
        expr_t** m_values; size_t n_values; // Sequence
        expr_t* m_separator;
        expr_t* m_end;
        stmt_t* m_overloaded;
    };
    static inline asr_t* make_FileWrite_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_iomsg, expr_t* a_iostat, expr_t* a_id, expr_t** a_values, size_t n_values, expr_t* a_separator, expr_t* a_end, stmt_t* a_overloaded) {
        FileWrite_t *n;
        n = al.make_new<FileWrite_t>();
        n->base.type = stmtType::FileWrite;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_iomsg = a_iomsg;
        n->m_iostat = a_iostat;
        n->m_id = a_id;
        n->m_values = a_values;
        n->n_values = n_values;
        n->m_separator = a_separator;
        n->m_end = a_end;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct Return_t // Constructor
    {
        const static stmtType class_type = stmtType::Return;
        typedef stmt_t parent_type;
        stmt_t base;
    };
    static inline asr_t* make_Return_t(Allocator &al, const Location &a_loc) {
        Return_t *n;
        n = al.make_new<Return_t>();
        n->base.type = stmtType::Return;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        return (asr_t*)n;
    }

    struct Select_t // Constructor
    {
        const static stmtType class_type = stmtType::Select;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_test;
        case_stmt_t** m_body; size_t n_body; // Sequence
        stmt_t** m_default; size_t n_default; // Sequence
        bool m_enable_fall_through;
    };
    static inline asr_t* make_Select_t(Allocator &al, const Location &a_loc, expr_t* a_test, case_stmt_t** a_body, size_t n_body, stmt_t** a_default, size_t n_default, bool a_enable_fall_through) {
        Select_t *n;
        n = al.make_new<Select_t>();
        n->base.type = stmtType::Select;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_default = a_default;
        n->n_default = n_default;
        n->m_enable_fall_through = a_enable_fall_through;
        return (asr_t*)n;
    }

    struct Stop_t // Constructor
    {
        const static stmtType class_type = stmtType::Stop;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_code;
    };
    static inline asr_t* make_Stop_t(Allocator &al, const Location &a_loc, expr_t* a_code) {
        Stop_t *n;
        n = al.make_new<Stop_t>();
        n->base.type = stmtType::Stop;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_code = a_code;
        return (asr_t*)n;
    }

    struct Assert_t // Constructor
    {
        const static stmtType class_type = stmtType::Assert;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_test;
        expr_t* m_msg;
    };
    static inline asr_t* make_Assert_t(Allocator &al, const Location &a_loc, expr_t* a_test, expr_t* a_msg) {
        Assert_t *n;
        n = al.make_new<Assert_t>();
        n->base.type = stmtType::Assert;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->m_msg = a_msg;
        return (asr_t*)n;
    }

    struct SubroutineCall_t // Constructor
    {
        const static stmtType class_type = stmtType::SubroutineCall;
        typedef stmt_t parent_type;
        stmt_t base;
        symbol_t* m_name;
        symbol_t* m_original_name;
        call_arg_t* m_args; size_t n_args; // Sequence
        expr_t* m_dt;
    };
    static inline asr_t* make_SubroutineCall_t(Allocator &al, const Location &a_loc, symbol_t* a_name, symbol_t* a_original_name, call_arg_t* a_args, size_t n_args, expr_t* a_dt) {
        SubroutineCall_t *n;
        n = al.make_new<SubroutineCall_t>();
        n->base.type = stmtType::SubroutineCall;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_name = a_name;
        n->m_original_name = a_original_name;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_dt = a_dt;
        return (asr_t*)n;
    }

    struct IntrinsicImpureSubroutine_t // Constructor
    {
        const static stmtType class_type = stmtType::IntrinsicImpureSubroutine;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_intrinsic_id;
        expr_t** m_args; size_t n_args; // Sequence
        int64_t m_overload_id;
    };
    static inline asr_t* make_IntrinsicImpureSubroutine_t(Allocator &al, const Location &a_loc, int64_t a_intrinsic_id, expr_t** a_args, size_t n_args, int64_t a_overload_id) {
        IntrinsicImpureSubroutine_t *n;
        n = al.make_new<IntrinsicImpureSubroutine_t>();
        n->base.type = stmtType::IntrinsicImpureSubroutine;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_intrinsic_id = a_intrinsic_id;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_overload_id = a_overload_id;
        return (asr_t*)n;
    }

    struct Where_t // Constructor
    {
        const static stmtType class_type = stmtType::Where;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_test;
        stmt_t** m_body; size_t n_body; // Sequence
        stmt_t** m_orelse; size_t n_orelse; // Sequence
    };
    static inline asr_t* make_Where_t(Allocator &al, const Location &a_loc, expr_t* a_test, stmt_t** a_body, size_t n_body, stmt_t** a_orelse, size_t n_orelse) {
        Where_t *n;
        n = al.make_new<Where_t>();
        n->base.type = stmtType::Where;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_orelse = a_orelse;
        n->n_orelse = n_orelse;
        return (asr_t*)n;
    }

    struct WhileLoop_t // Constructor
    {
        const static stmtType class_type = stmtType::WhileLoop;
        typedef stmt_t parent_type;
        stmt_t base;
        char* m_name;
        expr_t* m_test;
        stmt_t** m_body; size_t n_body; // Sequence
        stmt_t** m_orelse; size_t n_orelse; // Sequence
    };
    static inline asr_t* make_WhileLoop_t(Allocator &al, const Location &a_loc, char* a_name, expr_t* a_test, stmt_t** a_body, size_t n_body, stmt_t** a_orelse, size_t n_orelse) {
        WhileLoop_t *n;
        n = al.make_new<WhileLoop_t>();
        n->base.type = stmtType::WhileLoop;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_name = a_name;
        n->m_test = a_test;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_orelse = a_orelse;
        n->n_orelse = n_orelse;
        return (asr_t*)n;
    }

    struct Nullify_t // Constructor
    {
        const static stmtType class_type = stmtType::Nullify;
        typedef stmt_t parent_type;
        stmt_t base;
        symbol_t** m_vars; size_t n_vars; // Sequence
    };
    static inline asr_t* make_Nullify_t(Allocator &al, const Location &a_loc, symbol_t** a_vars, size_t n_vars) {
        Nullify_t *n;
        n = al.make_new<Nullify_t>();
        n->base.type = stmtType::Nullify;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_vars = a_vars;
        n->n_vars = n_vars;
        return (asr_t*)n;
    }

    struct Flush_t // Constructor
    {
        const static stmtType class_type = stmtType::Flush;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        expr_t* m_unit;
        expr_t* m_err;
        expr_t* m_iomsg;
        expr_t* m_iostat;
    };
    static inline asr_t* make_Flush_t(Allocator &al, const Location &a_loc, int64_t a_label, expr_t* a_unit, expr_t* a_err, expr_t* a_iomsg, expr_t* a_iostat) {
        Flush_t *n;
        n = al.make_new<Flush_t>();
        n->base.type = stmtType::Flush;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_unit = a_unit;
        n->m_err = a_err;
        n->m_iomsg = a_iomsg;
        n->m_iostat = a_iostat;
        return (asr_t*)n;
    }

    struct ListAppend_t // Constructor
    {
        const static stmtType class_type = stmtType::ListAppend;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
        expr_t* m_ele;
    };
    static inline asr_t* make_ListAppend_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_ele) {
        ListAppend_t *n;
        n = al.make_new<ListAppend_t>();
        n->base.type = stmtType::ListAppend;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_ele = a_ele;
        return (asr_t*)n;
    }

    struct AssociateBlockCall_t // Constructor
    {
        const static stmtType class_type = stmtType::AssociateBlockCall;
        typedef stmt_t parent_type;
        stmt_t base;
        symbol_t* m_m;
    };
    static inline asr_t* make_AssociateBlockCall_t(Allocator &al, const Location &a_loc, symbol_t* a_m) {
        AssociateBlockCall_t *n;
        n = al.make_new<AssociateBlockCall_t>();
        n->base.type = stmtType::AssociateBlockCall;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_m = a_m;
        return (asr_t*)n;
    }

    struct SelectType_t // Constructor
    {
        const static stmtType class_type = stmtType::SelectType;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_selector;
        type_stmt_t** m_body; size_t n_body; // Sequence
        stmt_t** m_default; size_t n_default; // Sequence
    };
    static inline asr_t* make_SelectType_t(Allocator &al, const Location &a_loc, expr_t* a_selector, type_stmt_t** a_body, size_t n_body, stmt_t** a_default, size_t n_default) {
        SelectType_t *n;
        n = al.make_new<SelectType_t>();
        n->base.type = stmtType::SelectType;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_selector = a_selector;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_default = a_default;
        n->n_default = n_default;
        return (asr_t*)n;
    }

    struct CPtrToPointer_t // Constructor
    {
        const static stmtType class_type = stmtType::CPtrToPointer;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_cptr;
        expr_t* m_ptr;
        expr_t* m_shape;
        expr_t* m_lower_bounds;
    };
    static inline asr_t* make_CPtrToPointer_t(Allocator &al, const Location &a_loc, expr_t* a_cptr, expr_t* a_ptr, expr_t* a_shape, expr_t* a_lower_bounds) {
        CPtrToPointer_t *n;
        n = al.make_new<CPtrToPointer_t>();
        n->base.type = stmtType::CPtrToPointer;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_cptr = a_cptr;
        n->m_ptr = a_ptr;
        n->m_shape = a_shape;
        n->m_lower_bounds = a_lower_bounds;
        return (asr_t*)n;
    }

    struct BlockCall_t // Constructor
    {
        const static stmtType class_type = stmtType::BlockCall;
        typedef stmt_t parent_type;
        stmt_t base;
        int64_t m_label;
        symbol_t* m_m;
    };
    static inline asr_t* make_BlockCall_t(Allocator &al, const Location &a_loc, int64_t a_label, symbol_t* a_m) {
        BlockCall_t *n;
        n = al.make_new<BlockCall_t>();
        n->base.type = stmtType::BlockCall;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_label = a_label;
        n->m_m = a_m;
        return (asr_t*)n;
    }

    struct SetInsert_t // Constructor
    {
        const static stmtType class_type = stmtType::SetInsert;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
        expr_t* m_ele;
    };
    static inline asr_t* make_SetInsert_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_ele) {
        SetInsert_t *n;
        n = al.make_new<SetInsert_t>();
        n->base.type = stmtType::SetInsert;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_ele = a_ele;
        return (asr_t*)n;
    }

    struct SetRemove_t // Constructor
    {
        const static stmtType class_type = stmtType::SetRemove;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
        expr_t* m_ele;
    };
    static inline asr_t* make_SetRemove_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_ele) {
        SetRemove_t *n;
        n = al.make_new<SetRemove_t>();
        n->base.type = stmtType::SetRemove;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_ele = a_ele;
        return (asr_t*)n;
    }

    struct ListInsert_t // Constructor
    {
        const static stmtType class_type = stmtType::ListInsert;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
        expr_t* m_pos;
        expr_t* m_ele;
    };
    static inline asr_t* make_ListInsert_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_pos, expr_t* a_ele) {
        ListInsert_t *n;
        n = al.make_new<ListInsert_t>();
        n->base.type = stmtType::ListInsert;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_pos = a_pos;
        n->m_ele = a_ele;
        return (asr_t*)n;
    }

    struct ListRemove_t // Constructor
    {
        const static stmtType class_type = stmtType::ListRemove;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
        expr_t* m_ele;
    };
    static inline asr_t* make_ListRemove_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_ele) {
        ListRemove_t *n;
        n = al.make_new<ListRemove_t>();
        n->base.type = stmtType::ListRemove;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_ele = a_ele;
        return (asr_t*)n;
    }

    struct ListClear_t // Constructor
    {
        const static stmtType class_type = stmtType::ListClear;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
    };
    static inline asr_t* make_ListClear_t(Allocator &al, const Location &a_loc, expr_t* a_a) {
        ListClear_t *n;
        n = al.make_new<ListClear_t>();
        n->base.type = stmtType::ListClear;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        return (asr_t*)n;
    }

    struct DictInsert_t // Constructor
    {
        const static stmtType class_type = stmtType::DictInsert;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_a;
        expr_t* m_key;
        expr_t* m_value;
    };
    static inline asr_t* make_DictInsert_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_key, expr_t* a_value) {
        DictInsert_t *n;
        n = al.make_new<DictInsert_t>();
        n->base.type = stmtType::DictInsert;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_key = a_key;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct Expr_t // Constructor
    {
        const static stmtType class_type = stmtType::Expr;
        typedef stmt_t parent_type;
        stmt_t base;
        expr_t* m_expression;
    };
    static inline asr_t* make_Expr_t(Allocator &al, const Location &a_loc, expr_t* a_expression) {
        Expr_t *n;
        n = al.make_new<Expr_t>();
        n->base.type = stmtType::Expr;
        n->base.base.type = asrType::stmt;
        n->base.base.loc = a_loc;
        n->m_expression = a_expression;
        return (asr_t*)n;
    }



enum exprType // Types
{
    IfExp, ComplexConstructor, NamedExpr, FunctionCall, IntrinsicElementalFunction, IntrinsicArrayFunction, IntrinsicImpureFunction, TypeInquiry, StructConstructor, EnumConstructor, UnionTypeConstructor, ImpliedDoLoop, IntegerConstant, IntegerBOZ, IntegerBitNot, IntegerUnaryMinus, IntegerCompare, IntegerBinOp, UnsignedIntegerConstant, UnsignedIntegerUnaryMinus, UnsignedIntegerBitNot, UnsignedIntegerCompare, UnsignedIntegerBinOp, RealConstant, RealUnaryMinus, RealCompare, RealBinOp, RealCopySign, ComplexConstant, ComplexUnaryMinus, ComplexCompare, ComplexBinOp, LogicalConstant, LogicalNot, LogicalCompare, LogicalBinOp, ListConstant, ListLen, ListConcat, ListCompare, ListCount, SetConstant, SetLen, TupleConstant, TupleLen, TupleCompare, TupleConcat, StringConstant, StringConcat, StringRepeat, StringLen, StringItem, StringSection, StringCompare, StringOrd, StringChr, StringFormat, CPtrCompare, SymbolicCompare, DictConstant, DictLen, Var, FunctionParam, ArrayConstructor, ArrayConstant, ArrayItem, ArraySection, ArraySize, ArrayBound, ArrayTranspose, ArrayPack, ArrayReshape, ArrayBroadcast, BitCast, StructInstanceMember, StructStaticMember, EnumStaticMember, UnionInstanceMember, EnumName, EnumValue, OverloadedCompare, OverloadedBinOp, OverloadedUnaryMinus, OverloadedStringConcat, Cast, ArrayPhysicalCast, ComplexRe, ComplexIm, DictItem, CLoc, PointerToCPtr, GetPointer, ListItem, TupleItem, ListSection, ListRepeat, DictPop, SetPop, IntegerBitLen, Ichar, Iachar, SizeOfType, PointerNullConstant, PointerAssociated, RealSqrt
};

struct expr_t // Sum
{
    const static asrType class_type = asrType::expr;
    asr_t base;
    exprType type;
};

    struct IfExp_t // Constructor
    {
        const static exprType class_type = exprType::IfExp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_test;
        expr_t* m_body;
        expr_t* m_orelse;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IfExp_t(Allocator &al, const Location &a_loc, expr_t* a_test, expr_t* a_body, expr_t* a_orelse, ttype_t* a_type, expr_t* a_value) {
        IfExp_t *n;
        n = al.make_new<IfExp_t>();
        n->base.type = exprType::IfExp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->m_body = a_body;
        n->m_orelse = a_orelse;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ComplexConstructor_t // Constructor
    {
        const static exprType class_type = exprType::ComplexConstructor;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_re;
        expr_t* m_im;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ComplexConstructor_t(Allocator &al, const Location &a_loc, expr_t* a_re, expr_t* a_im, ttype_t* a_type, expr_t* a_value) {
        ComplexConstructor_t *n;
        n = al.make_new<ComplexConstructor_t>();
        n->base.type = exprType::ComplexConstructor;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_re = a_re;
        n->m_im = a_im;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct NamedExpr_t // Constructor
    {
        const static exprType class_type = exprType::NamedExpr;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_target;
        expr_t* m_value;
        ttype_t* m_type;
    };
    static inline asr_t* make_NamedExpr_t(Allocator &al, const Location &a_loc, expr_t* a_target, expr_t* a_value, ttype_t* a_type) {
        NamedExpr_t *n;
        n = al.make_new<NamedExpr_t>();
        n->base.type = exprType::NamedExpr;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_target = a_target;
        n->m_value = a_value;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct FunctionCall_t // Constructor
    {
        const static exprType class_type = exprType::FunctionCall;
        typedef expr_t parent_type;
        expr_t base;
        symbol_t* m_name;
        symbol_t* m_original_name;
        call_arg_t* m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        expr_t* m_value;
        expr_t* m_dt;
    };
    static inline asr_t* make_FunctionCall_t(Allocator &al, const Location &a_loc, symbol_t* a_name, symbol_t* a_original_name, call_arg_t* a_args, size_t n_args, ttype_t* a_type, expr_t* a_value, expr_t* a_dt) {
        FunctionCall_t *n;
        n = al.make_new<FunctionCall_t>();
        n->base.type = exprType::FunctionCall;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_name = a_name;
        n->m_original_name = a_original_name;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_value = a_value;
        n->m_dt = a_dt;
        return (asr_t*)n;
    }

    struct IntrinsicElementalFunction_t // Constructor
    {
        const static exprType class_type = exprType::IntrinsicElementalFunction;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_intrinsic_id;
        expr_t** m_args; size_t n_args; // Sequence
        int64_t m_overload_id;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntrinsicElementalFunction_t(Allocator &al, const Location &a_loc, int64_t a_intrinsic_id, expr_t** a_args, size_t n_args, int64_t a_overload_id, ttype_t* a_type, expr_t* a_value) {
        IntrinsicElementalFunction_t *n;
        n = al.make_new<IntrinsicElementalFunction_t>();
        n->base.type = exprType::IntrinsicElementalFunction;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_intrinsic_id = a_intrinsic_id;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_overload_id = a_overload_id;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntrinsicArrayFunction_t // Constructor
    {
        const static exprType class_type = exprType::IntrinsicArrayFunction;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_arr_intrinsic_id;
        expr_t** m_args; size_t n_args; // Sequence
        int64_t m_overload_id;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntrinsicArrayFunction_t(Allocator &al, const Location &a_loc, int64_t a_arr_intrinsic_id, expr_t** a_args, size_t n_args, int64_t a_overload_id, ttype_t* a_type, expr_t* a_value) {
        IntrinsicArrayFunction_t *n;
        n = al.make_new<IntrinsicArrayFunction_t>();
        n->base.type = exprType::IntrinsicArrayFunction;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arr_intrinsic_id = a_arr_intrinsic_id;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_overload_id = a_overload_id;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntrinsicImpureFunction_t // Constructor
    {
        const static exprType class_type = exprType::IntrinsicImpureFunction;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_impure_intrinsic_id;
        expr_t** m_args; size_t n_args; // Sequence
        int64_t m_overload_id;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntrinsicImpureFunction_t(Allocator &al, const Location &a_loc, int64_t a_impure_intrinsic_id, expr_t** a_args, size_t n_args, int64_t a_overload_id, ttype_t* a_type, expr_t* a_value) {
        IntrinsicImpureFunction_t *n;
        n = al.make_new<IntrinsicImpureFunction_t>();
        n->base.type = exprType::IntrinsicImpureFunction;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_impure_intrinsic_id = a_impure_intrinsic_id;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_overload_id = a_overload_id;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct TypeInquiry_t // Constructor
    {
        const static exprType class_type = exprType::TypeInquiry;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_inquiry_id;
        ttype_t* m_arg_type;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_TypeInquiry_t(Allocator &al, const Location &a_loc, int64_t a_inquiry_id, ttype_t* a_arg_type, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        TypeInquiry_t *n;
        n = al.make_new<TypeInquiry_t>();
        n->base.type = exprType::TypeInquiry;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_inquiry_id = a_inquiry_id;
        n->m_arg_type = a_arg_type;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StructConstructor_t // Constructor
    {
        const static exprType class_type = exprType::StructConstructor;
        typedef expr_t parent_type;
        expr_t base;
        symbol_t* m_dt_sym;
        call_arg_t* m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StructConstructor_t(Allocator &al, const Location &a_loc, symbol_t* a_dt_sym, call_arg_t* a_args, size_t n_args, ttype_t* a_type, expr_t* a_value) {
        StructConstructor_t *n;
        n = al.make_new<StructConstructor_t>();
        n->base.type = exprType::StructConstructor;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_dt_sym = a_dt_sym;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct EnumConstructor_t // Constructor
    {
        const static exprType class_type = exprType::EnumConstructor;
        typedef expr_t parent_type;
        expr_t base;
        symbol_t* m_dt_sym;
        expr_t** m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_EnumConstructor_t(Allocator &al, const Location &a_loc, symbol_t* a_dt_sym, expr_t** a_args, size_t n_args, ttype_t* a_type, expr_t* a_value) {
        EnumConstructor_t *n;
        n = al.make_new<EnumConstructor_t>();
        n->base.type = exprType::EnumConstructor;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_dt_sym = a_dt_sym;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct UnionTypeConstructor_t // Constructor
    {
        const static exprType class_type = exprType::UnionTypeConstructor;
        typedef expr_t parent_type;
        expr_t base;
        symbol_t* m_dt_sym;
        expr_t** m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_UnionTypeConstructor_t(Allocator &al, const Location &a_loc, symbol_t* a_dt_sym, expr_t** a_args, size_t n_args, ttype_t* a_type, expr_t* a_value) {
        UnionTypeConstructor_t *n;
        n = al.make_new<UnionTypeConstructor_t>();
        n->base.type = exprType::UnionTypeConstructor;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_dt_sym = a_dt_sym;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ImpliedDoLoop_t // Constructor
    {
        const static exprType class_type = exprType::ImpliedDoLoop;
        typedef expr_t parent_type;
        expr_t base;
        expr_t** m_values; size_t n_values; // Sequence
        expr_t* m_var;
        expr_t* m_start;
        expr_t* m_end;
        expr_t* m_increment;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ImpliedDoLoop_t(Allocator &al, const Location &a_loc, expr_t** a_values, size_t n_values, expr_t* a_var, expr_t* a_start, expr_t* a_end, expr_t* a_increment, ttype_t* a_type, expr_t* a_value) {
        ImpliedDoLoop_t *n;
        n = al.make_new<ImpliedDoLoop_t>();
        n->base.type = exprType::ImpliedDoLoop;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_values = a_values;
        n->n_values = n_values;
        n->m_var = a_var;
        n->m_start = a_start;
        n->m_end = a_end;
        n->m_increment = a_increment;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntegerConstant_t // Constructor
    {
        const static exprType class_type = exprType::IntegerConstant;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_n;
        ttype_t* m_type;
    };
    static inline asr_t* make_IntegerConstant_t(Allocator &al, const Location &a_loc, int64_t a_n, ttype_t* a_type) {
        IntegerConstant_t *n;
        n = al.make_new<IntegerConstant_t>();
        n->base.type = exprType::IntegerConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_n = a_n;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct IntegerBOZ_t // Constructor
    {
        const static exprType class_type = exprType::IntegerBOZ;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_v;
        integerbozType m_intboz_type;
        ttype_t* m_type;
    };
    static inline asr_t* make_IntegerBOZ_t(Allocator &al, const Location &a_loc, int64_t a_v, integerbozType a_intboz_type, ttype_t* a_type) {
        IntegerBOZ_t *n;
        n = al.make_new<IntegerBOZ_t>();
        n->base.type = exprType::IntegerBOZ;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_intboz_type = a_intboz_type;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct IntegerBitNot_t // Constructor
    {
        const static exprType class_type = exprType::IntegerBitNot;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntegerBitNot_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        IntegerBitNot_t *n;
        n = al.make_new<IntegerBitNot_t>();
        n->base.type = exprType::IntegerBitNot;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntegerUnaryMinus_t // Constructor
    {
        const static exprType class_type = exprType::IntegerUnaryMinus;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntegerUnaryMinus_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        IntegerUnaryMinus_t *n;
        n = al.make_new<IntegerUnaryMinus_t>();
        n->base.type = exprType::IntegerUnaryMinus;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntegerCompare_t // Constructor
    {
        const static exprType class_type = exprType::IntegerCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntegerCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        IntegerCompare_t *n;
        n = al.make_new<IntegerCompare_t>();
        n->base.type = exprType::IntegerCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntegerBinOp_t // Constructor
    {
        const static exprType class_type = exprType::IntegerBinOp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        binopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntegerBinOp_t(Allocator &al, const Location &a_loc, expr_t* a_left, binopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        IntegerBinOp_t *n;
        n = al.make_new<IntegerBinOp_t>();
        n->base.type = exprType::IntegerBinOp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct UnsignedIntegerConstant_t // Constructor
    {
        const static exprType class_type = exprType::UnsignedIntegerConstant;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_n;
        ttype_t* m_type;
    };
    static inline asr_t* make_UnsignedIntegerConstant_t(Allocator &al, const Location &a_loc, int64_t a_n, ttype_t* a_type) {
        UnsignedIntegerConstant_t *n;
        n = al.make_new<UnsignedIntegerConstant_t>();
        n->base.type = exprType::UnsignedIntegerConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_n = a_n;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct UnsignedIntegerUnaryMinus_t // Constructor
    {
        const static exprType class_type = exprType::UnsignedIntegerUnaryMinus;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_UnsignedIntegerUnaryMinus_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        UnsignedIntegerUnaryMinus_t *n;
        n = al.make_new<UnsignedIntegerUnaryMinus_t>();
        n->base.type = exprType::UnsignedIntegerUnaryMinus;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct UnsignedIntegerBitNot_t // Constructor
    {
        const static exprType class_type = exprType::UnsignedIntegerBitNot;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_UnsignedIntegerBitNot_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        UnsignedIntegerBitNot_t *n;
        n = al.make_new<UnsignedIntegerBitNot_t>();
        n->base.type = exprType::UnsignedIntegerBitNot;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct UnsignedIntegerCompare_t // Constructor
    {
        const static exprType class_type = exprType::UnsignedIntegerCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_UnsignedIntegerCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        UnsignedIntegerCompare_t *n;
        n = al.make_new<UnsignedIntegerCompare_t>();
        n->base.type = exprType::UnsignedIntegerCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct UnsignedIntegerBinOp_t // Constructor
    {
        const static exprType class_type = exprType::UnsignedIntegerBinOp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        binopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_UnsignedIntegerBinOp_t(Allocator &al, const Location &a_loc, expr_t* a_left, binopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        UnsignedIntegerBinOp_t *n;
        n = al.make_new<UnsignedIntegerBinOp_t>();
        n->base.type = exprType::UnsignedIntegerBinOp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct RealConstant_t // Constructor
    {
        const static exprType class_type = exprType::RealConstant;
        typedef expr_t parent_type;
        expr_t base;
        double m_r;
        ttype_t* m_type;
    };
    static inline asr_t* make_RealConstant_t(Allocator &al, const Location &a_loc, double a_r, ttype_t* a_type) {
        RealConstant_t *n;
        n = al.make_new<RealConstant_t>();
        n->base.type = exprType::RealConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_r = a_r;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct RealUnaryMinus_t // Constructor
    {
        const static exprType class_type = exprType::RealUnaryMinus;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_RealUnaryMinus_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        RealUnaryMinus_t *n;
        n = al.make_new<RealUnaryMinus_t>();
        n->base.type = exprType::RealUnaryMinus;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct RealCompare_t // Constructor
    {
        const static exprType class_type = exprType::RealCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_RealCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        RealCompare_t *n;
        n = al.make_new<RealCompare_t>();
        n->base.type = exprType::RealCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct RealBinOp_t // Constructor
    {
        const static exprType class_type = exprType::RealBinOp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        binopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_RealBinOp_t(Allocator &al, const Location &a_loc, expr_t* a_left, binopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        RealBinOp_t *n;
        n = al.make_new<RealBinOp_t>();
        n->base.type = exprType::RealBinOp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct RealCopySign_t // Constructor
    {
        const static exprType class_type = exprType::RealCopySign;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_target;
        expr_t* m_source;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_RealCopySign_t(Allocator &al, const Location &a_loc, expr_t* a_target, expr_t* a_source, ttype_t* a_type, expr_t* a_value) {
        RealCopySign_t *n;
        n = al.make_new<RealCopySign_t>();
        n->base.type = exprType::RealCopySign;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_target = a_target;
        n->m_source = a_source;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ComplexConstant_t // Constructor
    {
        const static exprType class_type = exprType::ComplexConstant;
        typedef expr_t parent_type;
        expr_t base;
        double m_re;
        double m_im;
        ttype_t* m_type;
    };
    static inline asr_t* make_ComplexConstant_t(Allocator &al, const Location &a_loc, double a_re, double a_im, ttype_t* a_type) {
        ComplexConstant_t *n;
        n = al.make_new<ComplexConstant_t>();
        n->base.type = exprType::ComplexConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_re = a_re;
        n->m_im = a_im;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct ComplexUnaryMinus_t // Constructor
    {
        const static exprType class_type = exprType::ComplexUnaryMinus;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ComplexUnaryMinus_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        ComplexUnaryMinus_t *n;
        n = al.make_new<ComplexUnaryMinus_t>();
        n->base.type = exprType::ComplexUnaryMinus;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ComplexCompare_t // Constructor
    {
        const static exprType class_type = exprType::ComplexCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ComplexCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        ComplexCompare_t *n;
        n = al.make_new<ComplexCompare_t>();
        n->base.type = exprType::ComplexCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ComplexBinOp_t // Constructor
    {
        const static exprType class_type = exprType::ComplexBinOp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        binopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ComplexBinOp_t(Allocator &al, const Location &a_loc, expr_t* a_left, binopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        ComplexBinOp_t *n;
        n = al.make_new<ComplexBinOp_t>();
        n->base.type = exprType::ComplexBinOp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct LogicalConstant_t // Constructor
    {
        const static exprType class_type = exprType::LogicalConstant;
        typedef expr_t parent_type;
        expr_t base;
        bool m_value;
        ttype_t* m_type;
    };
    static inline asr_t* make_LogicalConstant_t(Allocator &al, const Location &a_loc, bool a_value, ttype_t* a_type) {
        LogicalConstant_t *n;
        n = al.make_new<LogicalConstant_t>();
        n->base.type = exprType::LogicalConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_value = a_value;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct LogicalNot_t // Constructor
    {
        const static exprType class_type = exprType::LogicalNot;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_LogicalNot_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        LogicalNot_t *n;
        n = al.make_new<LogicalNot_t>();
        n->base.type = exprType::LogicalNot;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct LogicalCompare_t // Constructor
    {
        const static exprType class_type = exprType::LogicalCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_LogicalCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        LogicalCompare_t *n;
        n = al.make_new<LogicalCompare_t>();
        n->base.type = exprType::LogicalCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct LogicalBinOp_t // Constructor
    {
        const static exprType class_type = exprType::LogicalBinOp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        logicalbinopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_LogicalBinOp_t(Allocator &al, const Location &a_loc, expr_t* a_left, logicalbinopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        LogicalBinOp_t *n;
        n = al.make_new<LogicalBinOp_t>();
        n->base.type = exprType::LogicalBinOp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListConstant_t // Constructor
    {
        const static exprType class_type = exprType::ListConstant;
        typedef expr_t parent_type;
        expr_t base;
        expr_t** m_args; size_t n_args; // Sequence
        ttype_t* m_type;
    };
    static inline asr_t* make_ListConstant_t(Allocator &al, const Location &a_loc, expr_t** a_args, size_t n_args, ttype_t* a_type) {
        ListConstant_t *n;
        n = al.make_new<ListConstant_t>();
        n->base.type = exprType::ListConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct ListLen_t // Constructor
    {
        const static exprType class_type = exprType::ListLen;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListLen_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        ListLen_t *n;
        n = al.make_new<ListLen_t>();
        n->base.type = exprType::ListLen;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListConcat_t // Constructor
    {
        const static exprType class_type = exprType::ListConcat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListConcat_t(Allocator &al, const Location &a_loc, expr_t* a_left, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        ListConcat_t *n;
        n = al.make_new<ListConcat_t>();
        n->base.type = exprType::ListConcat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListCompare_t // Constructor
    {
        const static exprType class_type = exprType::ListCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        ListCompare_t *n;
        n = al.make_new<ListCompare_t>();
        n->base.type = exprType::ListCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListCount_t // Constructor
    {
        const static exprType class_type = exprType::ListCount;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        expr_t* m_ele;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListCount_t(Allocator &al, const Location &a_loc, expr_t* a_arg, expr_t* a_ele, ttype_t* a_type, expr_t* a_value) {
        ListCount_t *n;
        n = al.make_new<ListCount_t>();
        n->base.type = exprType::ListCount;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_ele = a_ele;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct SetConstant_t // Constructor
    {
        const static exprType class_type = exprType::SetConstant;
        typedef expr_t parent_type;
        expr_t base;
        expr_t** m_elements; size_t n_elements; // Sequence
        ttype_t* m_type;
    };
    static inline asr_t* make_SetConstant_t(Allocator &al, const Location &a_loc, expr_t** a_elements, size_t n_elements, ttype_t* a_type) {
        SetConstant_t *n;
        n = al.make_new<SetConstant_t>();
        n->base.type = exprType::SetConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_elements = a_elements;
        n->n_elements = n_elements;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct SetLen_t // Constructor
    {
        const static exprType class_type = exprType::SetLen;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_SetLen_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        SetLen_t *n;
        n = al.make_new<SetLen_t>();
        n->base.type = exprType::SetLen;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct TupleConstant_t // Constructor
    {
        const static exprType class_type = exprType::TupleConstant;
        typedef expr_t parent_type;
        expr_t base;
        expr_t** m_elements; size_t n_elements; // Sequence
        ttype_t* m_type;
    };
    static inline asr_t* make_TupleConstant_t(Allocator &al, const Location &a_loc, expr_t** a_elements, size_t n_elements, ttype_t* a_type) {
        TupleConstant_t *n;
        n = al.make_new<TupleConstant_t>();
        n->base.type = exprType::TupleConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_elements = a_elements;
        n->n_elements = n_elements;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct TupleLen_t // Constructor
    {
        const static exprType class_type = exprType::TupleLen;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_TupleLen_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        TupleLen_t *n;
        n = al.make_new<TupleLen_t>();
        n->base.type = exprType::TupleLen;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct TupleCompare_t // Constructor
    {
        const static exprType class_type = exprType::TupleCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_TupleCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        TupleCompare_t *n;
        n = al.make_new<TupleCompare_t>();
        n->base.type = exprType::TupleCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct TupleConcat_t // Constructor
    {
        const static exprType class_type = exprType::TupleConcat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_TupleConcat_t(Allocator &al, const Location &a_loc, expr_t* a_left, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        TupleConcat_t *n;
        n = al.make_new<TupleConcat_t>();
        n->base.type = exprType::TupleConcat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringConstant_t // Constructor
    {
        const static exprType class_type = exprType::StringConstant;
        typedef expr_t parent_type;
        expr_t base;
        char* m_s;
        ttype_t* m_type;
    };
    static inline asr_t* make_StringConstant_t(Allocator &al, const Location &a_loc, char* a_s, ttype_t* a_type) {
        StringConstant_t *n;
        n = al.make_new<StringConstant_t>();
        n->base.type = exprType::StringConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_s = a_s;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct StringConcat_t // Constructor
    {
        const static exprType class_type = exprType::StringConcat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringConcat_t(Allocator &al, const Location &a_loc, expr_t* a_left, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        StringConcat_t *n;
        n = al.make_new<StringConcat_t>();
        n->base.type = exprType::StringConcat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringRepeat_t // Constructor
    {
        const static exprType class_type = exprType::StringRepeat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringRepeat_t(Allocator &al, const Location &a_loc, expr_t* a_left, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        StringRepeat_t *n;
        n = al.make_new<StringRepeat_t>();
        n->base.type = exprType::StringRepeat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringLen_t // Constructor
    {
        const static exprType class_type = exprType::StringLen;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringLen_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        StringLen_t *n;
        n = al.make_new<StringLen_t>();
        n->base.type = exprType::StringLen;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringItem_t // Constructor
    {
        const static exprType class_type = exprType::StringItem;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        expr_t* m_idx;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringItem_t(Allocator &al, const Location &a_loc, expr_t* a_arg, expr_t* a_idx, ttype_t* a_type, expr_t* a_value) {
        StringItem_t *n;
        n = al.make_new<StringItem_t>();
        n->base.type = exprType::StringItem;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_idx = a_idx;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringSection_t // Constructor
    {
        const static exprType class_type = exprType::StringSection;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        expr_t* m_start;
        expr_t* m_end;
        expr_t* m_step;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringSection_t(Allocator &al, const Location &a_loc, expr_t* a_arg, expr_t* a_start, expr_t* a_end, expr_t* a_step, ttype_t* a_type, expr_t* a_value) {
        StringSection_t *n;
        n = al.make_new<StringSection_t>();
        n->base.type = exprType::StringSection;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_start = a_start;
        n->m_end = a_end;
        n->m_step = a_step;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringCompare_t // Constructor
    {
        const static exprType class_type = exprType::StringCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        StringCompare_t *n;
        n = al.make_new<StringCompare_t>();
        n->base.type = exprType::StringCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringOrd_t // Constructor
    {
        const static exprType class_type = exprType::StringOrd;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringOrd_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        StringOrd_t *n;
        n = al.make_new<StringOrd_t>();
        n->base.type = exprType::StringOrd;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringChr_t // Constructor
    {
        const static exprType class_type = exprType::StringChr;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringChr_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        StringChr_t *n;
        n = al.make_new<StringChr_t>();
        n->base.type = exprType::StringChr;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StringFormat_t // Constructor
    {
        const static exprType class_type = exprType::StringFormat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_fmt;
        expr_t** m_args; size_t n_args; // Sequence
        string_format_kindType m_kind;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StringFormat_t(Allocator &al, const Location &a_loc, expr_t* a_fmt, expr_t** a_args, size_t n_args, string_format_kindType a_kind, ttype_t* a_type, expr_t* a_value) {
        StringFormat_t *n;
        n = al.make_new<StringFormat_t>();
        n->base.type = exprType::StringFormat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_fmt = a_fmt;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_kind = a_kind;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct CPtrCompare_t // Constructor
    {
        const static exprType class_type = exprType::CPtrCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_CPtrCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        CPtrCompare_t *n;
        n = al.make_new<CPtrCompare_t>();
        n->base.type = exprType::CPtrCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct SymbolicCompare_t // Constructor
    {
        const static exprType class_type = exprType::SymbolicCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_SymbolicCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        SymbolicCompare_t *n;
        n = al.make_new<SymbolicCompare_t>();
        n->base.type = exprType::SymbolicCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct DictConstant_t // Constructor
    {
        const static exprType class_type = exprType::DictConstant;
        typedef expr_t parent_type;
        expr_t base;
        expr_t** m_keys; size_t n_keys; // Sequence
        expr_t** m_values; size_t n_values; // Sequence
        ttype_t* m_type;
    };
    static inline asr_t* make_DictConstant_t(Allocator &al, const Location &a_loc, expr_t** a_keys, size_t n_keys, expr_t** a_values, size_t n_values, ttype_t* a_type) {
        DictConstant_t *n;
        n = al.make_new<DictConstant_t>();
        n->base.type = exprType::DictConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_keys = a_keys;
        n->n_keys = n_keys;
        n->m_values = a_values;
        n->n_values = n_values;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct DictLen_t // Constructor
    {
        const static exprType class_type = exprType::DictLen;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_DictLen_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        DictLen_t *n;
        n = al.make_new<DictLen_t>();
        n->base.type = exprType::DictLen;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct Var_t // Constructor
    {
        const static exprType class_type = exprType::Var;
        typedef expr_t parent_type;
        expr_t base;
        symbol_t* m_v;
    };
    static inline asr_t* make_Var_t(Allocator &al, const Location &a_loc, symbol_t* a_v) {
        Var_t *n;
        n = al.make_new<Var_t>();
        n->base.type = exprType::Var;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        return (asr_t*)n;
    }

    struct FunctionParam_t // Constructor
    {
        const static exprType class_type = exprType::FunctionParam;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_param_number;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_FunctionParam_t(Allocator &al, const Location &a_loc, int64_t a_param_number, ttype_t* a_type, expr_t* a_value) {
        FunctionParam_t *n;
        n = al.make_new<FunctionParam_t>();
        n->base.type = exprType::FunctionParam;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_param_number = a_param_number;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayConstructor_t // Constructor
    {
        const static exprType class_type = exprType::ArrayConstructor;
        typedef expr_t parent_type;
        expr_t base;
        expr_t** m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        expr_t* m_value;
        arraystorageType m_storage_format;
    };
    static inline asr_t* make_ArrayConstructor_t(Allocator &al, const Location &a_loc, expr_t** a_args, size_t n_args, ttype_t* a_type, expr_t* a_value, arraystorageType a_storage_format) {
        ArrayConstructor_t *n;
        n = al.make_new<ArrayConstructor_t>();
        n->base.type = exprType::ArrayConstructor;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_value = a_value;
        n->m_storage_format = a_storage_format;
        return (asr_t*)n;
    }

    struct ArrayConstant_t // Constructor
    {
        const static exprType class_type = exprType::ArrayConstant;
        typedef expr_t parent_type;
        expr_t base;
        int64_t m_n_data;
        void * m_data;
        ttype_t* m_type;
        arraystorageType m_storage_format;
    };
    static inline asr_t* make_ArrayConstant_t(Allocator &al, const Location &a_loc, int64_t a_n_data, void * a_data, ttype_t* a_type, arraystorageType a_storage_format) {
        ArrayConstant_t *n;
        n = al.make_new<ArrayConstant_t>();
        n->base.type = exprType::ArrayConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_n_data = a_n_data;
        n->m_data = a_data;
        n->m_type = a_type;
        n->m_storage_format = a_storage_format;
        return (asr_t*)n;
    }

    struct ArrayItem_t // Constructor
    {
        const static exprType class_type = exprType::ArrayItem;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        array_index_t* m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        arraystorageType m_storage_format;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayItem_t(Allocator &al, const Location &a_loc, expr_t* a_v, array_index_t* a_args, size_t n_args, ttype_t* a_type, arraystorageType a_storage_format, expr_t* a_value) {
        ArrayItem_t *n;
        n = al.make_new<ArrayItem_t>();
        n->base.type = exprType::ArrayItem;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_storage_format = a_storage_format;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArraySection_t // Constructor
    {
        const static exprType class_type = exprType::ArraySection;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        array_index_t* m_args; size_t n_args; // Sequence
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArraySection_t(Allocator &al, const Location &a_loc, expr_t* a_v, array_index_t* a_args, size_t n_args, ttype_t* a_type, expr_t* a_value) {
        ArraySection_t *n;
        n = al.make_new<ArraySection_t>();
        n->base.type = exprType::ArraySection;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_args = a_args;
        n->n_args = n_args;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArraySize_t // Constructor
    {
        const static exprType class_type = exprType::ArraySize;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        expr_t* m_dim;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArraySize_t(Allocator &al, const Location &a_loc, expr_t* a_v, expr_t* a_dim, ttype_t* a_type, expr_t* a_value) {
        ArraySize_t *n;
        n = al.make_new<ArraySize_t>();
        n->base.type = exprType::ArraySize;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_dim = a_dim;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayBound_t // Constructor
    {
        const static exprType class_type = exprType::ArrayBound;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        expr_t* m_dim;
        ttype_t* m_type;
        arrayboundType m_bound;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayBound_t(Allocator &al, const Location &a_loc, expr_t* a_v, expr_t* a_dim, ttype_t* a_type, arrayboundType a_bound, expr_t* a_value) {
        ArrayBound_t *n;
        n = al.make_new<ArrayBound_t>();
        n->base.type = exprType::ArrayBound;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_dim = a_dim;
        n->m_type = a_type;
        n->m_bound = a_bound;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayTranspose_t // Constructor
    {
        const static exprType class_type = exprType::ArrayTranspose;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_matrix;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayTranspose_t(Allocator &al, const Location &a_loc, expr_t* a_matrix, ttype_t* a_type, expr_t* a_value) {
        ArrayTranspose_t *n;
        n = al.make_new<ArrayTranspose_t>();
        n->base.type = exprType::ArrayTranspose;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_matrix = a_matrix;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayPack_t // Constructor
    {
        const static exprType class_type = exprType::ArrayPack;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_array;
        expr_t* m_mask;
        expr_t* m_vector;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayPack_t(Allocator &al, const Location &a_loc, expr_t* a_array, expr_t* a_mask, expr_t* a_vector, ttype_t* a_type, expr_t* a_value) {
        ArrayPack_t *n;
        n = al.make_new<ArrayPack_t>();
        n->base.type = exprType::ArrayPack;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_array = a_array;
        n->m_mask = a_mask;
        n->m_vector = a_vector;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayReshape_t // Constructor
    {
        const static exprType class_type = exprType::ArrayReshape;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_array;
        expr_t* m_shape;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayReshape_t(Allocator &al, const Location &a_loc, expr_t* a_array, expr_t* a_shape, ttype_t* a_type, expr_t* a_value) {
        ArrayReshape_t *n;
        n = al.make_new<ArrayReshape_t>();
        n->base.type = exprType::ArrayReshape;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_array = a_array;
        n->m_shape = a_shape;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayBroadcast_t // Constructor
    {
        const static exprType class_type = exprType::ArrayBroadcast;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_array;
        expr_t* m_shape;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayBroadcast_t(Allocator &al, const Location &a_loc, expr_t* a_array, expr_t* a_shape, ttype_t* a_type, expr_t* a_value) {
        ArrayBroadcast_t *n;
        n = al.make_new<ArrayBroadcast_t>();
        n->base.type = exprType::ArrayBroadcast;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_array = a_array;
        n->m_shape = a_shape;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct BitCast_t // Constructor
    {
        const static exprType class_type = exprType::BitCast;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_source;
        expr_t* m_mold;
        expr_t* m_size;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_BitCast_t(Allocator &al, const Location &a_loc, expr_t* a_source, expr_t* a_mold, expr_t* a_size, ttype_t* a_type, expr_t* a_value) {
        BitCast_t *n;
        n = al.make_new<BitCast_t>();
        n->base.type = exprType::BitCast;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_source = a_source;
        n->m_mold = a_mold;
        n->m_size = a_size;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StructInstanceMember_t // Constructor
    {
        const static exprType class_type = exprType::StructInstanceMember;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        symbol_t* m_m;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StructInstanceMember_t(Allocator &al, const Location &a_loc, expr_t* a_v, symbol_t* a_m, ttype_t* a_type, expr_t* a_value) {
        StructInstanceMember_t *n;
        n = al.make_new<StructInstanceMember_t>();
        n->base.type = exprType::StructInstanceMember;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_m = a_m;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct StructStaticMember_t // Constructor
    {
        const static exprType class_type = exprType::StructStaticMember;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        symbol_t* m_m;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_StructStaticMember_t(Allocator &al, const Location &a_loc, expr_t* a_v, symbol_t* a_m, ttype_t* a_type, expr_t* a_value) {
        StructStaticMember_t *n;
        n = al.make_new<StructStaticMember_t>();
        n->base.type = exprType::StructStaticMember;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_m = a_m;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct EnumStaticMember_t // Constructor
    {
        const static exprType class_type = exprType::EnumStaticMember;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        symbol_t* m_m;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_EnumStaticMember_t(Allocator &al, const Location &a_loc, expr_t* a_v, symbol_t* a_m, ttype_t* a_type, expr_t* a_value) {
        EnumStaticMember_t *n;
        n = al.make_new<EnumStaticMember_t>();
        n->base.type = exprType::EnumStaticMember;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_m = a_m;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct UnionInstanceMember_t // Constructor
    {
        const static exprType class_type = exprType::UnionInstanceMember;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        symbol_t* m_m;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_UnionInstanceMember_t(Allocator &al, const Location &a_loc, expr_t* a_v, symbol_t* a_m, ttype_t* a_type, expr_t* a_value) {
        UnionInstanceMember_t *n;
        n = al.make_new<UnionInstanceMember_t>();
        n->base.type = exprType::UnionInstanceMember;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_m = a_m;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct EnumName_t // Constructor
    {
        const static exprType class_type = exprType::EnumName;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        ttype_t* m_enum_type;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_EnumName_t(Allocator &al, const Location &a_loc, expr_t* a_v, ttype_t* a_enum_type, ttype_t* a_type, expr_t* a_value) {
        EnumName_t *n;
        n = al.make_new<EnumName_t>();
        n->base.type = exprType::EnumName;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_enum_type = a_enum_type;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct EnumValue_t // Constructor
    {
        const static exprType class_type = exprType::EnumValue;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_v;
        ttype_t* m_enum_type;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_EnumValue_t(Allocator &al, const Location &a_loc, expr_t* a_v, ttype_t* a_enum_type, ttype_t* a_type, expr_t* a_value) {
        EnumValue_t *n;
        n = al.make_new<EnumValue_t>();
        n->base.type = exprType::EnumValue;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_v = a_v;
        n->m_enum_type = a_enum_type;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct OverloadedCompare_t // Constructor
    {
        const static exprType class_type = exprType::OverloadedCompare;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        cmpopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
        expr_t* m_overloaded;
    };
    static inline asr_t* make_OverloadedCompare_t(Allocator &al, const Location &a_loc, expr_t* a_left, cmpopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value, expr_t* a_overloaded) {
        OverloadedCompare_t *n;
        n = al.make_new<OverloadedCompare_t>();
        n->base.type = exprType::OverloadedCompare;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct OverloadedBinOp_t // Constructor
    {
        const static exprType class_type = exprType::OverloadedBinOp;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        binopType m_op;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
        expr_t* m_overloaded;
    };
    static inline asr_t* make_OverloadedBinOp_t(Allocator &al, const Location &a_loc, expr_t* a_left, binopType a_op, expr_t* a_right, ttype_t* a_type, expr_t* a_value, expr_t* a_overloaded) {
        OverloadedBinOp_t *n;
        n = al.make_new<OverloadedBinOp_t>();
        n->base.type = exprType::OverloadedBinOp;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_op = a_op;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct OverloadedUnaryMinus_t // Constructor
    {
        const static exprType class_type = exprType::OverloadedUnaryMinus;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
        expr_t* m_overloaded;
    };
    static inline asr_t* make_OverloadedUnaryMinus_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value, expr_t* a_overloaded) {
        OverloadedUnaryMinus_t *n;
        n = al.make_new<OverloadedUnaryMinus_t>();
        n->base.type = exprType::OverloadedUnaryMinus;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct OverloadedStringConcat_t // Constructor
    {
        const static exprType class_type = exprType::OverloadedStringConcat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
        expr_t* m_overloaded;
    };
    static inline asr_t* make_OverloadedStringConcat_t(Allocator &al, const Location &a_loc, expr_t* a_left, expr_t* a_right, ttype_t* a_type, expr_t* a_value, expr_t* a_overloaded) {
        OverloadedStringConcat_t *n;
        n = al.make_new<OverloadedStringConcat_t>();
        n->base.type = exprType::OverloadedStringConcat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        n->m_overloaded = a_overloaded;
        return (asr_t*)n;
    }

    struct Cast_t // Constructor
    {
        const static exprType class_type = exprType::Cast;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        cast_kindType m_kind;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_Cast_t(Allocator &al, const Location &a_loc, expr_t* a_arg, cast_kindType a_kind, ttype_t* a_type, expr_t* a_value) {
        Cast_t *n;
        n = al.make_new<Cast_t>();
        n->base.type = exprType::Cast;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_kind = a_kind;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ArrayPhysicalCast_t // Constructor
    {
        const static exprType class_type = exprType::ArrayPhysicalCast;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        array_physical_typeType m_old;
        array_physical_typeType m_new;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ArrayPhysicalCast_t(Allocator &al, const Location &a_loc, expr_t* a_arg, array_physical_typeType a_old, array_physical_typeType a_new, ttype_t* a_type, expr_t* a_value) {
        ArrayPhysicalCast_t *n;
        n = al.make_new<ArrayPhysicalCast_t>();
        n->base.type = exprType::ArrayPhysicalCast;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_old = a_old;
        n->m_new = a_new;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ComplexRe_t // Constructor
    {
        const static exprType class_type = exprType::ComplexRe;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ComplexRe_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        ComplexRe_t *n;
        n = al.make_new<ComplexRe_t>();
        n->base.type = exprType::ComplexRe;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ComplexIm_t // Constructor
    {
        const static exprType class_type = exprType::ComplexIm;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ComplexIm_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        ComplexIm_t *n;
        n = al.make_new<ComplexIm_t>();
        n->base.type = exprType::ComplexIm;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct DictItem_t // Constructor
    {
        const static exprType class_type = exprType::DictItem;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        expr_t* m_key;
        expr_t* m_default;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_DictItem_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_key, expr_t* a_default, ttype_t* a_type, expr_t* a_value) {
        DictItem_t *n;
        n = al.make_new<DictItem_t>();
        n->base.type = exprType::DictItem;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_key = a_key;
        n->m_default = a_default;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct CLoc_t // Constructor
    {
        const static exprType class_type = exprType::CLoc;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_CLoc_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        CLoc_t *n;
        n = al.make_new<CLoc_t>();
        n->base.type = exprType::CLoc;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct PointerToCPtr_t // Constructor
    {
        const static exprType class_type = exprType::PointerToCPtr;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_PointerToCPtr_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        PointerToCPtr_t *n;
        n = al.make_new<PointerToCPtr_t>();
        n->base.type = exprType::PointerToCPtr;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct GetPointer_t // Constructor
    {
        const static exprType class_type = exprType::GetPointer;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_GetPointer_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        GetPointer_t *n;
        n = al.make_new<GetPointer_t>();
        n->base.type = exprType::GetPointer;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListItem_t // Constructor
    {
        const static exprType class_type = exprType::ListItem;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        expr_t* m_pos;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListItem_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_pos, ttype_t* a_type, expr_t* a_value) {
        ListItem_t *n;
        n = al.make_new<ListItem_t>();
        n->base.type = exprType::ListItem;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_pos = a_pos;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct TupleItem_t // Constructor
    {
        const static exprType class_type = exprType::TupleItem;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        expr_t* m_pos;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_TupleItem_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_pos, ttype_t* a_type, expr_t* a_value) {
        TupleItem_t *n;
        n = al.make_new<TupleItem_t>();
        n->base.type = exprType::TupleItem;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_pos = a_pos;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListSection_t // Constructor
    {
        const static exprType class_type = exprType::ListSection;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        array_index_t m_section;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListSection_t(Allocator &al, const Location &a_loc, expr_t* a_a, array_index_t a_section, ttype_t* a_type, expr_t* a_value) {
        ListSection_t *n;
        n = al.make_new<ListSection_t>();
        n->base.type = exprType::ListSection;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_section = a_section;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct ListRepeat_t // Constructor
    {
        const static exprType class_type = exprType::ListRepeat;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_left;
        expr_t* m_right;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_ListRepeat_t(Allocator &al, const Location &a_loc, expr_t* a_left, expr_t* a_right, ttype_t* a_type, expr_t* a_value) {
        ListRepeat_t *n;
        n = al.make_new<ListRepeat_t>();
        n->base.type = exprType::ListRepeat;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_left = a_left;
        n->m_right = a_right;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct DictPop_t // Constructor
    {
        const static exprType class_type = exprType::DictPop;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        expr_t* m_key;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_DictPop_t(Allocator &al, const Location &a_loc, expr_t* a_a, expr_t* a_key, ttype_t* a_type, expr_t* a_value) {
        DictPop_t *n;
        n = al.make_new<DictPop_t>();
        n->base.type = exprType::DictPop;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_key = a_key;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct SetPop_t // Constructor
    {
        const static exprType class_type = exprType::SetPop;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_SetPop_t(Allocator &al, const Location &a_loc, expr_t* a_a, ttype_t* a_type, expr_t* a_value) {
        SetPop_t *n;
        n = al.make_new<SetPop_t>();
        n->base.type = exprType::SetPop;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct IntegerBitLen_t // Constructor
    {
        const static exprType class_type = exprType::IntegerBitLen;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_a;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_IntegerBitLen_t(Allocator &al, const Location &a_loc, expr_t* a_a, ttype_t* a_type, expr_t* a_value) {
        IntegerBitLen_t *n;
        n = al.make_new<IntegerBitLen_t>();
        n->base.type = exprType::IntegerBitLen;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_a = a_a;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct Ichar_t // Constructor
    {
        const static exprType class_type = exprType::Ichar;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_Ichar_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        Ichar_t *n;
        n = al.make_new<Ichar_t>();
        n->base.type = exprType::Ichar;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct Iachar_t // Constructor
    {
        const static exprType class_type = exprType::Iachar;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_Iachar_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        Iachar_t *n;
        n = al.make_new<Iachar_t>();
        n->base.type = exprType::Iachar;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct SizeOfType_t // Constructor
    {
        const static exprType class_type = exprType::SizeOfType;
        typedef expr_t parent_type;
        expr_t base;
        ttype_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_SizeOfType_t(Allocator &al, const Location &a_loc, ttype_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        SizeOfType_t *n;
        n = al.make_new<SizeOfType_t>();
        n->base.type = exprType::SizeOfType;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct PointerNullConstant_t // Constructor
    {
        const static exprType class_type = exprType::PointerNullConstant;
        typedef expr_t parent_type;
        expr_t base;
        ttype_t* m_type;
    };
    static inline asr_t* make_PointerNullConstant_t(Allocator &al, const Location &a_loc, ttype_t* a_type) {
        PointerNullConstant_t *n;
        n = al.make_new<PointerNullConstant_t>();
        n->base.type = exprType::PointerNullConstant;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct PointerAssociated_t // Constructor
    {
        const static exprType class_type = exprType::PointerAssociated;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_ptr;
        expr_t* m_tgt;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_PointerAssociated_t(Allocator &al, const Location &a_loc, expr_t* a_ptr, expr_t* a_tgt, ttype_t* a_type, expr_t* a_value) {
        PointerAssociated_t *n;
        n = al.make_new<PointerAssociated_t>();
        n->base.type = exprType::PointerAssociated;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_ptr = a_ptr;
        n->m_tgt = a_tgt;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }

    struct RealSqrt_t // Constructor
    {
        const static exprType class_type = exprType::RealSqrt;
        typedef expr_t parent_type;
        expr_t base;
        expr_t* m_arg;
        ttype_t* m_type;
        expr_t* m_value;
    };
    static inline asr_t* make_RealSqrt_t(Allocator &al, const Location &a_loc, expr_t* a_arg, ttype_t* a_type, expr_t* a_value) {
        RealSqrt_t *n;
        n = al.make_new<RealSqrt_t>();
        n->base.type = exprType::RealSqrt;
        n->base.base.type = asrType::expr;
        n->base.base.loc = a_loc;
        n->m_arg = a_arg;
        n->m_type = a_type;
        n->m_value = a_value;
        return (asr_t*)n;
    }



enum ttypeType // Types
{
    Integer, UnsignedInteger, Real, Complex, Character, Logical, Set, List, Tuple, StructType, EnumType, Union, ClassType, Dict, Pointer, Allocatable, CPtr, SymbolicExpression, TypeParameter, Array, FunctionType
};

struct ttype_t // Sum
{
    const static asrType class_type = asrType::ttype;
    asr_t base;
    ttypeType type;
};

    struct Integer_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Integer;
        typedef ttype_t parent_type;
        ttype_t base;
        int64_t m_kind;
    };
    static inline asr_t* make_Integer_t(Allocator &al, const Location &a_loc, int64_t a_kind) {
        Integer_t *n;
        n = al.make_new<Integer_t>();
        n->base.type = ttypeType::Integer;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_kind = a_kind;
        return (asr_t*)n;
    }

    struct UnsignedInteger_t // Constructor
    {
        const static ttypeType class_type = ttypeType::UnsignedInteger;
        typedef ttype_t parent_type;
        ttype_t base;
        int64_t m_kind;
    };
    static inline asr_t* make_UnsignedInteger_t(Allocator &al, const Location &a_loc, int64_t a_kind) {
        UnsignedInteger_t *n;
        n = al.make_new<UnsignedInteger_t>();
        n->base.type = ttypeType::UnsignedInteger;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_kind = a_kind;
        return (asr_t*)n;
    }

    struct Real_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Real;
        typedef ttype_t parent_type;
        ttype_t base;
        int64_t m_kind;
    };
    static inline asr_t* make_Real_t(Allocator &al, const Location &a_loc, int64_t a_kind) {
        Real_t *n;
        n = al.make_new<Real_t>();
        n->base.type = ttypeType::Real;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_kind = a_kind;
        return (asr_t*)n;
    }

    struct Complex_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Complex;
        typedef ttype_t parent_type;
        ttype_t base;
        int64_t m_kind;
    };
    static inline asr_t* make_Complex_t(Allocator &al, const Location &a_loc, int64_t a_kind) {
        Complex_t *n;
        n = al.make_new<Complex_t>();
        n->base.type = ttypeType::Complex;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_kind = a_kind;
        return (asr_t*)n;
    }

    struct Character_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Character;
        typedef ttype_t parent_type;
        ttype_t base;
        int64_t m_kind;
        int64_t m_len;
        expr_t* m_len_expr;
    };
    static inline asr_t* make_Character_t(Allocator &al, const Location &a_loc, int64_t a_kind, int64_t a_len, expr_t* a_len_expr) {
        Character_t *n;
        n = al.make_new<Character_t>();
        n->base.type = ttypeType::Character;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_kind = a_kind;
        n->m_len = a_len;
        n->m_len_expr = a_len_expr;
        return (asr_t*)n;
    }

    struct Logical_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Logical;
        typedef ttype_t parent_type;
        ttype_t base;
        int64_t m_kind;
    };
    static inline asr_t* make_Logical_t(Allocator &al, const Location &a_loc, int64_t a_kind) {
        Logical_t *n;
        n = al.make_new<Logical_t>();
        n->base.type = ttypeType::Logical;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_kind = a_kind;
        return (asr_t*)n;
    }

    struct Set_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Set;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t* m_type;
    };
    static inline asr_t* make_Set_t(Allocator &al, const Location &a_loc, ttype_t* a_type) {
        Set_t *n;
        n = al.make_new<Set_t>();
        n->base.type = ttypeType::Set;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct List_t // Constructor
    {
        const static ttypeType class_type = ttypeType::List;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t* m_type;
    };
    static inline asr_t* make_List_t(Allocator &al, const Location &a_loc, ttype_t* a_type) {
        List_t *n;
        n = al.make_new<List_t>();
        n->base.type = ttypeType::List;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct Tuple_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Tuple;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t** m_type; size_t n_type; // Sequence
    };
    static inline asr_t* make_Tuple_t(Allocator &al, const Location &a_loc, ttype_t** a_type, size_t n_type) {
        Tuple_t *n;
        n = al.make_new<Tuple_t>();
        n->base.type = ttypeType::Tuple;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        n->n_type = n_type;
        return (asr_t*)n;
    }

    struct StructType_t // Constructor
    {
        const static ttypeType class_type = ttypeType::StructType;
        typedef ttype_t parent_type;
        ttype_t base;
        symbol_t* m_derived_type;
    };
    static inline asr_t* make_StructType_t(Allocator &al, const Location &a_loc, symbol_t* a_derived_type) {
        StructType_t *n;
        n = al.make_new<StructType_t>();
        n->base.type = ttypeType::StructType;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_derived_type = a_derived_type;
        return (asr_t*)n;
    }

    struct EnumType_t // Constructor
    {
        const static ttypeType class_type = ttypeType::EnumType;
        typedef ttype_t parent_type;
        ttype_t base;
        symbol_t* m_enum_type;
    };
    static inline asr_t* make_EnumType_t(Allocator &al, const Location &a_loc, symbol_t* a_enum_type) {
        EnumType_t *n;
        n = al.make_new<EnumType_t>();
        n->base.type = ttypeType::EnumType;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_enum_type = a_enum_type;
        return (asr_t*)n;
    }

    struct Union_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Union;
        typedef ttype_t parent_type;
        ttype_t base;
        symbol_t* m_union_type;
    };
    static inline asr_t* make_Union_t(Allocator &al, const Location &a_loc, symbol_t* a_union_type) {
        Union_t *n;
        n = al.make_new<Union_t>();
        n->base.type = ttypeType::Union;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_union_type = a_union_type;
        return (asr_t*)n;
    }

    struct ClassType_t // Constructor
    {
        const static ttypeType class_type = ttypeType::ClassType;
        typedef ttype_t parent_type;
        ttype_t base;
        symbol_t* m_class_type;
    };
    static inline asr_t* make_ClassType_t(Allocator &al, const Location &a_loc, symbol_t* a_class_type) {
        ClassType_t *n;
        n = al.make_new<ClassType_t>();
        n->base.type = ttypeType::ClassType;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_class_type = a_class_type;
        return (asr_t*)n;
    }

    struct Dict_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Dict;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t* m_key_type;
        ttype_t* m_value_type;
    };
    static inline asr_t* make_Dict_t(Allocator &al, const Location &a_loc, ttype_t* a_key_type, ttype_t* a_value_type) {
        Dict_t *n;
        n = al.make_new<Dict_t>();
        n->base.type = ttypeType::Dict;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_key_type = a_key_type;
        n->m_value_type = a_value_type;
        return (asr_t*)n;
    }

    struct Pointer_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Pointer;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t* m_type;
    };
    static inline asr_t* make_Pointer_t(Allocator &al, const Location &a_loc, ttype_t* a_type) {
        Pointer_t *n;
        n = al.make_new<Pointer_t>();
        n->base.type = ttypeType::Pointer;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct Allocatable_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Allocatable;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t* m_type;
    };
    static inline asr_t* make_Allocatable_t(Allocator &al, const Location &a_loc, ttype_t* a_type) {
        Allocatable_t *n;
        n = al.make_new<Allocatable_t>();
        n->base.type = ttypeType::Allocatable;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        return (asr_t*)n;
    }

    struct CPtr_t // Constructor
    {
        const static ttypeType class_type = ttypeType::CPtr;
        typedef ttype_t parent_type;
        ttype_t base;
    };
    static inline asr_t* make_CPtr_t(Allocator &al, const Location &a_loc) {
        CPtr_t *n;
        n = al.make_new<CPtr_t>();
        n->base.type = ttypeType::CPtr;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        return (asr_t*)n;
    }

    struct SymbolicExpression_t // Constructor
    {
        const static ttypeType class_type = ttypeType::SymbolicExpression;
        typedef ttype_t parent_type;
        ttype_t base;
    };
    static inline asr_t* make_SymbolicExpression_t(Allocator &al, const Location &a_loc) {
        SymbolicExpression_t *n;
        n = al.make_new<SymbolicExpression_t>();
        n->base.type = ttypeType::SymbolicExpression;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        return (asr_t*)n;
    }

    struct TypeParameter_t // Constructor
    {
        const static ttypeType class_type = ttypeType::TypeParameter;
        typedef ttype_t parent_type;
        ttype_t base;
        char* m_param;
    };
    static inline asr_t* make_TypeParameter_t(Allocator &al, const Location &a_loc, char* a_param) {
        TypeParameter_t *n;
        n = al.make_new<TypeParameter_t>();
        n->base.type = ttypeType::TypeParameter;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_param = a_param;
        return (asr_t*)n;
    }

    struct Array_t // Constructor
    {
        const static ttypeType class_type = ttypeType::Array;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t* m_type;
        dimension_t* m_dims; size_t n_dims; // Sequence
        array_physical_typeType m_physical_type;
    };
    static inline asr_t* make_Array_t(Allocator &al, const Location &a_loc, ttype_t* a_type, dimension_t* a_dims, size_t n_dims, array_physical_typeType a_physical_type) {
        Array_t *n;
        n = al.make_new<Array_t>();
        n->base.type = ttypeType::Array;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        n->m_dims = a_dims;
        n->n_dims = n_dims;
        n->m_physical_type = a_physical_type;
        return (asr_t*)n;
    }

    struct FunctionType_t // Constructor
    {
        const static ttypeType class_type = ttypeType::FunctionType;
        typedef ttype_t parent_type;
        ttype_t base;
        ttype_t** m_arg_types; size_t n_arg_types; // Sequence
        ttype_t* m_return_var_type;
        abiType m_abi;
        deftypeType m_deftype;
        char* m_bindc_name;
        bool m_elemental;
        bool m_pure;
        bool m_module;
        bool m_inline;
        bool m_static;
        symbol_t** m_restrictions; size_t n_restrictions; // Sequence
        bool m_is_restriction;
    };
    static inline asr_t* make_FunctionType_t(Allocator &al, const Location &a_loc, ttype_t** a_arg_types, size_t n_arg_types, ttype_t* a_return_var_type, abiType a_abi, deftypeType a_deftype, char* a_bindc_name, bool a_elemental, bool a_pure, bool a_module, bool a_inline, bool a_static, symbol_t** a_restrictions, size_t n_restrictions, bool a_is_restriction) {
        FunctionType_t *n;
        n = al.make_new<FunctionType_t>();
        n->base.type = ttypeType::FunctionType;
        n->base.base.type = asrType::ttype;
        n->base.base.loc = a_loc;
        n->m_arg_types = a_arg_types;
        n->n_arg_types = n_arg_types;
        n->m_return_var_type = a_return_var_type;
        n->m_abi = a_abi;
        n->m_deftype = a_deftype;
        n->m_bindc_name = a_bindc_name;
        n->m_elemental = a_elemental;
        n->m_pure = a_pure;
        n->m_module = a_module;
        n->m_inline = a_inline;
        n->m_static = a_static;
        n->m_restrictions = a_restrictions;
        n->n_restrictions = n_restrictions;
        n->m_is_restriction = a_is_restriction;
        return (asr_t*)n;
    }



enum attributeType // Types
{
    Attribute
};

struct attribute_t // Sum
{
    const static asrType class_type = asrType::attribute;
    asr_t base;
    attributeType type;
};

    struct Attribute_t // Constructor
    {
        const static attributeType class_type = attributeType::Attribute;
        typedef attribute_t parent_type;
        attribute_t base;
        char* m_name;
        attribute_arg_t* m_args; size_t n_args; // Sequence
    };
    static inline asr_t* make_Attribute_t(Allocator &al, const Location &a_loc, char* a_name, attribute_arg_t* a_args, size_t n_args) {
        Attribute_t *n;
        n = al.make_new<Attribute_t>();
        n->base.type = attributeType::Attribute;
        n->base.base.type = asrType::attribute;
        n->base.base.loc = a_loc;
        n->m_name = a_name;
        n->m_args = a_args;
        n->n_args = n_args;
        return (asr_t*)n;
    }



enum tbindType // Types
{
    Bind
};

struct tbind_t // Sum
{
    const static asrType class_type = asrType::tbind;
    asr_t base;
    tbindType type;
};

    struct Bind_t // Constructor
    {
        const static tbindType class_type = tbindType::Bind;
        typedef tbind_t parent_type;
        tbind_t base;
        char* m_lang;
        char* m_name;
    };
    static inline asr_t* make_Bind_t(Allocator &al, const Location &a_loc, char* a_lang, char* a_name) {
        Bind_t *n;
        n = al.make_new<Bind_t>();
        n->base.type = tbindType::Bind;
        n->base.base.type = asrType::tbind;
        n->base.base.loc = a_loc;
        n->m_lang = a_lang;
        n->m_name = a_name;
        return (asr_t*)n;
    }



enum case_stmtType // Types
{
    CaseStmt, CaseStmt_Range
};

struct case_stmt_t // Sum
{
    const static asrType class_type = asrType::case_stmt;
    asr_t base;
    case_stmtType type;
};

    struct CaseStmt_t // Constructor
    {
        const static case_stmtType class_type = case_stmtType::CaseStmt;
        typedef case_stmt_t parent_type;
        case_stmt_t base;
        expr_t** m_test; size_t n_test; // Sequence
        stmt_t** m_body; size_t n_body; // Sequence
        bool m_fall_through;
    };
    static inline asr_t* make_CaseStmt_t(Allocator &al, const Location &a_loc, expr_t** a_test, size_t n_test, stmt_t** a_body, size_t n_body, bool a_fall_through) {
        CaseStmt_t *n;
        n = al.make_new<CaseStmt_t>();
        n->base.type = case_stmtType::CaseStmt;
        n->base.base.type = asrType::case_stmt;
        n->base.base.loc = a_loc;
        n->m_test = a_test;
        n->n_test = n_test;
        n->m_body = a_body;
        n->n_body = n_body;
        n->m_fall_through = a_fall_through;
        return (asr_t*)n;
    }

    struct CaseStmt_Range_t // Constructor
    {
        const static case_stmtType class_type = case_stmtType::CaseStmt_Range;
        typedef case_stmt_t parent_type;
        case_stmt_t base;
        expr_t* m_start;
        expr_t* m_end;
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_CaseStmt_Range_t(Allocator &al, const Location &a_loc, expr_t* a_start, expr_t* a_end, stmt_t** a_body, size_t n_body) {
        CaseStmt_Range_t *n;
        n = al.make_new<CaseStmt_Range_t>();
        n->base.type = case_stmtType::CaseStmt_Range;
        n->base.base.type = asrType::case_stmt;
        n->base.base.loc = a_loc;
        n->m_start = a_start;
        n->m_end = a_end;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }



enum type_stmtType // Types
{
    TypeStmtName, ClassStmt, TypeStmtType
};

struct type_stmt_t // Sum
{
    const static asrType class_type = asrType::type_stmt;
    asr_t base;
    type_stmtType type;
};

    struct TypeStmtName_t // Constructor
    {
        const static type_stmtType class_type = type_stmtType::TypeStmtName;
        typedef type_stmt_t parent_type;
        type_stmt_t base;
        symbol_t* m_sym;
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_TypeStmtName_t(Allocator &al, const Location &a_loc, symbol_t* a_sym, stmt_t** a_body, size_t n_body) {
        TypeStmtName_t *n;
        n = al.make_new<TypeStmtName_t>();
        n->base.type = type_stmtType::TypeStmtName;
        n->base.base.type = asrType::type_stmt;
        n->base.base.loc = a_loc;
        n->m_sym = a_sym;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }

    struct ClassStmt_t // Constructor
    {
        const static type_stmtType class_type = type_stmtType::ClassStmt;
        typedef type_stmt_t parent_type;
        type_stmt_t base;
        symbol_t* m_sym;
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_ClassStmt_t(Allocator &al, const Location &a_loc, symbol_t* a_sym, stmt_t** a_body, size_t n_body) {
        ClassStmt_t *n;
        n = al.make_new<ClassStmt_t>();
        n->base.type = type_stmtType::ClassStmt;
        n->base.base.type = asrType::type_stmt;
        n->base.base.loc = a_loc;
        n->m_sym = a_sym;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }

    struct TypeStmtType_t // Constructor
    {
        const static type_stmtType class_type = type_stmtType::TypeStmtType;
        typedef type_stmt_t parent_type;
        type_stmt_t base;
        ttype_t* m_type;
        stmt_t** m_body; size_t n_body; // Sequence
    };
    static inline asr_t* make_TypeStmtType_t(Allocator &al, const Location &a_loc, ttype_t* a_type, stmt_t** a_body, size_t n_body) {
        TypeStmtType_t *n;
        n = al.make_new<TypeStmtType_t>();
        n->base.type = type_stmtType::TypeStmtType;
        n->base.base.type = asrType::type_stmt;
        n->base.base.loc = a_loc;
        n->m_type = a_type;
        n->m_body = a_body;
        n->n_body = n_body;
        return (asr_t*)n;
    }



enum require_instantiationType // Types
{
    Require
};

struct require_instantiation_t // Sum
{
    const static asrType class_type = asrType::require_instantiation;
    asr_t base;
    require_instantiationType type;
};

    struct Require_t // Constructor
    {
        const static require_instantiationType class_type = require_instantiationType::Require;
        typedef require_instantiation_t parent_type;
        require_instantiation_t base;
        char* m_name;
        char** m_args; size_t n_args; // Sequence
    };
    static inline asr_t* make_Require_t(Allocator &al, const Location &a_loc, char* a_name, char** a_args, size_t n_args) {
        Require_t *n;
        n = al.make_new<Require_t>();
        n->base.type = require_instantiationType::Require;
        n->base.base.type = asrType::require_instantiation;
        n->base.base.loc = a_loc;
        n->m_name = a_name;
        n->m_args = a_args;
        n->n_args = n_args;
        return (asr_t*)n;
    }





/******************************************************************************/
// Visitor functions

template <class Visitor>
static void visit_unit_t(const unit_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::unit)
    switch (x.type) {
        case unitType::TranslationUnit: { v.visit_TranslationUnit((const TranslationUnit_t &)x); return; }
    }
}

template <class Visitor>
static void visit_symbol_t(const symbol_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::symbol)
    switch (x.type) {
        case symbolType::Program: { v.visit_Program((const Program_t &)x); return; }
        case symbolType::Module: { v.visit_Module((const Module_t &)x); return; }
        case symbolType::Function: { v.visit_Function((const Function_t &)x); return; }
        case symbolType::GenericProcedure: { v.visit_GenericProcedure((const GenericProcedure_t &)x); return; }
        case symbolType::CustomOperator: { v.visit_CustomOperator((const CustomOperator_t &)x); return; }
        case symbolType::ExternalSymbol: { v.visit_ExternalSymbol((const ExternalSymbol_t &)x); return; }
        case symbolType::Struct: { v.visit_Struct((const Struct_t &)x); return; }
        case symbolType::Enum: { v.visit_Enum((const Enum_t &)x); return; }
        case symbolType::UnionType: { v.visit_UnionType((const UnionType_t &)x); return; }
        case symbolType::Variable: { v.visit_Variable((const Variable_t &)x); return; }
        case symbolType::Class: { v.visit_Class((const Class_t &)x); return; }
        case symbolType::ClassProcedure: { v.visit_ClassProcedure((const ClassProcedure_t &)x); return; }
        case symbolType::AssociateBlock: { v.visit_AssociateBlock((const AssociateBlock_t &)x); return; }
        case symbolType::Block: { v.visit_Block((const Block_t &)x); return; }
        case symbolType::Requirement: { v.visit_Requirement((const Requirement_t &)x); return; }
        case symbolType::Template: { v.visit_Template((const Template_t &)x); return; }
    }
}

template <class Visitor>
static void visit_stmt_t(const stmt_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::stmt)
    switch (x.type) {
        case stmtType::Allocate: { v.visit_Allocate((const Allocate_t &)x); return; }
        case stmtType::ReAlloc: { v.visit_ReAlloc((const ReAlloc_t &)x); return; }
        case stmtType::Assign: { v.visit_Assign((const Assign_t &)x); return; }
        case stmtType::Assignment: { v.visit_Assignment((const Assignment_t &)x); return; }
        case stmtType::Associate: { v.visit_Associate((const Associate_t &)x); return; }
        case stmtType::Cycle: { v.visit_Cycle((const Cycle_t &)x); return; }
        case stmtType::ExplicitDeallocate: { v.visit_ExplicitDeallocate((const ExplicitDeallocate_t &)x); return; }
        case stmtType::ImplicitDeallocate: { v.visit_ImplicitDeallocate((const ImplicitDeallocate_t &)x); return; }
        case stmtType::DoConcurrentLoop: { v.visit_DoConcurrentLoop((const DoConcurrentLoop_t &)x); return; }
        case stmtType::DoLoop: { v.visit_DoLoop((const DoLoop_t &)x); return; }
        case stmtType::ErrorStop: { v.visit_ErrorStop((const ErrorStop_t &)x); return; }
        case stmtType::Exit: { v.visit_Exit((const Exit_t &)x); return; }
        case stmtType::ForAllSingle: { v.visit_ForAllSingle((const ForAllSingle_t &)x); return; }
        case stmtType::GoTo: { v.visit_GoTo((const GoTo_t &)x); return; }
        case stmtType::GoToTarget: { v.visit_GoToTarget((const GoToTarget_t &)x); return; }
        case stmtType::If: { v.visit_If((const If_t &)x); return; }
        case stmtType::IfArithmetic: { v.visit_IfArithmetic((const IfArithmetic_t &)x); return; }
        case stmtType::Print: { v.visit_Print((const Print_t &)x); return; }
        case stmtType::FileOpen: { v.visit_FileOpen((const FileOpen_t &)x); return; }
        case stmtType::FileClose: { v.visit_FileClose((const FileClose_t &)x); return; }
        case stmtType::FileRead: { v.visit_FileRead((const FileRead_t &)x); return; }
        case stmtType::FileBackspace: { v.visit_FileBackspace((const FileBackspace_t &)x); return; }
        case stmtType::FileRewind: { v.visit_FileRewind((const FileRewind_t &)x); return; }
        case stmtType::FileInquire: { v.visit_FileInquire((const FileInquire_t &)x); return; }
        case stmtType::FileWrite: { v.visit_FileWrite((const FileWrite_t &)x); return; }
        case stmtType::Return: { v.visit_Return((const Return_t &)x); return; }
        case stmtType::Select: { v.visit_Select((const Select_t &)x); return; }
        case stmtType::Stop: { v.visit_Stop((const Stop_t &)x); return; }
        case stmtType::Assert: { v.visit_Assert((const Assert_t &)x); return; }
        case stmtType::SubroutineCall: { v.visit_SubroutineCall((const SubroutineCall_t &)x); return; }
        case stmtType::IntrinsicImpureSubroutine: { v.visit_IntrinsicImpureSubroutine((const IntrinsicImpureSubroutine_t &)x); return; }
        case stmtType::Where: { v.visit_Where((const Where_t &)x); return; }
        case stmtType::WhileLoop: { v.visit_WhileLoop((const WhileLoop_t &)x); return; }
        case stmtType::Nullify: { v.visit_Nullify((const Nullify_t &)x); return; }
        case stmtType::Flush: { v.visit_Flush((const Flush_t &)x); return; }
        case stmtType::ListAppend: { v.visit_ListAppend((const ListAppend_t &)x); return; }
        case stmtType::AssociateBlockCall: { v.visit_AssociateBlockCall((const AssociateBlockCall_t &)x); return; }
        case stmtType::SelectType: { v.visit_SelectType((const SelectType_t &)x); return; }
        case stmtType::CPtrToPointer: { v.visit_CPtrToPointer((const CPtrToPointer_t &)x); return; }
        case stmtType::BlockCall: { v.visit_BlockCall((const BlockCall_t &)x); return; }
        case stmtType::SetInsert: { v.visit_SetInsert((const SetInsert_t &)x); return; }
        case stmtType::SetRemove: { v.visit_SetRemove((const SetRemove_t &)x); return; }
        case stmtType::ListInsert: { v.visit_ListInsert((const ListInsert_t &)x); return; }
        case stmtType::ListRemove: { v.visit_ListRemove((const ListRemove_t &)x); return; }
        case stmtType::ListClear: { v.visit_ListClear((const ListClear_t &)x); return; }
        case stmtType::DictInsert: { v.visit_DictInsert((const DictInsert_t &)x); return; }
        case stmtType::Expr: { v.visit_Expr((const Expr_t &)x); return; }
    }
}

template <class Visitor>
static void visit_expr_t(const expr_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::expr)
    switch (x.type) {
        case exprType::IfExp: { v.visit_IfExp((const IfExp_t &)x); return; }
        case exprType::ComplexConstructor: { v.visit_ComplexConstructor((const ComplexConstructor_t &)x); return; }
        case exprType::NamedExpr: { v.visit_NamedExpr((const NamedExpr_t &)x); return; }
        case exprType::FunctionCall: { v.visit_FunctionCall((const FunctionCall_t &)x); return; }
        case exprType::IntrinsicElementalFunction: { v.visit_IntrinsicElementalFunction((const IntrinsicElementalFunction_t &)x); return; }
        case exprType::IntrinsicArrayFunction: { v.visit_IntrinsicArrayFunction((const IntrinsicArrayFunction_t &)x); return; }
        case exprType::IntrinsicImpureFunction: { v.visit_IntrinsicImpureFunction((const IntrinsicImpureFunction_t &)x); return; }
        case exprType::TypeInquiry: { v.visit_TypeInquiry((const TypeInquiry_t &)x); return; }
        case exprType::StructConstructor: { v.visit_StructConstructor((const StructConstructor_t &)x); return; }
        case exprType::EnumConstructor: { v.visit_EnumConstructor((const EnumConstructor_t &)x); return; }
        case exprType::UnionTypeConstructor: { v.visit_UnionTypeConstructor((const UnionTypeConstructor_t &)x); return; }
        case exprType::ImpliedDoLoop: { v.visit_ImpliedDoLoop((const ImpliedDoLoop_t &)x); return; }
        case exprType::IntegerConstant: { v.visit_IntegerConstant((const IntegerConstant_t &)x); return; }
        case exprType::IntegerBOZ: { v.visit_IntegerBOZ((const IntegerBOZ_t &)x); return; }
        case exprType::IntegerBitNot: { v.visit_IntegerBitNot((const IntegerBitNot_t &)x); return; }
        case exprType::IntegerUnaryMinus: { v.visit_IntegerUnaryMinus((const IntegerUnaryMinus_t &)x); return; }
        case exprType::IntegerCompare: { v.visit_IntegerCompare((const IntegerCompare_t &)x); return; }
        case exprType::IntegerBinOp: { v.visit_IntegerBinOp((const IntegerBinOp_t &)x); return; }
        case exprType::UnsignedIntegerConstant: { v.visit_UnsignedIntegerConstant((const UnsignedIntegerConstant_t &)x); return; }
        case exprType::UnsignedIntegerUnaryMinus: { v.visit_UnsignedIntegerUnaryMinus((const UnsignedIntegerUnaryMinus_t &)x); return; }
        case exprType::UnsignedIntegerBitNot: { v.visit_UnsignedIntegerBitNot((const UnsignedIntegerBitNot_t &)x); return; }
        case exprType::UnsignedIntegerCompare: { v.visit_UnsignedIntegerCompare((const UnsignedIntegerCompare_t &)x); return; }
        case exprType::UnsignedIntegerBinOp: { v.visit_UnsignedIntegerBinOp((const UnsignedIntegerBinOp_t &)x); return; }
        case exprType::RealConstant: { v.visit_RealConstant((const RealConstant_t &)x); return; }
        case exprType::RealUnaryMinus: { v.visit_RealUnaryMinus((const RealUnaryMinus_t &)x); return; }
        case exprType::RealCompare: { v.visit_RealCompare((const RealCompare_t &)x); return; }
        case exprType::RealBinOp: { v.visit_RealBinOp((const RealBinOp_t &)x); return; }
        case exprType::RealCopySign: { v.visit_RealCopySign((const RealCopySign_t &)x); return; }
        case exprType::ComplexConstant: { v.visit_ComplexConstant((const ComplexConstant_t &)x); return; }
        case exprType::ComplexUnaryMinus: { v.visit_ComplexUnaryMinus((const ComplexUnaryMinus_t &)x); return; }
        case exprType::ComplexCompare: { v.visit_ComplexCompare((const ComplexCompare_t &)x); return; }
        case exprType::ComplexBinOp: { v.visit_ComplexBinOp((const ComplexBinOp_t &)x); return; }
        case exprType::LogicalConstant: { v.visit_LogicalConstant((const LogicalConstant_t &)x); return; }
        case exprType::LogicalNot: { v.visit_LogicalNot((const LogicalNot_t &)x); return; }
        case exprType::LogicalCompare: { v.visit_LogicalCompare((const LogicalCompare_t &)x); return; }
        case exprType::LogicalBinOp: { v.visit_LogicalBinOp((const LogicalBinOp_t &)x); return; }
        case exprType::ListConstant: { v.visit_ListConstant((const ListConstant_t &)x); return; }
        case exprType::ListLen: { v.visit_ListLen((const ListLen_t &)x); return; }
        case exprType::ListConcat: { v.visit_ListConcat((const ListConcat_t &)x); return; }
        case exprType::ListCompare: { v.visit_ListCompare((const ListCompare_t &)x); return; }
        case exprType::ListCount: { v.visit_ListCount((const ListCount_t &)x); return; }
        case exprType::SetConstant: { v.visit_SetConstant((const SetConstant_t &)x); return; }
        case exprType::SetLen: { v.visit_SetLen((const SetLen_t &)x); return; }
        case exprType::TupleConstant: { v.visit_TupleConstant((const TupleConstant_t &)x); return; }
        case exprType::TupleLen: { v.visit_TupleLen((const TupleLen_t &)x); return; }
        case exprType::TupleCompare: { v.visit_TupleCompare((const TupleCompare_t &)x); return; }
        case exprType::TupleConcat: { v.visit_TupleConcat((const TupleConcat_t &)x); return; }
        case exprType::StringConstant: { v.visit_StringConstant((const StringConstant_t &)x); return; }
        case exprType::StringConcat: { v.visit_StringConcat((const StringConcat_t &)x); return; }
        case exprType::StringRepeat: { v.visit_StringRepeat((const StringRepeat_t &)x); return; }
        case exprType::StringLen: { v.visit_StringLen((const StringLen_t &)x); return; }
        case exprType::StringItem: { v.visit_StringItem((const StringItem_t &)x); return; }
        case exprType::StringSection: { v.visit_StringSection((const StringSection_t &)x); return; }
        case exprType::StringCompare: { v.visit_StringCompare((const StringCompare_t &)x); return; }
        case exprType::StringOrd: { v.visit_StringOrd((const StringOrd_t &)x); return; }
        case exprType::StringChr: { v.visit_StringChr((const StringChr_t &)x); return; }
        case exprType::StringFormat: { v.visit_StringFormat((const StringFormat_t &)x); return; }
        case exprType::CPtrCompare: { v.visit_CPtrCompare((const CPtrCompare_t &)x); return; }
        case exprType::SymbolicCompare: { v.visit_SymbolicCompare((const SymbolicCompare_t &)x); return; }
        case exprType::DictConstant: { v.visit_DictConstant((const DictConstant_t &)x); return; }
        case exprType::DictLen: { v.visit_DictLen((const DictLen_t &)x); return; }
        case exprType::Var: { v.visit_Var((const Var_t &)x); return; }
        case exprType::FunctionParam: { v.visit_FunctionParam((const FunctionParam_t &)x); return; }
        case exprType::ArrayConstructor: { v.visit_ArrayConstructor((const ArrayConstructor_t &)x); return; }
        case exprType::ArrayConstant: { v.visit_ArrayConstant((const ArrayConstant_t &)x); return; }
        case exprType::ArrayItem: { v.visit_ArrayItem((const ArrayItem_t &)x); return; }
        case exprType::ArraySection: { v.visit_ArraySection((const ArraySection_t &)x); return; }
        case exprType::ArraySize: { v.visit_ArraySize((const ArraySize_t &)x); return; }
        case exprType::ArrayBound: { v.visit_ArrayBound((const ArrayBound_t &)x); return; }
        case exprType::ArrayTranspose: { v.visit_ArrayTranspose((const ArrayTranspose_t &)x); return; }
        case exprType::ArrayPack: { v.visit_ArrayPack((const ArrayPack_t &)x); return; }
        case exprType::ArrayReshape: { v.visit_ArrayReshape((const ArrayReshape_t &)x); return; }
        case exprType::ArrayBroadcast: { v.visit_ArrayBroadcast((const ArrayBroadcast_t &)x); return; }
        case exprType::BitCast: { v.visit_BitCast((const BitCast_t &)x); return; }
        case exprType::StructInstanceMember: { v.visit_StructInstanceMember((const StructInstanceMember_t &)x); return; }
        case exprType::StructStaticMember: { v.visit_StructStaticMember((const StructStaticMember_t &)x); return; }
        case exprType::EnumStaticMember: { v.visit_EnumStaticMember((const EnumStaticMember_t &)x); return; }
        case exprType::UnionInstanceMember: { v.visit_UnionInstanceMember((const UnionInstanceMember_t &)x); return; }
        case exprType::EnumName: { v.visit_EnumName((const EnumName_t &)x); return; }
        case exprType::EnumValue: { v.visit_EnumValue((const EnumValue_t &)x); return; }
        case exprType::OverloadedCompare: { v.visit_OverloadedCompare((const OverloadedCompare_t &)x); return; }
        case exprType::OverloadedBinOp: { v.visit_OverloadedBinOp((const OverloadedBinOp_t &)x); return; }
        case exprType::OverloadedUnaryMinus: { v.visit_OverloadedUnaryMinus((const OverloadedUnaryMinus_t &)x); return; }
        case exprType::OverloadedStringConcat: { v.visit_OverloadedStringConcat((const OverloadedStringConcat_t &)x); return; }
        case exprType::Cast: { v.visit_Cast((const Cast_t &)x); return; }
        case exprType::ArrayPhysicalCast: { v.visit_ArrayPhysicalCast((const ArrayPhysicalCast_t &)x); return; }
        case exprType::ComplexRe: { v.visit_ComplexRe((const ComplexRe_t &)x); return; }
        case exprType::ComplexIm: { v.visit_ComplexIm((const ComplexIm_t &)x); return; }
        case exprType::DictItem: { v.visit_DictItem((const DictItem_t &)x); return; }
        case exprType::CLoc: { v.visit_CLoc((const CLoc_t &)x); return; }
        case exprType::PointerToCPtr: { v.visit_PointerToCPtr((const PointerToCPtr_t &)x); return; }
        case exprType::GetPointer: { v.visit_GetPointer((const GetPointer_t &)x); return; }
        case exprType::ListItem: { v.visit_ListItem((const ListItem_t &)x); return; }
        case exprType::TupleItem: { v.visit_TupleItem((const TupleItem_t &)x); return; }
        case exprType::ListSection: { v.visit_ListSection((const ListSection_t &)x); return; }
        case exprType::ListRepeat: { v.visit_ListRepeat((const ListRepeat_t &)x); return; }
        case exprType::DictPop: { v.visit_DictPop((const DictPop_t &)x); return; }
        case exprType::SetPop: { v.visit_SetPop((const SetPop_t &)x); return; }
        case exprType::IntegerBitLen: { v.visit_IntegerBitLen((const IntegerBitLen_t &)x); return; }
        case exprType::Ichar: { v.visit_Ichar((const Ichar_t &)x); return; }
        case exprType::Iachar: { v.visit_Iachar((const Iachar_t &)x); return; }
        case exprType::SizeOfType: { v.visit_SizeOfType((const SizeOfType_t &)x); return; }
        case exprType::PointerNullConstant: { v.visit_PointerNullConstant((const PointerNullConstant_t &)x); return; }
        case exprType::PointerAssociated: { v.visit_PointerAssociated((const PointerAssociated_t &)x); return; }
        case exprType::RealSqrt: { v.visit_RealSqrt((const RealSqrt_t &)x); return; }
    }
}

template <class Visitor>
static void visit_ttype_t(const ttype_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::ttype)
    switch (x.type) {
        case ttypeType::Integer: { v.visit_Integer((const Integer_t &)x); return; }
        case ttypeType::UnsignedInteger: { v.visit_UnsignedInteger((const UnsignedInteger_t &)x); return; }
        case ttypeType::Real: { v.visit_Real((const Real_t &)x); return; }
        case ttypeType::Complex: { v.visit_Complex((const Complex_t &)x); return; }
        case ttypeType::Character: { v.visit_Character((const Character_t &)x); return; }
        case ttypeType::Logical: { v.visit_Logical((const Logical_t &)x); return; }
        case ttypeType::Set: { v.visit_Set((const Set_t &)x); return; }
        case ttypeType::List: { v.visit_List((const List_t &)x); return; }
        case ttypeType::Tuple: { v.visit_Tuple((const Tuple_t &)x); return; }
        case ttypeType::StructType: { v.visit_StructType((const StructType_t &)x); return; }
        case ttypeType::EnumType: { v.visit_EnumType((const EnumType_t &)x); return; }
        case ttypeType::Union: { v.visit_Union((const Union_t &)x); return; }
        case ttypeType::ClassType: { v.visit_ClassType((const ClassType_t &)x); return; }
        case ttypeType::Dict: { v.visit_Dict((const Dict_t &)x); return; }
        case ttypeType::Pointer: { v.visit_Pointer((const Pointer_t &)x); return; }
        case ttypeType::Allocatable: { v.visit_Allocatable((const Allocatable_t &)x); return; }
        case ttypeType::CPtr: { v.visit_CPtr((const CPtr_t &)x); return; }
        case ttypeType::SymbolicExpression: { v.visit_SymbolicExpression((const SymbolicExpression_t &)x); return; }
        case ttypeType::TypeParameter: { v.visit_TypeParameter((const TypeParameter_t &)x); return; }
        case ttypeType::Array: { v.visit_Array((const Array_t &)x); return; }
        case ttypeType::FunctionType: { v.visit_FunctionType((const FunctionType_t &)x); return; }
    }
}

template <class Visitor>
static void visit_attribute_t(const attribute_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::attribute)
    switch (x.type) {
        case attributeType::Attribute: { v.visit_Attribute((const Attribute_t &)x); return; }
    }
}

template <class Visitor>
static void visit_tbind_t(const tbind_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::tbind)
    switch (x.type) {
        case tbindType::Bind: { v.visit_Bind((const Bind_t &)x); return; }
    }
}

template <class Visitor>
static void visit_case_stmt_t(const case_stmt_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::case_stmt)
    switch (x.type) {
        case case_stmtType::CaseStmt: { v.visit_CaseStmt((const CaseStmt_t &)x); return; }
        case case_stmtType::CaseStmt_Range: { v.visit_CaseStmt_Range((const CaseStmt_Range_t &)x); return; }
    }
}

template <class Visitor>
static void visit_type_stmt_t(const type_stmt_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::type_stmt)
    switch (x.type) {
        case type_stmtType::TypeStmtName: { v.visit_TypeStmtName((const TypeStmtName_t &)x); return; }
        case type_stmtType::ClassStmt: { v.visit_ClassStmt((const ClassStmt_t &)x); return; }
        case type_stmtType::TypeStmtType: { v.visit_TypeStmtType((const TypeStmtType_t &)x); return; }
    }
}

template <class Visitor>
static void visit_require_instantiation_t(const require_instantiation_t &x, Visitor &v) {
    LCOMPILERS_ASSERT(x.base.type == asrType::require_instantiation)
    switch (x.type) {
        case require_instantiationType::Require: { v.visit_Require((const Require_t &)x); return; }
    }
}



template <class Visitor>
static void visit_asr_t(const asr_t &x, Visitor &v) {
    switch (x.type) {
        case asrType::unit: { v.visit_unit((const unit_t &)x); return; }
        case asrType::symbol: { v.visit_symbol((const symbol_t &)x); return; }
        case asrType::stmt: { v.visit_stmt((const stmt_t &)x); return; }
        case asrType::expr: { v.visit_expr((const expr_t &)x); return; }
        case asrType::ttype: { v.visit_ttype((const ttype_t &)x); return; }
        case asrType::attribute: { v.visit_attribute((const attribute_t &)x); return; }
        case asrType::tbind: { v.visit_tbind((const tbind_t &)x); return; }
        case asrType::case_stmt: { v.visit_case_stmt((const case_stmt_t &)x); return; }
        case asrType::type_stmt: { v.visit_type_stmt((const type_stmt_t &)x); return; }
        case asrType::require_instantiation: { v.visit_require_instantiation((const require_instantiation_t &)x); return; }
    }
}



/******************************************************************************/
// Visitor base class

template <class StructType>
class BaseVisitor
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    void visit_asr(const asr_t &b) { visit_asr_t(b, self()); }
    void visit_unit(const unit_t &b) { visit_unit_t(b, self()); }
        void visit_TranslationUnit(const TranslationUnit_t & /* x */) { throw LCompilersException("visit_TranslationUnit() not implemented"); }
    void visit_symbol(const symbol_t &b) { visit_symbol_t(b, self()); }
        void visit_Program(const Program_t & /* x */) { throw LCompilersException("visit_Program() not implemented"); }
        void visit_Module(const Module_t & /* x */) { throw LCompilersException("visit_Module() not implemented"); }
        void visit_Function(const Function_t & /* x */) { throw LCompilersException("visit_Function() not implemented"); }
        void visit_GenericProcedure(const GenericProcedure_t & /* x */) { throw LCompilersException("visit_GenericProcedure() not implemented"); }
        void visit_CustomOperator(const CustomOperator_t & /* x */) { throw LCompilersException("visit_CustomOperator() not implemented"); }
        void visit_ExternalSymbol(const ExternalSymbol_t & /* x */) { throw LCompilersException("visit_ExternalSymbol() not implemented"); }
        void visit_Struct(const Struct_t & /* x */) { throw LCompilersException("visit_Struct() not implemented"); }
        void visit_Enum(const Enum_t & /* x */) { throw LCompilersException("visit_Enum() not implemented"); }
        void visit_UnionType(const UnionType_t & /* x */) { throw LCompilersException("visit_UnionType() not implemented"); }
        void visit_Variable(const Variable_t & /* x */) { throw LCompilersException("visit_Variable() not implemented"); }
        void visit_Class(const Class_t & /* x */) { throw LCompilersException("visit_Class() not implemented"); }
        void visit_ClassProcedure(const ClassProcedure_t & /* x */) { throw LCompilersException("visit_ClassProcedure() not implemented"); }
        void visit_AssociateBlock(const AssociateBlock_t & /* x */) { throw LCompilersException("visit_AssociateBlock() not implemented"); }
        void visit_Block(const Block_t & /* x */) { throw LCompilersException("visit_Block() not implemented"); }
        void visit_Requirement(const Requirement_t & /* x */) { throw LCompilersException("visit_Requirement() not implemented"); }
        void visit_Template(const Template_t & /* x */) { throw LCompilersException("visit_Template() not implemented"); }
    void visit_stmt(const stmt_t &b) { visit_stmt_t(b, self()); }
        void visit_Allocate(const Allocate_t & /* x */) { throw LCompilersException("visit_Allocate() not implemented"); }
        void visit_ReAlloc(const ReAlloc_t & /* x */) { throw LCompilersException("visit_ReAlloc() not implemented"); }
        void visit_Assign(const Assign_t & /* x */) { throw LCompilersException("visit_Assign() not implemented"); }
        void visit_Assignment(const Assignment_t & /* x */) { throw LCompilersException("visit_Assignment() not implemented"); }
        void visit_Associate(const Associate_t & /* x */) { throw LCompilersException("visit_Associate() not implemented"); }
        void visit_Cycle(const Cycle_t & /* x */) { throw LCompilersException("visit_Cycle() not implemented"); }
        void visit_ExplicitDeallocate(const ExplicitDeallocate_t & /* x */) { throw LCompilersException("visit_ExplicitDeallocate() not implemented"); }
        void visit_ImplicitDeallocate(const ImplicitDeallocate_t & /* x */) { throw LCompilersException("visit_ImplicitDeallocate() not implemented"); }
        void visit_DoConcurrentLoop(const DoConcurrentLoop_t & /* x */) { throw LCompilersException("visit_DoConcurrentLoop() not implemented"); }
        void visit_DoLoop(const DoLoop_t & /* x */) { throw LCompilersException("visit_DoLoop() not implemented"); }
        void visit_ErrorStop(const ErrorStop_t & /* x */) { throw LCompilersException("visit_ErrorStop() not implemented"); }
        void visit_Exit(const Exit_t & /* x */) { throw LCompilersException("visit_Exit() not implemented"); }
        void visit_ForAllSingle(const ForAllSingle_t & /* x */) { throw LCompilersException("visit_ForAllSingle() not implemented"); }
        void visit_GoTo(const GoTo_t & /* x */) { throw LCompilersException("visit_GoTo() not implemented"); }
        void visit_GoToTarget(const GoToTarget_t & /* x */) { throw LCompilersException("visit_GoToTarget() not implemented"); }
        void visit_If(const If_t & /* x */) { throw LCompilersException("visit_If() not implemented"); }
        void visit_IfArithmetic(const IfArithmetic_t & /* x */) { throw LCompilersException("visit_IfArithmetic() not implemented"); }
        void visit_Print(const Print_t & /* x */) { throw LCompilersException("visit_Print() not implemented"); }
        void visit_FileOpen(const FileOpen_t & /* x */) { throw LCompilersException("visit_FileOpen() not implemented"); }
        void visit_FileClose(const FileClose_t & /* x */) { throw LCompilersException("visit_FileClose() not implemented"); }
        void visit_FileRead(const FileRead_t & /* x */) { throw LCompilersException("visit_FileRead() not implemented"); }
        void visit_FileBackspace(const FileBackspace_t & /* x */) { throw LCompilersException("visit_FileBackspace() not implemented"); }
        void visit_FileRewind(const FileRewind_t & /* x */) { throw LCompilersException("visit_FileRewind() not implemented"); }
        void visit_FileInquire(const FileInquire_t & /* x */) { throw LCompilersException("visit_FileInquire() not implemented"); }
        void visit_FileWrite(const FileWrite_t & /* x */) { throw LCompilersException("visit_FileWrite() not implemented"); }
        void visit_Return(const Return_t & /* x */) { throw LCompilersException("visit_Return() not implemented"); }
        void visit_Select(const Select_t & /* x */) { throw LCompilersException("visit_Select() not implemented"); }
        void visit_Stop(const Stop_t & /* x */) { throw LCompilersException("visit_Stop() not implemented"); }
        void visit_Assert(const Assert_t & /* x */) { throw LCompilersException("visit_Assert() not implemented"); }
        void visit_SubroutineCall(const SubroutineCall_t & /* x */) { throw LCompilersException("visit_SubroutineCall() not implemented"); }
        void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t & /* x */) { throw LCompilersException("visit_IntrinsicImpureSubroutine() not implemented"); }
        void visit_Where(const Where_t & /* x */) { throw LCompilersException("visit_Where() not implemented"); }
        void visit_WhileLoop(const WhileLoop_t & /* x */) { throw LCompilersException("visit_WhileLoop() not implemented"); }
        void visit_Nullify(const Nullify_t & /* x */) { throw LCompilersException("visit_Nullify() not implemented"); }
        void visit_Flush(const Flush_t & /* x */) { throw LCompilersException("visit_Flush() not implemented"); }
        void visit_ListAppend(const ListAppend_t & /* x */) { throw LCompilersException("visit_ListAppend() not implemented"); }
        void visit_AssociateBlockCall(const AssociateBlockCall_t & /* x */) { throw LCompilersException("visit_AssociateBlockCall() not implemented"); }
        void visit_SelectType(const SelectType_t & /* x */) { throw LCompilersException("visit_SelectType() not implemented"); }
        void visit_CPtrToPointer(const CPtrToPointer_t & /* x */) { throw LCompilersException("visit_CPtrToPointer() not implemented"); }
        void visit_BlockCall(const BlockCall_t & /* x */) { throw LCompilersException("visit_BlockCall() not implemented"); }
        void visit_SetInsert(const SetInsert_t & /* x */) { throw LCompilersException("visit_SetInsert() not implemented"); }
        void visit_SetRemove(const SetRemove_t & /* x */) { throw LCompilersException("visit_SetRemove() not implemented"); }
        void visit_ListInsert(const ListInsert_t & /* x */) { throw LCompilersException("visit_ListInsert() not implemented"); }
        void visit_ListRemove(const ListRemove_t & /* x */) { throw LCompilersException("visit_ListRemove() not implemented"); }
        void visit_ListClear(const ListClear_t & /* x */) { throw LCompilersException("visit_ListClear() not implemented"); }
        void visit_DictInsert(const DictInsert_t & /* x */) { throw LCompilersException("visit_DictInsert() not implemented"); }
        void visit_Expr(const Expr_t & /* x */) { throw LCompilersException("visit_Expr() not implemented"); }
    void visit_expr(const expr_t &b) { visit_expr_t(b, self()); }
        void visit_IfExp(const IfExp_t & /* x */) { throw LCompilersException("visit_IfExp() not implemented"); }
        void visit_ComplexConstructor(const ComplexConstructor_t & /* x */) { throw LCompilersException("visit_ComplexConstructor() not implemented"); }
        void visit_NamedExpr(const NamedExpr_t & /* x */) { throw LCompilersException("visit_NamedExpr() not implemented"); }
        void visit_FunctionCall(const FunctionCall_t & /* x */) { throw LCompilersException("visit_FunctionCall() not implemented"); }
        void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t & /* x */) { throw LCompilersException("visit_IntrinsicElementalFunction() not implemented"); }
        void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t & /* x */) { throw LCompilersException("visit_IntrinsicArrayFunction() not implemented"); }
        void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t & /* x */) { throw LCompilersException("visit_IntrinsicImpureFunction() not implemented"); }
        void visit_TypeInquiry(const TypeInquiry_t & /* x */) { throw LCompilersException("visit_TypeInquiry() not implemented"); }
        void visit_StructConstructor(const StructConstructor_t & /* x */) { throw LCompilersException("visit_StructConstructor() not implemented"); }
        void visit_EnumConstructor(const EnumConstructor_t & /* x */) { throw LCompilersException("visit_EnumConstructor() not implemented"); }
        void visit_UnionTypeConstructor(const UnionTypeConstructor_t & /* x */) { throw LCompilersException("visit_UnionTypeConstructor() not implemented"); }
        void visit_ImpliedDoLoop(const ImpliedDoLoop_t & /* x */) { throw LCompilersException("visit_ImpliedDoLoop() not implemented"); }
        void visit_IntegerConstant(const IntegerConstant_t & /* x */) { throw LCompilersException("visit_IntegerConstant() not implemented"); }
        void visit_IntegerBOZ(const IntegerBOZ_t & /* x */) { throw LCompilersException("visit_IntegerBOZ() not implemented"); }
        void visit_IntegerBitNot(const IntegerBitNot_t & /* x */) { throw LCompilersException("visit_IntegerBitNot() not implemented"); }
        void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t & /* x */) { throw LCompilersException("visit_IntegerUnaryMinus() not implemented"); }
        void visit_IntegerCompare(const IntegerCompare_t & /* x */) { throw LCompilersException("visit_IntegerCompare() not implemented"); }
        void visit_IntegerBinOp(const IntegerBinOp_t & /* x */) { throw LCompilersException("visit_IntegerBinOp() not implemented"); }
        void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t & /* x */) { throw LCompilersException("visit_UnsignedIntegerConstant() not implemented"); }
        void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t & /* x */) { throw LCompilersException("visit_UnsignedIntegerUnaryMinus() not implemented"); }
        void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t & /* x */) { throw LCompilersException("visit_UnsignedIntegerBitNot() not implemented"); }
        void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t & /* x */) { throw LCompilersException("visit_UnsignedIntegerCompare() not implemented"); }
        void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t & /* x */) { throw LCompilersException("visit_UnsignedIntegerBinOp() not implemented"); }
        void visit_RealConstant(const RealConstant_t & /* x */) { throw LCompilersException("visit_RealConstant() not implemented"); }
        void visit_RealUnaryMinus(const RealUnaryMinus_t & /* x */) { throw LCompilersException("visit_RealUnaryMinus() not implemented"); }
        void visit_RealCompare(const RealCompare_t & /* x */) { throw LCompilersException("visit_RealCompare() not implemented"); }
        void visit_RealBinOp(const RealBinOp_t & /* x */) { throw LCompilersException("visit_RealBinOp() not implemented"); }
        void visit_RealCopySign(const RealCopySign_t & /* x */) { throw LCompilersException("visit_RealCopySign() not implemented"); }
        void visit_ComplexConstant(const ComplexConstant_t & /* x */) { throw LCompilersException("visit_ComplexConstant() not implemented"); }
        void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t & /* x */) { throw LCompilersException("visit_ComplexUnaryMinus() not implemented"); }
        void visit_ComplexCompare(const ComplexCompare_t & /* x */) { throw LCompilersException("visit_ComplexCompare() not implemented"); }
        void visit_ComplexBinOp(const ComplexBinOp_t & /* x */) { throw LCompilersException("visit_ComplexBinOp() not implemented"); }
        void visit_LogicalConstant(const LogicalConstant_t & /* x */) { throw LCompilersException("visit_LogicalConstant() not implemented"); }
        void visit_LogicalNot(const LogicalNot_t & /* x */) { throw LCompilersException("visit_LogicalNot() not implemented"); }
        void visit_LogicalCompare(const LogicalCompare_t & /* x */) { throw LCompilersException("visit_LogicalCompare() not implemented"); }
        void visit_LogicalBinOp(const LogicalBinOp_t & /* x */) { throw LCompilersException("visit_LogicalBinOp() not implemented"); }
        void visit_ListConstant(const ListConstant_t & /* x */) { throw LCompilersException("visit_ListConstant() not implemented"); }
        void visit_ListLen(const ListLen_t & /* x */) { throw LCompilersException("visit_ListLen() not implemented"); }
        void visit_ListConcat(const ListConcat_t & /* x */) { throw LCompilersException("visit_ListConcat() not implemented"); }
        void visit_ListCompare(const ListCompare_t & /* x */) { throw LCompilersException("visit_ListCompare() not implemented"); }
        void visit_ListCount(const ListCount_t & /* x */) { throw LCompilersException("visit_ListCount() not implemented"); }
        void visit_SetConstant(const SetConstant_t & /* x */) { throw LCompilersException("visit_SetConstant() not implemented"); }
        void visit_SetLen(const SetLen_t & /* x */) { throw LCompilersException("visit_SetLen() not implemented"); }
        void visit_TupleConstant(const TupleConstant_t & /* x */) { throw LCompilersException("visit_TupleConstant() not implemented"); }
        void visit_TupleLen(const TupleLen_t & /* x */) { throw LCompilersException("visit_TupleLen() not implemented"); }
        void visit_TupleCompare(const TupleCompare_t & /* x */) { throw LCompilersException("visit_TupleCompare() not implemented"); }
        void visit_TupleConcat(const TupleConcat_t & /* x */) { throw LCompilersException("visit_TupleConcat() not implemented"); }
        void visit_StringConstant(const StringConstant_t & /* x */) { throw LCompilersException("visit_StringConstant() not implemented"); }
        void visit_StringConcat(const StringConcat_t & /* x */) { throw LCompilersException("visit_StringConcat() not implemented"); }
        void visit_StringRepeat(const StringRepeat_t & /* x */) { throw LCompilersException("visit_StringRepeat() not implemented"); }
        void visit_StringLen(const StringLen_t & /* x */) { throw LCompilersException("visit_StringLen() not implemented"); }
        void visit_StringItem(const StringItem_t & /* x */) { throw LCompilersException("visit_StringItem() not implemented"); }
        void visit_StringSection(const StringSection_t & /* x */) { throw LCompilersException("visit_StringSection() not implemented"); }
        void visit_StringCompare(const StringCompare_t & /* x */) { throw LCompilersException("visit_StringCompare() not implemented"); }
        void visit_StringOrd(const StringOrd_t & /* x */) { throw LCompilersException("visit_StringOrd() not implemented"); }
        void visit_StringChr(const StringChr_t & /* x */) { throw LCompilersException("visit_StringChr() not implemented"); }
        void visit_StringFormat(const StringFormat_t & /* x */) { throw LCompilersException("visit_StringFormat() not implemented"); }
        void visit_CPtrCompare(const CPtrCompare_t & /* x */) { throw LCompilersException("visit_CPtrCompare() not implemented"); }
        void visit_SymbolicCompare(const SymbolicCompare_t & /* x */) { throw LCompilersException("visit_SymbolicCompare() not implemented"); }
        void visit_DictConstant(const DictConstant_t & /* x */) { throw LCompilersException("visit_DictConstant() not implemented"); }
        void visit_DictLen(const DictLen_t & /* x */) { throw LCompilersException("visit_DictLen() not implemented"); }
        void visit_Var(const Var_t & /* x */) { throw LCompilersException("visit_Var() not implemented"); }
        void visit_FunctionParam(const FunctionParam_t & /* x */) { throw LCompilersException("visit_FunctionParam() not implemented"); }
        void visit_ArrayConstructor(const ArrayConstructor_t & /* x */) { throw LCompilersException("visit_ArrayConstructor() not implemented"); }
        void visit_ArrayConstant(const ArrayConstant_t & /* x */) { throw LCompilersException("visit_ArrayConstant() not implemented"); }
        void visit_ArrayItem(const ArrayItem_t & /* x */) { throw LCompilersException("visit_ArrayItem() not implemented"); }
        void visit_ArraySection(const ArraySection_t & /* x */) { throw LCompilersException("visit_ArraySection() not implemented"); }
        void visit_ArraySize(const ArraySize_t & /* x */) { throw LCompilersException("visit_ArraySize() not implemented"); }
        void visit_ArrayBound(const ArrayBound_t & /* x */) { throw LCompilersException("visit_ArrayBound() not implemented"); }
        void visit_ArrayTranspose(const ArrayTranspose_t & /* x */) { throw LCompilersException("visit_ArrayTranspose() not implemented"); }
        void visit_ArrayPack(const ArrayPack_t & /* x */) { throw LCompilersException("visit_ArrayPack() not implemented"); }
        void visit_ArrayReshape(const ArrayReshape_t & /* x */) { throw LCompilersException("visit_ArrayReshape() not implemented"); }
        void visit_ArrayBroadcast(const ArrayBroadcast_t & /* x */) { throw LCompilersException("visit_ArrayBroadcast() not implemented"); }
        void visit_BitCast(const BitCast_t & /* x */) { throw LCompilersException("visit_BitCast() not implemented"); }
        void visit_StructInstanceMember(const StructInstanceMember_t & /* x */) { throw LCompilersException("visit_StructInstanceMember() not implemented"); }
        void visit_StructStaticMember(const StructStaticMember_t & /* x */) { throw LCompilersException("visit_StructStaticMember() not implemented"); }
        void visit_EnumStaticMember(const EnumStaticMember_t & /* x */) { throw LCompilersException("visit_EnumStaticMember() not implemented"); }
        void visit_UnionInstanceMember(const UnionInstanceMember_t & /* x */) { throw LCompilersException("visit_UnionInstanceMember() not implemented"); }
        void visit_EnumName(const EnumName_t & /* x */) { throw LCompilersException("visit_EnumName() not implemented"); }
        void visit_EnumValue(const EnumValue_t & /* x */) { throw LCompilersException("visit_EnumValue() not implemented"); }
        void visit_OverloadedCompare(const OverloadedCompare_t & /* x */) { throw LCompilersException("visit_OverloadedCompare() not implemented"); }
        void visit_OverloadedBinOp(const OverloadedBinOp_t & /* x */) { throw LCompilersException("visit_OverloadedBinOp() not implemented"); }
        void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t & /* x */) { throw LCompilersException("visit_OverloadedUnaryMinus() not implemented"); }
        void visit_OverloadedStringConcat(const OverloadedStringConcat_t & /* x */) { throw LCompilersException("visit_OverloadedStringConcat() not implemented"); }
        void visit_Cast(const Cast_t & /* x */) { throw LCompilersException("visit_Cast() not implemented"); }
        void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t & /* x */) { throw LCompilersException("visit_ArrayPhysicalCast() not implemented"); }
        void visit_ComplexRe(const ComplexRe_t & /* x */) { throw LCompilersException("visit_ComplexRe() not implemented"); }
        void visit_ComplexIm(const ComplexIm_t & /* x */) { throw LCompilersException("visit_ComplexIm() not implemented"); }
        void visit_DictItem(const DictItem_t & /* x */) { throw LCompilersException("visit_DictItem() not implemented"); }
        void visit_CLoc(const CLoc_t & /* x */) { throw LCompilersException("visit_CLoc() not implemented"); }
        void visit_PointerToCPtr(const PointerToCPtr_t & /* x */) { throw LCompilersException("visit_PointerToCPtr() not implemented"); }
        void visit_GetPointer(const GetPointer_t & /* x */) { throw LCompilersException("visit_GetPointer() not implemented"); }
        void visit_ListItem(const ListItem_t & /* x */) { throw LCompilersException("visit_ListItem() not implemented"); }
        void visit_TupleItem(const TupleItem_t & /* x */) { throw LCompilersException("visit_TupleItem() not implemented"); }
        void visit_ListSection(const ListSection_t & /* x */) { throw LCompilersException("visit_ListSection() not implemented"); }
        void visit_ListRepeat(const ListRepeat_t & /* x */) { throw LCompilersException("visit_ListRepeat() not implemented"); }
        void visit_DictPop(const DictPop_t & /* x */) { throw LCompilersException("visit_DictPop() not implemented"); }
        void visit_SetPop(const SetPop_t & /* x */) { throw LCompilersException("visit_SetPop() not implemented"); }
        void visit_IntegerBitLen(const IntegerBitLen_t & /* x */) { throw LCompilersException("visit_IntegerBitLen() not implemented"); }
        void visit_Ichar(const Ichar_t & /* x */) { throw LCompilersException("visit_Ichar() not implemented"); }
        void visit_Iachar(const Iachar_t & /* x */) { throw LCompilersException("visit_Iachar() not implemented"); }
        void visit_SizeOfType(const SizeOfType_t & /* x */) { throw LCompilersException("visit_SizeOfType() not implemented"); }
        void visit_PointerNullConstant(const PointerNullConstant_t & /* x */) { throw LCompilersException("visit_PointerNullConstant() not implemented"); }
        void visit_PointerAssociated(const PointerAssociated_t & /* x */) { throw LCompilersException("visit_PointerAssociated() not implemented"); }
        void visit_RealSqrt(const RealSqrt_t & /* x */) { throw LCompilersException("visit_RealSqrt() not implemented"); }
    void visit_ttype(const ttype_t &b) { visit_ttype_t(b, self()); }
        void visit_Integer(const Integer_t & /* x */) { throw LCompilersException("visit_Integer() not implemented"); }
        void visit_UnsignedInteger(const UnsignedInteger_t & /* x */) { throw LCompilersException("visit_UnsignedInteger() not implemented"); }
        void visit_Real(const Real_t & /* x */) { throw LCompilersException("visit_Real() not implemented"); }
        void visit_Complex(const Complex_t & /* x */) { throw LCompilersException("visit_Complex() not implemented"); }
        void visit_Character(const Character_t & /* x */) { throw LCompilersException("visit_Character() not implemented"); }
        void visit_Logical(const Logical_t & /* x */) { throw LCompilersException("visit_Logical() not implemented"); }
        void visit_Set(const Set_t & /* x */) { throw LCompilersException("visit_Set() not implemented"); }
        void visit_List(const List_t & /* x */) { throw LCompilersException("visit_List() not implemented"); }
        void visit_Tuple(const Tuple_t & /* x */) { throw LCompilersException("visit_Tuple() not implemented"); }
        void visit_StructType(const StructType_t & /* x */) { throw LCompilersException("visit_StructType() not implemented"); }
        void visit_EnumType(const EnumType_t & /* x */) { throw LCompilersException("visit_EnumType() not implemented"); }
        void visit_Union(const Union_t & /* x */) { throw LCompilersException("visit_Union() not implemented"); }
        void visit_ClassType(const ClassType_t & /* x */) { throw LCompilersException("visit_ClassType() not implemented"); }
        void visit_Dict(const Dict_t & /* x */) { throw LCompilersException("visit_Dict() not implemented"); }
        void visit_Pointer(const Pointer_t & /* x */) { throw LCompilersException("visit_Pointer() not implemented"); }
        void visit_Allocatable(const Allocatable_t & /* x */) { throw LCompilersException("visit_Allocatable() not implemented"); }
        void visit_CPtr(const CPtr_t & /* x */) { throw LCompilersException("visit_CPtr() not implemented"); }
        void visit_SymbolicExpression(const SymbolicExpression_t & /* x */) { throw LCompilersException("visit_SymbolicExpression() not implemented"); }
        void visit_TypeParameter(const TypeParameter_t & /* x */) { throw LCompilersException("visit_TypeParameter() not implemented"); }
        void visit_Array(const Array_t & /* x */) { throw LCompilersException("visit_Array() not implemented"); }
        void visit_FunctionType(const FunctionType_t & /* x */) { throw LCompilersException("visit_FunctionType() not implemented"); }
    void visit_attribute(const attribute_t &b) { visit_attribute_t(b, self()); }
        void visit_Attribute(const Attribute_t & /* x */) { throw LCompilersException("visit_Attribute() not implemented"); }
    void visit_tbind(const tbind_t &b) { visit_tbind_t(b, self()); }
        void visit_Bind(const Bind_t & /* x */) { throw LCompilersException("visit_Bind() not implemented"); }
    void visit_case_stmt(const case_stmt_t &b) { visit_case_stmt_t(b, self()); }
        void visit_CaseStmt(const CaseStmt_t & /* x */) { throw LCompilersException("visit_CaseStmt() not implemented"); }
        void visit_CaseStmt_Range(const CaseStmt_Range_t & /* x */) { throw LCompilersException("visit_CaseStmt_Range() not implemented"); }
    void visit_type_stmt(const type_stmt_t &b) { visit_type_stmt_t(b, self()); }
        void visit_TypeStmtName(const TypeStmtName_t & /* x */) { throw LCompilersException("visit_TypeStmtName() not implemented"); }
        void visit_ClassStmt(const ClassStmt_t & /* x */) { throw LCompilersException("visit_ClassStmt() not implemented"); }
        void visit_TypeStmtType(const TypeStmtType_t & /* x */) { throw LCompilersException("visit_TypeStmtType() not implemented"); }
    void visit_require_instantiation(const require_instantiation_t &b) { visit_require_instantiation_t(b, self()); }
        void visit_Require(const Require_t & /* x */) { throw LCompilersException("visit_Require() not implemented"); }
};


/******************************************************************************/
// Walk Visitor base class

template <class StructType>
class BaseWalkVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
    }
    void visit_Program(const Program_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Module(const Module_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
    }
    void visit_Function(const Function_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().visit_ttype(*x.m_function_signature);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        if (x.m_return_var)
            self().visit_expr(*x.m_return_var);
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        for (size_t i=0; i<x.n_procs; i++) {
        }
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        for (size_t i=0; i<x.n_procs; i++) {
        }
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Struct(const Struct_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        if (x.m_alignment)
            self().visit_expr(*x.m_alignment);
    }
    void visit_Enum(const Enum_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_UnionType(const UnionType_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
    }
    void visit_Variable(const Variable_t &x) {
        if (x.m_symbolic_value)
            self().visit_expr(*x.m_symbolic_value);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_ttype(*x.m_type);
    }
    void visit_Class(const Class_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Block(const Block_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Requirement(const Requirement_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
    }
    void visit_Template(const Template_t &x) {
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
    }
    void visit_Allocate(const Allocate_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
        if (x.m_stat)
            self().visit_expr(*x.m_stat);
        if (x.m_errmsg)
            self().visit_expr(*x.m_errmsg);
        if (x.m_source)
            self().visit_expr(*x.m_source);
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
    }
    void visit_Assign(const Assign_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
        if (x.m_overloaded)
            self().visit_stmt(*x.m_overloaded);
    }
    void visit_Associate(const Associate_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
    }
    void visit_Cycle(const Cycle_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        self().visit_do_loop_head(x.m_head);
        for (size_t i=0; i<x.n_shared; i++) {
            self().visit_expr(*x.m_shared[i]);
        }
        for (size_t i=0; i<x.n_local; i++) {
            self().visit_expr(*x.m_local[i]);
        }
        for (size_t i=0; i<x.n_reduction; i++) {
            self().visit_reduction_expr(x.m_reduction[i]);
        }
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_DoLoop(const DoLoop_t &x) {
        self().visit_do_loop_head(x.m_head);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        if (x.m_code)
            self().visit_expr(*x.m_code);
    }
    void visit_Exit(const Exit_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        self().visit_do_loop_head(x.m_head);
        self().visit_stmt(*x.m_assign_stmt);
    }
    void visit_GoTo(const GoTo_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        self().visit_expr(*x.m_test);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        self().visit_expr(*x.m_test);
    }
    void visit_Print(const Print_t &x) {
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator)
            self().visit_expr(*x.m_separator);
        if (x.m_end)
            self().visit_expr(*x.m_end);
    }
    void visit_FileOpen(const FileOpen_t &x) {
        if (x.m_newunit)
            self().visit_expr(*x.m_newunit);
        if (x.m_filename)
            self().visit_expr(*x.m_filename);
        if (x.m_status)
            self().visit_expr(*x.m_status);
        if (x.m_form)
            self().visit_expr(*x.m_form);
    }
    void visit_FileClose(const FileClose_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_err)
            self().visit_expr(*x.m_err);
        if (x.m_status)
            self().visit_expr(*x.m_status);
    }
    void visit_FileRead(const FileRead_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_fmt)
            self().visit_expr(*x.m_fmt);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_size)
            self().visit_expr(*x.m_size);
        if (x.m_id)
            self().visit_expr(*x.m_id);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_overloaded)
            self().visit_stmt(*x.m_overloaded);
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_err)
            self().visit_expr(*x.m_err);
    }
    void visit_FileRewind(const FileRewind_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_err)
            self().visit_expr(*x.m_err);
    }
    void visit_FileInquire(const FileInquire_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_file)
            self().visit_expr(*x.m_file);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_err)
            self().visit_expr(*x.m_err);
        if (x.m_exist)
            self().visit_expr(*x.m_exist);
        if (x.m_opened)
            self().visit_expr(*x.m_opened);
        if (x.m_number)
            self().visit_expr(*x.m_number);
        if (x.m_named)
            self().visit_expr(*x.m_named);
        if (x.m_name)
            self().visit_expr(*x.m_name);
        if (x.m_access)
            self().visit_expr(*x.m_access);
        if (x.m_sequential)
            self().visit_expr(*x.m_sequential);
        if (x.m_direct)
            self().visit_expr(*x.m_direct);
        if (x.m_form)
            self().visit_expr(*x.m_form);
        if (x.m_formatted)
            self().visit_expr(*x.m_formatted);
        if (x.m_unformatted)
            self().visit_expr(*x.m_unformatted);
        if (x.m_recl)
            self().visit_expr(*x.m_recl);
        if (x.m_nextrec)
            self().visit_expr(*x.m_nextrec);
        if (x.m_blank)
            self().visit_expr(*x.m_blank);
        if (x.m_position)
            self().visit_expr(*x.m_position);
        if (x.m_action)
            self().visit_expr(*x.m_action);
        if (x.m_read)
            self().visit_expr(*x.m_read);
        if (x.m_write)
            self().visit_expr(*x.m_write);
        if (x.m_readwrite)
            self().visit_expr(*x.m_readwrite);
        if (x.m_delim)
            self().visit_expr(*x.m_delim);
        if (x.m_pad)
            self().visit_expr(*x.m_pad);
        if (x.m_flen)
            self().visit_expr(*x.m_flen);
        if (x.m_blocksize)
            self().visit_expr(*x.m_blocksize);
        if (x.m_convert)
            self().visit_expr(*x.m_convert);
        if (x.m_carriagecontrol)
            self().visit_expr(*x.m_carriagecontrol);
        if (x.m_iolength)
            self().visit_expr(*x.m_iolength);
    }
    void visit_FileWrite(const FileWrite_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_id)
            self().visit_expr(*x.m_id);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator)
            self().visit_expr(*x.m_separator);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        if (x.m_overloaded)
            self().visit_stmt(*x.m_overloaded);
    }
    void visit_Return(const Return_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        self().visit_expr(*x.m_test);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_case_stmt(*x.m_body[i]);
        }
        for (size_t i=0; i<x.n_default; i++) {
            self().visit_stmt(*x.m_default[i]);
        }
    }
    void visit_Stop(const Stop_t &x) {
        if (x.m_code)
            self().visit_expr(*x.m_code);
    }
    void visit_Assert(const Assert_t &x) {
        self().visit_expr(*x.m_test);
        if (x.m_msg)
            self().visit_expr(*x.m_msg);
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        if (x.m_dt)
            self().visit_expr(*x.m_dt);
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
    }
    void visit_Where(const Where_t &x) {
        self().visit_expr(*x.m_test);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        self().visit_expr(*x.m_test);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_Nullify(const Nullify_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
        }
    }
    void visit_Flush(const Flush_t &x) {
        self().visit_expr(*x.m_unit);
        if (x.m_err)
            self().visit_expr(*x.m_err);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
    }
    void visit_ListAppend(const ListAppend_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SelectType(const SelectType_t &x) {
        self().visit_expr(*x.m_selector);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_type_stmt(*x.m_body[i]);
        }
        for (size_t i=0; i<x.n_default; i++) {
            self().visit_stmt(*x.m_default[i]);
        }
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        self().visit_expr(*x.m_cptr);
        self().visit_expr(*x.m_ptr);
        if (x.m_shape)
            self().visit_expr(*x.m_shape);
        if (x.m_lower_bounds)
            self().visit_expr(*x.m_lower_bounds);
    }
    void visit_BlockCall(const BlockCall_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetInsert(const SetInsert_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_SetRemove(const SetRemove_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListInsert(const ListInsert_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListRemove(const ListRemove_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListClear(const ListClear_t &x) {
        self().visit_expr(*x.m_a);
    }
    void visit_DictInsert(const DictInsert_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        self().visit_expr(*x.m_value);
    }
    void visit_Expr(const Expr_t &x) {
        self().visit_expr(*x.m_expression);
    }
    void visit_IfExp(const IfExp_t &x) {
        self().visit_expr(*x.m_test);
        self().visit_expr(*x.m_body);
        self().visit_expr(*x.m_orelse);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        self().visit_expr(*x.m_re);
        self().visit_expr(*x.m_im);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
        self().visit_ttype(*x.m_type);
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        if (x.m_dt)
            self().visit_expr(*x.m_dt);
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type)
            self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type)
            self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type)
            self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        self().visit_ttype(*x.m_arg_type);
        if (x.m_arg)
            self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        self().visit_expr(*x.m_value);
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_expr(*x.m_var);
        self().visit_expr(*x.m_start);
        self().visit_expr(*x.m_end);
        if (x.m_increment)
            self().visit_expr(*x.m_increment);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        if (x.m_type)
            self().visit_ttype(*x.m_type);
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealConstant(const RealConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealCompare(const RealCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_source);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListConstant(const ListConstant_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_ListLen(const ListLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListConcat(const ListConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListCompare(const ListCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListCount(const ListCount_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_expr(*x.m_ele);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SetConstant(const SetConstant_t &x) {
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_SetLen(const SetLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_TupleLen(const TupleLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        self().visit_expr(*x.m_value);
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringConstant(const StringConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_StringConcat(const StringConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringLen(const StringLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringItem(const StringItem_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_expr(*x.m_idx);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringSection(const StringSection_t &x) {
        self().visit_expr(*x.m_arg);
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        if (x.m_step)
            self().visit_expr(*x.m_step);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringCompare(const StringCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringOrd(const StringOrd_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringChr(const StringChr_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringFormat(const StringFormat_t &x) {
        self().visit_expr(*x.m_fmt);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_DictConstant(const DictConstant_t &x) {
        for (size_t i=0; i<x.n_keys; i++) {
            self().visit_expr(*x.m_keys[i]);
        }
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_DictLen(const DictLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Var(const Var_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        self().visit_expr(*x.m_v);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArraySection(const ArraySection_t &x) {
        self().visit_expr(*x.m_v);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArraySize(const ArraySize_t &x) {
        self().visit_expr(*x.m_v);
        if (x.m_dim)
            self().visit_expr(*x.m_dim);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        self().visit_expr(*x.m_v);
        if (x.m_dim)
            self().visit_expr(*x.m_dim);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        self().visit_expr(*x.m_matrix);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_mask);
        if (x.m_vector)
            self().visit_expr(*x.m_vector);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_BitCast(const BitCast_t &x) {
        self().visit_expr(*x.m_source);
        self().visit_expr(*x.m_mold);
        if (x.m_size)
            self().visit_expr(*x.m_size);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumName(const EnumName_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumValue(const EnumValue_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_Cast(const Cast_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_DictItem(const DictItem_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        if (x.m_default)
            self().visit_expr(*x.m_default);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_CLoc(const CLoc_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_GetPointer(const GetPointer_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListItem(const ListItem_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TupleItem(const TupleItem_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListSection(const ListSection_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_array_index(x.m_section);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_DictPop(const DictPop_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SetPop(const SetPop_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Ichar(const Ichar_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Iachar(const Iachar_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        self().visit_ttype(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        self().visit_expr(*x.m_ptr);
        if (x.m_tgt)
            self().visit_expr(*x.m_tgt);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Integer(const Integer_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        if (x.m_len_expr)
            self().visit_expr(*x.m_len_expr);
    }
    void visit_Logical(const Logical_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_List(const List_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_Tuple(const Tuple_t &x) {
        for (size_t i=0; i<x.n_type; i++) {
            self().visit_ttype(*x.m_type[i]);
        }
    }
    void visit_StructType(const StructType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumType(const EnumType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Union(const Union_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassType(const ClassType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Dict(const Dict_t &x) {
        self().visit_ttype(*x.m_key_type);
        self().visit_ttype(*x.m_value_type);
    }
    void visit_Pointer(const Pointer_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_Allocatable(const Allocatable_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_CPtr(const CPtr_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        self().visit_ttype(*x.m_type);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
    }
    void visit_FunctionType(const FunctionType_t &x) {
        for (size_t i=0; i<x.n_arg_types; i++) {
            self().visit_ttype(*x.m_arg_types[i]);
        }
        if (x.m_return_var_type)
            self().visit_ttype(*x.m_return_var_type);
        for (size_t i=0; i<x.n_restrictions; i++) {
        }
    }
    void visit_dimension(const dimension_t &x) {
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_length)
            self().visit_expr(*x.m_length);
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        self().visit_expr(*x.m_a);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
        if (x.m_len_expr)
            self().visit_expr(*x.m_len_expr);
        if (x.m_type)
            self().visit_ttype(*x.m_type);
    }
    void visit_Attribute(const Attribute_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_attribute_arg(x.m_args[i]);
        }
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        self().visit_expr(*x.m_arg);
    }
    void visit_Bind(const Bind_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        if (x.m_left)
            self().visit_expr(*x.m_left);
        if (x.m_right)
            self().visit_expr(*x.m_right);
        if (x.m_step)
            self().visit_expr(*x.m_step);
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        if (x.m_v)
            self().visit_expr(*x.m_v);
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        if (x.m_increment)
            self().visit_expr(*x.m_increment);
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        for (size_t i=0; i<x.n_test; i++) {
            self().visit_expr(*x.m_test[i]);
        }
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        self().visit_ttype(*x.m_type);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Require(const Require_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
};


/******************************************************************************/
// Tree Visitor base class

template <class StructType>
class TreeBaseVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    std::string s, indtd;
    bool use_colors;
    bool start_line = true;
    bool last, attached;
    int indent_level = 0, indent_spaces = 2, lvl = 0;
public:
    TreeBaseVisitor() : use_colors(false), last(true), attached(false) { s.reserve(100000); }
    void inc_indent() {
        indent_level++;
        indtd += "  ";
    }
    void inc_lindent() {
        indent_level++;
        indtd += "| ";
    }
    void dec_indent() {
        indent_level--;
        LCOMPILERS_ASSERT(indent_level >= 0);
        indtd = indtd.substr(0, indent_level*indent_spaces);
    }
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TranslationUnit");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "└-" + "items=↧");
        attached = false;
        for (size_t i=0; i<x.n_items; i++) {
            inc_indent();
            last = i == x.n_items-1;
            attached = false;
            self().visit_asr(*x.m_items[i]);
            dec_indent();
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Program(const Program_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Program");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Module(const Module_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Module");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "loaded_from_mod=");
        if (x.m_loaded_from_mod) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "└-" + "intrinsic=");
        if (x.m_intrinsic) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Function(const Function_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Function");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "function_signature=");
        attached = true;
        self().visit_ttype(*x.m_function_signature);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "return_var=");
        if (x.m_return_var) {
            self().visit_expr(*x.m_return_var);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "accessType=");
        visit_accessType(x.m_access);
        s.append("\n" + indtd + "|-" + "deterministic=");
        if (x.m_deterministic) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "side_effect_free=");
        if (x.m_side_effect_free) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "└-" + "module_file=");
        if (x.m_module_file) {
            s.append("\"" + std::string(x.m_module_file) + "\"");
        } else {
            s.append("()");
        }
        dec_indent();
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GenericProcedure");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "parent_symtab=");
        s.append(x.m_parent_symtab->get_counter());
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "procs=↧");
        for (size_t i=0; i<x.n_procs; i++) {
            inc_lindent();
            last = i == x.n_procs-1;
            attached = false;
            self().visit_symbol(*x.m_procs[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "accessType=");
        visit_accessType(x.m_access);
        dec_indent();
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CustomOperator");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "parent_symtab=");
        s.append(x.m_parent_symtab->get_counter());
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "procs=↧");
        for (size_t i=0; i<x.n_procs; i++) {
            inc_lindent();
            last = i == x.n_procs-1;
            attached = false;
            self().visit_symbol(*x.m_procs[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "accessType=");
        visit_accessType(x.m_access);
        dec_indent();
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ExternalSymbol");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "parent_symtab=");
        s.append(x.m_parent_symtab->get_counter());
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "external=");
        attached = true;
        self().visit_symbol(*x.m_external);
        s.append("\n" + indtd + "|-" + "module_name=");
        s.append(x.m_module_name);
        s.append("\n" + indtd + "|-" + "scope_names=");
        for (size_t i=0; i<x.n_scope_names; i++) {
            s.append(x.m_scope_names[i]);
            if (i < x.n_scope_names-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "original_name=");
        s.append(x.m_original_name);
        s.append("\n" + indtd + "└-" + "accessType=");
        visit_accessType(x.m_access);
        dec_indent();
    }
    void visit_Struct(const Struct_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Struct");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "members=");
        for (size_t i=0; i<x.n_members; i++) {
            s.append(x.m_members[i]);
            if (i < x.n_members-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "|-" + "accessType=");
        visit_accessType(x.m_access);
        s.append("\n" + indtd + "|-" + "is_packed=");
        if (x.m_is_packed) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "is_abstract=");
        if (x.m_is_abstract) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "initializers=↧");
        for (size_t i=0; i<x.n_initializers; i++) {
            inc_lindent();
            last = i == x.n_initializers-1;
            attached = false;
            self().visit_call_arg(x.m_initializers[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "alignment=");
        if (x.m_alignment) {
            self().visit_expr(*x.m_alignment);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "parent=");
        last = true;
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Enum(const Enum_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Enum");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "members=");
        for (size_t i=0; i<x.n_members; i++) {
            s.append(x.m_members[i]);
            if (i < x.n_members-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "|-" + "accessType=");
        visit_accessType(x.m_access);
        s.append("\n" + indtd + "|-" + "enumtypeType=");
        visit_enumtypeType(x.m_enum_value_type);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "parent=");
        last = true;
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnionType(const UnionType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnionType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "members=");
        for (size_t i=0; i<x.n_members; i++) {
            s.append(x.m_members[i]);
            if (i < x.n_members-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "|-" + "accessType=");
        visit_accessType(x.m_access);
        s.append("\n" + indtd + "|-" + "initializers=↧");
        for (size_t i=0; i<x.n_initializers; i++) {
            inc_lindent();
            last = i == x.n_initializers-1;
            attached = false;
            self().visit_call_arg(x.m_initializers[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "parent=");
        last = true;
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Variable(const Variable_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Variable");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "parent_symtab=");
        s.append(x.m_parent_symtab->get_counter());
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "dependencies=");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) s.append(" ");
        }
        s.append("\n" + indtd + "|-" + "intentType=");
        visit_intentType(x.m_intent);
        s.append("\n" + indtd + "|-" + "symbolic_value=");
        if (x.m_symbolic_value) {
            self().visit_expr(*x.m_symbolic_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "storage_typeType=");
        visit_storage_typeType(x.m_storage);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "type_declaration=");
        if (x.m_type_declaration) {
            self().visit_symbol(*x.m_type_declaration);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "|-" + "accessType=");
        visit_accessType(x.m_access);
        s.append("\n" + indtd + "|-" + "presenceType=");
        visit_presenceType(x.m_presence);
        s.append("\n" + indtd + "└-" + "value_attr=");
        if (x.m_value_attr) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        dec_indent();
    }
    void visit_Class(const Class_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Class");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "└-" + "accessType=");
        visit_accessType(x.m_access);
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ClassProcedure");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "parent_symtab=");
        s.append(x.m_parent_symtab->get_counter());
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "self_argument=");
        if (x.m_self_argument) {
            s.append(x.m_self_argument);
        } else {
            s.append("()");
        }
        s.append("\n" + indtd + "|-" + "proc_name=");
        s.append(x.m_proc_name);
        s.append("\n" + indtd + "|-" + "proc=");
        attached = true;
        self().visit_symbol(*x.m_proc);
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "|-" + "is_deferred=");
        if (x.m_is_deferred) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "└-" + "is_nopass=");
        if (x.m_is_nopass) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        dec_indent();
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("AssociateBlock");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Block(const Block_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Block");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Requirement(const Requirement_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Requirement");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "args=");
        for (size_t i=0; i<x.n_args; i++) {
            s.append(x.m_args[i]);
            if (i < x.n_args-1) s.append(" ");
        }
        s.append("\n" + indtd + "└-" + "requires=↧");
        for (size_t i=0; i<x.n_requires; i++) {
            inc_indent();
            last = i == x.n_requires-1;
            attached = false;
            self().visit_require_instantiation(*x.m_requires[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Template(const Template_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Template");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-");
        inc_lindent();
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        s.append("\n" + indtd + "|-counter=");
        s.append(x.m_symtab->get_counter());
        size_t i = 0;
        s.append("\n" + indtd + "└-scope=↧");
        for (auto &a : x.m_symtab->get_scope()) {
            i++;
            inc_indent();
            last = i == x.m_symtab->get_scope().size();
            s.append("\n" + indtd + (last ? "└-" : "|-") + a.first + ": ");
            this->visit_symbol(*a.second);
            dec_indent();
        }
        dec_indent();
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "|-" + "args=");
        for (size_t i=0; i<x.n_args; i++) {
            s.append(x.m_args[i]);
            if (i < x.n_args-1) s.append(" ");
        }
        s.append("\n" + indtd + "└-" + "requires=↧");
        for (size_t i=0; i<x.n_requires; i++) {
            inc_indent();
            last = i == x.n_requires-1;
            attached = false;
            self().visit_require_instantiation(*x.m_requires[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Allocate(const Allocate_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Allocate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_alloc_arg(x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "stat=");
        if (x.m_stat) {
            self().visit_expr(*x.m_stat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "errmsg=");
        if (x.m_errmsg) {
            self().visit_expr(*x.m_errmsg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "source=");
        last = true;
        if (x.m_source) {
            self().visit_expr(*x.m_source);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ReAlloc");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_indent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_alloc_arg(x.m_args[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Assign(const Assign_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Assign");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "└-" + "variable=");
        s.append(x.m_variable);
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Assignment");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "target=");
        attached = true;
        self().visit_expr(*x.m_target);
        s.append("\n" + indtd + "|-" + "value=");
        attached = true;
        self().visit_expr(*x.m_value);
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Associate(const Associate_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Associate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "target=");
        attached = true;
        self().visit_expr(*x.m_target);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_value);
        dec_indent();
    }
    void visit_Cycle(const Cycle_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Cycle");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "stmt_name=");
        if (x.m_stmt_name) {
            s.append(x.m_stmt_name);
        } else {
            s.append("()");
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ExplicitDeallocate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "vars=↧");
        for (size_t i=0; i<x.n_vars; i++) {
            inc_indent();
            last = i == x.n_vars-1;
            attached = false;
            self().visit_expr(*x.m_vars[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ImplicitDeallocate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "vars=↧");
        for (size_t i=0; i<x.n_vars; i++) {
            inc_indent();
            last = i == x.n_vars-1;
            attached = false;
            self().visit_expr(*x.m_vars[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DoConcurrentLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "head=");
        attached = true;
        self().visit_do_loop_head(x.m_head);
        s.append("\n" + indtd + "|-" + "shared=↧");
        for (size_t i=0; i<x.n_shared; i++) {
            inc_lindent();
            last = i == x.n_shared-1;
            attached = false;
            self().visit_expr(*x.m_shared[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "local=↧");
        for (size_t i=0; i<x.n_local; i++) {
            inc_lindent();
            last = i == x.n_local-1;
            attached = false;
            self().visit_expr(*x.m_local[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "reduction=↧");
        for (size_t i=0; i<x.n_reduction; i++) {
            inc_lindent();
            last = i == x.n_reduction-1;
            attached = false;
            self().visit_reduction_expr(x.m_reduction[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_DoLoop(const DoLoop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DoLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "name=");
        if (x.m_name) {
            s.append(x.m_name);
        } else {
            s.append("()");
        }
        s.append("\n" + indtd + "|-" + "head=");
        attached = true;
        self().visit_do_loop_head(x.m_head);
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "orelse=↧");
        for (size_t i=0; i<x.n_orelse; i++) {
            inc_indent();
            last = i == x.n_orelse-1;
            attached = false;
            self().visit_stmt(*x.m_orelse[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ErrorStop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "code=");
        last = true;
        if (x.m_code) {
            self().visit_expr(*x.m_code);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Exit(const Exit_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Exit");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "stmt_name=");
        if (x.m_stmt_name) {
            s.append(x.m_stmt_name);
        } else {
            s.append("()");
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ForAllSingle");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "head=");
        attached = true;
        self().visit_do_loop_head(x.m_head);
        s.append("\n" + indtd + "└-" + "assign_stmt=");
        last = true;
        attached = true;
        self().visit_stmt(*x.m_assign_stmt);
        dec_indent();
    }
    void visit_GoTo(const GoTo_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GoTo");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "target_id=");
        s.append(std::to_string(x.m_target_id));
        s.append("\n" + indtd + "└-" + "name=");
        s.append(x.m_name);
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GoToTarget");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "id=");
        s.append(std::to_string(x.m_id));
        s.append("\n" + indtd + "└-" + "name=");
        s.append(x.m_name);
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("If");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "orelse=↧");
        for (size_t i=0; i<x.n_orelse; i++) {
            inc_indent();
            last = i == x.n_orelse-1;
            attached = false;
            self().visit_stmt(*x.m_orelse[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IfArithmetic");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "|-" + "lt_label=");
        s.append(std::to_string(x.m_lt_label));
        s.append("\n" + indtd + "|-" + "eq_label=");
        s.append(std::to_string(x.m_eq_label));
        s.append("\n" + indtd + "└-" + "gt_label=");
        s.append(std::to_string(x.m_gt_label));
        dec_indent();
    }
    void visit_Print(const Print_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Print");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "values=↧");
        for (size_t i=0; i<x.n_values; i++) {
            inc_lindent();
            last = i == x.n_values-1;
            attached = false;
            self().visit_expr(*x.m_values[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "separator=");
        if (x.m_separator) {
            self().visit_expr(*x.m_separator);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "end=");
        last = true;
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileOpen(const FileOpen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileOpen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "newunit=");
        if (x.m_newunit) {
            self().visit_expr(*x.m_newunit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "filename=");
        if (x.m_filename) {
            self().visit_expr(*x.m_filename);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "status=");
        if (x.m_status) {
            self().visit_expr(*x.m_status);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "form=");
        last = true;
        if (x.m_form) {
            self().visit_expr(*x.m_form);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileClose(const FileClose_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileClose");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iostat=");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iomsg=");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "err=");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "status=");
        last = true;
        if (x.m_status) {
            self().visit_expr(*x.m_status);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileRead(const FileRead_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileRead");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "fmt=");
        if (x.m_fmt) {
            self().visit_expr(*x.m_fmt);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iomsg=");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iostat=");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "size=");
        if (x.m_size) {
            self().visit_expr(*x.m_size);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "id=");
        if (x.m_id) {
            self().visit_expr(*x.m_id);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "values=↧");
        for (size_t i=0; i<x.n_values; i++) {
            inc_lindent();
            last = i == x.n_values-1;
            attached = false;
            self().visit_expr(*x.m_values[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileBackspace");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iostat=");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "err=");
        last = true;
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileRewind(const FileRewind_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileRewind");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iostat=");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "err=");
        last = true;
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileInquire(const FileInquire_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileInquire");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "file=");
        if (x.m_file) {
            self().visit_expr(*x.m_file);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iostat=");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "err=");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "exist=");
        if (x.m_exist) {
            self().visit_expr(*x.m_exist);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "opened=");
        if (x.m_opened) {
            self().visit_expr(*x.m_opened);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "number=");
        if (x.m_number) {
            self().visit_expr(*x.m_number);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "named=");
        if (x.m_named) {
            self().visit_expr(*x.m_named);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "name=");
        if (x.m_name) {
            self().visit_expr(*x.m_name);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "access=");
        if (x.m_access) {
            self().visit_expr(*x.m_access);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "sequential=");
        if (x.m_sequential) {
            self().visit_expr(*x.m_sequential);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "direct=");
        if (x.m_direct) {
            self().visit_expr(*x.m_direct);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "form=");
        if (x.m_form) {
            self().visit_expr(*x.m_form);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "formatted=");
        if (x.m_formatted) {
            self().visit_expr(*x.m_formatted);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "unformatted=");
        if (x.m_unformatted) {
            self().visit_expr(*x.m_unformatted);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "recl=");
        if (x.m_recl) {
            self().visit_expr(*x.m_recl);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "nextrec=");
        if (x.m_nextrec) {
            self().visit_expr(*x.m_nextrec);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "blank=");
        if (x.m_blank) {
            self().visit_expr(*x.m_blank);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "position=");
        if (x.m_position) {
            self().visit_expr(*x.m_position);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "action=");
        if (x.m_action) {
            self().visit_expr(*x.m_action);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "read=");
        if (x.m_read) {
            self().visit_expr(*x.m_read);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "write=");
        if (x.m_write) {
            self().visit_expr(*x.m_write);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "readwrite=");
        if (x.m_readwrite) {
            self().visit_expr(*x.m_readwrite);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "delim=");
        if (x.m_delim) {
            self().visit_expr(*x.m_delim);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "pad=");
        if (x.m_pad) {
            self().visit_expr(*x.m_pad);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "flen=");
        if (x.m_flen) {
            self().visit_expr(*x.m_flen);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "blocksize=");
        if (x.m_blocksize) {
            self().visit_expr(*x.m_blocksize);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "convert=");
        if (x.m_convert) {
            self().visit_expr(*x.m_convert);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "carriagecontrol=");
        if (x.m_carriagecontrol) {
            self().visit_expr(*x.m_carriagecontrol);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "iolength=");
        last = true;
        if (x.m_iolength) {
            self().visit_expr(*x.m_iolength);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_FileWrite(const FileWrite_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileWrite");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iomsg=");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iostat=");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "id=");
        if (x.m_id) {
            self().visit_expr(*x.m_id);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "values=↧");
        for (size_t i=0; i<x.n_values; i++) {
            inc_lindent();
            last = i == x.n_values-1;
            attached = false;
            self().visit_expr(*x.m_values[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "separator=");
        if (x.m_separator) {
            self().visit_expr(*x.m_separator);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "end=");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Return(const Return_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Return");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Select");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_case_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "default=↧");
        for (size_t i=0; i<x.n_default; i++) {
            inc_lindent();
            last = i == x.n_default-1;
            attached = false;
            self().visit_stmt(*x.m_default[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "enable_fall_through=");
        if (x.m_enable_fall_through) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        dec_indent();
    }
    void visit_Stop(const Stop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Stop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "code=");
        last = true;
        if (x.m_code) {
            self().visit_expr(*x.m_code);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Assert(const Assert_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Assert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "└-" + "msg=");
        last = true;
        if (x.m_msg) {
            self().visit_expr(*x.m_msg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SubroutineCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "name=");
        attached = true;
        self().visit_symbol(*x.m_name);
        s.append("\n" + indtd + "|-" + "original_name=");
        if (x.m_original_name) {
            self().visit_symbol(*x.m_original_name);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_call_arg(x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "dt=");
        last = true;
        if (x.m_dt) {
            self().visit_expr(*x.m_dt);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicImpureSubroutine");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "intrinsic_id=");
        s.append(std::to_string(x.m_intrinsic_id));
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "overload_id=");
        s.append(std::to_string(x.m_overload_id));
        dec_indent();
    }
    void visit_Where(const Where_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Where");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "orelse=↧");
        for (size_t i=0; i<x.n_orelse; i++) {
            inc_indent();
            last = i == x.n_orelse-1;
            attached = false;
            self().visit_stmt(*x.m_orelse[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("WhileLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "name=");
        if (x.m_name) {
            s.append(x.m_name);
        } else {
            s.append("()");
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "orelse=↧");
        for (size_t i=0; i<x.n_orelse; i++) {
            inc_indent();
            last = i == x.n_orelse-1;
            attached = false;
            self().visit_stmt(*x.m_orelse[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Nullify(const Nullify_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Nullify");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "vars=↧");
        for (size_t i=0; i<x.n_vars; i++) {
            inc_indent();
            last = i == x.n_vars-1;
            attached = false;
            self().visit_symbol(*x.m_vars[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_Flush(const Flush_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Flush");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "|-" + "unit=");
        attached = true;
        self().visit_expr(*x.m_unit);
        s.append("\n" + indtd + "|-" + "err=");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "iomsg=");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "iostat=");
        last = true;
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListAppend(const ListAppend_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListAppend");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "└-" + "ele=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_ele);
        dec_indent();
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("AssociateBlockCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "m=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_m);
        dec_indent();
    }
    void visit_SelectType(const SelectType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SelectType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "selector=");
        attached = true;
        self().visit_expr(*x.m_selector);
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_type_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "default=↧");
        for (size_t i=0; i<x.n_default; i++) {
            inc_indent();
            last = i == x.n_default-1;
            attached = false;
            self().visit_stmt(*x.m_default[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CPtrToPointer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "cptr=");
        attached = true;
        self().visit_expr(*x.m_cptr);
        s.append("\n" + indtd + "|-" + "ptr=");
        attached = true;
        self().visit_expr(*x.m_ptr);
        s.append("\n" + indtd + "|-" + "shape=");
        if (x.m_shape) {
            self().visit_expr(*x.m_shape);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "lower_bounds=");
        last = true;
        if (x.m_lower_bounds) {
            self().visit_expr(*x.m_lower_bounds);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_BlockCall(const BlockCall_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("BlockCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "label=");
        s.append(std::to_string(x.m_label));
        s.append("\n" + indtd + "└-" + "m=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_m);
        dec_indent();
    }
    void visit_SetInsert(const SetInsert_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetInsert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "└-" + "ele=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_ele);
        dec_indent();
    }
    void visit_SetRemove(const SetRemove_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetRemove");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "└-" + "ele=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_ele);
        dec_indent();
    }
    void visit_ListInsert(const ListInsert_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListInsert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "pos=");
        attached = true;
        self().visit_expr(*x.m_pos);
        s.append("\n" + indtd + "└-" + "ele=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_ele);
        dec_indent();
    }
    void visit_ListRemove(const ListRemove_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListRemove");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "└-" + "ele=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_ele);
        dec_indent();
    }
    void visit_ListClear(const ListClear_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListClear");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "a=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_a);
        dec_indent();
    }
    void visit_DictInsert(const DictInsert_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictInsert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "key=");
        attached = true;
        self().visit_expr(*x.m_key);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_value);
        dec_indent();
    }
    void visit_Expr(const Expr_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Expr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "expression=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_expression);
        dec_indent();
    }
    void visit_IfExp(const IfExp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IfExp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=");
        attached = true;
        self().visit_expr(*x.m_test);
        s.append("\n" + indtd + "|-" + "body=");
        attached = true;
        self().visit_expr(*x.m_body);
        s.append("\n" + indtd + "|-" + "orelse=");
        attached = true;
        self().visit_expr(*x.m_orelse);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "re=");
        attached = true;
        self().visit_expr(*x.m_re);
        s.append("\n" + indtd + "|-" + "im=");
        attached = true;
        self().visit_expr(*x.m_im);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("NamedExpr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "target=");
        attached = true;
        self().visit_expr(*x.m_target);
        s.append("\n" + indtd + "|-" + "value=");
        attached = true;
        self().visit_expr(*x.m_value);
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FunctionCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "name=");
        attached = true;
        self().visit_symbol(*x.m_name);
        s.append("\n" + indtd + "|-" + "original_name=");
        if (x.m_original_name) {
            self().visit_symbol(*x.m_original_name);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_call_arg(x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "dt=");
        last = true;
        if (x.m_dt) {
            self().visit_expr(*x.m_dt);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicElementalFunction");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "intrinsic_id=");
        s.append(std::to_string(x.m_intrinsic_id));
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "overload_id=");
        s.append(std::to_string(x.m_overload_id));
        s.append("\n" + indtd + "|-" + "type=");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicArrayFunction");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arr_intrinsic_id=");
        s.append(std::to_string(x.m_arr_intrinsic_id));
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "overload_id=");
        s.append(std::to_string(x.m_overload_id));
        s.append("\n" + indtd + "|-" + "type=");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicImpureFunction");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "impure_intrinsic_id=");
        s.append(std::to_string(x.m_impure_intrinsic_id));
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "overload_id=");
        s.append(std::to_string(x.m_overload_id));
        s.append("\n" + indtd + "|-" + "type=");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeInquiry");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "inquiry_id=");
        s.append(std::to_string(x.m_inquiry_id));
        s.append("\n" + indtd + "|-" + "arg_type=");
        attached = true;
        self().visit_ttype(*x.m_arg_type);
        s.append("\n" + indtd + "|-" + "arg=");
        if (x.m_arg) {
            self().visit_expr(*x.m_arg);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_value);
        dec_indent();
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "dt_sym=");
        attached = true;
        self().visit_symbol(*x.m_dt_sym);
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_call_arg(x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "dt_sym=");
        attached = true;
        self().visit_symbol(*x.m_dt_sym);
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnionTypeConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "dt_sym=");
        attached = true;
        self().visit_symbol(*x.m_dt_sym);
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ImpliedDoLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "values=↧");
        for (size_t i=0; i<x.n_values; i++) {
            inc_lindent();
            last = i == x.n_values-1;
            attached = false;
            self().visit_expr(*x.m_values[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "var=");
        attached = true;
        self().visit_expr(*x.m_var);
        s.append("\n" + indtd + "|-" + "start=");
        attached = true;
        self().visit_expr(*x.m_start);
        s.append("\n" + indtd + "|-" + "end=");
        attached = true;
        self().visit_expr(*x.m_end);
        s.append("\n" + indtd + "|-" + "increment=");
        if (x.m_increment) {
            self().visit_expr(*x.m_increment);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "n=");
        s.append(std::to_string(x.m_n));
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBOZ");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        s.append(std::to_string(x.m_v));
        s.append("\n" + indtd + "|-" + "integerbozType=");
        visit_integerbozType(x.m_intboz_type);
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBitNot");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "binopType=");
        visit_binopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "n=");
        s.append(std::to_string(x.m_n));
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerBitNot");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "binopType=");
        visit_binopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_RealConstant(const RealConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "r=");
        s.append(std::to_string(x.m_r));
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_RealCompare(const RealCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "binopType=");
        visit_binopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealCopySign");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "target=");
        attached = true;
        self().visit_expr(*x.m_target);
        s.append("\n" + indtd + "|-" + "source=");
        attached = true;
        self().visit_expr(*x.m_source);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "re=");
        s.append(std::to_string(x.m_re));
        s.append("\n" + indtd + "|-" + "im=");
        s.append(std::to_string(x.m_im));
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "binopType=");
        visit_binopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalNot");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "logicalbinopType=");
        visit_logicalbinopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListConstant(const ListConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_ListLen(const ListLen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListConcat(const ListConcat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListCompare(const ListCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListCount(const ListCount_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListCount");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "ele=");
        attached = true;
        self().visit_expr(*x.m_ele);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_SetConstant(const SetConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "elements=↧");
        for (size_t i=0; i<x.n_elements; i++) {
            inc_lindent();
            last = i == x.n_elements-1;
            attached = false;
            self().visit_expr(*x.m_elements[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_SetLen(const SetLen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "elements=↧");
        for (size_t i=0; i<x.n_elements; i++) {
            inc_lindent();
            last = i == x.n_elements-1;
            attached = false;
            self().visit_expr(*x.m_elements[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_TupleLen(const TupleLen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_value);
        dec_indent();
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringConstant(const StringConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "s=");
        s.append("\"" + std::string(x.m_s) + "\"");
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_StringConcat(const StringConcat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringRepeat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringLen(const StringLen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringItem(const StringItem_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "idx=");
        attached = true;
        self().visit_expr(*x.m_idx);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringSection(const StringSection_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringSection");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "start=");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "end=");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "step=");
        if (x.m_step) {
            self().visit_expr(*x.m_step);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringCompare(const StringCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringOrd(const StringOrd_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringOrd");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringChr(const StringChr_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringChr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StringFormat(const StringFormat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringFormat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "fmt=");
        attached = true;
        self().visit_expr(*x.m_fmt);
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "string_format_kindType=");
        visit_string_format_kindType(x.m_kind);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CPtrCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SymbolicCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_DictConstant(const DictConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "keys=↧");
        for (size_t i=0; i<x.n_keys; i++) {
            inc_lindent();
            last = i == x.n_keys-1;
            attached = false;
            self().visit_expr(*x.m_keys[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "values=↧");
        for (size_t i=0; i<x.n_values; i++) {
            inc_lindent();
            last = i == x.n_values-1;
            attached = false;
            self().visit_expr(*x.m_values[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_DictLen(const DictLen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Var(const Var_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Var");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "v=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_v);
        dec_indent();
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FunctionParam");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "param_number=");
        s.append(std::to_string(x.m_param_number));
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_expr(*x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "arraystorageType=");
        visit_arraystorageType(x.m_storage_format);
        dec_indent();
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "n_data=");
        s.append(std::to_string(x.m_n_data));
        s.append("Unimplementedvoid");
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "arraystorageType=");
        visit_arraystorageType(x.m_storage_format);
        dec_indent();
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_array_index(x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "arraystorageType=");
        visit_arraystorageType(x.m_storage_format);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArraySection(const ArraySection_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArraySection");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_lindent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_array_index(x.m_args[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArraySize(const ArraySize_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArraySize");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "dim=");
        if (x.m_dim) {
            self().visit_expr(*x.m_dim);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayBound");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "dim=");
        if (x.m_dim) {
            self().visit_expr(*x.m_dim);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "arrayboundType=");
        visit_arrayboundType(x.m_bound);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayTranspose");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "matrix=");
        attached = true;
        self().visit_expr(*x.m_matrix);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayPack");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "array=");
        attached = true;
        self().visit_expr(*x.m_array);
        s.append("\n" + indtd + "|-" + "mask=");
        attached = true;
        self().visit_expr(*x.m_mask);
        s.append("\n" + indtd + "|-" + "vector=");
        if (x.m_vector) {
            self().visit_expr(*x.m_vector);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayReshape");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "array=");
        attached = true;
        self().visit_expr(*x.m_array);
        s.append("\n" + indtd + "|-" + "shape=");
        attached = true;
        self().visit_expr(*x.m_shape);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayBroadcast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "array=");
        attached = true;
        self().visit_expr(*x.m_array);
        s.append("\n" + indtd + "|-" + "shape=");
        attached = true;
        self().visit_expr(*x.m_shape);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_BitCast(const BitCast_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("BitCast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "source=");
        attached = true;
        self().visit_expr(*x.m_source);
        s.append("\n" + indtd + "|-" + "mold=");
        attached = true;
        self().visit_expr(*x.m_mold);
        s.append("\n" + indtd + "|-" + "size=");
        if (x.m_size) {
            self().visit_expr(*x.m_size);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructInstanceMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "m=");
        attached = true;
        self().visit_symbol(*x.m_m);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructStaticMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "m=");
        attached = true;
        self().visit_symbol(*x.m_m);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumStaticMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "m=");
        attached = true;
        self().visit_symbol(*x.m_m);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnionInstanceMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "m=");
        attached = true;
        self().visit_symbol(*x.m_m);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_EnumName(const EnumName_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumName");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "enum_type=");
        attached = true;
        self().visit_ttype(*x.m_enum_type);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_EnumValue(const EnumValue_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumValue");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "v=");
        attached = true;
        self().visit_expr(*x.m_v);
        s.append("\n" + indtd + "|-" + "enum_type=");
        attached = true;
        self().visit_ttype(*x.m_enum_type);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "cmpopType=");
        visit_cmpopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_overloaded);
        dec_indent();
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "binopType=");
        visit_binopType(x.m_op);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_overloaded);
        dec_indent();
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_overloaded);
        dec_indent();
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedStringConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "value=");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "overloaded=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_overloaded);
        dec_indent();
    }
    void visit_Cast(const Cast_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Cast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "cast_kindType=");
        visit_cast_kindType(x.m_kind);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayPhysicalCast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "array_physical_typeType=");
        visit_array_physical_typeType(x.m_old);
        s.append("\n" + indtd + "|-" + "array_physical_typeType=");
        visit_array_physical_typeType(x.m_new);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexRe");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexIm");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_DictItem(const DictItem_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "key=");
        attached = true;
        self().visit_expr(*x.m_key);
        s.append("\n" + indtd + "|-" + "default=");
        if (x.m_default) {
            self().visit_expr(*x.m_default);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_CLoc(const CLoc_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CLoc");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("PointerToCPtr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_GetPointer(const GetPointer_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GetPointer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListItem(const ListItem_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "pos=");
        attached = true;
        self().visit_expr(*x.m_pos);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_TupleItem(const TupleItem_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "pos=");
        attached = true;
        self().visit_expr(*x.m_pos);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListSection(const ListSection_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListSection");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "section=");
        attached = true;
        self().visit_array_index(x.m_section);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListRepeat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "left=");
        attached = true;
        self().visit_expr(*x.m_left);
        s.append("\n" + indtd + "|-" + "right=");
        attached = true;
        self().visit_expr(*x.m_right);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_DictPop(const DictPop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictPop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "key=");
        attached = true;
        self().visit_expr(*x.m_key);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_SetPop(const SetPop_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetPop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBitLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Ichar(const Ichar_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Ichar");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Iachar(const Iachar_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Iachar");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SizeOfType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_ttype(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("PointerNullConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("PointerAssociated");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "ptr=");
        attached = true;
        self().visit_expr(*x.m_ptr);
        s.append("\n" + indtd + "|-" + "tgt=");
        if (x.m_tgt) {
            self().visit_expr(*x.m_tgt);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealSqrt");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg=");
        attached = true;
        self().visit_expr(*x.m_arg);
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Integer(const Integer_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Integer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "kind=");
        s.append(std::to_string(x.m_kind));
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedInteger");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "kind=");
        s.append(std::to_string(x.m_kind));
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Real");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "kind=");
        s.append(std::to_string(x.m_kind));
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Complex");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "kind=");
        s.append(std::to_string(x.m_kind));
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Character");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "kind=");
        s.append(std::to_string(x.m_kind));
        s.append("\n" + indtd + "|-" + "len=");
        s.append(std::to_string(x.m_len));
        s.append("\n" + indtd + "└-" + "len_expr=");
        last = true;
        if (x.m_len_expr) {
            self().visit_expr(*x.m_len_expr);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Logical(const Logical_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Logical");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "kind=");
        s.append(std::to_string(x.m_kind));
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Set");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_List(const List_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("List");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_Tuple(const Tuple_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Tuple");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "type=↧");
        for (size_t i=0; i<x.n_type; i++) {
            inc_indent();
            last = i == x.n_type-1;
            attached = false;
            self().visit_ttype(*x.m_type[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_StructType(const StructType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "derived_type=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_derived_type);
        dec_indent();
    }
    void visit_EnumType(const EnumType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "enum_type=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_enum_type);
        dec_indent();
    }
    void visit_Union(const Union_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Union");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "union_type=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_union_type);
        dec_indent();
    }
    void visit_ClassType(const ClassType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ClassType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "class_type=");
        last = true;
        attached = true;
        self().visit_symbol(*x.m_class_type);
        dec_indent();
    }
    void visit_Dict(const Dict_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Dict");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "key_type=");
        attached = true;
        self().visit_ttype(*x.m_key_type);
        s.append("\n" + indtd + "└-" + "value_type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_value_type);
        dec_indent();
    }
    void visit_Pointer(const Pointer_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Pointer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_Allocatable(const Allocatable_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Allocatable");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        attached = true;
        self().visit_ttype(*x.m_type);
        dec_indent();
    }
    void visit_CPtr(const CPtr_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CPtr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SymbolicExpression");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeParameter");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "└-" + "param=");
        s.append(x.m_param);
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Array");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "|-" + "dims=↧");
        for (size_t i=0; i<x.n_dims; i++) {
            inc_lindent();
            last = i == x.n_dims-1;
            attached = false;
            self().visit_dimension(x.m_dims[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "array_physical_typeType=");
        visit_array_physical_typeType(x.m_physical_type);
        dec_indent();
    }
    void visit_FunctionType(const FunctionType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FunctionType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "arg_types=↧");
        for (size_t i=0; i<x.n_arg_types; i++) {
            inc_lindent();
            last = i == x.n_arg_types-1;
            attached = false;
            self().visit_ttype(*x.m_arg_types[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "return_var_type=");
        if (x.m_return_var_type) {
            self().visit_ttype(*x.m_return_var_type);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "abiType=");
        visit_abiType(x.m_abi);
        s.append("\n" + indtd + "|-" + "deftypeType=");
        visit_deftypeType(x.m_deftype);
        s.append("\n" + indtd + "|-" + "bindc_name=");
        if (x.m_bindc_name) {
            s.append("\"" + std::string(x.m_bindc_name) + "\"");
        } else {
            s.append("()");
        }
        s.append("\n" + indtd + "|-" + "elemental=");
        if (x.m_elemental) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "pure=");
        if (x.m_pure) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "module=");
        if (x.m_module) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "inline=");
        if (x.m_inline) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "static=");
        if (x.m_static) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        s.append("\n" + indtd + "|-" + "restrictions=↧");
        for (size_t i=0; i<x.n_restrictions; i++) {
            inc_lindent();
            last = i == x.n_restrictions-1;
            attached = false;
            self().visit_symbol(*x.m_restrictions[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "is_restriction=");
        if (x.m_is_restriction) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        dec_indent();
    }
    void visit_cast_kindType(const cast_kindType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (cast_kindType::RealToInteger) : {
                s.append("RealToInteger");
             break; }
            case (cast_kindType::IntegerToReal) : {
                s.append("IntegerToReal");
             break; }
            case (cast_kindType::LogicalToReal) : {
                s.append("LogicalToReal");
             break; }
            case (cast_kindType::RealToReal) : {
                s.append("RealToReal");
             break; }
            case (cast_kindType::IntegerToInteger) : {
                s.append("IntegerToInteger");
             break; }
            case (cast_kindType::RealToComplex) : {
                s.append("RealToComplex");
             break; }
            case (cast_kindType::IntegerToComplex) : {
                s.append("IntegerToComplex");
             break; }
            case (cast_kindType::IntegerToLogical) : {
                s.append("IntegerToLogical");
             break; }
            case (cast_kindType::RealToLogical) : {
                s.append("RealToLogical");
             break; }
            case (cast_kindType::CharacterToLogical) : {
                s.append("CharacterToLogical");
             break; }
            case (cast_kindType::CharacterToInteger) : {
                s.append("CharacterToInteger");
             break; }
            case (cast_kindType::CharacterToList) : {
                s.append("CharacterToList");
             break; }
            case (cast_kindType::ComplexToLogical) : {
                s.append("ComplexToLogical");
             break; }
            case (cast_kindType::ComplexToComplex) : {
                s.append("ComplexToComplex");
             break; }
            case (cast_kindType::ComplexToReal) : {
                s.append("ComplexToReal");
             break; }
            case (cast_kindType::ComplexToInteger) : {
                s.append("ComplexToInteger");
             break; }
            case (cast_kindType::LogicalToInteger) : {
                s.append("LogicalToInteger");
             break; }
            case (cast_kindType::RealToCharacter) : {
                s.append("RealToCharacter");
             break; }
            case (cast_kindType::IntegerToCharacter) : {
                s.append("IntegerToCharacter");
             break; }
            case (cast_kindType::LogicalToCharacter) : {
                s.append("LogicalToCharacter");
             break; }
            case (cast_kindType::UnsignedIntegerToInteger) : {
                s.append("UnsignedIntegerToInteger");
             break; }
            case (cast_kindType::UnsignedIntegerToUnsignedInteger) : {
                s.append("UnsignedIntegerToUnsignedInteger");
             break; }
            case (cast_kindType::UnsignedIntegerToReal) : {
                s.append("UnsignedIntegerToReal");
             break; }
            case (cast_kindType::UnsignedIntegerToLogical) : {
                s.append("UnsignedIntegerToLogical");
             break; }
            case (cast_kindType::IntegerToUnsignedInteger) : {
                s.append("IntegerToUnsignedInteger");
             break; }
            case (cast_kindType::RealToUnsignedInteger) : {
                s.append("RealToUnsignedInteger");
             break; }
            case (cast_kindType::CPtrToUnsignedInteger) : {
                s.append("CPtrToUnsignedInteger");
             break; }
            case (cast_kindType::UnsignedIntegerToCPtr) : {
                s.append("UnsignedIntegerToCPtr");
             break; }
            case (cast_kindType::IntegerToSymbolicExpression) : {
                s.append("IntegerToSymbolicExpression");
             break; }
            case (cast_kindType::ListToArray) : {
                s.append("ListToArray");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_storage_typeType(const storage_typeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (storage_typeType::Default) : {
                s.append("Default");
             break; }
            case (storage_typeType::Save) : {
                s.append("Save");
             break; }
            case (storage_typeType::Parameter) : {
                s.append("Parameter");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_accessType(const accessType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (accessType::Public) : {
                s.append("Public");
             break; }
            case (accessType::Private) : {
                s.append("Private");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_intentType(const intentType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (intentType::Local) : {
                s.append("Local");
             break; }
            case (intentType::In) : {
                s.append("In");
             break; }
            case (intentType::Out) : {
                s.append("Out");
             break; }
            case (intentType::InOut) : {
                s.append("InOut");
             break; }
            case (intentType::ReturnVar) : {
                s.append("ReturnVar");
             break; }
            case (intentType::Unspecified) : {
                s.append("Unspecified");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_deftypeType(const deftypeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (deftypeType::Implementation) : {
                s.append("Implementation");
             break; }
            case (deftypeType::Interface) : {
                s.append("Interface");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_presenceType(const presenceType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (presenceType::Required) : {
                s.append("Required");
             break; }
            case (presenceType::Optional) : {
                s.append("Optional");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_abiType(const abiType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (abiType::Source) : {
                s.append("Source");
             break; }
            case (abiType::LFortranModule) : {
                s.append("LFortranModule");
             break; }
            case (abiType::GFortranModule) : {
                s.append("GFortranModule");
             break; }
            case (abiType::BindC) : {
                s.append("BindC");
             break; }
            case (abiType::BindPython) : {
                s.append("BindPython");
             break; }
            case (abiType::BindJS) : {
                s.append("BindJS");
             break; }
            case (abiType::Interactive) : {
                s.append("Interactive");
             break; }
            case (abiType::Intrinsic) : {
                s.append("Intrinsic");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_dimension(const dimension_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "|-" + "start=");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "length=");
        last = true;
        if (x.m_length) {
            self().visit_expr(*x.m_length);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "|-" + "a=");
        attached = true;
        self().visit_expr(*x.m_a);
        s.append("\n" + indtd + "|-" + "dims=↧");
        for (size_t i=0; i<x.n_dims; i++) {
            inc_lindent();
            last = i == x.n_dims-1;
            attached = false;
            self().visit_dimension(x.m_dims[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "len_expr=");
        if (x.m_len_expr) {
            self().visit_expr(*x.m_len_expr);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "type=");
        last = true;
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_Attribute(const Attribute_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Attribute");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "└-" + "args=↧");
        for (size_t i=0; i<x.n_args; i++) {
            inc_indent();
            last = i == x.n_args-1;
            attached = false;
            self().visit_attribute_arg(x.m_args[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "└-" + "arg=");
        s.append(x.m_arg);
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "└-" + "value=");
        last = true;
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "|-" + "reduction_opType=");
        visit_reduction_opType(x.m_op);
        s.append("\n" + indtd + "└-" + "arg=");
        last = true;
        attached = true;
        self().visit_expr(*x.m_arg);
        dec_indent();
    }
    void visit_Bind(const Bind_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Bind");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "lang=");
        s.append("\"" + std::string(x.m_lang) + "\"");
        s.append("\n" + indtd + "└-" + "name=");
        s.append("\"" + std::string(x.m_name) + "\"");
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "|-" + "left=");
        if (x.m_left) {
            self().visit_expr(*x.m_left);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "right=");
        if (x.m_right) {
            self().visit_expr(*x.m_right);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "step=");
        last = true;
        if (x.m_step) {
            self().visit_expr(*x.m_step);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        s.append("\n" + indtd + "|-" + "v=");
        if (x.m_v) {
            self().visit_expr(*x.m_v);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "start=");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "end=");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "increment=");
        last = true;
        if (x.m_increment) {
            self().visit_expr(*x.m_increment);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        dec_indent();
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CaseStmt");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "test=↧");
        for (size_t i=0; i<x.n_test; i++) {
            inc_lindent();
            last = i == x.n_test-1;
            attached = false;
            self().visit_expr(*x.m_test[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "|-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_lindent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        s.append("\n" + indtd + "└-" + "fall_through=");
        if (x.m_fall_through) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        dec_indent();
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CaseStmt_Range");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "start=");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "|-" + "end=");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        last = false;
        attached = false;
        }
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeStmtName");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "sym=");
        attached = true;
        self().visit_symbol(*x.m_sym);
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ClassStmt");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "sym=");
        attached = true;
        self().visit_symbol(*x.m_sym);
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeStmtType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "type=");
        attached = true;
        self().visit_ttype(*x.m_type);
        s.append("\n" + indtd + "└-" + "body=↧");
        for (size_t i=0; i<x.n_body; i++) {
            inc_indent();
            last = i == x.n_body-1;
            attached = false;
            self().visit_stmt(*x.m_body[i]);
            dec_indent();
        }
        dec_indent();
    }
    void visit_enumtypeType(const enumtypeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (enumtypeType::IntegerConsecutiveFromZero) : {
                s.append("IntegerConsecutiveFromZero");
             break; }
            case (enumtypeType::IntegerUnique) : {
                s.append("IntegerUnique");
             break; }
            case (enumtypeType::IntegerNotUnique) : {
                s.append("IntegerNotUnique");
             break; }
            case (enumtypeType::NonInteger) : {
                s.append("NonInteger");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_Require(const Require_t &x) {
        if(!attached) {
            if(start_line) {
                start_line = false;
                s.append(indtd);
            } else {
                s.append("\n"+indtd);
            }
            last ? s.append("└-") : s.append("|-");
        }
        last ? inc_indent() : inc_lindent();
        attached = true;
        last = false;
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Require");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append("\n" + indtd + "|-" + "name=");
        s.append(x.m_name);
        s.append("\n" + indtd + "└-" + "args=");
        for (size_t i=0; i<x.n_args; i++) {
            s.append(x.m_args[i]);
            if (i < x.n_args-1) s.append(" ");
        }
        dec_indent();
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_physical_typeType(const array_physical_typeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (array_physical_typeType::DescriptorArray) : {
                s.append("DescriptorArray");
             break; }
            case (array_physical_typeType::PointerToDataArray) : {
                s.append("PointerToDataArray");
             break; }
            case (array_physical_typeType::UnboundedPointerToDataArray) : {
                s.append("UnboundedPointerToDataArray");
             break; }
            case (array_physical_typeType::FixedSizeArray) : {
                s.append("FixedSizeArray");
             break; }
            case (array_physical_typeType::CharacterArraySinglePointer) : {
                s.append("CharacterArraySinglePointer");
             break; }
            case (array_physical_typeType::NumPyArray) : {
                s.append("NumPyArray");
             break; }
            case (array_physical_typeType::ISODescriptorArray) : {
                s.append("ISODescriptorArray");
             break; }
            case (array_physical_typeType::SIMDArray) : {
                s.append("SIMDArray");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_binopType(const binopType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (binopType::Add) : {
                s.append("Add");
             break; }
            case (binopType::Sub) : {
                s.append("Sub");
             break; }
            case (binopType::Mul) : {
                s.append("Mul");
             break; }
            case (binopType::Div) : {
                s.append("Div");
             break; }
            case (binopType::Pow) : {
                s.append("Pow");
             break; }
            case (binopType::BitAnd) : {
                s.append("BitAnd");
             break; }
            case (binopType::BitOr) : {
                s.append("BitOr");
             break; }
            case (binopType::BitXor) : {
                s.append("BitXor");
             break; }
            case (binopType::BitLShift) : {
                s.append("BitLShift");
             break; }
            case (binopType::BitRShift) : {
                s.append("BitRShift");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_reduction_opType(const reduction_opType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (reduction_opType::ReduceAdd) : {
                s.append("ReduceAdd");
             break; }
            case (reduction_opType::ReduceSub) : {
                s.append("ReduceSub");
             break; }
            case (reduction_opType::ReduceMul) : {
                s.append("ReduceMul");
             break; }
            case (reduction_opType::ReduceMIN) : {
                s.append("ReduceMIN");
             break; }
            case (reduction_opType::ReduceMAX) : {
                s.append("ReduceMAX");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_logicalbinopType(const logicalbinopType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (logicalbinopType::And) : {
                s.append("And");
             break; }
            case (logicalbinopType::Or) : {
                s.append("Or");
             break; }
            case (logicalbinopType::Xor) : {
                s.append("Xor");
             break; }
            case (logicalbinopType::NEqv) : {
                s.append("NEqv");
             break; }
            case (logicalbinopType::Eqv) : {
                s.append("Eqv");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_cmpopType(const cmpopType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (cmpopType::Eq) : {
                s.append("Eq");
             break; }
            case (cmpopType::NotEq) : {
                s.append("NotEq");
             break; }
            case (cmpopType::Lt) : {
                s.append("Lt");
             break; }
            case (cmpopType::LtE) : {
                s.append("LtE");
             break; }
            case (cmpopType::Gt) : {
                s.append("Gt");
             break; }
            case (cmpopType::GtE) : {
                s.append("GtE");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_integerbozType(const integerbozType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (integerbozType::Binary) : {
                s.append("Binary");
             break; }
            case (integerbozType::Hex) : {
                s.append("Hex");
             break; }
            case (integerbozType::Octal) : {
                s.append("Octal");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_arrayboundType(const arrayboundType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (arrayboundType::LBound) : {
                s.append("LBound");
             break; }
            case (arrayboundType::UBound) : {
                s.append("UBound");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_arraystorageType(const arraystorageType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (arraystorageType::RowMajor) : {
                s.append("RowMajor");
             break; }
            case (arraystorageType::ColMajor) : {
                s.append("ColMajor");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_string_format_kindType(const string_format_kindType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (string_format_kindType::FormatFortran) : {
                s.append("FormatFortran");
             break; }
            case (string_format_kindType::FormatC) : {
                s.append("FormatC");
             break; }
            case (string_format_kindType::FormatPythonPercent) : {
                s.append("FormatPythonPercent");
             break; }
            case (string_format_kindType::FormatPythonFString) : {
                s.append("FormatPythonFString");
             break; }
            case (string_format_kindType::FormatPythonFormat) : {
                s.append("FormatPythonFormat");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
};


/******************************************************************************/
// Pickle Visitor base class

template <class StructType>
class PickleBaseVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    std::string s, indented = "";
    bool use_colors;
    bool indent;
    int indent_level = 0, indent_spaces = 4;
public:
    PickleBaseVisitor() : use_colors(false), indent(false) { s.reserve(100000); }
    void inc_indent() {
        indent_level++;
        indented = std::string(indent_level*indent_spaces, ' ');
    }
    void dec_indent() {
        indent_level--;
        LCOMPILERS_ASSERT(indent_level >= 0);
        indented = std::string(indent_level*indent_spaces, ' ');
    }
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TranslationUnit");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_items; i++) {
            self().visit_asr(*x.m_items[i]);
            if (i < x.n_items-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Program(const Program_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Program");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Module(const Module_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Module");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_loaded_from_mod) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_intrinsic) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Function(const Function_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Function");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_function_signature);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_return_var) {
            self().visit_expr(*x.m_return_var);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_deterministic) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_side_effect_free) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_module_file) {
            s.append("\"" + str_escape_c(x.m_module_file) + "\"");
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GenericProcedure");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_parent_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_procs; i++) {
            self().visit_symbol(*x.m_procs[i]);
            if (i < x.n_procs-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CustomOperator");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_parent_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_procs; i++) {
            self().visit_symbol(*x.m_procs[i]);
            if (i < x.n_procs-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ExternalSymbol");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_parent_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_external);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_module_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_scope_names; i++) {
            s.append(x.m_scope_names[i]);
            if (i < x.n_scope_names-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_original_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Struct(const Struct_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Struct");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_members; i++) {
            s.append(x.m_members[i]);
            if (i < x.n_members-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_is_packed) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_is_abstract) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
            if (i < x.n_initializers-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_alignment) {
            self().visit_expr(*x.m_alignment);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Enum(const Enum_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Enum");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_members; i++) {
            s.append(x.m_members[i]);
            if (i < x.n_members-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_enumtypeType(x.m_enum_value_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnionType(const UnionType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnionType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_members; i++) {
            s.append(x.m_members[i]);
            if (i < x.n_members-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
            if (i < x.n_initializers-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Variable(const Variable_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Variable");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_parent_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dependencies; i++) {
            s.append(x.m_dependencies[i]);
            if (i < x.n_dependencies-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_intentType(x.m_intent);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_symbolic_value) {
            self().visit_expr(*x.m_symbolic_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_storage_typeType(x.m_storage);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_type_declaration) {
            self().visit_symbol(*x.m_type_declaration);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_presenceType(x.m_presence);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value_attr) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Class(const Class_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Class");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_accessType(x.m_access);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ClassProcedure");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_parent_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_self_argument) {
            s.append(x.m_self_argument);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_proc_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_proc);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_is_deferred) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_is_nopass) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("AssociateBlock");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Block(const Block_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Block");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Requirement(const Requirement_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Requirement");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            s.append(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
            if (i < x.n_requires-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Template(const Template_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Template");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("(");
        if (use_colors) {
            s.append(color(fg::yellow));
        }
        s.append("SymbolTable");
        if (use_colors) {
            s.append(color(fg::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_symtab->get_counter());
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("{");
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        }
        {
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append(a.first + ":");
                if(indent) {
                    inc_indent();
                    s.append("\n" + indented);
                } else {
                    s.append(" ");
                }
                this->visit_symbol(*a.second);
                if(indent) dec_indent();
                if (i < x.m_symtab->get_scope().size()-1) {
                    s.append(",");
                    if(indent) s.append("\n" + indented);
                    else s.append(" ");
                }
                i++;
            }
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append("})");
        if(indent) dec_indent();
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            s.append(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
            if (i < x.n_requires-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Allocate(const Allocate_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Allocate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_stat) {
            self().visit_expr(*x.m_stat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_errmsg) {
            self().visit_expr(*x.m_errmsg);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_source) {
            self().visit_expr(*x.m_source);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ReAlloc");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Assign(const Assign_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Assign");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_variable);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Assignment");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_target);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_value);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Associate(const Associate_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Associate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_target);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_value);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Cycle(const Cycle_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Cycle");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_stmt_name) {
            s.append(x.m_stmt_name);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ExplicitDeallocate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
            if (i < x.n_vars-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ImplicitDeallocate");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
            if (i < x.n_vars-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DoConcurrentLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_do_loop_head(x.m_head);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_shared; i++) {
            self().visit_expr(*x.m_shared[i]);
            if (i < x.n_shared-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_local; i++) {
            self().visit_expr(*x.m_local[i]);
            if (i < x.n_local-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_reduction; i++) {
            self().visit_reduction_expr(x.m_reduction[i]);
            if (i < x.n_reduction-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DoLoop(const DoLoop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DoLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_name) {
            s.append(x.m_name);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_do_loop_head(x.m_head);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
            if (i < x.n_orelse-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ErrorStop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_code) {
            self().visit_expr(*x.m_code);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Exit(const Exit_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Exit");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_stmt_name) {
            s.append(x.m_stmt_name);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ForAllSingle");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_do_loop_head(x.m_head);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_stmt(*x.m_assign_stmt);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_GoTo(const GoTo_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GoTo");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_target_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GoToTarget");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(x.m_name);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("If");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
            if (i < x.n_orelse-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IfArithmetic");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_lt_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_eq_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_gt_label));
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Print(const Print_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Print");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
            if (i < x.n_values-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_separator) {
            self().visit_expr(*x.m_separator);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileOpen(const FileOpen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileOpen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_newunit) {
            self().visit_expr(*x.m_newunit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_filename) {
            self().visit_expr(*x.m_filename);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_status) {
            self().visit_expr(*x.m_status);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_form) {
            self().visit_expr(*x.m_form);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileClose(const FileClose_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileClose");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_status) {
            self().visit_expr(*x.m_status);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileRead(const FileRead_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileRead");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_fmt) {
            self().visit_expr(*x.m_fmt);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_size) {
            self().visit_expr(*x.m_size);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_id) {
            self().visit_expr(*x.m_id);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
            if (i < x.n_values-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileBackspace");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileRewind(const FileRewind_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileRewind");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileInquire(const FileInquire_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileInquire");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_file) {
            self().visit_expr(*x.m_file);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_exist) {
            self().visit_expr(*x.m_exist);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_opened) {
            self().visit_expr(*x.m_opened);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_number) {
            self().visit_expr(*x.m_number);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_named) {
            self().visit_expr(*x.m_named);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_name) {
            self().visit_expr(*x.m_name);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_access) {
            self().visit_expr(*x.m_access);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_sequential) {
            self().visit_expr(*x.m_sequential);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_direct) {
            self().visit_expr(*x.m_direct);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_form) {
            self().visit_expr(*x.m_form);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_formatted) {
            self().visit_expr(*x.m_formatted);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unformatted) {
            self().visit_expr(*x.m_unformatted);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_recl) {
            self().visit_expr(*x.m_recl);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_nextrec) {
            self().visit_expr(*x.m_nextrec);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_blank) {
            self().visit_expr(*x.m_blank);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_position) {
            self().visit_expr(*x.m_position);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_action) {
            self().visit_expr(*x.m_action);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_read) {
            self().visit_expr(*x.m_read);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_write) {
            self().visit_expr(*x.m_write);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_readwrite) {
            self().visit_expr(*x.m_readwrite);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_delim) {
            self().visit_expr(*x.m_delim);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_pad) {
            self().visit_expr(*x.m_pad);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_flen) {
            self().visit_expr(*x.m_flen);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_blocksize) {
            self().visit_expr(*x.m_blocksize);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_convert) {
            self().visit_expr(*x.m_convert);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_carriagecontrol) {
            self().visit_expr(*x.m_carriagecontrol);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iolength) {
            self().visit_expr(*x.m_iolength);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FileWrite(const FileWrite_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FileWrite");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_id) {
            self().visit_expr(*x.m_id);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
            if (i < x.n_values-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_separator) {
            self().visit_expr(*x.m_separator);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Return(const Return_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Return");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Select");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_case_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_default; i++) {
            self().visit_stmt(*x.m_default[i]);
            if (i < x.n_default-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_enable_fall_through) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Stop(const Stop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Stop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_code) {
            self().visit_expr(*x.m_code);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Assert(const Assert_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Assert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_msg) {
            self().visit_expr(*x.m_msg);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SubroutineCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_original_name) {
            self().visit_symbol(*x.m_original_name);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_dt) {
            self().visit_expr(*x.m_dt);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicImpureSubroutine");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(self().convert_intrinsic_id(x.m_intrinsic_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_overload_id));
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Where(const Where_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Where");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
            if (i < x.n_orelse-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("WhileLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_name) {
            s.append(x.m_name);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
            if (i < x.n_orelse-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Nullify(const Nullify_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Nullify");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_symbol(*x.m_vars[i]);
            if (i < x.n_vars-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Flush(const Flush_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Flush");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_unit);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListAppend(const ListAppend_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListAppend");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ele);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("AssociateBlockCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_m);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SelectType(const SelectType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SelectType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_selector);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_type_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_default; i++) {
            self().visit_stmt(*x.m_default[i]);
            if (i < x.n_default-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CPtrToPointer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_cptr);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ptr);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_shape) {
            self().visit_expr(*x.m_shape);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_lower_bounds) {
            self().visit_expr(*x.m_lower_bounds);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_BlockCall(const BlockCall_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("BlockCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_label));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_m);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SetInsert(const SetInsert_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetInsert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ele);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SetRemove(const SetRemove_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetRemove");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ele);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListInsert(const ListInsert_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListInsert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_pos);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ele);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListRemove(const ListRemove_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListRemove");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ele);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListClear(const ListClear_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListClear");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DictInsert(const DictInsert_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictInsert");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_key);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_value);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Expr(const Expr_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Expr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_expression);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IfExp(const IfExp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IfExp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_test);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_body);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_orelse);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_re);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_im);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("NamedExpr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_target);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_value);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FunctionCall");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_original_name) {
            self().visit_symbol(*x.m_original_name);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_dt) {
            self().visit_expr(*x.m_dt);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicElementalFunction");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(self().convert_intrinsic_id(x.m_intrinsic_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_overload_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicArrayFunction");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(self().convert_array_intrinsic_id(x.m_arr_intrinsic_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_overload_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntrinsicImpureFunction");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(self().convert_impure_intrinsic_id(x.m_impure_intrinsic_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_overload_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeInquiry");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(self().convert_intrinsic_id(x.m_inquiry_id));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_arg_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_arg) {
            self().visit_expr(*x.m_arg);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_value);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_dt_sym);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_dt_sym);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnionTypeConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_dt_sym);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ImpliedDoLoop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
            if (i < x.n_values-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_var);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_start);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_end);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_increment) {
            self().visit_expr(*x.m_increment);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_n));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBOZ");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_v));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_integerbozType(x.m_intboz_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBitNot");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_binopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_n));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerBitNot");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedIntegerBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_binopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_RealConstant(const RealConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_r));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_RealCompare(const RealCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_binopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealCopySign");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_target);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_source);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_re));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append(std::to_string(x.m_im));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_binopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_value) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalNot");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("LogicalBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_logicalbinopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListConstant(const ListConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListLen(const ListLen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListConcat(const ListConcat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListCompare(const ListCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListCount(const ListCount_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListCount");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_ele);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SetConstant(const SetConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
            if (i < x.n_elements-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SetLen(const SetLen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
            if (i < x.n_elements-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TupleLen(const TupleLen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_value);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringConstant(const StringConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("\"" + str_escape_c(x.m_s) + "\"");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringConcat(const StringConcat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringRepeat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringLen(const StringLen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringItem(const StringItem_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_idx);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringSection(const StringSection_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringSection");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_step) {
            self().visit_expr(*x.m_step);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringCompare(const StringCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringOrd(const StringOrd_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringOrd");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringChr(const StringChr_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringChr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StringFormat(const StringFormat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StringFormat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_fmt);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_string_format_kindType(x.m_kind);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CPtrCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SymbolicCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DictConstant(const DictConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_keys; i++) {
            self().visit_expr(*x.m_keys[i]);
            if (i < x.n_keys-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
            if (i < x.n_values-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DictLen(const DictLen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Var(const Var_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Var");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(" ");
        self().visit_symbol(*x.m_v);
        s.append(")");
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FunctionParam");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_param_number));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayConstructor");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_arraystorageType(x.m_storage_format);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_n_data));
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_arraystorageType(x.m_storage_format);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_arraystorageType(x.m_storage_format);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArraySection(const ArraySection_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArraySection");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArraySize(const ArraySize_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArraySize");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_dim) {
            self().visit_expr(*x.m_dim);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayBound");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_dim) {
            self().visit_expr(*x.m_dim);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_arrayboundType(x.m_bound);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayTranspose");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_matrix);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayPack");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_array);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_mask);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_vector) {
            self().visit_expr(*x.m_vector);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayReshape");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_array);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_shape);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayBroadcast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_array);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_shape);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_BitCast(const BitCast_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("BitCast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_source);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_mold);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_size) {
            self().visit_expr(*x.m_size);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructInstanceMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_m);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructStaticMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_m);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumStaticMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_m);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnionInstanceMember");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_symbol(*x.m_m);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_EnumName(const EnumName_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumName");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_enum_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_EnumValue(const EnumValue_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumValue");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_v);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_enum_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedCompare");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cmpopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_overloaded);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedBinOp");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_binopType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_overloaded);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedUnaryMinus");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_overloaded);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("OverloadedStringConcat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_overloaded);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Cast(const Cast_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Cast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_cast_kindType(x.m_kind);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ArrayPhysicalCast");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_array_physical_typeType(x.m_old);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_array_physical_typeType(x.m_new);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexRe");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ComplexIm");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DictItem(const DictItem_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_key);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_default) {
            self().visit_expr(*x.m_default);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_CLoc(const CLoc_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CLoc");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("PointerToCPtr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_GetPointer(const GetPointer_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("GetPointer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListItem(const ListItem_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_pos);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TupleItem(const TupleItem_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TupleItem");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_pos);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListSection(const ListSection_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListSection");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_array_index(x.m_section);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ListRepeat");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_left);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_right);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_DictPop(const DictPop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("DictPop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_key);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SetPop(const SetPop_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SetPop");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("IntegerBitLen");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Ichar(const Ichar_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Ichar");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Iachar(const Iachar_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Iachar");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SizeOfType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("PointerNullConstant");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("PointerAssociated");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_ptr);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_tgt) {
            self().visit_expr(*x.m_tgt);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("RealSqrt");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_expr(*x.m_arg);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Integer(const Integer_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Integer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(" ");
        s.append(std::to_string(x.m_kind));
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("UnsignedInteger");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(std::to_string(x.m_kind));
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Real");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(" ");
        s.append(std::to_string(x.m_kind));
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Complex");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(" ");
        s.append(std::to_string(x.m_kind));
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Character");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(" ");
        s.append(std::to_string(x.m_kind));
        s.append(" ");
        s.append(std::to_string(x.m_len));
        s.append(" ");
        if (x.m_len_expr) {
            self().visit_expr(*x.m_len_expr);
        } else {
            s.append("()");
        }
        s.append(")");
    }
    void visit_Logical(const Logical_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Logical");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(" ");
        s.append(std::to_string(x.m_kind));
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Set");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_List(const List_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("List");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Tuple(const Tuple_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Tuple");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_type; i++) {
            self().visit_ttype(*x.m_type[i]);
            if (i < x.n_type-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_StructType(const StructType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("StructType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_derived_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_EnumType(const EnumType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("EnumType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_enum_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Union(const Union_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Union");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_union_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ClassType(const ClassType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ClassType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_class_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Dict(const Dict_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Dict");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_key_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_ttype(*x.m_value_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Pointer(const Pointer_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Pointer");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_Allocatable(const Allocatable_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Allocatable");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_CPtr(const CPtr_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CPtr");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("SymbolicExpression");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeParameter");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_param);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Array");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
            if (i < x.n_dims-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_array_physical_typeType(x.m_physical_type);
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_FunctionType(const FunctionType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("FunctionType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_arg_types; i++) {
            self().visit_ttype(*x.m_arg_types[i]);
            if (i < x.n_arg_types-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_return_var_type) {
            self().visit_ttype(*x.m_return_var_type);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_abiType(x.m_abi);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        visit_deftypeType(x.m_deftype);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_bindc_name) {
            s.append("\"" + str_escape_c(x.m_bindc_name) + "\"");
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_elemental) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_pure) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_module) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_inline) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_static) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_restrictions; i++) {
            self().visit_symbol(*x.m_restrictions[i]);
            if (i < x.n_restrictions-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_is_restriction) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_cast_kindType(const cast_kindType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (cast_kindType::RealToInteger) : {
                s.append("RealToInteger");
             break; }
            case (cast_kindType::IntegerToReal) : {
                s.append("IntegerToReal");
             break; }
            case (cast_kindType::LogicalToReal) : {
                s.append("LogicalToReal");
             break; }
            case (cast_kindType::RealToReal) : {
                s.append("RealToReal");
             break; }
            case (cast_kindType::IntegerToInteger) : {
                s.append("IntegerToInteger");
             break; }
            case (cast_kindType::RealToComplex) : {
                s.append("RealToComplex");
             break; }
            case (cast_kindType::IntegerToComplex) : {
                s.append("IntegerToComplex");
             break; }
            case (cast_kindType::IntegerToLogical) : {
                s.append("IntegerToLogical");
             break; }
            case (cast_kindType::RealToLogical) : {
                s.append("RealToLogical");
             break; }
            case (cast_kindType::CharacterToLogical) : {
                s.append("CharacterToLogical");
             break; }
            case (cast_kindType::CharacterToInteger) : {
                s.append("CharacterToInteger");
             break; }
            case (cast_kindType::CharacterToList) : {
                s.append("CharacterToList");
             break; }
            case (cast_kindType::ComplexToLogical) : {
                s.append("ComplexToLogical");
             break; }
            case (cast_kindType::ComplexToComplex) : {
                s.append("ComplexToComplex");
             break; }
            case (cast_kindType::ComplexToReal) : {
                s.append("ComplexToReal");
             break; }
            case (cast_kindType::ComplexToInteger) : {
                s.append("ComplexToInteger");
             break; }
            case (cast_kindType::LogicalToInteger) : {
                s.append("LogicalToInteger");
             break; }
            case (cast_kindType::RealToCharacter) : {
                s.append("RealToCharacter");
             break; }
            case (cast_kindType::IntegerToCharacter) : {
                s.append("IntegerToCharacter");
             break; }
            case (cast_kindType::LogicalToCharacter) : {
                s.append("LogicalToCharacter");
             break; }
            case (cast_kindType::UnsignedIntegerToInteger) : {
                s.append("UnsignedIntegerToInteger");
             break; }
            case (cast_kindType::UnsignedIntegerToUnsignedInteger) : {
                s.append("UnsignedIntegerToUnsignedInteger");
             break; }
            case (cast_kindType::UnsignedIntegerToReal) : {
                s.append("UnsignedIntegerToReal");
             break; }
            case (cast_kindType::UnsignedIntegerToLogical) : {
                s.append("UnsignedIntegerToLogical");
             break; }
            case (cast_kindType::IntegerToUnsignedInteger) : {
                s.append("IntegerToUnsignedInteger");
             break; }
            case (cast_kindType::RealToUnsignedInteger) : {
                s.append("RealToUnsignedInteger");
             break; }
            case (cast_kindType::CPtrToUnsignedInteger) : {
                s.append("CPtrToUnsignedInteger");
             break; }
            case (cast_kindType::UnsignedIntegerToCPtr) : {
                s.append("UnsignedIntegerToCPtr");
             break; }
            case (cast_kindType::IntegerToSymbolicExpression) : {
                s.append("IntegerToSymbolicExpression");
             break; }
            case (cast_kindType::ListToArray) : {
                s.append("ListToArray");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_storage_typeType(const storage_typeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (storage_typeType::Default) : {
                s.append("Default");
             break; }
            case (storage_typeType::Save) : {
                s.append("Save");
             break; }
            case (storage_typeType::Parameter) : {
                s.append("Parameter");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_accessType(const accessType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (accessType::Public) : {
                s.append("Public");
             break; }
            case (accessType::Private) : {
                s.append("Private");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_intentType(const intentType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (intentType::Local) : {
                s.append("Local");
             break; }
            case (intentType::In) : {
                s.append("In");
             break; }
            case (intentType::Out) : {
                s.append("Out");
             break; }
            case (intentType::InOut) : {
                s.append("InOut");
             break; }
            case (intentType::ReturnVar) : {
                s.append("ReturnVar");
             break; }
            case (intentType::Unspecified) : {
                s.append("Unspecified");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_deftypeType(const deftypeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (deftypeType::Implementation) : {
                s.append("Implementation");
             break; }
            case (deftypeType::Interface) : {
                s.append("Interface");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_presenceType(const presenceType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (presenceType::Required) : {
                s.append("Required");
             break; }
            case (presenceType::Optional) : {
                s.append("Optional");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_abiType(const abiType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (abiType::Source) : {
                s.append("Source");
             break; }
            case (abiType::LFortranModule) : {
                s.append("LFortranModule");
             break; }
            case (abiType::GFortranModule) : {
                s.append("GFortranModule");
             break; }
            case (abiType::BindC) : {
                s.append("BindC");
             break; }
            case (abiType::BindPython) : {
                s.append("BindPython");
             break; }
            case (abiType::BindJS) : {
                s.append("BindJS");
             break; }
            case (abiType::Interactive) : {
                s.append("Interactive");
             break; }
            case (abiType::Intrinsic) : {
                s.append("Intrinsic");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_dimension(const dimension_t &x) {
        s.append("(");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_length) {
            self().visit_expr(*x.m_length);
        } else {
            s.append("()");
        }
        s.append(")");
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        s.append("(");
        self().visit_expr(*x.m_a);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
            if (i < x.n_dims-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_len_expr) {
            self().visit_expr(*x.m_len_expr);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("()");
        }
        s.append(")");
    }
    void visit_Attribute(const Attribute_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Attribute");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_attribute_arg(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        s.append("(");
        s.append(x.m_arg);
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        s.append("(");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("()");
        }
        s.append(")");
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        s.append("(");
        visit_reduction_opType(x.m_op);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        self().visit_expr(*x.m_arg);
        s.append(")");
    }
    void visit_Bind(const Bind_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Bind");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("\"" + str_escape_c(x.m_lang) + "\"");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("\"" + str_escape_c(x.m_name) + "\"");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        s.append("(");
        if (x.m_left) {
            self().visit_expr(*x.m_left);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_right) {
            self().visit_expr(*x.m_right);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_step) {
            self().visit_expr(*x.m_step);
        } else {
            s.append("()");
        }
        s.append(")");
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        s.append("(");
        if (x.m_v) {
            self().visit_expr(*x.m_v);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_increment) {
            self().visit_expr(*x.m_increment);
        } else {
            s.append("()");
        }
        s.append(")");
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CaseStmt");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append("[");
        for (size_t i=0; i<x.n_test; i++) {
            self().visit_expr(*x.m_test[i]);
            if (i < x.n_test-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_fall_through) {
            s.append(".true.");
        } else {
            s.append(".false.");
        }
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("CaseStmt_Range");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("()");
        }
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeStmtName");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_sym);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("ClassStmt");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_symbol(*x.m_sym);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("TypeStmtType");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        self().visit_ttype(*x.m_type);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
            if (i < x.n_body-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
    }
    void visit_enumtypeType(const enumtypeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (enumtypeType::IntegerConsecutiveFromZero) : {
                s.append("IntegerConsecutiveFromZero");
             break; }
            case (enumtypeType::IntegerUnique) : {
                s.append("IntegerUnique");
             break; }
            case (enumtypeType::IntegerNotUnique) : {
                s.append("IntegerNotUnique");
             break; }
            case (enumtypeType::NonInteger) : {
                s.append("NonInteger");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_Require(const Require_t &x) {
        s.append("(");
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::magenta));
        }
        s.append("Require");
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
        if(indent) {
            inc_indent();
            s.append("\n" + indented);
        } else {
            s.append(" ");
        }
        s.append(x.m_name);
        if(indent) s.append("\n" + indented);
        else s.append(" ");
        s.append("[");
        for (size_t i=0; i<x.n_args; i++) {
            s.append(x.m_args[i]);
            if (i < x.n_args-1) {
                if (indent) s.append("\n" + indented);
                else s.append(" ");
            };
        }
        s.append("]");
        if(indent) {
            dec_indent();
            s.append("\n" + indented);
        }
        s.append(")");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_physical_typeType(const array_physical_typeType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (array_physical_typeType::DescriptorArray) : {
                s.append("DescriptorArray");
             break; }
            case (array_physical_typeType::PointerToDataArray) : {
                s.append("PointerToDataArray");
             break; }
            case (array_physical_typeType::UnboundedPointerToDataArray) : {
                s.append("UnboundedPointerToDataArray");
             break; }
            case (array_physical_typeType::FixedSizeArray) : {
                s.append("FixedSizeArray");
             break; }
            case (array_physical_typeType::CharacterArraySinglePointer) : {
                s.append("CharacterArraySinglePointer");
             break; }
            case (array_physical_typeType::NumPyArray) : {
                s.append("NumPyArray");
             break; }
            case (array_physical_typeType::ISODescriptorArray) : {
                s.append("ISODescriptorArray");
             break; }
            case (array_physical_typeType::SIMDArray) : {
                s.append("SIMDArray");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_binopType(const binopType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (binopType::Add) : {
                s.append("Add");
             break; }
            case (binopType::Sub) : {
                s.append("Sub");
             break; }
            case (binopType::Mul) : {
                s.append("Mul");
             break; }
            case (binopType::Div) : {
                s.append("Div");
             break; }
            case (binopType::Pow) : {
                s.append("Pow");
             break; }
            case (binopType::BitAnd) : {
                s.append("BitAnd");
             break; }
            case (binopType::BitOr) : {
                s.append("BitOr");
             break; }
            case (binopType::BitXor) : {
                s.append("BitXor");
             break; }
            case (binopType::BitLShift) : {
                s.append("BitLShift");
             break; }
            case (binopType::BitRShift) : {
                s.append("BitRShift");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_reduction_opType(const reduction_opType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (reduction_opType::ReduceAdd) : {
                s.append("ReduceAdd");
             break; }
            case (reduction_opType::ReduceSub) : {
                s.append("ReduceSub");
             break; }
            case (reduction_opType::ReduceMul) : {
                s.append("ReduceMul");
             break; }
            case (reduction_opType::ReduceMIN) : {
                s.append("ReduceMIN");
             break; }
            case (reduction_opType::ReduceMAX) : {
                s.append("ReduceMAX");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_logicalbinopType(const logicalbinopType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (logicalbinopType::And) : {
                s.append("And");
             break; }
            case (logicalbinopType::Or) : {
                s.append("Or");
             break; }
            case (logicalbinopType::Xor) : {
                s.append("Xor");
             break; }
            case (logicalbinopType::NEqv) : {
                s.append("NEqv");
             break; }
            case (logicalbinopType::Eqv) : {
                s.append("Eqv");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_cmpopType(const cmpopType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (cmpopType::Eq) : {
                s.append("Eq");
             break; }
            case (cmpopType::NotEq) : {
                s.append("NotEq");
             break; }
            case (cmpopType::Lt) : {
                s.append("Lt");
             break; }
            case (cmpopType::LtE) : {
                s.append("LtE");
             break; }
            case (cmpopType::Gt) : {
                s.append("Gt");
             break; }
            case (cmpopType::GtE) : {
                s.append("GtE");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_integerbozType(const integerbozType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (integerbozType::Binary) : {
                s.append("Binary");
             break; }
            case (integerbozType::Hex) : {
                s.append("Hex");
             break; }
            case (integerbozType::Octal) : {
                s.append("Octal");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_arrayboundType(const arrayboundType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (arrayboundType::LBound) : {
                s.append("LBound");
             break; }
            case (arrayboundType::UBound) : {
                s.append("UBound");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_arraystorageType(const arraystorageType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (arraystorageType::RowMajor) : {
                s.append("RowMajor");
             break; }
            case (arraystorageType::ColMajor) : {
                s.append("ColMajor");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
    void visit_string_format_kindType(const string_format_kindType &x) {
        if (use_colors) {
            s.append(color(style::bold));
            s.append(color(fg::green));
        }
        switch (x) {
            case (string_format_kindType::FormatFortran) : {
                s.append("FormatFortran");
             break; }
            case (string_format_kindType::FormatC) : {
                s.append("FormatC");
             break; }
            case (string_format_kindType::FormatPythonPercent) : {
                s.append("FormatPythonPercent");
             break; }
            case (string_format_kindType::FormatPythonFString) : {
                s.append("FormatPythonFString");
             break; }
            case (string_format_kindType::FormatPythonFormat) : {
                s.append("FormatPythonFormat");
             break; }
        }
        if (use_colors) {
            s.append(color(fg::reset));
            s.append(color(style::reset));
        }
    }
};


/******************************************************************************/
// Json Visitor base class

template <class StructType>
class JsonBaseVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    std::string s, indtd = "";
    bool no_loc = false;
    int indent_level = 0, indent_spaces = 4;
    LocationManager &lm;
public:
    JsonBaseVisitor(LocationManager &lmref) : lm(lmref) {
        s.reserve(100000);
    }
    void inc_indent() {
        indent_level++;
        indtd = std::string(indent_level*indent_spaces, ' ');
    }
    void dec_indent() {
        indent_level--;
        LCOMPILERS_ASSERT(indent_level >= 0);
        indtd = std::string(indent_level*indent_spaces, ' ');
    }
    void append_location(std::string &s, uint32_t first, uint32_t last) {
        if (no_loc) return;
        s.append(",\n" + indtd);
        s.append("\"loc\": {");
        inc_indent();
        s.append("\n" + indtd);
        s.append("\"first\": " + std::to_string(first));
        s.append(",\n" + indtd);
        s.append("\"last\": " + std::to_string(last));

        uint32_t first_line = 0, first_col = 0;
        std::string first_filename;
        uint32_t last_line = 0, last_col = 0;
        std::string last_filename;

        lm.pos_to_linecol(first, first_line, first_col, first_filename);
        lm.pos_to_linecol(last, last_line, last_col, last_filename);

        s.append(",\n" + indtd);
        s.append("\"first_filename\": \"" + first_filename + "\"");
        s.append(",\n" + indtd);
        s.append("\"first_line\": " + std::to_string(first_line));
        s.append(",\n" + indtd);
        s.append("\"first_column\": " + std::to_string(first_col));
        s.append(",\n" + indtd);
        s.append("\"last_filename\": \"" + last_filename + "\"");
        s.append(",\n" + indtd);
        s.append("\"last_line\": " + std::to_string(last_line));
        s.append(",\n" + indtd);
        s.append("\"last_column\": " + std::to_string(last_col));

        dec_indent();
        s.append("\n" + indtd);
        s.append("}");
    }
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TranslationUnit\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"items\": ");
        s.append("[");
        if (x.n_items > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_items; i++) {
                self().visit_asr(*x.m_items[i]);
                if (i < x.n_items-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Program(const Program_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Program\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Module(const Module_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Module\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"loaded_from_mod\": ");
        if (x.m_loaded_from_mod) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"intrinsic\": ");
        if (x.m_intrinsic) {
            s.append("true");
        } else {
            s.append("false");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Function(const Function_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Function\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"function_signature\": ");
        self().visit_ttype(*x.m_function_signature);
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"return_var\": ");
        if (x.m_return_var) {
            self().visit_expr(*x.m_return_var);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        s.append(",\n" + indtd);
        s.append("\"deterministic\": ");
        if (x.m_deterministic) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"side_effect_free\": ");
        if (x.m_side_effect_free) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"module_file\": ");
        if (x.m_module_file) {
            s.append("\"" + str_escape_c(x.m_module_file) + "\"");
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"GenericProcedure\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"parent_symtab\": ");
        s.append(x.m_parent_symtab->get_counter());
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"procs\": ");
        s.append("[");
        if (x.n_procs > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_procs; i++) {
                self().visit_symbol(*x.m_procs[i]);
                if (i < x.n_procs-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CustomOperator\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"parent_symtab\": ");
        s.append(x.m_parent_symtab->get_counter());
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"procs\": ");
        s.append("[");
        if (x.n_procs > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_procs; i++) {
                self().visit_symbol(*x.m_procs[i]);
                if (i < x.n_procs-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ExternalSymbol\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"parent_symtab\": ");
        s.append(x.m_parent_symtab->get_counter());
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"external\": ");
        self().visit_symbol(*x.m_external);
        s.append(",\n" + indtd);
        s.append("\"module_name\": ");
        s.append("\"" + std::string(x.m_module_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"scope_names\": ");
        s.append("[");
        if (x.n_scope_names > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_scope_names; i++) {
                s.append("\"" + std::string(x.m_scope_names[i]) + "\"");
                if (i < x.n_scope_names-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"original_name\": ");
        s.append("\"" + std::string(x.m_original_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Struct(const Struct_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Struct\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"members\": ");
        s.append("[");
        if (x.n_members > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_members; i++) {
                s.append("\"" + std::string(x.m_members[i]) + "\"");
                if (i < x.n_members-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        s.append(",\n" + indtd);
        s.append("\"is_packed\": ");
        if (x.m_is_packed) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"is_abstract\": ");
        if (x.m_is_abstract) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"initializers\": ");
        s.append("[");
        if (x.n_initializers > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_initializers; i++) {
                self().visit_call_arg(x.m_initializers[i]);
                if (i < x.n_initializers-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"alignment\": ");
        if (x.m_alignment) {
            self().visit_expr(*x.m_alignment);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"parent\": ");
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Enum(const Enum_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Enum\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"members\": ");
        s.append("[");
        if (x.n_members > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_members; i++) {
                s.append("\"" + std::string(x.m_members[i]) + "\"");
                if (i < x.n_members-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        s.append(",\n" + indtd);
        s.append("\"enum_value_type\": ");
        visit_enumtypeType(x.m_enum_value_type);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"parent\": ");
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnionType(const UnionType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnionType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"members\": ");
        s.append("[");
        if (x.n_members > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_members; i++) {
                s.append("\"" + std::string(x.m_members[i]) + "\"");
                if (i < x.n_members-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        s.append(",\n" + indtd);
        s.append("\"initializers\": ");
        s.append("[");
        if (x.n_initializers > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_initializers; i++) {
                self().visit_call_arg(x.m_initializers[i]);
                if (i < x.n_initializers-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"parent\": ");
        if (x.m_parent) {
            self().visit_symbol(*x.m_parent);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Variable(const Variable_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Variable\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"parent_symtab\": ");
        s.append(x.m_parent_symtab->get_counter());
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"dependencies\": ");
        s.append("[");
        if (x.n_dependencies > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dependencies; i++) {
                s.append("\"" + std::string(x.m_dependencies[i]) + "\"");
                if (i < x.n_dependencies-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"intent\": ");
        visit_intentType(x.m_intent);
        s.append(",\n" + indtd);
        s.append("\"symbolic_value\": ");
        if (x.m_symbolic_value) {
            self().visit_expr(*x.m_symbolic_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"storage\": ");
        visit_storage_typeType(x.m_storage);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"type_declaration\": ");
        if (x.m_type_declaration) {
            self().visit_symbol(*x.m_type_declaration);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        s.append(",\n" + indtd);
        s.append("\"presence\": ");
        visit_presenceType(x.m_presence);
        s.append(",\n" + indtd);
        s.append("\"value_attr\": ");
        if (x.m_value_attr) {
            s.append("true");
        } else {
            s.append("false");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Class(const Class_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Class\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        visit_accessType(x.m_access);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ClassProcedure\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"parent_symtab\": ");
        s.append(x.m_parent_symtab->get_counter());
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"self_argument\": ");
        if (x.m_self_argument) {
            s.append("\"" + std::string(x.m_self_argument) + "\"");
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"proc_name\": ");
        s.append("\"" + std::string(x.m_proc_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"proc\": ");
        self().visit_symbol(*x.m_proc);
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"is_deferred\": ");
        if (x.m_is_deferred) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"is_nopass\": ");
        if (x.m_is_nopass) {
            s.append("true");
        } else {
            s.append("false");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"AssociateBlock\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Block(const Block_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Block\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Requirement(const Requirement_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Requirement\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                s.append("\"" + std::string(x.m_args[i]) + "\"");
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"requires\": ");
        s.append("[");
        if (x.n_requires > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_requires; i++) {
                self().visit_require_instantiation(*x.m_requires[i]);
                if (i < x.n_requires-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Template(const Template_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Template\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"symtab\": ");
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolTable" + x.m_symtab->get_counter() +"\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        if (x.m_symtab->get_scope().size() > 0) {
            inc_indent(); s.append("\n" + indtd);
            size_t i = 0;
            for (auto &a : x.m_symtab->get_scope()) {
                s.append("\"" + a.first + "\": ");
                this->visit_symbol(*a.second);
                if (i < x.m_symtab->get_scope().size()-1) { 
                        s.append(",\n" + indtd);
                }
                i++;
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("}");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                s.append("\"" + std::string(x.m_args[i]) + "\"");
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"requires\": ");
        s.append("[");
        if (x.n_requires > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_requires; i++) {
                self().visit_require_instantiation(*x.m_requires[i]);
                if (i < x.n_requires-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Allocate(const Allocate_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Allocate\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_alloc_arg(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"stat\": ");
        if (x.m_stat) {
            self().visit_expr(*x.m_stat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"errmsg\": ");
        if (x.m_errmsg) {
            self().visit_expr(*x.m_errmsg);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"source\": ");
        if (x.m_source) {
            self().visit_expr(*x.m_source);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ReAlloc\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_alloc_arg(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assign(const Assign_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Assign\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"variable\": ");
        s.append("\"" + std::string(x.m_variable) + "\"");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Assignment\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"target\": ");
        self().visit_expr(*x.m_target);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        self().visit_expr(*x.m_value);
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Associate(const Associate_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Associate\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"target\": ");
        self().visit_expr(*x.m_target);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        self().visit_expr(*x.m_value);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Cycle(const Cycle_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Cycle\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"stmt_name\": ");
        if (x.m_stmt_name) {
            s.append("\"" + std::string(x.m_stmt_name) + "\"");
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ExplicitDeallocate\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"vars\": ");
        s.append("[");
        if (x.n_vars > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_vars; i++) {
                self().visit_expr(*x.m_vars[i]);
                if (i < x.n_vars-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ImplicitDeallocate\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"vars\": ");
        s.append("[");
        if (x.n_vars > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_vars; i++) {
                self().visit_expr(*x.m_vars[i]);
                if (i < x.n_vars-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DoConcurrentLoop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"head\": ");
        self().visit_do_loop_head(x.m_head);
        s.append(",\n" + indtd);
        s.append("\"shared\": ");
        s.append("[");
        if (x.n_shared > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_shared; i++) {
                self().visit_expr(*x.m_shared[i]);
                if (i < x.n_shared-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"local\": ");
        s.append("[");
        if (x.n_local > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_local; i++) {
                self().visit_expr(*x.m_local[i]);
                if (i < x.n_local-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"reduction\": ");
        s.append("[");
        if (x.n_reduction > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_reduction; i++) {
                self().visit_reduction_expr(x.m_reduction[i]);
                if (i < x.n_reduction-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DoLoop(const DoLoop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DoLoop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"name\": ");
        if (x.m_name) {
            s.append("\"" + std::string(x.m_name) + "\"");
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"head\": ");
        self().visit_do_loop_head(x.m_head);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"orelse\": ");
        s.append("[");
        if (x.n_orelse > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_orelse; i++) {
                self().visit_stmt(*x.m_orelse[i]);
                if (i < x.n_orelse-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ErrorStop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"code\": ");
        if (x.m_code) {
            self().visit_expr(*x.m_code);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Exit(const Exit_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Exit\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"stmt_name\": ");
        if (x.m_stmt_name) {
            s.append("\"" + std::string(x.m_stmt_name) + "\"");
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ForAllSingle\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"head\": ");
        self().visit_do_loop_head(x.m_head);
        s.append(",\n" + indtd);
        s.append("\"assign_stmt\": ");
        self().visit_stmt(*x.m_assign_stmt);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoTo(const GoTo_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"GoTo\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"target_id\": ");
        s.append(std::to_string(x.m_target_id));
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"GoToTarget\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"id\": ");
        s.append(std::to_string(x.m_id));
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"If\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"orelse\": ");
        s.append("[");
        if (x.n_orelse > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_orelse; i++) {
                self().visit_stmt(*x.m_orelse[i]);
                if (i < x.n_orelse-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IfArithmetic\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"lt_label\": ");
        s.append(std::to_string(x.m_lt_label));
        s.append(",\n" + indtd);
        s.append("\"eq_label\": ");
        s.append(std::to_string(x.m_eq_label));
        s.append(",\n" + indtd);
        s.append("\"gt_label\": ");
        s.append(std::to_string(x.m_gt_label));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Print(const Print_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Print\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"values\": ");
        s.append("[");
        if (x.n_values > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_values; i++) {
                self().visit_expr(*x.m_values[i]);
                if (i < x.n_values-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"separator\": ");
        if (x.m_separator) {
            self().visit_expr(*x.m_separator);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"end\": ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileOpen(const FileOpen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileOpen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"newunit\": ");
        if (x.m_newunit) {
            self().visit_expr(*x.m_newunit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"filename\": ");
        if (x.m_filename) {
            self().visit_expr(*x.m_filename);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"status\": ");
        if (x.m_status) {
            self().visit_expr(*x.m_status);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"form\": ");
        if (x.m_form) {
            self().visit_expr(*x.m_form);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileClose(const FileClose_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileClose\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iomsg\": ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"err\": ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"status\": ");
        if (x.m_status) {
            self().visit_expr(*x.m_status);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileRead(const FileRead_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileRead\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"fmt\": ");
        if (x.m_fmt) {
            self().visit_expr(*x.m_fmt);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iomsg\": ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"size\": ");
        if (x.m_size) {
            self().visit_expr(*x.m_size);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"id\": ");
        if (x.m_id) {
            self().visit_expr(*x.m_id);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"values\": ");
        s.append("[");
        if (x.n_values > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_values; i++) {
                self().visit_expr(*x.m_values[i]);
                if (i < x.n_values-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileBackspace\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"err\": ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileRewind(const FileRewind_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileRewind\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"err\": ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileInquire(const FileInquire_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileInquire\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"file\": ");
        if (x.m_file) {
            self().visit_expr(*x.m_file);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"err\": ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"exist\": ");
        if (x.m_exist) {
            self().visit_expr(*x.m_exist);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"opened\": ");
        if (x.m_opened) {
            self().visit_expr(*x.m_opened);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"number\": ");
        if (x.m_number) {
            self().visit_expr(*x.m_number);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"named\": ");
        if (x.m_named) {
            self().visit_expr(*x.m_named);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        if (x.m_name) {
            self().visit_expr(*x.m_name);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"access\": ");
        if (x.m_access) {
            self().visit_expr(*x.m_access);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"sequential\": ");
        if (x.m_sequential) {
            self().visit_expr(*x.m_sequential);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"direct\": ");
        if (x.m_direct) {
            self().visit_expr(*x.m_direct);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"form\": ");
        if (x.m_form) {
            self().visit_expr(*x.m_form);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"formatted\": ");
        if (x.m_formatted) {
            self().visit_expr(*x.m_formatted);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"unformatted\": ");
        if (x.m_unformatted) {
            self().visit_expr(*x.m_unformatted);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"recl\": ");
        if (x.m_recl) {
            self().visit_expr(*x.m_recl);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"nextrec\": ");
        if (x.m_nextrec) {
            self().visit_expr(*x.m_nextrec);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"blank\": ");
        if (x.m_blank) {
            self().visit_expr(*x.m_blank);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"position\": ");
        if (x.m_position) {
            self().visit_expr(*x.m_position);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"action\": ");
        if (x.m_action) {
            self().visit_expr(*x.m_action);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"read\": ");
        if (x.m_read) {
            self().visit_expr(*x.m_read);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"write\": ");
        if (x.m_write) {
            self().visit_expr(*x.m_write);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"readwrite\": ");
        if (x.m_readwrite) {
            self().visit_expr(*x.m_readwrite);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"delim\": ");
        if (x.m_delim) {
            self().visit_expr(*x.m_delim);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"pad\": ");
        if (x.m_pad) {
            self().visit_expr(*x.m_pad);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"flen\": ");
        if (x.m_flen) {
            self().visit_expr(*x.m_flen);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"blocksize\": ");
        if (x.m_blocksize) {
            self().visit_expr(*x.m_blocksize);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"convert\": ");
        if (x.m_convert) {
            self().visit_expr(*x.m_convert);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"carriagecontrol\": ");
        if (x.m_carriagecontrol) {
            self().visit_expr(*x.m_carriagecontrol);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iolength\": ");
        if (x.m_iolength) {
            self().visit_expr(*x.m_iolength);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FileWrite(const FileWrite_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FileWrite\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        if (x.m_unit) {
            self().visit_expr(*x.m_unit);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iomsg\": ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"id\": ");
        if (x.m_id) {
            self().visit_expr(*x.m_id);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"values\": ");
        s.append("[");
        if (x.n_values > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_values; i++) {
                self().visit_expr(*x.m_values[i]);
                if (i < x.n_values-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"separator\": ");
        if (x.m_separator) {
            self().visit_expr(*x.m_separator);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"end\": ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Return(const Return_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Return\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Select\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_case_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"default\": ");
        s.append("[");
        if (x.n_default > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_default; i++) {
                self().visit_stmt(*x.m_default[i]);
                if (i < x.n_default-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"enable_fall_through\": ");
        if (x.m_enable_fall_through) {
            s.append("true");
        } else {
            s.append("false");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Stop(const Stop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Stop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"code\": ");
        if (x.m_code) {
            self().visit_expr(*x.m_code);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assert(const Assert_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Assert\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"msg\": ");
        if (x.m_msg) {
            self().visit_expr(*x.m_msg);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SubroutineCall\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"name\": ");
        self().visit_symbol(*x.m_name);
        s.append(",\n" + indtd);
        s.append("\"original_name\": ");
        if (x.m_original_name) {
            self().visit_symbol(*x.m_original_name);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_call_arg(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"dt\": ");
        if (x.m_dt) {
            self().visit_expr(*x.m_dt);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntrinsicImpureSubroutine\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"intrinsic_id\": ");
        s.append(std::to_string(x.m_intrinsic_id));
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"overload_id\": ");
        s.append(std::to_string(x.m_overload_id));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Where(const Where_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Where\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"orelse\": ");
        s.append("[");
        if (x.n_orelse > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_orelse; i++) {
                self().visit_stmt(*x.m_orelse[i]);
                if (i < x.n_orelse-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"WhileLoop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"name\": ");
        if (x.m_name) {
            s.append("\"" + std::string(x.m_name) + "\"");
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"orelse\": ");
        s.append("[");
        if (x.n_orelse > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_orelse; i++) {
                self().visit_stmt(*x.m_orelse[i]);
                if (i < x.n_orelse-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Nullify(const Nullify_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Nullify\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"vars\": ");
        s.append("[");
        if (x.n_vars > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_vars; i++) {
                self().visit_symbol(*x.m_vars[i]);
                if (i < x.n_vars-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Flush(const Flush_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Flush\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"unit\": ");
        self().visit_expr(*x.m_unit);
        s.append(",\n" + indtd);
        s.append("\"err\": ");
        if (x.m_err) {
            self().visit_expr(*x.m_err);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iomsg\": ");
        if (x.m_iomsg) {
            self().visit_expr(*x.m_iomsg);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"iostat\": ");
        if (x.m_iostat) {
            self().visit_expr(*x.m_iostat);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListAppend(const ListAppend_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListAppend\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"ele\": ");
        self().visit_expr(*x.m_ele);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"AssociateBlockCall\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"m\": ");
        self().visit_symbol(*x.m_m);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SelectType(const SelectType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SelectType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"selector\": ");
        self().visit_expr(*x.m_selector);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_type_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"default\": ");
        s.append("[");
        if (x.n_default > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_default; i++) {
                self().visit_stmt(*x.m_default[i]);
                if (i < x.n_default-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CPtrToPointer\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"cptr\": ");
        self().visit_expr(*x.m_cptr);
        s.append(",\n" + indtd);
        s.append("\"ptr\": ");
        self().visit_expr(*x.m_ptr);
        s.append(",\n" + indtd);
        s.append("\"shape\": ");
        if (x.m_shape) {
            self().visit_expr(*x.m_shape);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"lower_bounds\": ");
        if (x.m_lower_bounds) {
            self().visit_expr(*x.m_lower_bounds);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_BlockCall(const BlockCall_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"BlockCall\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"label\": ");
        s.append(std::to_string(x.m_label));
        s.append(",\n" + indtd);
        s.append("\"m\": ");
        self().visit_symbol(*x.m_m);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetInsert(const SetInsert_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SetInsert\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"ele\": ");
        self().visit_expr(*x.m_ele);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetRemove(const SetRemove_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SetRemove\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"ele\": ");
        self().visit_expr(*x.m_ele);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListInsert(const ListInsert_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListInsert\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"pos\": ");
        self().visit_expr(*x.m_pos);
        s.append(",\n" + indtd);
        s.append("\"ele\": ");
        self().visit_expr(*x.m_ele);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListRemove(const ListRemove_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListRemove\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"ele\": ");
        self().visit_expr(*x.m_ele);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListClear(const ListClear_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListClear\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DictInsert(const DictInsert_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DictInsert\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"key\": ");
        self().visit_expr(*x.m_key);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        self().visit_expr(*x.m_value);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Expr(const Expr_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Expr\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"expression\": ");
        self().visit_expr(*x.m_expression);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IfExp(const IfExp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IfExp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        self().visit_expr(*x.m_test);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        self().visit_expr(*x.m_body);
        s.append(",\n" + indtd);
        s.append("\"orelse\": ");
        self().visit_expr(*x.m_orelse);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexConstructor\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"re\": ");
        self().visit_expr(*x.m_re);
        s.append(",\n" + indtd);
        s.append("\"im\": ");
        self().visit_expr(*x.m_im);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"NamedExpr\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"target\": ");
        self().visit_expr(*x.m_target);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        self().visit_expr(*x.m_value);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FunctionCall\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"name\": ");
        self().visit_symbol(*x.m_name);
        s.append(",\n" + indtd);
        s.append("\"original_name\": ");
        if (x.m_original_name) {
            self().visit_symbol(*x.m_original_name);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_call_arg(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"dt\": ");
        if (x.m_dt) {
            self().visit_expr(*x.m_dt);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntrinsicElementalFunction\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"intrinsic_id\": ");
        s.append(std::to_string(x.m_intrinsic_id));
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"overload_id\": ");
        s.append(std::to_string(x.m_overload_id));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntrinsicArrayFunction\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arr_intrinsic_id\": ");
        s.append(std::to_string(x.m_arr_intrinsic_id));
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"overload_id\": ");
        s.append(std::to_string(x.m_overload_id));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntrinsicImpureFunction\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"impure_intrinsic_id\": ");
        s.append(std::to_string(x.m_impure_intrinsic_id));
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"overload_id\": ");
        s.append(std::to_string(x.m_overload_id));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TypeInquiry\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"inquiry_id\": ");
        s.append(std::to_string(x.m_inquiry_id));
        s.append(",\n" + indtd);
        s.append("\"arg_type\": ");
        self().visit_ttype(*x.m_arg_type);
        s.append(",\n" + indtd);
        s.append("\"arg\": ");
        if (x.m_arg) {
            self().visit_expr(*x.m_arg);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        self().visit_expr(*x.m_value);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StructConstructor\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"dt_sym\": ");
        self().visit_symbol(*x.m_dt_sym);
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_call_arg(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"EnumConstructor\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"dt_sym\": ");
        self().visit_symbol(*x.m_dt_sym);
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnionTypeConstructor\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"dt_sym\": ");
        self().visit_symbol(*x.m_dt_sym);
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ImpliedDoLoop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"values\": ");
        s.append("[");
        if (x.n_values > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_values; i++) {
                self().visit_expr(*x.m_values[i]);
                if (i < x.n_values-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"var\": ");
        self().visit_expr(*x.m_var);
        s.append(",\n" + indtd);
        s.append("\"start\": ");
        self().visit_expr(*x.m_start);
        s.append(",\n" + indtd);
        s.append("\"end\": ");
        self().visit_expr(*x.m_end);
        s.append(",\n" + indtd);
        s.append("\"increment\": ");
        if (x.m_increment) {
            self().visit_expr(*x.m_increment);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"n\": ");
        s.append(std::to_string(x.m_n));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerBOZ\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        s.append(std::to_string(x.m_v));
        s.append(",\n" + indtd);
        s.append("\"intboz_type\": ");
        visit_integerbozType(x.m_intboz_type);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerBitNot\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerUnaryMinus\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerBinOp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_binopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnsignedIntegerConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"n\": ");
        s.append(std::to_string(x.m_n));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnsignedIntegerUnaryMinus\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnsignedIntegerBitNot\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnsignedIntegerCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnsignedIntegerBinOp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_binopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_RealConstant(const RealConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"RealConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"r\": ");
        s.append(std::to_string(x.m_r));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"RealUnaryMinus\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_RealCompare(const RealCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"RealCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"RealBinOp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_binopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"RealCopySign\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"target\": ");
        self().visit_expr(*x.m_target);
        s.append(",\n" + indtd);
        s.append("\"source\": ");
        self().visit_expr(*x.m_source);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"re\": ");
        s.append(std::to_string(x.m_re));
        s.append(",\n" + indtd);
        s.append("\"im\": ");
        s.append(std::to_string(x.m_im));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexUnaryMinus\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexBinOp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_binopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"LogicalConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"LogicalNot\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"LogicalCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"LogicalBinOp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_logicalbinopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListConstant(const ListConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListLen(const ListLen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListLen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListConcat(const ListConcat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListConcat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListCompare(const ListCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListCount(const ListCount_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListCount\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"ele\": ");
        self().visit_expr(*x.m_ele);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetConstant(const SetConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SetConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"elements\": ");
        s.append("[");
        if (x.n_elements > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_elements; i++) {
                self().visit_expr(*x.m_elements[i]);
                if (i < x.n_elements-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetLen(const SetLen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SetLen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TupleConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"elements\": ");
        s.append("[");
        if (x.n_elements > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_elements; i++) {
                self().visit_expr(*x.m_elements[i]);
                if (i < x.n_elements-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TupleLen(const TupleLen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TupleLen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        self().visit_expr(*x.m_value);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TupleCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TupleConcat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringConstant(const StringConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"s\": ");
        s.append("\"" + str_escape_c(x.m_s) + "\"");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringConcat(const StringConcat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringConcat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringRepeat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringLen(const StringLen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringLen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringItem(const StringItem_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringItem\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"idx\": ");
        self().visit_expr(*x.m_idx);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringSection(const StringSection_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringSection\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"start\": ");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"end\": ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"step\": ");
        if (x.m_step) {
            self().visit_expr(*x.m_step);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringCompare(const StringCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringOrd(const StringOrd_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringOrd\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringChr(const StringChr_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringChr\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StringFormat(const StringFormat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StringFormat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"fmt\": ");
        self().visit_expr(*x.m_fmt);
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"kind\": ");
        visit_string_format_kindType(x.m_kind);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CPtrCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolicCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DictConstant(const DictConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DictConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"keys\": ");
        s.append("[");
        if (x.n_keys > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_keys; i++) {
                self().visit_expr(*x.m_keys[i]);
                if (i < x.n_keys-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"values\": ");
        s.append("[");
        if (x.n_values > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_values; i++) {
                self().visit_expr(*x.m_values[i]);
                if (i < x.n_values-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DictLen(const DictLen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DictLen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Var(const Var_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Var\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_symbol(*x.m_v);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FunctionParam\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"param_number\": ");
        s.append(std::to_string(x.m_param_number));
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayConstructor\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_expr(*x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"storage_format\": ");
        visit_arraystorageType(x.m_storage_format);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"n_data\": ");
        s.append(std::to_string(x.m_n_data));
        s.append(",\n" + indtd);
        s.append("\"data\": ");
        s.append("\"Unimplementedvoid\"");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"storage_format\": ");
        visit_arraystorageType(x.m_storage_format);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayItem\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_array_index(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"storage_format\": ");
        visit_arraystorageType(x.m_storage_format);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArraySection(const ArraySection_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArraySection\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_array_index(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArraySize(const ArraySize_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArraySize\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"dim\": ");
        if (x.m_dim) {
            self().visit_expr(*x.m_dim);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayBound\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"dim\": ");
        if (x.m_dim) {
            self().visit_expr(*x.m_dim);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"bound\": ");
        visit_arrayboundType(x.m_bound);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayTranspose\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"matrix\": ");
        self().visit_expr(*x.m_matrix);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayPack\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"array\": ");
        self().visit_expr(*x.m_array);
        s.append(",\n" + indtd);
        s.append("\"mask\": ");
        self().visit_expr(*x.m_mask);
        s.append(",\n" + indtd);
        s.append("\"vector\": ");
        if (x.m_vector) {
            self().visit_expr(*x.m_vector);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayReshape\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"array\": ");
        self().visit_expr(*x.m_array);
        s.append(",\n" + indtd);
        s.append("\"shape\": ");
        self().visit_expr(*x.m_shape);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayBroadcast\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"array\": ");
        self().visit_expr(*x.m_array);
        s.append(",\n" + indtd);
        s.append("\"shape\": ");
        self().visit_expr(*x.m_shape);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_BitCast(const BitCast_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"BitCast\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"source\": ");
        self().visit_expr(*x.m_source);
        s.append(",\n" + indtd);
        s.append("\"mold\": ");
        self().visit_expr(*x.m_mold);
        s.append(",\n" + indtd);
        s.append("\"size\": ");
        if (x.m_size) {
            self().visit_expr(*x.m_size);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StructInstanceMember\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"m\": ");
        self().visit_symbol(*x.m_m);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StructStaticMember\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"m\": ");
        self().visit_symbol(*x.m_m);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"EnumStaticMember\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"m\": ");
        self().visit_symbol(*x.m_m);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnionInstanceMember\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"m\": ");
        self().visit_symbol(*x.m_m);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumName(const EnumName_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"EnumName\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"enum_type\": ");
        self().visit_ttype(*x.m_enum_type);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumValue(const EnumValue_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"EnumValue\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        self().visit_expr(*x.m_v);
        s.append(",\n" + indtd);
        s.append("\"enum_type\": ");
        self().visit_ttype(*x.m_enum_type);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"OverloadedCompare\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_cmpopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        self().visit_expr(*x.m_overloaded);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"OverloadedBinOp\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"op\": ");
        visit_binopType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        self().visit_expr(*x.m_overloaded);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"OverloadedUnaryMinus\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        self().visit_expr(*x.m_overloaded);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"OverloadedStringConcat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"overloaded\": ");
        self().visit_expr(*x.m_overloaded);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Cast(const Cast_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Cast\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"kind\": ");
        visit_cast_kindType(x.m_kind);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ArrayPhysicalCast\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"old\": ");
        visit_array_physical_typeType(x.m_old);
        s.append(",\n" + indtd);
        s.append("\"new\": ");
        visit_array_physical_typeType(x.m_new);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexRe\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ComplexIm\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DictItem(const DictItem_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DictItem\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"key\": ");
        self().visit_expr(*x.m_key);
        s.append(",\n" + indtd);
        s.append("\"default\": ");
        if (x.m_default) {
            self().visit_expr(*x.m_default);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CLoc(const CLoc_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CLoc\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"PointerToCPtr\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GetPointer(const GetPointer_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"GetPointer\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListItem(const ListItem_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListItem\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"pos\": ");
        self().visit_expr(*x.m_pos);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TupleItem(const TupleItem_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TupleItem\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"pos\": ");
        self().visit_expr(*x.m_pos);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListSection(const ListSection_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListSection\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"section\": ");
        self().visit_array_index(x.m_section);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ListRepeat\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        self().visit_expr(*x.m_left);
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        self().visit_expr(*x.m_right);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_DictPop(const DictPop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"DictPop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"key\": ");
        self().visit_expr(*x.m_key);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetPop(const SetPop_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SetPop\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"IntegerBitLen\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Ichar(const Ichar_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Ichar\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Iachar(const Iachar_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Iachar\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SizeOfType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_ttype(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"PointerNullConstant\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"PointerAssociated\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"ptr\": ");
        self().visit_expr(*x.m_ptr);
        s.append(",\n" + indtd);
        s.append("\"tgt\": ");
        if (x.m_tgt) {
            self().visit_expr(*x.m_tgt);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"RealSqrt\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Integer(const Integer_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Integer\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"kind\": ");
        s.append(std::to_string(x.m_kind));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"UnsignedInteger\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"kind\": ");
        s.append(std::to_string(x.m_kind));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Real\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"kind\": ");
        s.append(std::to_string(x.m_kind));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Complex\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"kind\": ");
        s.append(std::to_string(x.m_kind));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Character\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"kind\": ");
        s.append(std::to_string(x.m_kind));
        s.append(",\n" + indtd);
        s.append("\"len\": ");
        s.append(std::to_string(x.m_len));
        s.append(",\n" + indtd);
        s.append("\"len_expr\": ");
        if (x.m_len_expr) {
            self().visit_expr(*x.m_len_expr);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Logical(const Logical_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Logical\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"kind\": ");
        s.append(std::to_string(x.m_kind));
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Set\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_List(const List_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"List\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Tuple(const Tuple_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Tuple\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        s.append("[");
        if (x.n_type > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_type; i++) {
                self().visit_ttype(*x.m_type[i]);
                if (i < x.n_type-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_StructType(const StructType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"StructType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"derived_type\": ");
        self().visit_symbol(*x.m_derived_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumType(const EnumType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"EnumType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"enum_type\": ");
        self().visit_symbol(*x.m_enum_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Union(const Union_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Union\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"union_type\": ");
        self().visit_symbol(*x.m_union_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassType(const ClassType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ClassType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"class_type\": ");
        self().visit_symbol(*x.m_class_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Dict(const Dict_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Dict\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"key_type\": ");
        self().visit_ttype(*x.m_key_type);
        s.append(",\n" + indtd);
        s.append("\"value_type\": ");
        self().visit_ttype(*x.m_value_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Pointer(const Pointer_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Pointer\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Allocatable(const Allocatable_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Allocatable\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CPtr(const CPtr_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CPtr\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"SymbolicExpression\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TypeParameter\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"param\": ");
        s.append("\"" + std::string(x.m_param) + "\"");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Array\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"dims\": ");
        s.append("[");
        if (x.n_dims > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dims; i++) {
                self().visit_dimension(x.m_dims[i]);
                if (i < x.n_dims-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"physical_type\": ");
        visit_array_physical_typeType(x.m_physical_type);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FunctionType(const FunctionType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"FunctionType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg_types\": ");
        s.append("[");
        if (x.n_arg_types > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_arg_types; i++) {
                self().visit_ttype(*x.m_arg_types[i]);
                if (i < x.n_arg_types-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"return_var_type\": ");
        if (x.m_return_var_type) {
            self().visit_ttype(*x.m_return_var_type);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"abi\": ");
        visit_abiType(x.m_abi);
        s.append(",\n" + indtd);
        s.append("\"deftype\": ");
        visit_deftypeType(x.m_deftype);
        s.append(",\n" + indtd);
        s.append("\"bindc_name\": ");
        if (x.m_bindc_name) {
            s.append("\"" + str_escape_c(x.m_bindc_name) + "\"");
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"elemental\": ");
        if (x.m_elemental) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"pure\": ");
        if (x.m_pure) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"module\": ");
        if (x.m_module) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"inline\": ");
        if (x.m_inline) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"static\": ");
        if (x.m_static) {
            s.append("true");
        } else {
            s.append("false");
        }
        s.append(",\n" + indtd);
        s.append("\"restrictions\": ");
        s.append("[");
        if (x.n_restrictions > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_restrictions; i++) {
                self().visit_symbol(*x.m_restrictions[i]);
                if (i < x.n_restrictions-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"is_restriction\": ");
        if (x.m_is_restriction) {
            s.append("true");
        } else {
            s.append("false");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_cast_kindType(const cast_kindType &x) {
        switch (x) {
            case (cast_kindType::RealToInteger) : {
                s.append("\"RealToInteger\"");
             break; }
            case (cast_kindType::IntegerToReal) : {
                s.append("\"IntegerToReal\"");
             break; }
            case (cast_kindType::LogicalToReal) : {
                s.append("\"LogicalToReal\"");
             break; }
            case (cast_kindType::RealToReal) : {
                s.append("\"RealToReal\"");
             break; }
            case (cast_kindType::IntegerToInteger) : {
                s.append("\"IntegerToInteger\"");
             break; }
            case (cast_kindType::RealToComplex) : {
                s.append("\"RealToComplex\"");
             break; }
            case (cast_kindType::IntegerToComplex) : {
                s.append("\"IntegerToComplex\"");
             break; }
            case (cast_kindType::IntegerToLogical) : {
                s.append("\"IntegerToLogical\"");
             break; }
            case (cast_kindType::RealToLogical) : {
                s.append("\"RealToLogical\"");
             break; }
            case (cast_kindType::CharacterToLogical) : {
                s.append("\"CharacterToLogical\"");
             break; }
            case (cast_kindType::CharacterToInteger) : {
                s.append("\"CharacterToInteger\"");
             break; }
            case (cast_kindType::CharacterToList) : {
                s.append("\"CharacterToList\"");
             break; }
            case (cast_kindType::ComplexToLogical) : {
                s.append("\"ComplexToLogical\"");
             break; }
            case (cast_kindType::ComplexToComplex) : {
                s.append("\"ComplexToComplex\"");
             break; }
            case (cast_kindType::ComplexToReal) : {
                s.append("\"ComplexToReal\"");
             break; }
            case (cast_kindType::ComplexToInteger) : {
                s.append("\"ComplexToInteger\"");
             break; }
            case (cast_kindType::LogicalToInteger) : {
                s.append("\"LogicalToInteger\"");
             break; }
            case (cast_kindType::RealToCharacter) : {
                s.append("\"RealToCharacter\"");
             break; }
            case (cast_kindType::IntegerToCharacter) : {
                s.append("\"IntegerToCharacter\"");
             break; }
            case (cast_kindType::LogicalToCharacter) : {
                s.append("\"LogicalToCharacter\"");
             break; }
            case (cast_kindType::UnsignedIntegerToInteger) : {
                s.append("\"UnsignedIntegerToInteger\"");
             break; }
            case (cast_kindType::UnsignedIntegerToUnsignedInteger) : {
                s.append("\"UnsignedIntegerToUnsignedInteger\"");
             break; }
            case (cast_kindType::UnsignedIntegerToReal) : {
                s.append("\"UnsignedIntegerToReal\"");
             break; }
            case (cast_kindType::UnsignedIntegerToLogical) : {
                s.append("\"UnsignedIntegerToLogical\"");
             break; }
            case (cast_kindType::IntegerToUnsignedInteger) : {
                s.append("\"IntegerToUnsignedInteger\"");
             break; }
            case (cast_kindType::RealToUnsignedInteger) : {
                s.append("\"RealToUnsignedInteger\"");
             break; }
            case (cast_kindType::CPtrToUnsignedInteger) : {
                s.append("\"CPtrToUnsignedInteger\"");
             break; }
            case (cast_kindType::UnsignedIntegerToCPtr) : {
                s.append("\"UnsignedIntegerToCPtr\"");
             break; }
            case (cast_kindType::IntegerToSymbolicExpression) : {
                s.append("\"IntegerToSymbolicExpression\"");
             break; }
            case (cast_kindType::ListToArray) : {
                s.append("\"ListToArray\"");
             break; }
        }
    }
    void visit_storage_typeType(const storage_typeType &x) {
        switch (x) {
            case (storage_typeType::Default) : {
                s.append("\"Default\"");
             break; }
            case (storage_typeType::Save) : {
                s.append("\"Save\"");
             break; }
            case (storage_typeType::Parameter) : {
                s.append("\"Parameter\"");
             break; }
        }
    }
    void visit_accessType(const accessType &x) {
        switch (x) {
            case (accessType::Public) : {
                s.append("\"Public\"");
             break; }
            case (accessType::Private) : {
                s.append("\"Private\"");
             break; }
        }
    }
    void visit_intentType(const intentType &x) {
        switch (x) {
            case (intentType::Local) : {
                s.append("\"Local\"");
             break; }
            case (intentType::In) : {
                s.append("\"In\"");
             break; }
            case (intentType::Out) : {
                s.append("\"Out\"");
             break; }
            case (intentType::InOut) : {
                s.append("\"InOut\"");
             break; }
            case (intentType::ReturnVar) : {
                s.append("\"ReturnVar\"");
             break; }
            case (intentType::Unspecified) : {
                s.append("\"Unspecified\"");
             break; }
        }
    }
    void visit_deftypeType(const deftypeType &x) {
        switch (x) {
            case (deftypeType::Implementation) : {
                s.append("\"Implementation\"");
             break; }
            case (deftypeType::Interface) : {
                s.append("\"Interface\"");
             break; }
        }
    }
    void visit_presenceType(const presenceType &x) {
        switch (x) {
            case (presenceType::Required) : {
                s.append("\"Required\"");
             break; }
            case (presenceType::Optional) : {
                s.append("\"Optional\"");
             break; }
        }
    }
    void visit_abiType(const abiType &x) {
        switch (x) {
            case (abiType::Source) : {
                s.append("\"Source\"");
             break; }
            case (abiType::LFortranModule) : {
                s.append("\"LFortranModule\"");
             break; }
            case (abiType::GFortranModule) : {
                s.append("\"GFortranModule\"");
             break; }
            case (abiType::BindC) : {
                s.append("\"BindC\"");
             break; }
            case (abiType::BindPython) : {
                s.append("\"BindPython\"");
             break; }
            case (abiType::BindJS) : {
                s.append("\"BindJS\"");
             break; }
            case (abiType::Interactive) : {
                s.append("\"Interactive\"");
             break; }
            case (abiType::Intrinsic) : {
                s.append("\"Intrinsic\"");
             break; }
        }
    }
    void visit_dimension(const dimension_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"dimension\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"start\": ");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"length\": ");
        if (x.m_length) {
            self().visit_expr(*x.m_length);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"alloc_arg\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"a\": ");
        self().visit_expr(*x.m_a);
        s.append(",\n" + indtd);
        s.append("\"dims\": ");
        s.append("[");
        if (x.n_dims > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_dims; i++) {
                self().visit_dimension(x.m_dims[i]);
                if (i < x.n_dims-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"len_expr\": ");
        if (x.m_len_expr) {
            self().visit_expr(*x.m_len_expr);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"type\": ");
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Attribute(const Attribute_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Attribute\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                self().visit_attribute_arg(x.m_args[i]);
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"attribute_arg\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"arg\": ");
        s.append("\"" + std::string(x.m_arg) + "\"");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"call_arg\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"value\": ");
        if (x.m_value) {
            self().visit_expr(*x.m_value);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"reduction_expr\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"op\": ");
        visit_reduction_opType(x.m_op);
        s.append(",\n" + indtd);
        s.append("\"arg\": ");
        self().visit_expr(*x.m_arg);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Bind(const Bind_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Bind\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"lang\": ");
        s.append("\"" + str_escape_c(x.m_lang) + "\"");
        s.append(",\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + str_escape_c(x.m_name) + "\"");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"array_index\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"left\": ");
        if (x.m_left) {
            self().visit_expr(*x.m_left);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"right\": ");
        if (x.m_right) {
            self().visit_expr(*x.m_right);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"step\": ");
        if (x.m_step) {
            self().visit_expr(*x.m_step);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"do_loop_head\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"v\": ");
        if (x.m_v) {
            self().visit_expr(*x.m_v);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"start\": ");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"end\": ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"increment\": ");
        if (x.m_increment) {
            self().visit_expr(*x.m_increment);
        } else {
            s.append("[]");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.loc.first, x.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CaseStmt\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"test\": ");
        s.append("[");
        if (x.n_test > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_test; i++) {
                self().visit_expr(*x.m_test[i]);
                if (i < x.n_test-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        s.append(",\n" + indtd);
        s.append("\"fall_through\": ");
        if (x.m_fall_through) {
            s.append("true");
        } else {
            s.append("false");
        }
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"CaseStmt_Range\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"start\": ");
        if (x.m_start) {
            self().visit_expr(*x.m_start);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"end\": ");
        if (x.m_end) {
            self().visit_expr(*x.m_end);
        } else {
            s.append("[]");
        }
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TypeStmtName\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"sym\": ");
        self().visit_symbol(*x.m_sym);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"ClassStmt\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"sym\": ");
        self().visit_symbol(*x.m_sym);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"TypeStmtType\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"type\": ");
        self().visit_ttype(*x.m_type);
        s.append(",\n" + indtd);
        s.append("\"body\": ");
        s.append("[");
        if (x.n_body > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_body; i++) {
                self().visit_stmt(*x.m_body[i]);
                if (i < x.n_body-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_enumtypeType(const enumtypeType &x) {
        switch (x) {
            case (enumtypeType::IntegerConsecutiveFromZero) : {
                s.append("\"IntegerConsecutiveFromZero\"");
             break; }
            case (enumtypeType::IntegerUnique) : {
                s.append("\"IntegerUnique\"");
             break; }
            case (enumtypeType::IntegerNotUnique) : {
                s.append("\"IntegerNotUnique\"");
             break; }
            case (enumtypeType::NonInteger) : {
                s.append("\"NonInteger\"");
             break; }
        }
    }
    void visit_Require(const Require_t &x) {
        s.append("{");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"node\": \"Require\"");
        s.append(",\n" + indtd);
        s.append("\"fields\": {");
        inc_indent(); s.append("\n" + indtd);
        s.append("\"name\": ");
        s.append("\"" + std::string(x.m_name) + "\"");
        s.append(",\n" + indtd);
        s.append("\"args\": ");
        s.append("[");
        if (x.n_args > 0) {
            inc_indent(); s.append("\n" + indtd);
            for (size_t i=0; i<x.n_args; i++) {
                s.append("\"" + std::string(x.m_args[i]) + "\"");
                if (i < x.n_args-1) {
                    s.append(",\n" + indtd);
                };
            }
            dec_indent(); s.append("\n" + indtd);
        }
        s.append("]");
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        append_location(s, x.base.base.loc.first, x.base.base.loc.last);
        dec_indent(); s.append("\n" + indtd);
        s.append("}");
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_physical_typeType(const array_physical_typeType &x) {
        switch (x) {
            case (array_physical_typeType::DescriptorArray) : {
                s.append("\"DescriptorArray\"");
             break; }
            case (array_physical_typeType::PointerToDataArray) : {
                s.append("\"PointerToDataArray\"");
             break; }
            case (array_physical_typeType::UnboundedPointerToDataArray) : {
                s.append("\"UnboundedPointerToDataArray\"");
             break; }
            case (array_physical_typeType::FixedSizeArray) : {
                s.append("\"FixedSizeArray\"");
             break; }
            case (array_physical_typeType::CharacterArraySinglePointer) : {
                s.append("\"CharacterArraySinglePointer\"");
             break; }
            case (array_physical_typeType::NumPyArray) : {
                s.append("\"NumPyArray\"");
             break; }
            case (array_physical_typeType::ISODescriptorArray) : {
                s.append("\"ISODescriptorArray\"");
             break; }
            case (array_physical_typeType::SIMDArray) : {
                s.append("\"SIMDArray\"");
             break; }
        }
    }
    void visit_binopType(const binopType &x) {
        switch (x) {
            case (binopType::Add) : {
                s.append("\"Add\"");
             break; }
            case (binopType::Sub) : {
                s.append("\"Sub\"");
             break; }
            case (binopType::Mul) : {
                s.append("\"Mul\"");
             break; }
            case (binopType::Div) : {
                s.append("\"Div\"");
             break; }
            case (binopType::Pow) : {
                s.append("\"Pow\"");
             break; }
            case (binopType::BitAnd) : {
                s.append("\"BitAnd\"");
             break; }
            case (binopType::BitOr) : {
                s.append("\"BitOr\"");
             break; }
            case (binopType::BitXor) : {
                s.append("\"BitXor\"");
             break; }
            case (binopType::BitLShift) : {
                s.append("\"BitLShift\"");
             break; }
            case (binopType::BitRShift) : {
                s.append("\"BitRShift\"");
             break; }
        }
    }
    void visit_reduction_opType(const reduction_opType &x) {
        switch (x) {
            case (reduction_opType::ReduceAdd) : {
                s.append("\"ReduceAdd\"");
             break; }
            case (reduction_opType::ReduceSub) : {
                s.append("\"ReduceSub\"");
             break; }
            case (reduction_opType::ReduceMul) : {
                s.append("\"ReduceMul\"");
             break; }
            case (reduction_opType::ReduceMIN) : {
                s.append("\"ReduceMIN\"");
             break; }
            case (reduction_opType::ReduceMAX) : {
                s.append("\"ReduceMAX\"");
             break; }
        }
    }
    void visit_logicalbinopType(const logicalbinopType &x) {
        switch (x) {
            case (logicalbinopType::And) : {
                s.append("\"And\"");
             break; }
            case (logicalbinopType::Or) : {
                s.append("\"Or\"");
             break; }
            case (logicalbinopType::Xor) : {
                s.append("\"Xor\"");
             break; }
            case (logicalbinopType::NEqv) : {
                s.append("\"NEqv\"");
             break; }
            case (logicalbinopType::Eqv) : {
                s.append("\"Eqv\"");
             break; }
        }
    }
    void visit_cmpopType(const cmpopType &x) {
        switch (x) {
            case (cmpopType::Eq) : {
                s.append("\"Eq\"");
             break; }
            case (cmpopType::NotEq) : {
                s.append("\"NotEq\"");
             break; }
            case (cmpopType::Lt) : {
                s.append("\"Lt\"");
             break; }
            case (cmpopType::LtE) : {
                s.append("\"LtE\"");
             break; }
            case (cmpopType::Gt) : {
                s.append("\"Gt\"");
             break; }
            case (cmpopType::GtE) : {
                s.append("\"GtE\"");
             break; }
        }
    }
    void visit_integerbozType(const integerbozType &x) {
        switch (x) {
            case (integerbozType::Binary) : {
                s.append("\"Binary\"");
             break; }
            case (integerbozType::Hex) : {
                s.append("\"Hex\"");
             break; }
            case (integerbozType::Octal) : {
                s.append("\"Octal\"");
             break; }
        }
    }
    void visit_arrayboundType(const arrayboundType &x) {
        switch (x) {
            case (arrayboundType::LBound) : {
                s.append("\"LBound\"");
             break; }
            case (arrayboundType::UBound) : {
                s.append("\"UBound\"");
             break; }
        }
    }
    void visit_arraystorageType(const arraystorageType &x) {
        switch (x) {
            case (arraystorageType::RowMajor) : {
                s.append("\"RowMajor\"");
             break; }
            case (arraystorageType::ColMajor) : {
                s.append("\"ColMajor\"");
             break; }
        }
    }
    void visit_string_format_kindType(const string_format_kindType &x) {
        switch (x) {
            case (string_format_kindType::FormatFortran) : {
                s.append("\"FormatFortran\"");
             break; }
            case (string_format_kindType::FormatC) : {
                s.append("\"FormatC\"");
             break; }
            case (string_format_kindType::FormatPythonPercent) : {
                s.append("\"FormatPythonPercent\"");
             break; }
            case (string_format_kindType::FormatPythonFString) : {
                s.append("\"FormatPythonFString\"");
             break; }
            case (string_format_kindType::FormatPythonFormat) : {
                s.append("\"FormatPythonFormat\"");
             break; }
        }
    }
};


/******************************************************************************/
// Serialization Visitor base class

template <class StructType>
class SerializationBaseVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_int64(x.n_items);
        for (size_t i=0; i<x.n_items; i++) {
            self().write_int8(x.m_items[i]->type);
            self().visit_asr(*x.m_items[i]);
        }
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Program(const Program_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Module(const Module_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        if (x.m_loaded_from_mod) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_intrinsic) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Function(const Function_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().visit_ttype(*x.m_function_signature);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        if (x.m_return_var) {
            self().write_bool(true);
            self().visit_expr(*x.m_return_var);
        } else {
            self().write_bool(false);
        }
        visit_accessType(x.m_access);
        if (x.m_deterministic) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_side_effect_free) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_module_file) {
            self().write_bool(true);
            self().write_string(x.m_module_file);
        } else {
            self().write_bool(false);
        }
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_parent_symtab->counter);
        self().write_string(x.m_name);
        self().write_int64(x.n_procs);
        for (size_t i=0; i<x.n_procs; i++) {
            self().write_symbol(*x.m_procs[i]);
        }
        visit_accessType(x.m_access);
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_parent_symtab->counter);
        self().write_string(x.m_name);
        self().write_int64(x.n_procs);
        for (size_t i=0; i<x.n_procs; i++) {
            self().write_symbol(*x.m_procs[i]);
        }
        visit_accessType(x.m_access);
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_parent_symtab->counter);
        self().write_string(x.m_name);
        // We skip the symbol for ExternalSymbol
        self().write_string(x.m_module_name);
        self().write_int64(x.n_scope_names);
        for (size_t i=0; i<x.n_scope_names; i++) {
            self().write_string(x.m_scope_names[i]);
        }
        self().write_string(x.m_original_name);
        visit_accessType(x.m_access);
    }
    void visit_Struct(const Struct_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        self().write_int64(x.n_members);
        for (size_t i=0; i<x.n_members; i++) {
            self().write_string(x.m_members[i]);
        }
        visit_abiType(x.m_abi);
        visit_accessType(x.m_access);
        if (x.m_is_packed) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_is_abstract) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_initializers);
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        if (x.m_alignment) {
            self().write_bool(true);
            self().visit_expr(*x.m_alignment);
        } else {
            self().write_bool(false);
        }
        if (x.m_parent) {
            self().write_bool(true);
            self().write_symbol(*x.m_parent);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Enum(const Enum_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        self().write_int64(x.n_members);
        for (size_t i=0; i<x.n_members; i++) {
            self().write_string(x.m_members[i]);
        }
        visit_abiType(x.m_abi);
        visit_accessType(x.m_access);
        visit_enumtypeType(x.m_enum_value_type);
        self().visit_ttype(*x.m_type);
        if (x.m_parent) {
            self().write_bool(true);
            self().write_symbol(*x.m_parent);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnionType(const UnionType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        self().write_int64(x.n_members);
        for (size_t i=0; i<x.n_members; i++) {
            self().write_string(x.m_members[i]);
        }
        visit_abiType(x.m_abi);
        visit_accessType(x.m_access);
        self().write_int64(x.n_initializers);
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        if (x.m_parent) {
            self().write_bool(true);
            self().write_symbol(*x.m_parent);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Variable(const Variable_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_parent_symtab->counter);
        self().write_string(x.m_name);
        self().write_int64(x.n_dependencies);
        for (size_t i=0; i<x.n_dependencies; i++) {
            self().write_string(x.m_dependencies[i]);
        }
        visit_intentType(x.m_intent);
        if (x.m_symbolic_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_symbolic_value);
        } else {
            self().write_bool(false);
        }
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        visit_storage_typeType(x.m_storage);
        self().visit_ttype(*x.m_type);
        if (x.m_type_declaration) {
            self().write_bool(true);
            self().write_symbol(*x.m_type_declaration);
        } else {
            self().write_bool(false);
        }
        visit_abiType(x.m_abi);
        visit_accessType(x.m_access);
        visit_presenceType(x.m_presence);
        if (x.m_value_attr) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Class(const Class_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        visit_abiType(x.m_abi);
        visit_accessType(x.m_access);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_parent_symtab->counter);
        self().write_string(x.m_name);
        if (x.m_self_argument) {
            self().write_bool(true);
            self().write_string(x.m_self_argument);
        } else {
            self().write_bool(false);
        }
        self().write_string(x.m_proc_name);
        self().write_symbol(*x.m_proc);
        visit_abiType(x.m_abi);
        if (x.m_is_deferred) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_is_nopass) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Block(const Block_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_Requirement(const Requirement_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().write_string(x.m_args[i]);
        }
        self().write_int64(x.n_requires);
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
    }
    void visit_Template(const Template_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_symtab->counter);
        self().write_int64(x.m_symtab->get_scope().size());
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                continue;
            }
            self().write_string(a.first);
            this->visit_symbol(*a.second);
        }
        for (auto &a : x.m_symtab->get_scope()) {
            if (ASR::is_a<ASR::Function_t>(*a.second)) {
                self().write_string(a.first);
                this->visit_symbol(*a.second);
            }
        }
        self().write_string(x.m_name);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().write_string(x.m_args[i]);
        }
        self().write_int64(x.n_requires);
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
    }
    void visit_Allocate(const Allocate_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
        if (x.m_stat) {
            self().write_bool(true);
            self().visit_expr(*x.m_stat);
        } else {
            self().write_bool(false);
        }
        if (x.m_errmsg) {
            self().write_bool(true);
            self().visit_expr(*x.m_errmsg);
        } else {
            self().write_bool(false);
        }
        if (x.m_source) {
            self().write_bool(true);
            self().visit_expr(*x.m_source);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
    }
    void visit_Assign(const Assign_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        self().write_string(x.m_variable);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
        if (x.m_overloaded) {
            self().write_bool(true);
            self().visit_stmt(*x.m_overloaded);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Associate(const Associate_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
    }
    void visit_Cycle(const Cycle_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_stmt_name) {
            self().write_bool(true);
            self().write_string(x.m_stmt_name);
        } else {
            self().write_bool(false);
        }
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_vars);
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_vars);
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_do_loop_head(x.m_head);
        self().write_int64(x.n_shared);
        for (size_t i=0; i<x.n_shared; i++) {
            self().visit_expr(*x.m_shared[i]);
        }
        self().write_int64(x.n_local);
        for (size_t i=0; i<x.n_local; i++) {
            self().visit_expr(*x.m_local[i]);
        }
        self().write_int64(x.n_reduction);
        for (size_t i=0; i<x.n_reduction; i++) {
            self().visit_reduction_expr(x.m_reduction[i]);
        }
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_DoLoop(const DoLoop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_name) {
            self().write_bool(true);
            self().write_string(x.m_name);
        } else {
            self().write_bool(false);
        }
        self().visit_do_loop_head(x.m_head);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        self().write_int64(x.n_orelse);
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_code) {
            self().write_bool(true);
            self().visit_expr(*x.m_code);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Exit(const Exit_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_stmt_name) {
            self().write_bool(true);
            self().write_string(x.m_stmt_name);
        } else {
            self().write_bool(false);
        }
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_do_loop_head(x.m_head);
        self().visit_stmt(*x.m_assign_stmt);
    }
    void visit_GoTo(const GoTo_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_target_id);
        self().write_string(x.m_name);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_id);
        self().write_string(x.m_name);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_test);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        self().write_int64(x.n_orelse);
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_test);
        self().write_int64(x.m_lt_label);
        self().write_int64(x.m_eq_label);
        self().write_int64(x.m_gt_label);
    }
    void visit_Print(const Print_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_values);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator) {
            self().write_bool(true);
            self().visit_expr(*x.m_separator);
        } else {
            self().write_bool(false);
        }
        if (x.m_end) {
            self().write_bool(true);
            self().visit_expr(*x.m_end);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileOpen(const FileOpen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_newunit) {
            self().write_bool(true);
            self().visit_expr(*x.m_newunit);
        } else {
            self().write_bool(false);
        }
        if (x.m_filename) {
            self().write_bool(true);
            self().visit_expr(*x.m_filename);
        } else {
            self().write_bool(false);
        }
        if (x.m_status) {
            self().write_bool(true);
            self().visit_expr(*x.m_status);
        } else {
            self().write_bool(false);
        }
        if (x.m_form) {
            self().write_bool(true);
            self().visit_expr(*x.m_form);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileClose(const FileClose_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_unit) {
            self().write_bool(true);
            self().visit_expr(*x.m_unit);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
        if (x.m_iomsg) {
            self().write_bool(true);
            self().visit_expr(*x.m_iomsg);
        } else {
            self().write_bool(false);
        }
        if (x.m_err) {
            self().write_bool(true);
            self().visit_expr(*x.m_err);
        } else {
            self().write_bool(false);
        }
        if (x.m_status) {
            self().write_bool(true);
            self().visit_expr(*x.m_status);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileRead(const FileRead_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_unit) {
            self().write_bool(true);
            self().visit_expr(*x.m_unit);
        } else {
            self().write_bool(false);
        }
        if (x.m_fmt) {
            self().write_bool(true);
            self().visit_expr(*x.m_fmt);
        } else {
            self().write_bool(false);
        }
        if (x.m_iomsg) {
            self().write_bool(true);
            self().visit_expr(*x.m_iomsg);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
        if (x.m_size) {
            self().write_bool(true);
            self().visit_expr(*x.m_size);
        } else {
            self().write_bool(false);
        }
        if (x.m_id) {
            self().write_bool(true);
            self().visit_expr(*x.m_id);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_values);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_overloaded) {
            self().write_bool(true);
            self().visit_stmt(*x.m_overloaded);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_unit) {
            self().write_bool(true);
            self().visit_expr(*x.m_unit);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
        if (x.m_err) {
            self().write_bool(true);
            self().visit_expr(*x.m_err);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileRewind(const FileRewind_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_unit) {
            self().write_bool(true);
            self().visit_expr(*x.m_unit);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
        if (x.m_err) {
            self().write_bool(true);
            self().visit_expr(*x.m_err);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileInquire(const FileInquire_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_unit) {
            self().write_bool(true);
            self().visit_expr(*x.m_unit);
        } else {
            self().write_bool(false);
        }
        if (x.m_file) {
            self().write_bool(true);
            self().visit_expr(*x.m_file);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
        if (x.m_err) {
            self().write_bool(true);
            self().visit_expr(*x.m_err);
        } else {
            self().write_bool(false);
        }
        if (x.m_exist) {
            self().write_bool(true);
            self().visit_expr(*x.m_exist);
        } else {
            self().write_bool(false);
        }
        if (x.m_opened) {
            self().write_bool(true);
            self().visit_expr(*x.m_opened);
        } else {
            self().write_bool(false);
        }
        if (x.m_number) {
            self().write_bool(true);
            self().visit_expr(*x.m_number);
        } else {
            self().write_bool(false);
        }
        if (x.m_named) {
            self().write_bool(true);
            self().visit_expr(*x.m_named);
        } else {
            self().write_bool(false);
        }
        if (x.m_name) {
            self().write_bool(true);
            self().visit_expr(*x.m_name);
        } else {
            self().write_bool(false);
        }
        if (x.m_access) {
            self().write_bool(true);
            self().visit_expr(*x.m_access);
        } else {
            self().write_bool(false);
        }
        if (x.m_sequential) {
            self().write_bool(true);
            self().visit_expr(*x.m_sequential);
        } else {
            self().write_bool(false);
        }
        if (x.m_direct) {
            self().write_bool(true);
            self().visit_expr(*x.m_direct);
        } else {
            self().write_bool(false);
        }
        if (x.m_form) {
            self().write_bool(true);
            self().visit_expr(*x.m_form);
        } else {
            self().write_bool(false);
        }
        if (x.m_formatted) {
            self().write_bool(true);
            self().visit_expr(*x.m_formatted);
        } else {
            self().write_bool(false);
        }
        if (x.m_unformatted) {
            self().write_bool(true);
            self().visit_expr(*x.m_unformatted);
        } else {
            self().write_bool(false);
        }
        if (x.m_recl) {
            self().write_bool(true);
            self().visit_expr(*x.m_recl);
        } else {
            self().write_bool(false);
        }
        if (x.m_nextrec) {
            self().write_bool(true);
            self().visit_expr(*x.m_nextrec);
        } else {
            self().write_bool(false);
        }
        if (x.m_blank) {
            self().write_bool(true);
            self().visit_expr(*x.m_blank);
        } else {
            self().write_bool(false);
        }
        if (x.m_position) {
            self().write_bool(true);
            self().visit_expr(*x.m_position);
        } else {
            self().write_bool(false);
        }
        if (x.m_action) {
            self().write_bool(true);
            self().visit_expr(*x.m_action);
        } else {
            self().write_bool(false);
        }
        if (x.m_read) {
            self().write_bool(true);
            self().visit_expr(*x.m_read);
        } else {
            self().write_bool(false);
        }
        if (x.m_write) {
            self().write_bool(true);
            self().visit_expr(*x.m_write);
        } else {
            self().write_bool(false);
        }
        if (x.m_readwrite) {
            self().write_bool(true);
            self().visit_expr(*x.m_readwrite);
        } else {
            self().write_bool(false);
        }
        if (x.m_delim) {
            self().write_bool(true);
            self().visit_expr(*x.m_delim);
        } else {
            self().write_bool(false);
        }
        if (x.m_pad) {
            self().write_bool(true);
            self().visit_expr(*x.m_pad);
        } else {
            self().write_bool(false);
        }
        if (x.m_flen) {
            self().write_bool(true);
            self().visit_expr(*x.m_flen);
        } else {
            self().write_bool(false);
        }
        if (x.m_blocksize) {
            self().write_bool(true);
            self().visit_expr(*x.m_blocksize);
        } else {
            self().write_bool(false);
        }
        if (x.m_convert) {
            self().write_bool(true);
            self().visit_expr(*x.m_convert);
        } else {
            self().write_bool(false);
        }
        if (x.m_carriagecontrol) {
            self().write_bool(true);
            self().visit_expr(*x.m_carriagecontrol);
        } else {
            self().write_bool(false);
        }
        if (x.m_iolength) {
            self().write_bool(true);
            self().visit_expr(*x.m_iolength);
        } else {
            self().write_bool(false);
        }
    }
    void visit_FileWrite(const FileWrite_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        if (x.m_unit) {
            self().write_bool(true);
            self().visit_expr(*x.m_unit);
        } else {
            self().write_bool(false);
        }
        if (x.m_iomsg) {
            self().write_bool(true);
            self().visit_expr(*x.m_iomsg);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
        if (x.m_id) {
            self().write_bool(true);
            self().visit_expr(*x.m_id);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_values);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator) {
            self().write_bool(true);
            self().visit_expr(*x.m_separator);
        } else {
            self().write_bool(false);
        }
        if (x.m_end) {
            self().write_bool(true);
            self().visit_expr(*x.m_end);
        } else {
            self().write_bool(false);
        }
        if (x.m_overloaded) {
            self().write_bool(true);
            self().visit_stmt(*x.m_overloaded);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Return(const Return_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_test);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_case_stmt(*x.m_body[i]);
        }
        self().write_int64(x.n_default);
        for (size_t i=0; i<x.n_default; i++) {
            self().visit_stmt(*x.m_default[i]);
        }
        if (x.m_enable_fall_through) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Stop(const Stop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_code) {
            self().write_bool(true);
            self().visit_expr(*x.m_code);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Assert(const Assert_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_test);
        if (x.m_msg) {
            self().write_bool(true);
            self().visit_expr(*x.m_msg);
        } else {
            self().write_bool(false);
        }
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_name);
        if (x.m_original_name) {
            self().write_bool(true);
            self().write_symbol(*x.m_original_name);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        if (x.m_dt) {
            self().write_bool(true);
            self().visit_expr(*x.m_dt);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_intrinsic_id);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().write_int64(x.m_overload_id);
    }
    void visit_Where(const Where_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_test);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        self().write_int64(x.n_orelse);
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_name) {
            self().write_bool(true);
            self().write_string(x.m_name);
        } else {
            self().write_bool(false);
        }
        self().visit_expr(*x.m_test);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        self().write_int64(x.n_orelse);
        for (size_t i=0; i<x.n_orelse; i++) {
            self().visit_stmt(*x.m_orelse[i]);
        }
    }
    void visit_Nullify(const Nullify_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_vars);
        for (size_t i=0; i<x.n_vars; i++) {
            self().write_symbol(*x.m_vars[i]);
        }
    }
    void visit_Flush(const Flush_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        self().visit_expr(*x.m_unit);
        if (x.m_err) {
            self().write_bool(true);
            self().visit_expr(*x.m_err);
        } else {
            self().write_bool(false);
        }
        if (x.m_iomsg) {
            self().write_bool(true);
            self().visit_expr(*x.m_iomsg);
        } else {
            self().write_bool(false);
        }
        if (x.m_iostat) {
            self().write_bool(true);
            self().visit_expr(*x.m_iostat);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListAppend(const ListAppend_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_m);
    }
    void visit_SelectType(const SelectType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_selector);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_type_stmt(*x.m_body[i]);
        }
        self().write_int64(x.n_default);
        for (size_t i=0; i<x.n_default; i++) {
            self().visit_stmt(*x.m_default[i]);
        }
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_cptr);
        self().visit_expr(*x.m_ptr);
        if (x.m_shape) {
            self().write_bool(true);
            self().visit_expr(*x.m_shape);
        } else {
            self().write_bool(false);
        }
        if (x.m_lower_bounds) {
            self().write_bool(true);
            self().visit_expr(*x.m_lower_bounds);
        } else {
            self().write_bool(false);
        }
    }
    void visit_BlockCall(const BlockCall_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_label);
        self().write_symbol(*x.m_m);
    }
    void visit_SetInsert(const SetInsert_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_SetRemove(const SetRemove_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListInsert(const ListInsert_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListRemove(const ListRemove_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListClear(const ListClear_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
    }
    void visit_DictInsert(const DictInsert_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        self().visit_expr(*x.m_value);
    }
    void visit_Expr(const Expr_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_expression);
    }
    void visit_IfExp(const IfExp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_test);
        self().visit_expr(*x.m_body);
        self().visit_expr(*x.m_orelse);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_re);
        self().visit_expr(*x.m_im);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
        self().visit_ttype(*x.m_type);
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_name);
        if (x.m_original_name) {
            self().write_bool(true);
            self().write_symbol(*x.m_original_name);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        if (x.m_dt) {
            self().write_bool(true);
            self().visit_expr(*x.m_dt);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_intrinsic_id);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().write_int64(x.m_overload_id);
        if (x.m_type) {
            self().write_bool(true);
            self().visit_ttype(*x.m_type);
        } else {
            self().write_bool(false);
        }
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_arr_intrinsic_id);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().write_int64(x.m_overload_id);
        if (x.m_type) {
            self().write_bool(true);
            self().visit_ttype(*x.m_type);
        } else {
            self().write_bool(false);
        }
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_impure_intrinsic_id);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().write_int64(x.m_overload_id);
        if (x.m_type) {
            self().write_bool(true);
            self().visit_ttype(*x.m_type);
        } else {
            self().write_bool(false);
        }
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_inquiry_id);
        self().visit_ttype(*x.m_arg_type);
        if (x.m_arg) {
            self().write_bool(true);
            self().visit_expr(*x.m_arg);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        self().visit_expr(*x.m_value);
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_dt_sym);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_dt_sym);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_dt_sym);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_values);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_expr(*x.m_var);
        self().visit_expr(*x.m_start);
        self().visit_expr(*x.m_end);
        if (x.m_increment) {
            self().write_bool(true);
            self().visit_expr(*x.m_increment);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_n);
        self().visit_ttype(*x.m_type);
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_v);
        visit_integerbozType(x.m_intboz_type);
        if (x.m_type) {
            self().write_bool(true);
            self().visit_ttype(*x.m_type);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_binopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_n);
        self().visit_ttype(*x.m_type);
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_binopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_RealConstant(const RealConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_float64(x.m_r);
        self().visit_ttype(*x.m_type);
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_RealCompare(const RealCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_binopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_source);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_float64(x.m_re);
        self().write_float64(x.m_im);
        self().visit_ttype(*x.m_type);
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_binopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_value) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_logicalbinopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListConstant(const ListConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_ListLen(const ListLen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListConcat(const ListConcat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListCompare(const ListCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListCount(const ListCount_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_expr(*x.m_ele);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_SetConstant(const SetConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_elements);
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_SetLen(const SetLen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_elements);
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_TupleLen(const TupleLen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        self().visit_expr(*x.m_value);
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringConstant(const StringConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_string(x.m_s);
        self().visit_ttype(*x.m_type);
    }
    void visit_StringConcat(const StringConcat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringLen(const StringLen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringItem(const StringItem_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_expr(*x.m_idx);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringSection(const StringSection_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        if (x.m_start) {
            self().write_bool(true);
            self().visit_expr(*x.m_start);
        } else {
            self().write_bool(false);
        }
        if (x.m_end) {
            self().write_bool(true);
            self().visit_expr(*x.m_end);
        } else {
            self().write_bool(false);
        }
        if (x.m_step) {
            self().write_bool(true);
            self().visit_expr(*x.m_step);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringCompare(const StringCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringOrd(const StringOrd_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringChr(const StringChr_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StringFormat(const StringFormat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_fmt);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        visit_string_format_kindType(x.m_kind);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_DictConstant(const DictConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_keys);
        for (size_t i=0; i<x.n_keys; i++) {
            self().visit_expr(*x.m_keys[i]);
        }
        self().write_int64(x.n_values);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_DictLen(const DictLen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Var(const Var_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_v);
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_param_number);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        visit_arraystorageType(x.m_storage_format);
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_n_data);
        self().write_void(x.m_data, x.m_n_data);
        self().visit_ttype(*x.m_type);
        visit_arraystorageType(x.m_storage_format);
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        visit_arraystorageType(x.m_storage_format);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArraySection(const ArraySection_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArraySize(const ArraySize_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        if (x.m_dim) {
            self().write_bool(true);
            self().visit_expr(*x.m_dim);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        if (x.m_dim) {
            self().write_bool(true);
            self().visit_expr(*x.m_dim);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        visit_arrayboundType(x.m_bound);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_matrix);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_mask);
        if (x.m_vector) {
            self().write_bool(true);
            self().visit_expr(*x.m_vector);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_BitCast(const BitCast_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_source);
        self().visit_expr(*x.m_mold);
        if (x.m_size) {
            self().write_bool(true);
            self().visit_expr(*x.m_size);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().write_symbol(*x.m_m);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().write_symbol(*x.m_m);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().write_symbol(*x.m_m);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().write_symbol(*x.m_m);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_EnumName(const EnumName_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_EnumValue(const EnumValue_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_cmpopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        visit_binopType(x.m_op);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
        self().visit_expr(*x.m_overloaded);
    }
    void visit_Cast(const Cast_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        visit_cast_kindType(x.m_kind);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        visit_array_physical_typeType(x.m_old);
        visit_array_physical_typeType(x.m_new);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_DictItem(const DictItem_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        if (x.m_default) {
            self().write_bool(true);
            self().visit_expr(*x.m_default);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_CLoc(const CLoc_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_GetPointer(const GetPointer_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListItem(const ListItem_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_TupleItem(const TupleItem_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListSection(const ListSection_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_array_index(x.m_section);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_DictPop(const DictPop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_SetPop(const SetPop_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Ichar(const Ichar_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Iachar(const Iachar_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_ptr);
        if (x.m_tgt) {
            self().write_bool(true);
            self().visit_expr(*x.m_tgt);
        } else {
            self().write_bool(false);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Integer(const Integer_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_kind);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_kind);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_kind);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_kind);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_kind);
        self().write_int64(x.m_len);
        if (x.m_len_expr) {
            self().write_bool(true);
            self().visit_expr(*x.m_len_expr);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Logical(const Logical_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.m_kind);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
    }
    void visit_List(const List_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
    }
    void visit_Tuple(const Tuple_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_type);
        for (size_t i=0; i<x.n_type; i++) {
            self().visit_ttype(*x.m_type[i]);
        }
    }
    void visit_StructType(const StructType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_derived_type);
    }
    void visit_EnumType(const EnumType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_enum_type);
    }
    void visit_Union(const Union_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_union_type);
    }
    void visit_ClassType(const ClassType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_class_type);
    }
    void visit_Dict(const Dict_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_key_type);
        self().visit_ttype(*x.m_value_type);
    }
    void visit_Pointer(const Pointer_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
    }
    void visit_Allocatable(const Allocatable_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
    }
    void visit_CPtr(const CPtr_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_string(x.m_param);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
        self().write_int64(x.n_dims);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
        visit_array_physical_typeType(x.m_physical_type);
    }
    void visit_FunctionType(const FunctionType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_arg_types);
        for (size_t i=0; i<x.n_arg_types; i++) {
            self().visit_ttype(*x.m_arg_types[i]);
        }
        if (x.m_return_var_type) {
            self().write_bool(true);
            self().visit_ttype(*x.m_return_var_type);
        } else {
            self().write_bool(false);
        }
        visit_abiType(x.m_abi);
        visit_deftypeType(x.m_deftype);
        if (x.m_bindc_name) {
            self().write_bool(true);
            self().write_string(x.m_bindc_name);
        } else {
            self().write_bool(false);
        }
        if (x.m_elemental) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_pure) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_module) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_inline) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        if (x.m_static) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_restrictions);
        for (size_t i=0; i<x.n_restrictions; i++) {
            self().write_symbol(*x.m_restrictions[i]);
        }
        if (x.m_is_restriction) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
    }
    void visit_cast_kindType(const cast_kindType &x) {
        self().write_int8(x);
    }
    void visit_storage_typeType(const storage_typeType &x) {
        self().write_int8(x);
    }
    void visit_accessType(const accessType &x) {
        self().write_int8(x);
    }
    void visit_intentType(const intentType &x) {
        self().write_int8(x);
    }
    void visit_deftypeType(const deftypeType &x) {
        self().write_int8(x);
    }
    void visit_presenceType(const presenceType &x) {
        self().write_int8(x);
    }
    void visit_abiType(const abiType &x) {
        self().write_int8(x);
    }
    void visit_dimension(const dimension_t &x) {
        if (x.m_start) {
            self().write_bool(true);
            self().visit_expr(*x.m_start);
        } else {
            self().write_bool(false);
        }
        if (x.m_length) {
            self().write_bool(true);
            self().visit_expr(*x.m_length);
        } else {
            self().write_bool(false);
        }
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        self().visit_expr(*x.m_a);
        self().write_int64(x.n_dims);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
        if (x.m_len_expr) {
            self().write_bool(true);
            self().visit_expr(*x.m_len_expr);
        } else {
            self().write_bool(false);
        }
        if (x.m_type) {
            self().write_bool(true);
            self().visit_ttype(*x.m_type);
        } else {
            self().write_bool(false);
        }
    }
    void visit_Attribute(const Attribute_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_string(x.m_name);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_attribute_arg(x.m_args[i]);
        }
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        self().write_string(x.m_arg);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        if (x.m_value) {
            self().write_bool(true);
            self().visit_expr(*x.m_value);
        } else {
            self().write_bool(false);
        }
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        visit_reduction_opType(x.m_op);
        self().visit_expr(*x.m_arg);
    }
    void visit_Bind(const Bind_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_string(x.m_lang);
        self().write_string(x.m_name);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        if (x.m_left) {
            self().write_bool(true);
            self().visit_expr(*x.m_left);
        } else {
            self().write_bool(false);
        }
        if (x.m_right) {
            self().write_bool(true);
            self().visit_expr(*x.m_right);
        } else {
            self().write_bool(false);
        }
        if (x.m_step) {
            self().write_bool(true);
            self().visit_expr(*x.m_step);
        } else {
            self().write_bool(false);
        }
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        if (x.m_v) {
            self().write_bool(true);
            self().visit_expr(*x.m_v);
        } else {
            self().write_bool(false);
        }
        if (x.m_start) {
            self().write_bool(true);
            self().visit_expr(*x.m_start);
        } else {
            self().write_bool(false);
        }
        if (x.m_end) {
            self().write_bool(true);
            self().visit_expr(*x.m_end);
        } else {
            self().write_bool(false);
        }
        if (x.m_increment) {
            self().write_bool(true);
            self().visit_expr(*x.m_increment);
        } else {
            self().write_bool(false);
        }
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_int64(x.n_test);
        for (size_t i=0; i<x.n_test; i++) {
            self().visit_expr(*x.m_test[i]);
        }
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
        if (x.m_fall_through) {
            self().write_bool(true);
        } else {
            self().write_bool(false);
        }
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        if (x.m_start) {
            self().write_bool(true);
            self().visit_expr(*x.m_start);
        } else {
            self().write_bool(false);
        }
        if (x.m_end) {
            self().write_bool(true);
            self().visit_expr(*x.m_end);
        } else {
            self().write_bool(false);
        }
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_sym);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_symbol(*x.m_sym);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().visit_ttype(*x.m_type);
        self().write_int64(x.n_body);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_stmt(*x.m_body[i]);
        }
    }
    void visit_enumtypeType(const enumtypeType &x) {
        self().write_int8(x);
    }
    void visit_Require(const Require_t &x) {
        self().write_int8(x.base.type);
        self().write_int64(x.base.base.loc.first);
        self().write_int64(x.base.base.loc.last);
        self().write_string(x.m_name);
        self().write_int64(x.n_args);
        for (size_t i=0; i<x.n_args; i++) {
            self().write_string(x.m_args[i]);
        }
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_physical_typeType(const array_physical_typeType &x) {
        self().write_int8(x);
    }
    void visit_binopType(const binopType &x) {
        self().write_int8(x);
    }
    void visit_reduction_opType(const reduction_opType &x) {
        self().write_int8(x);
    }
    void visit_logicalbinopType(const logicalbinopType &x) {
        self().write_int8(x);
    }
    void visit_cmpopType(const cmpopType &x) {
        self().write_int8(x);
    }
    void visit_integerbozType(const integerbozType &x) {
        self().write_int8(x);
    }
    void visit_arrayboundType(const arrayboundType &x) {
        self().write_int8(x);
    }
    void visit_arraystorageType(const arraystorageType &x) {
        self().write_int8(x);
    }
    void visit_string_format_kindType(const string_format_kindType &x) {
        self().write_int8(x);
    }
};


/******************************************************************************/
// Deserialization Visitor base class

template <class StructType>
class DeserializationBaseVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    Allocator &al;
    bool load_symtab_id;
    std::map<uint64_t,SymbolTable*> id_symtab_map;
    DeserializationBaseVisitor(Allocator &al, bool load_symtab_id) : al{al}, load_symtab_id{load_symtab_id} {}
    asr_t* deserialize_node() {
        uint8_t t = self().read_int8();
        ASR::asrType ty = static_cast<ASR::asrType>(t);
        switch (ty) {
            case (ASR::asrType::unit) : return self().deserialize_unit();
            case (ASR::asrType::symbol) : return self().deserialize_symbol();
            case (ASR::asrType::stmt) : return self().deserialize_stmt();
            case (ASR::asrType::expr) : return self().deserialize_expr();
            case (ASR::asrType::ttype) : return self().deserialize_ttype();
            case (ASR::asrType::attribute) : return self().deserialize_attribute();
            case (ASR::asrType::tbind) : return self().deserialize_tbind();
            case (ASR::asrType::case_stmt) : return self().deserialize_case_stmt();
            case (ASR::asrType::type_stmt) : return self().deserialize_type_stmt();
            case (ASR::asrType::require_instantiation) : return self().deserialize_require_instantiation();
            default : throw LCompilersException("Unknown type in deserialize_node()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    asr_t* deserialize_TranslationUnit() {
        size_t n_items; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        n_items = self().read_int64();
        Vec<asr_t*> v_items;
        v_items.reserve(al, n_items);
        for (size_t i=0; i<n_items; i++) {
            v_items.push_back(al, self().deserialize_node());
        }
        return ASR::make_TranslationUnit_t(al, loc, m_symtab, v_items.p, v_items.n);
    }
    asr_t* deserialize_unit() {
        uint8_t t = self().read_int8();
        ASR::unitType ty = static_cast<ASR::unitType>(t);
        switch (ty) {
            case (ASR::unitType::TranslationUnit) : return self().deserialize_TranslationUnit();
            default : throw LCompilersException("Unknown type in deserialize_unit()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    asr_t* deserialize_Program() {
        size_t n_dependencies; // Sequence
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_Program_t(al, loc, m_symtab, m_name, v_dependencies.p, v_dependencies.n, v_body.p, v_body.n);
    }
    asr_t* deserialize_Module() {
        size_t n_dependencies; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        bool m_loaded_from_mod = self().read_bool();
        bool m_intrinsic = self().read_bool();
        return ASR::make_Module_t(al, loc, m_symtab, m_name, v_dependencies.p, v_dependencies.n, m_loaded_from_mod, m_intrinsic);
    }
    asr_t* deserialize_Function() {
        size_t n_dependencies; // Sequence
        size_t n_args; // Sequence
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        ASR::ttype_t *m_function_signature;
        m_function_signature = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        ASR::expr_t *m_return_var;
        if (self().read_bool()) {
        m_return_var = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_return_var = nullptr;
        }
        ASR::accessType m_access = self().deserialize_access();
        bool m_deterministic = self().read_bool();
        bool m_side_effect_free = self().read_bool();
        char *m_module_file;
        bool m_module_file_present = self().read_bool();
        if (m_module_file_present) {
        m_module_file = self().read_cstring();
        } else {
        m_module_file = nullptr;
        }
        return ASR::make_Function_t(al, loc, m_symtab, m_name, m_function_signature, v_dependencies.p, v_dependencies.n, v_args.p, v_args.n, v_body.p, v_body.n, m_return_var, m_access, m_deterministic, m_side_effect_free, m_module_file);
    }
    asr_t* deserialize_GenericProcedure() {
        size_t n_procs; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_parent_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_parent_symtab_counter) != id_symtab_map.end());
        SymbolTable *m_parent_symtab = id_symtab_map[m_parent_symtab_counter];
        char *m_name;
        m_name = self().read_cstring();
        n_procs = self().read_int64();
        Vec<symbol_t*> v_procs;
        v_procs.reserve(al, n_procs);
        for (size_t i=0; i<n_procs; i++) {
            v_procs.push_back(al, self().read_symbol());
        }
        ASR::accessType m_access = self().deserialize_access();
        return ASR::make_GenericProcedure_t(al, loc, m_parent_symtab, m_name, v_procs.p, v_procs.n, m_access);
    }
    asr_t* deserialize_CustomOperator() {
        size_t n_procs; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_parent_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_parent_symtab_counter) != id_symtab_map.end());
        SymbolTable *m_parent_symtab = id_symtab_map[m_parent_symtab_counter];
        char *m_name;
        m_name = self().read_cstring();
        n_procs = self().read_int64();
        Vec<symbol_t*> v_procs;
        v_procs.reserve(al, n_procs);
        for (size_t i=0; i<n_procs; i++) {
            v_procs.push_back(al, self().read_symbol());
        }
        ASR::accessType m_access = self().deserialize_access();
        return ASR::make_CustomOperator_t(al, loc, m_parent_symtab, m_name, v_procs.p, v_procs.n, m_access);
    }
    asr_t* deserialize_ExternalSymbol() {
        size_t n_scope_names; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_parent_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_parent_symtab_counter) != id_symtab_map.end());
        SymbolTable *m_parent_symtab = id_symtab_map[m_parent_symtab_counter];
        char *m_name;
        m_name = self().read_cstring();
        ASR::symbol_t *m_external;
        // We skip the symbol for ExternalSymbol
        m_external = nullptr;
        char *m_module_name;
        m_module_name = self().read_cstring();
        n_scope_names = self().read_int64();
        Vec<char*> v_scope_names;
        v_scope_names.reserve(al, n_scope_names);
        for (size_t i=0; i<n_scope_names; i++) {
            v_scope_names.push_back(al, self().read_cstring());
        }
        char *m_original_name;
        m_original_name = self().read_cstring();
        ASR::accessType m_access = self().deserialize_access();
        return ASR::make_ExternalSymbol_t(al, loc, m_parent_symtab, m_name, m_external, m_module_name, v_scope_names.p, v_scope_names.n, m_original_name, m_access);
    }
    asr_t* deserialize_Struct() {
        size_t n_dependencies; // Sequence
        size_t n_members; // Sequence
        size_t n_initializers; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        n_members = self().read_int64();
        Vec<char*> v_members;
        v_members.reserve(al, n_members);
        for (size_t i=0; i<n_members; i++) {
            v_members.push_back(al, self().read_cstring());
        }
        ASR::abiType m_abi = self().deserialize_abi();
        ASR::accessType m_access = self().deserialize_access();
        bool m_is_packed = self().read_bool();
        bool m_is_abstract = self().read_bool();
        n_initializers = self().read_int64();
        Vec<call_arg_t> v_initializers;
        v_initializers.reserve(al, n_initializers);
        for (size_t i=0; i<n_initializers; i++) {
            v_initializers.push_back(al, self().deserialize_call_arg());
        }
        ASR::expr_t *m_alignment;
        if (self().read_bool()) {
        m_alignment = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_alignment = nullptr;
        }
        ASR::symbol_t *m_parent;
        if (self().read_bool()) {
        m_parent = self().read_symbol();
        } else {
        m_parent = nullptr;
        }
        return ASR::make_Struct_t(al, loc, m_symtab, m_name, v_dependencies.p, v_dependencies.n, v_members.p, v_members.n, m_abi, m_access, m_is_packed, m_is_abstract, v_initializers.p, v_initializers.n, m_alignment, m_parent);
    }
    asr_t* deserialize_Enum() {
        size_t n_dependencies; // Sequence
        size_t n_members; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        n_members = self().read_int64();
        Vec<char*> v_members;
        v_members.reserve(al, n_members);
        for (size_t i=0; i<n_members; i++) {
            v_members.push_back(al, self().read_cstring());
        }
        ASR::abiType m_abi = self().deserialize_abi();
        ASR::accessType m_access = self().deserialize_access();
        ASR::enumtypeType m_enum_value_type = self().deserialize_enumtype();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::symbol_t *m_parent;
        if (self().read_bool()) {
        m_parent = self().read_symbol();
        } else {
        m_parent = nullptr;
        }
        return ASR::make_Enum_t(al, loc, m_symtab, m_name, v_dependencies.p, v_dependencies.n, v_members.p, v_members.n, m_abi, m_access, m_enum_value_type, m_type, m_parent);
    }
    asr_t* deserialize_UnionType() {
        size_t n_dependencies; // Sequence
        size_t n_members; // Sequence
        size_t n_initializers; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        n_members = self().read_int64();
        Vec<char*> v_members;
        v_members.reserve(al, n_members);
        for (size_t i=0; i<n_members; i++) {
            v_members.push_back(al, self().read_cstring());
        }
        ASR::abiType m_abi = self().deserialize_abi();
        ASR::accessType m_access = self().deserialize_access();
        n_initializers = self().read_int64();
        Vec<call_arg_t> v_initializers;
        v_initializers.reserve(al, n_initializers);
        for (size_t i=0; i<n_initializers; i++) {
            v_initializers.push_back(al, self().deserialize_call_arg());
        }
        ASR::symbol_t *m_parent;
        if (self().read_bool()) {
        m_parent = self().read_symbol();
        } else {
        m_parent = nullptr;
        }
        return ASR::make_UnionType_t(al, loc, m_symtab, m_name, v_dependencies.p, v_dependencies.n, v_members.p, v_members.n, m_abi, m_access, v_initializers.p, v_initializers.n, m_parent);
    }
    asr_t* deserialize_Variable() {
        size_t n_dependencies; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_parent_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_parent_symtab_counter) != id_symtab_map.end());
        SymbolTable *m_parent_symtab = id_symtab_map[m_parent_symtab_counter];
        char *m_name;
        m_name = self().read_cstring();
        n_dependencies = self().read_int64();
        Vec<char*> v_dependencies;
        v_dependencies.reserve(al, n_dependencies);
        for (size_t i=0; i<n_dependencies; i++) {
            v_dependencies.push_back(al, self().read_cstring());
        }
        ASR::intentType m_intent = self().deserialize_intent();
        ASR::expr_t *m_symbolic_value;
        if (self().read_bool()) {
        m_symbolic_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_symbolic_value = nullptr;
        }
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::storage_typeType m_storage = self().deserialize_storage_type();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::symbol_t *m_type_declaration;
        if (self().read_bool()) {
        m_type_declaration = self().read_symbol();
        } else {
        m_type_declaration = nullptr;
        }
        ASR::abiType m_abi = self().deserialize_abi();
        ASR::accessType m_access = self().deserialize_access();
        ASR::presenceType m_presence = self().deserialize_presence();
        bool m_value_attr = self().read_bool();
        return ASR::make_Variable_t(al, loc, m_parent_symtab, m_name, v_dependencies.p, v_dependencies.n, m_intent, m_symbolic_value, m_value, m_storage, m_type, m_type_declaration, m_abi, m_access, m_presence, m_value_attr);
    }
    asr_t* deserialize_Class() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        ASR::abiType m_abi = self().deserialize_abi();
        ASR::accessType m_access = self().deserialize_access();
        return ASR::make_Class_t(al, loc, m_symtab, m_name, m_abi, m_access);
    }
    asr_t* deserialize_ClassProcedure() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_parent_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_parent_symtab_counter) != id_symtab_map.end());
        SymbolTable *m_parent_symtab = id_symtab_map[m_parent_symtab_counter];
        char *m_name;
        m_name = self().read_cstring();
        char *m_self_argument;
        bool m_self_argument_present = self().read_bool();
        if (m_self_argument_present) {
        m_self_argument = self().read_cstring();
        } else {
        m_self_argument = nullptr;
        }
        char *m_proc_name;
        m_proc_name = self().read_cstring();
        ASR::symbol_t *m_proc;
        m_proc = self().read_symbol();
        ASR::abiType m_abi = self().deserialize_abi();
        bool m_is_deferred = self().read_bool();
        bool m_is_nopass = self().read_bool();
        return ASR::make_ClassProcedure_t(al, loc, m_parent_symtab, m_name, m_self_argument, m_proc_name, m_proc, m_abi, m_is_deferred, m_is_nopass);
    }
    asr_t* deserialize_AssociateBlock() {
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_AssociateBlock_t(al, loc, m_symtab, m_name, v_body.p, v_body.n);
    }
    asr_t* deserialize_Block() {
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_Block_t(al, loc, m_symtab, m_name, v_body.p, v_body.n);
    }
    asr_t* deserialize_Requirement() {
        size_t n_args; // Sequence
        size_t n_requires; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_args = self().read_int64();
        Vec<char*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().read_cstring());
        }
        n_requires = self().read_int64();
        Vec<require_instantiation_t*> v_requires;
        v_requires.reserve(al, n_requires);
        for (size_t i=0; i<n_requires; i++) {
            v_requires.push_back(al, ASR::down_cast<ASR::require_instantiation_t>(self().deserialize_require_instantiation()));
        }
        return ASR::make_Requirement_t(al, loc, m_symtab, m_name, v_args.p, v_args.n, v_requires.p, v_requires.n);
    }
    asr_t* deserialize_Template() {
        size_t n_args; // Sequence
        size_t n_requires; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        uint64_t m_symtab_counter = self().read_int64();
        LCOMPILERS_ASSERT(id_symtab_map.find(m_symtab_counter) == id_symtab_map.end());
        SymbolTable *m_symtab = al.make_new<SymbolTable>(nullptr);
        if (load_symtab_id) m_symtab->counter = m_symtab_counter;
        id_symtab_map[m_symtab_counter] = m_symtab;
        {
            size_t n = self().read_int64();
            for (size_t i=0; i<n; i++) {
                std::string name = self().read_string();
                symbol_t *sym = down_cast<symbol_t>(deserialize_symbol());
                self().symtab_insert_symbol(*m_symtab, name, sym);
            }
        }
        char *m_name;
        m_name = self().read_cstring();
        n_args = self().read_int64();
        Vec<char*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().read_cstring());
        }
        n_requires = self().read_int64();
        Vec<require_instantiation_t*> v_requires;
        v_requires.reserve(al, n_requires);
        for (size_t i=0; i<n_requires; i++) {
            v_requires.push_back(al, ASR::down_cast<ASR::require_instantiation_t>(self().deserialize_require_instantiation()));
        }
        return ASR::make_Template_t(al, loc, m_symtab, m_name, v_args.p, v_args.n, v_requires.p, v_requires.n);
    }
    asr_t* deserialize_symbol() {
        uint8_t t = self().read_int8();
        ASR::symbolType ty = static_cast<ASR::symbolType>(t);
        switch (ty) {
            case (ASR::symbolType::Program) : return self().deserialize_Program();
            case (ASR::symbolType::Module) : return self().deserialize_Module();
            case (ASR::symbolType::Function) : return self().deserialize_Function();
            case (ASR::symbolType::GenericProcedure) : return self().deserialize_GenericProcedure();
            case (ASR::symbolType::CustomOperator) : return self().deserialize_CustomOperator();
            case (ASR::symbolType::ExternalSymbol) : return self().deserialize_ExternalSymbol();
            case (ASR::symbolType::Struct) : return self().deserialize_Struct();
            case (ASR::symbolType::Enum) : return self().deserialize_Enum();
            case (ASR::symbolType::UnionType) : return self().deserialize_UnionType();
            case (ASR::symbolType::Variable) : return self().deserialize_Variable();
            case (ASR::symbolType::Class) : return self().deserialize_Class();
            case (ASR::symbolType::ClassProcedure) : return self().deserialize_ClassProcedure();
            case (ASR::symbolType::AssociateBlock) : return self().deserialize_AssociateBlock();
            case (ASR::symbolType::Block) : return self().deserialize_Block();
            case (ASR::symbolType::Requirement) : return self().deserialize_Requirement();
            case (ASR::symbolType::Template) : return self().deserialize_Template();
            default : throw LCompilersException("Unknown type in deserialize_symbol()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    asr_t* deserialize_Allocate() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_args = self().read_int64();
        Vec<alloc_arg_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_alloc_arg());
        }
        ASR::expr_t *m_stat;
        if (self().read_bool()) {
        m_stat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_stat = nullptr;
        }
        ASR::expr_t *m_errmsg;
        if (self().read_bool()) {
        m_errmsg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_errmsg = nullptr;
        }
        ASR::expr_t *m_source;
        if (self().read_bool()) {
        m_source = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_source = nullptr;
        }
        return ASR::make_Allocate_t(al, loc, v_args.p, v_args.n, m_stat, m_errmsg, m_source);
    }
    asr_t* deserialize_ReAlloc() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_args = self().read_int64();
        Vec<alloc_arg_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_alloc_arg());
        }
        return ASR::make_ReAlloc_t(al, loc, v_args.p, v_args.n);
    }
    asr_t* deserialize_Assign() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        char *m_variable;
        m_variable = self().read_cstring();
        return ASR::make_Assign_t(al, loc, m_label, m_variable);
    }
    asr_t* deserialize_Assignment() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_target;
        m_target = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_value;
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::stmt_t *m_overloaded;
        if (self().read_bool()) {
        m_overloaded = ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt());
        } else {
        m_overloaded = nullptr;
        }
        return ASR::make_Assignment_t(al, loc, m_target, m_value, m_overloaded);
    }
    asr_t* deserialize_Associate() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_target;
        m_target = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_value;
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_Associate_t(al, loc, m_target, m_value);
    }
    asr_t* deserialize_Cycle() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_stmt_name;
        bool m_stmt_name_present = self().read_bool();
        if (m_stmt_name_present) {
        m_stmt_name = self().read_cstring();
        } else {
        m_stmt_name = nullptr;
        }
        return ASR::make_Cycle_t(al, loc, m_stmt_name);
    }
    asr_t* deserialize_ExplicitDeallocate() {
        size_t n_vars; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_vars = self().read_int64();
        Vec<expr_t*> v_vars;
        v_vars.reserve(al, n_vars);
        for (size_t i=0; i<n_vars; i++) {
            v_vars.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        return ASR::make_ExplicitDeallocate_t(al, loc, v_vars.p, v_vars.n);
    }
    asr_t* deserialize_ImplicitDeallocate() {
        size_t n_vars; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_vars = self().read_int64();
        Vec<expr_t*> v_vars;
        v_vars.reserve(al, n_vars);
        for (size_t i=0; i<n_vars; i++) {
            v_vars.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        return ASR::make_ImplicitDeallocate_t(al, loc, v_vars.p, v_vars.n);
    }
    asr_t* deserialize_DoConcurrentLoop() {
        size_t n_shared; // Sequence
        size_t n_local; // Sequence
        size_t n_reduction; // Sequence
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::do_loop_head_t m_head = self().deserialize_do_loop_head();
        n_shared = self().read_int64();
        Vec<expr_t*> v_shared;
        v_shared.reserve(al, n_shared);
        for (size_t i=0; i<n_shared; i++) {
            v_shared.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        n_local = self().read_int64();
        Vec<expr_t*> v_local;
        v_local.reserve(al, n_local);
        for (size_t i=0; i<n_local; i++) {
            v_local.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        n_reduction = self().read_int64();
        Vec<reduction_expr_t> v_reduction;
        v_reduction.reserve(al, n_reduction);
        for (size_t i=0; i<n_reduction; i++) {
            v_reduction.push_back(al, self().deserialize_reduction_expr());
        }
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_DoConcurrentLoop_t(al, loc, m_head, v_shared.p, v_shared.n, v_local.p, v_local.n, v_reduction.p, v_reduction.n, v_body.p, v_body.n);
    }
    asr_t* deserialize_DoLoop() {
        size_t n_body; // Sequence
        size_t n_orelse; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_name;
        bool m_name_present = self().read_bool();
        if (m_name_present) {
        m_name = self().read_cstring();
        } else {
        m_name = nullptr;
        }
        ASR::do_loop_head_t m_head = self().deserialize_do_loop_head();
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        n_orelse = self().read_int64();
        Vec<stmt_t*> v_orelse;
        v_orelse.reserve(al, n_orelse);
        for (size_t i=0; i<n_orelse; i++) {
            v_orelse.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_DoLoop_t(al, loc, m_name, m_head, v_body.p, v_body.n, v_orelse.p, v_orelse.n);
    }
    asr_t* deserialize_ErrorStop() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_code;
        if (self().read_bool()) {
        m_code = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_code = nullptr;
        }
        return ASR::make_ErrorStop_t(al, loc, m_code);
    }
    asr_t* deserialize_Exit() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_stmt_name;
        bool m_stmt_name_present = self().read_bool();
        if (m_stmt_name_present) {
        m_stmt_name = self().read_cstring();
        } else {
        m_stmt_name = nullptr;
        }
        return ASR::make_Exit_t(al, loc, m_stmt_name);
    }
    asr_t* deserialize_ForAllSingle() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::do_loop_head_t m_head = self().deserialize_do_loop_head();
        ASR::stmt_t *m_assign_stmt;
        m_assign_stmt = ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt());
        return ASR::make_ForAllSingle_t(al, loc, m_head, m_assign_stmt);
    }
    asr_t* deserialize_GoTo() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_target_id = self().read_int64();
        char *m_name;
        m_name = self().read_cstring();
        return ASR::make_GoTo_t(al, loc, m_target_id, m_name);
    }
    asr_t* deserialize_GoToTarget() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_id = self().read_int64();
        char *m_name;
        m_name = self().read_cstring();
        return ASR::make_GoToTarget_t(al, loc, m_id, m_name);
    }
    asr_t* deserialize_If() {
        size_t n_body; // Sequence
        size_t n_orelse; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        n_orelse = self().read_int64();
        Vec<stmt_t*> v_orelse;
        v_orelse.reserve(al, n_orelse);
        for (size_t i=0; i<n_orelse; i++) {
            v_orelse.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_If_t(al, loc, m_test, v_body.p, v_body.n, v_orelse.p, v_orelse.n);
    }
    asr_t* deserialize_IfArithmetic() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        int64_t m_lt_label = self().read_int64();
        int64_t m_eq_label = self().read_int64();
        int64_t m_gt_label = self().read_int64();
        return ASR::make_IfArithmetic_t(al, loc, m_test, m_lt_label, m_eq_label, m_gt_label);
    }
    asr_t* deserialize_Print() {
        size_t n_values; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_values = self().read_int64();
        Vec<expr_t*> v_values;
        v_values.reserve(al, n_values);
        for (size_t i=0; i<n_values; i++) {
            v_values.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::expr_t *m_separator;
        if (self().read_bool()) {
        m_separator = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_separator = nullptr;
        }
        ASR::expr_t *m_end;
        if (self().read_bool()) {
        m_end = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_end = nullptr;
        }
        return ASR::make_Print_t(al, loc, v_values.p, v_values.n, m_separator, m_end);
    }
    asr_t* deserialize_FileOpen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_newunit;
        if (self().read_bool()) {
        m_newunit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_newunit = nullptr;
        }
        ASR::expr_t *m_filename;
        if (self().read_bool()) {
        m_filename = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_filename = nullptr;
        }
        ASR::expr_t *m_status;
        if (self().read_bool()) {
        m_status = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_status = nullptr;
        }
        ASR::expr_t *m_form;
        if (self().read_bool()) {
        m_form = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_form = nullptr;
        }
        return ASR::make_FileOpen_t(al, loc, m_label, m_newunit, m_filename, m_status, m_form);
    }
    asr_t* deserialize_FileClose() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        if (self().read_bool()) {
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unit = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        ASR::expr_t *m_iomsg;
        if (self().read_bool()) {
        m_iomsg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iomsg = nullptr;
        }
        ASR::expr_t *m_err;
        if (self().read_bool()) {
        m_err = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_err = nullptr;
        }
        ASR::expr_t *m_status;
        if (self().read_bool()) {
        m_status = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_status = nullptr;
        }
        return ASR::make_FileClose_t(al, loc, m_label, m_unit, m_iostat, m_iomsg, m_err, m_status);
    }
    asr_t* deserialize_FileRead() {
        size_t n_values; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        if (self().read_bool()) {
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unit = nullptr;
        }
        ASR::expr_t *m_fmt;
        if (self().read_bool()) {
        m_fmt = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_fmt = nullptr;
        }
        ASR::expr_t *m_iomsg;
        if (self().read_bool()) {
        m_iomsg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iomsg = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        ASR::expr_t *m_size;
        if (self().read_bool()) {
        m_size = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_size = nullptr;
        }
        ASR::expr_t *m_id;
        if (self().read_bool()) {
        m_id = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_id = nullptr;
        }
        n_values = self().read_int64();
        Vec<expr_t*> v_values;
        v_values.reserve(al, n_values);
        for (size_t i=0; i<n_values; i++) {
            v_values.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::stmt_t *m_overloaded;
        if (self().read_bool()) {
        m_overloaded = ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt());
        } else {
        m_overloaded = nullptr;
        }
        return ASR::make_FileRead_t(al, loc, m_label, m_unit, m_fmt, m_iomsg, m_iostat, m_size, m_id, v_values.p, v_values.n, m_overloaded);
    }
    asr_t* deserialize_FileBackspace() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        if (self().read_bool()) {
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unit = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        ASR::expr_t *m_err;
        if (self().read_bool()) {
        m_err = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_err = nullptr;
        }
        return ASR::make_FileBackspace_t(al, loc, m_label, m_unit, m_iostat, m_err);
    }
    asr_t* deserialize_FileRewind() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        if (self().read_bool()) {
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unit = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        ASR::expr_t *m_err;
        if (self().read_bool()) {
        m_err = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_err = nullptr;
        }
        return ASR::make_FileRewind_t(al, loc, m_label, m_unit, m_iostat, m_err);
    }
    asr_t* deserialize_FileInquire() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        if (self().read_bool()) {
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unit = nullptr;
        }
        ASR::expr_t *m_file;
        if (self().read_bool()) {
        m_file = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_file = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        ASR::expr_t *m_err;
        if (self().read_bool()) {
        m_err = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_err = nullptr;
        }
        ASR::expr_t *m_exist;
        if (self().read_bool()) {
        m_exist = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_exist = nullptr;
        }
        ASR::expr_t *m_opened;
        if (self().read_bool()) {
        m_opened = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_opened = nullptr;
        }
        ASR::expr_t *m_number;
        if (self().read_bool()) {
        m_number = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_number = nullptr;
        }
        ASR::expr_t *m_named;
        if (self().read_bool()) {
        m_named = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_named = nullptr;
        }
        ASR::expr_t *m_name;
        if (self().read_bool()) {
        m_name = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_name = nullptr;
        }
        ASR::expr_t *m_access;
        if (self().read_bool()) {
        m_access = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_access = nullptr;
        }
        ASR::expr_t *m_sequential;
        if (self().read_bool()) {
        m_sequential = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_sequential = nullptr;
        }
        ASR::expr_t *m_direct;
        if (self().read_bool()) {
        m_direct = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_direct = nullptr;
        }
        ASR::expr_t *m_form;
        if (self().read_bool()) {
        m_form = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_form = nullptr;
        }
        ASR::expr_t *m_formatted;
        if (self().read_bool()) {
        m_formatted = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_formatted = nullptr;
        }
        ASR::expr_t *m_unformatted;
        if (self().read_bool()) {
        m_unformatted = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unformatted = nullptr;
        }
        ASR::expr_t *m_recl;
        if (self().read_bool()) {
        m_recl = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_recl = nullptr;
        }
        ASR::expr_t *m_nextrec;
        if (self().read_bool()) {
        m_nextrec = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_nextrec = nullptr;
        }
        ASR::expr_t *m_blank;
        if (self().read_bool()) {
        m_blank = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_blank = nullptr;
        }
        ASR::expr_t *m_position;
        if (self().read_bool()) {
        m_position = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_position = nullptr;
        }
        ASR::expr_t *m_action;
        if (self().read_bool()) {
        m_action = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_action = nullptr;
        }
        ASR::expr_t *m_read;
        if (self().read_bool()) {
        m_read = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_read = nullptr;
        }
        ASR::expr_t *m_write;
        if (self().read_bool()) {
        m_write = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_write = nullptr;
        }
        ASR::expr_t *m_readwrite;
        if (self().read_bool()) {
        m_readwrite = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_readwrite = nullptr;
        }
        ASR::expr_t *m_delim;
        if (self().read_bool()) {
        m_delim = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_delim = nullptr;
        }
        ASR::expr_t *m_pad;
        if (self().read_bool()) {
        m_pad = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_pad = nullptr;
        }
        ASR::expr_t *m_flen;
        if (self().read_bool()) {
        m_flen = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_flen = nullptr;
        }
        ASR::expr_t *m_blocksize;
        if (self().read_bool()) {
        m_blocksize = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_blocksize = nullptr;
        }
        ASR::expr_t *m_convert;
        if (self().read_bool()) {
        m_convert = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_convert = nullptr;
        }
        ASR::expr_t *m_carriagecontrol;
        if (self().read_bool()) {
        m_carriagecontrol = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_carriagecontrol = nullptr;
        }
        ASR::expr_t *m_iolength;
        if (self().read_bool()) {
        m_iolength = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iolength = nullptr;
        }
        return ASR::make_FileInquire_t(al, loc, m_label, m_unit, m_file, m_iostat, m_err, m_exist, m_opened, m_number, m_named, m_name, m_access, m_sequential, m_direct, m_form, m_formatted, m_unformatted, m_recl, m_nextrec, m_blank, m_position, m_action, m_read, m_write, m_readwrite, m_delim, m_pad, m_flen, m_blocksize, m_convert, m_carriagecontrol, m_iolength);
    }
    asr_t* deserialize_FileWrite() {
        size_t n_values; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        if (self().read_bool()) {
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_unit = nullptr;
        }
        ASR::expr_t *m_iomsg;
        if (self().read_bool()) {
        m_iomsg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iomsg = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        ASR::expr_t *m_id;
        if (self().read_bool()) {
        m_id = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_id = nullptr;
        }
        n_values = self().read_int64();
        Vec<expr_t*> v_values;
        v_values.reserve(al, n_values);
        for (size_t i=0; i<n_values; i++) {
            v_values.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::expr_t *m_separator;
        if (self().read_bool()) {
        m_separator = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_separator = nullptr;
        }
        ASR::expr_t *m_end;
        if (self().read_bool()) {
        m_end = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_end = nullptr;
        }
        ASR::stmt_t *m_overloaded;
        if (self().read_bool()) {
        m_overloaded = ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt());
        } else {
        m_overloaded = nullptr;
        }
        return ASR::make_FileWrite_t(al, loc, m_label, m_unit, m_iomsg, m_iostat, m_id, v_values.p, v_values.n, m_separator, m_end, m_overloaded);
    }
    asr_t* deserialize_Return() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        return ASR::make_Return_t(al, loc);
    }
    asr_t* deserialize_Select() {
        size_t n_body; // Sequence
        size_t n_default; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_body = self().read_int64();
        Vec<case_stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::case_stmt_t>(self().deserialize_case_stmt()));
        }
        n_default = self().read_int64();
        Vec<stmt_t*> v_default;
        v_default.reserve(al, n_default);
        for (size_t i=0; i<n_default; i++) {
            v_default.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        bool m_enable_fall_through = self().read_bool();
        return ASR::make_Select_t(al, loc, m_test, v_body.p, v_body.n, v_default.p, v_default.n, m_enable_fall_through);
    }
    asr_t* deserialize_Stop() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_code;
        if (self().read_bool()) {
        m_code = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_code = nullptr;
        }
        return ASR::make_Stop_t(al, loc, m_code);
    }
    asr_t* deserialize_Assert() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_msg;
        if (self().read_bool()) {
        m_msg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_msg = nullptr;
        }
        return ASR::make_Assert_t(al, loc, m_test, m_msg);
    }
    asr_t* deserialize_SubroutineCall() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_name;
        m_name = self().read_symbol();
        ASR::symbol_t *m_original_name;
        if (self().read_bool()) {
        m_original_name = self().read_symbol();
        } else {
        m_original_name = nullptr;
        }
        n_args = self().read_int64();
        Vec<call_arg_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_call_arg());
        }
        ASR::expr_t *m_dt;
        if (self().read_bool()) {
        m_dt = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_dt = nullptr;
        }
        return ASR::make_SubroutineCall_t(al, loc, m_name, m_original_name, v_args.p, v_args.n, m_dt);
    }
    asr_t* deserialize_IntrinsicImpureSubroutine() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_intrinsic_id = self().read_int64();
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        int64_t m_overload_id = self().read_int64();
        return ASR::make_IntrinsicImpureSubroutine_t(al, loc, m_intrinsic_id, v_args.p, v_args.n, m_overload_id);
    }
    asr_t* deserialize_Where() {
        size_t n_body; // Sequence
        size_t n_orelse; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        n_orelse = self().read_int64();
        Vec<stmt_t*> v_orelse;
        v_orelse.reserve(al, n_orelse);
        for (size_t i=0; i<n_orelse; i++) {
            v_orelse.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_Where_t(al, loc, m_test, v_body.p, v_body.n, v_orelse.p, v_orelse.n);
    }
    asr_t* deserialize_WhileLoop() {
        size_t n_body; // Sequence
        size_t n_orelse; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_name;
        bool m_name_present = self().read_bool();
        if (m_name_present) {
        m_name = self().read_cstring();
        } else {
        m_name = nullptr;
        }
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        n_orelse = self().read_int64();
        Vec<stmt_t*> v_orelse;
        v_orelse.reserve(al, n_orelse);
        for (size_t i=0; i<n_orelse; i++) {
            v_orelse.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_WhileLoop_t(al, loc, m_name, m_test, v_body.p, v_body.n, v_orelse.p, v_orelse.n);
    }
    asr_t* deserialize_Nullify() {
        size_t n_vars; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_vars = self().read_int64();
        Vec<symbol_t*> v_vars;
        v_vars.reserve(al, n_vars);
        for (size_t i=0; i<n_vars; i++) {
            v_vars.push_back(al, self().read_symbol());
        }
        return ASR::make_Nullify_t(al, loc, v_vars.p, v_vars.n);
    }
    asr_t* deserialize_Flush() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::expr_t *m_unit;
        m_unit = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_err;
        if (self().read_bool()) {
        m_err = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_err = nullptr;
        }
        ASR::expr_t *m_iomsg;
        if (self().read_bool()) {
        m_iomsg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iomsg = nullptr;
        }
        ASR::expr_t *m_iostat;
        if (self().read_bool()) {
        m_iostat = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_iostat = nullptr;
        }
        return ASR::make_Flush_t(al, loc, m_label, m_unit, m_err, m_iomsg, m_iostat);
    }
    asr_t* deserialize_ListAppend() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ele;
        m_ele = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_ListAppend_t(al, loc, m_a, m_ele);
    }
    asr_t* deserialize_AssociateBlockCall() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_m;
        m_m = self().read_symbol();
        return ASR::make_AssociateBlockCall_t(al, loc, m_m);
    }
    asr_t* deserialize_SelectType() {
        size_t n_body; // Sequence
        size_t n_default; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_selector;
        m_selector = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_body = self().read_int64();
        Vec<type_stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::type_stmt_t>(self().deserialize_type_stmt()));
        }
        n_default = self().read_int64();
        Vec<stmt_t*> v_default;
        v_default.reserve(al, n_default);
        for (size_t i=0; i<n_default; i++) {
            v_default.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_SelectType_t(al, loc, m_selector, v_body.p, v_body.n, v_default.p, v_default.n);
    }
    asr_t* deserialize_CPtrToPointer() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_cptr;
        m_cptr = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ptr;
        m_ptr = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_shape;
        if (self().read_bool()) {
        m_shape = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_shape = nullptr;
        }
        ASR::expr_t *m_lower_bounds;
        if (self().read_bool()) {
        m_lower_bounds = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_lower_bounds = nullptr;
        }
        return ASR::make_CPtrToPointer_t(al, loc, m_cptr, m_ptr, m_shape, m_lower_bounds);
    }
    asr_t* deserialize_BlockCall() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_label = self().read_int64();
        ASR::symbol_t *m_m;
        m_m = self().read_symbol();
        return ASR::make_BlockCall_t(al, loc, m_label, m_m);
    }
    asr_t* deserialize_SetInsert() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ele;
        m_ele = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_SetInsert_t(al, loc, m_a, m_ele);
    }
    asr_t* deserialize_SetRemove() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ele;
        m_ele = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_SetRemove_t(al, loc, m_a, m_ele);
    }
    asr_t* deserialize_ListInsert() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_pos;
        m_pos = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ele;
        m_ele = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_ListInsert_t(al, loc, m_a, m_pos, m_ele);
    }
    asr_t* deserialize_ListRemove() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ele;
        m_ele = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_ListRemove_t(al, loc, m_a, m_ele);
    }
    asr_t* deserialize_ListClear() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_ListClear_t(al, loc, m_a);
    }
    asr_t* deserialize_DictInsert() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_key;
        m_key = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_value;
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_DictInsert_t(al, loc, m_a, m_key, m_value);
    }
    asr_t* deserialize_Expr() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_expression;
        m_expression = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_Expr_t(al, loc, m_expression);
    }
    asr_t* deserialize_stmt() {
        uint8_t t = self().read_int8();
        ASR::stmtType ty = static_cast<ASR::stmtType>(t);
        switch (ty) {
            case (ASR::stmtType::Allocate) : return self().deserialize_Allocate();
            case (ASR::stmtType::ReAlloc) : return self().deserialize_ReAlloc();
            case (ASR::stmtType::Assign) : return self().deserialize_Assign();
            case (ASR::stmtType::Assignment) : return self().deserialize_Assignment();
            case (ASR::stmtType::Associate) : return self().deserialize_Associate();
            case (ASR::stmtType::Cycle) : return self().deserialize_Cycle();
            case (ASR::stmtType::ExplicitDeallocate) : return self().deserialize_ExplicitDeallocate();
            case (ASR::stmtType::ImplicitDeallocate) : return self().deserialize_ImplicitDeallocate();
            case (ASR::stmtType::DoConcurrentLoop) : return self().deserialize_DoConcurrentLoop();
            case (ASR::stmtType::DoLoop) : return self().deserialize_DoLoop();
            case (ASR::stmtType::ErrorStop) : return self().deserialize_ErrorStop();
            case (ASR::stmtType::Exit) : return self().deserialize_Exit();
            case (ASR::stmtType::ForAllSingle) : return self().deserialize_ForAllSingle();
            case (ASR::stmtType::GoTo) : return self().deserialize_GoTo();
            case (ASR::stmtType::GoToTarget) : return self().deserialize_GoToTarget();
            case (ASR::stmtType::If) : return self().deserialize_If();
            case (ASR::stmtType::IfArithmetic) : return self().deserialize_IfArithmetic();
            case (ASR::stmtType::Print) : return self().deserialize_Print();
            case (ASR::stmtType::FileOpen) : return self().deserialize_FileOpen();
            case (ASR::stmtType::FileClose) : return self().deserialize_FileClose();
            case (ASR::stmtType::FileRead) : return self().deserialize_FileRead();
            case (ASR::stmtType::FileBackspace) : return self().deserialize_FileBackspace();
            case (ASR::stmtType::FileRewind) : return self().deserialize_FileRewind();
            case (ASR::stmtType::FileInquire) : return self().deserialize_FileInquire();
            case (ASR::stmtType::FileWrite) : return self().deserialize_FileWrite();
            case (ASR::stmtType::Return) : return self().deserialize_Return();
            case (ASR::stmtType::Select) : return self().deserialize_Select();
            case (ASR::stmtType::Stop) : return self().deserialize_Stop();
            case (ASR::stmtType::Assert) : return self().deserialize_Assert();
            case (ASR::stmtType::SubroutineCall) : return self().deserialize_SubroutineCall();
            case (ASR::stmtType::IntrinsicImpureSubroutine) : return self().deserialize_IntrinsicImpureSubroutine();
            case (ASR::stmtType::Where) : return self().deserialize_Where();
            case (ASR::stmtType::WhileLoop) : return self().deserialize_WhileLoop();
            case (ASR::stmtType::Nullify) : return self().deserialize_Nullify();
            case (ASR::stmtType::Flush) : return self().deserialize_Flush();
            case (ASR::stmtType::ListAppend) : return self().deserialize_ListAppend();
            case (ASR::stmtType::AssociateBlockCall) : return self().deserialize_AssociateBlockCall();
            case (ASR::stmtType::SelectType) : return self().deserialize_SelectType();
            case (ASR::stmtType::CPtrToPointer) : return self().deserialize_CPtrToPointer();
            case (ASR::stmtType::BlockCall) : return self().deserialize_BlockCall();
            case (ASR::stmtType::SetInsert) : return self().deserialize_SetInsert();
            case (ASR::stmtType::SetRemove) : return self().deserialize_SetRemove();
            case (ASR::stmtType::ListInsert) : return self().deserialize_ListInsert();
            case (ASR::stmtType::ListRemove) : return self().deserialize_ListRemove();
            case (ASR::stmtType::ListClear) : return self().deserialize_ListClear();
            case (ASR::stmtType::DictInsert) : return self().deserialize_DictInsert();
            case (ASR::stmtType::Expr) : return self().deserialize_Expr();
            default : throw LCompilersException("Unknown type in deserialize_stmt()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    asr_t* deserialize_IfExp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_test;
        m_test = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_body;
        m_body = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_orelse;
        m_orelse = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IfExp_t(al, loc, m_test, m_body, m_orelse, m_type, m_value);
    }
    asr_t* deserialize_ComplexConstructor() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_re;
        m_re = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_im;
        m_im = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ComplexConstructor_t(al, loc, m_re, m_im, m_type, m_value);
    }
    asr_t* deserialize_NamedExpr() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_target;
        m_target = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_value;
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_NamedExpr_t(al, loc, m_target, m_value, m_type);
    }
    asr_t* deserialize_FunctionCall() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_name;
        m_name = self().read_symbol();
        ASR::symbol_t *m_original_name;
        if (self().read_bool()) {
        m_original_name = self().read_symbol();
        } else {
        m_original_name = nullptr;
        }
        n_args = self().read_int64();
        Vec<call_arg_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_call_arg());
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::expr_t *m_dt;
        if (self().read_bool()) {
        m_dt = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_dt = nullptr;
        }
        return ASR::make_FunctionCall_t(al, loc, m_name, m_original_name, v_args.p, v_args.n, m_type, m_value, m_dt);
    }
    asr_t* deserialize_IntrinsicElementalFunction() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_intrinsic_id = self().read_int64();
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        int64_t m_overload_id = self().read_int64();
        ASR::ttype_t *m_type;
        if (self().read_bool()) {
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        } else {
        m_type = nullptr;
        }
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntrinsicElementalFunction_t(al, loc, m_intrinsic_id, v_args.p, v_args.n, m_overload_id, m_type, m_value);
    }
    asr_t* deserialize_IntrinsicArrayFunction() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_arr_intrinsic_id = self().read_int64();
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        int64_t m_overload_id = self().read_int64();
        ASR::ttype_t *m_type;
        if (self().read_bool()) {
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        } else {
        m_type = nullptr;
        }
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntrinsicArrayFunction_t(al, loc, m_arr_intrinsic_id, v_args.p, v_args.n, m_overload_id, m_type, m_value);
    }
    asr_t* deserialize_IntrinsicImpureFunction() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_impure_intrinsic_id = self().read_int64();
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        int64_t m_overload_id = self().read_int64();
        ASR::ttype_t *m_type;
        if (self().read_bool()) {
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        } else {
        m_type = nullptr;
        }
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntrinsicImpureFunction_t(al, loc, m_impure_intrinsic_id, v_args.p, v_args.n, m_overload_id, m_type, m_value);
    }
    asr_t* deserialize_TypeInquiry() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_inquiry_id = self().read_int64();
        ASR::ttype_t *m_arg_type;
        m_arg_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_arg;
        if (self().read_bool()) {
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_arg = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_TypeInquiry_t(al, loc, m_inquiry_id, m_arg_type, m_arg, m_type, m_value);
    }
    asr_t* deserialize_StructConstructor() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_dt_sym;
        m_dt_sym = self().read_symbol();
        n_args = self().read_int64();
        Vec<call_arg_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_call_arg());
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StructConstructor_t(al, loc, m_dt_sym, v_args.p, v_args.n, m_type, m_value);
    }
    asr_t* deserialize_EnumConstructor() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_dt_sym;
        m_dt_sym = self().read_symbol();
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_EnumConstructor_t(al, loc, m_dt_sym, v_args.p, v_args.n, m_type, m_value);
    }
    asr_t* deserialize_UnionTypeConstructor() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_dt_sym;
        m_dt_sym = self().read_symbol();
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_UnionTypeConstructor_t(al, loc, m_dt_sym, v_args.p, v_args.n, m_type, m_value);
    }
    asr_t* deserialize_ImpliedDoLoop() {
        size_t n_values; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_values = self().read_int64();
        Vec<expr_t*> v_values;
        v_values.reserve(al, n_values);
        for (size_t i=0; i<n_values; i++) {
            v_values.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::expr_t *m_var;
        m_var = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_start;
        m_start = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_end;
        m_end = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_increment;
        if (self().read_bool()) {
        m_increment = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_increment = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ImpliedDoLoop_t(al, loc, v_values.p, v_values.n, m_var, m_start, m_end, m_increment, m_type, m_value);
    }
    asr_t* deserialize_IntegerConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_n = self().read_int64();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_IntegerConstant_t(al, loc, m_n, m_type);
    }
    asr_t* deserialize_IntegerBOZ() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_v = self().read_int64();
        ASR::integerbozType m_intboz_type = self().deserialize_integerboz();
        ASR::ttype_t *m_type;
        if (self().read_bool()) {
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        } else {
        m_type = nullptr;
        }
        return ASR::make_IntegerBOZ_t(al, loc, m_v, m_intboz_type, m_type);
    }
    asr_t* deserialize_IntegerBitNot() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntegerBitNot_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_IntegerUnaryMinus() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntegerUnaryMinus_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_IntegerCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntegerCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_IntegerBinOp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::binopType m_op = self().deserialize_binop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntegerBinOp_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_UnsignedIntegerConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_n = self().read_int64();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_UnsignedIntegerConstant_t(al, loc, m_n, m_type);
    }
    asr_t* deserialize_UnsignedIntegerUnaryMinus() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_UnsignedIntegerUnaryMinus_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_UnsignedIntegerBitNot() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_UnsignedIntegerBitNot_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_UnsignedIntegerCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_UnsignedIntegerCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_UnsignedIntegerBinOp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::binopType m_op = self().deserialize_binop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_UnsignedIntegerBinOp_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_RealConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        double m_r = self().read_float64();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_RealConstant_t(al, loc, m_r, m_type);
    }
    asr_t* deserialize_RealUnaryMinus() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_RealUnaryMinus_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_RealCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_RealCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_RealBinOp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::binopType m_op = self().deserialize_binop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_RealBinOp_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_RealCopySign() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_target;
        m_target = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_source;
        m_source = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_RealCopySign_t(al, loc, m_target, m_source, m_type, m_value);
    }
    asr_t* deserialize_ComplexConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        double m_re = self().read_float64();
        double m_im = self().read_float64();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_ComplexConstant_t(al, loc, m_re, m_im, m_type);
    }
    asr_t* deserialize_ComplexUnaryMinus() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ComplexUnaryMinus_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_ComplexCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ComplexCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_ComplexBinOp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::binopType m_op = self().deserialize_binop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ComplexBinOp_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_LogicalConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        bool m_value = self().read_bool();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_LogicalConstant_t(al, loc, m_value, m_type);
    }
    asr_t* deserialize_LogicalNot() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_LogicalNot_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_LogicalCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_LogicalCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_LogicalBinOp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::logicalbinopType m_op = self().deserialize_logicalbinop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_LogicalBinOp_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_ListConstant() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_ListConstant_t(al, loc, v_args.p, v_args.n, m_type);
    }
    asr_t* deserialize_ListLen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListLen_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_ListConcat() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListConcat_t(al, loc, m_left, m_right, m_type, m_value);
    }
    asr_t* deserialize_ListCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_ListCount() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_ele;
        m_ele = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListCount_t(al, loc, m_arg, m_ele, m_type, m_value);
    }
    asr_t* deserialize_SetConstant() {
        size_t n_elements; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_elements = self().read_int64();
        Vec<expr_t*> v_elements;
        v_elements.reserve(al, n_elements);
        for (size_t i=0; i<n_elements; i++) {
            v_elements.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_SetConstant_t(al, loc, v_elements.p, v_elements.n, m_type);
    }
    asr_t* deserialize_SetLen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_SetLen_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_TupleConstant() {
        size_t n_elements; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_elements = self().read_int64();
        Vec<expr_t*> v_elements;
        v_elements.reserve(al, n_elements);
        for (size_t i=0; i<n_elements; i++) {
            v_elements.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_TupleConstant_t(al, loc, v_elements.p, v_elements.n, m_type);
    }
    asr_t* deserialize_TupleLen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_TupleLen_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_TupleCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_TupleCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_TupleConcat() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_TupleConcat_t(al, loc, m_left, m_right, m_type, m_value);
    }
    asr_t* deserialize_StringConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_s;
        m_s = self().read_cstring();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_StringConstant_t(al, loc, m_s, m_type);
    }
    asr_t* deserialize_StringConcat() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringConcat_t(al, loc, m_left, m_right, m_type, m_value);
    }
    asr_t* deserialize_StringRepeat() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringRepeat_t(al, loc, m_left, m_right, m_type, m_value);
    }
    asr_t* deserialize_StringLen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringLen_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_StringItem() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_idx;
        m_idx = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringItem_t(al, loc, m_arg, m_idx, m_type, m_value);
    }
    asr_t* deserialize_StringSection() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_start;
        if (self().read_bool()) {
        m_start = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_start = nullptr;
        }
        ASR::expr_t *m_end;
        if (self().read_bool()) {
        m_end = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_end = nullptr;
        }
        ASR::expr_t *m_step;
        if (self().read_bool()) {
        m_step = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_step = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringSection_t(al, loc, m_arg, m_start, m_end, m_step, m_type, m_value);
    }
    asr_t* deserialize_StringCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_StringOrd() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringOrd_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_StringChr() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringChr_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_StringFormat() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_fmt;
        m_fmt = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::string_format_kindType m_kind = self().deserialize_string_format_kind();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StringFormat_t(al, loc, m_fmt, v_args.p, v_args.n, m_kind, m_type, m_value);
    }
    asr_t* deserialize_CPtrCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_CPtrCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_SymbolicCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_SymbolicCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value);
    }
    asr_t* deserialize_DictConstant() {
        size_t n_keys; // Sequence
        size_t n_values; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_keys = self().read_int64();
        Vec<expr_t*> v_keys;
        v_keys.reserve(al, n_keys);
        for (size_t i=0; i<n_keys; i++) {
            v_keys.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        n_values = self().read_int64();
        Vec<expr_t*> v_values;
        v_values.reserve(al, n_values);
        for (size_t i=0; i<n_values; i++) {
            v_values.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_DictConstant_t(al, loc, v_keys.p, v_keys.n, v_values.p, v_values.n, m_type);
    }
    asr_t* deserialize_DictLen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_DictLen_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_Var() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_v;
        m_v = self().read_symbol();
        return ASR::make_Var_t(al, loc, m_v);
    }
    asr_t* deserialize_FunctionParam() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_param_number = self().read_int64();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_FunctionParam_t(al, loc, m_param_number, m_type, m_value);
    }
    asr_t* deserialize_ArrayConstructor() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_args = self().read_int64();
        Vec<expr_t*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::arraystorageType m_storage_format = self().deserialize_arraystorage();
        return ASR::make_ArrayConstructor_t(al, loc, v_args.p, v_args.n, m_type, m_value, m_storage_format);
    }
    asr_t* deserialize_ArrayConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_n_data = self().read_int64();
        void *m_data = self().read_void(m_n_data);
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::arraystorageType m_storage_format = self().deserialize_arraystorage();
        return ASR::make_ArrayConstant_t(al, loc, m_n_data, m_data, m_type, m_storage_format);
    }
    asr_t* deserialize_ArrayItem() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_args = self().read_int64();
        Vec<array_index_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_array_index());
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::arraystorageType m_storage_format = self().deserialize_arraystorage();
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayItem_t(al, loc, m_v, v_args.p, v_args.n, m_type, m_storage_format, m_value);
    }
    asr_t* deserialize_ArraySection() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        n_args = self().read_int64();
        Vec<array_index_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_array_index());
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArraySection_t(al, loc, m_v, v_args.p, v_args.n, m_type, m_value);
    }
    asr_t* deserialize_ArraySize() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_dim;
        if (self().read_bool()) {
        m_dim = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_dim = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArraySize_t(al, loc, m_v, m_dim, m_type, m_value);
    }
    asr_t* deserialize_ArrayBound() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_dim;
        if (self().read_bool()) {
        m_dim = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_dim = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::arrayboundType m_bound = self().deserialize_arraybound();
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayBound_t(al, loc, m_v, m_dim, m_type, m_bound, m_value);
    }
    asr_t* deserialize_ArrayTranspose() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_matrix;
        m_matrix = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayTranspose_t(al, loc, m_matrix, m_type, m_value);
    }
    asr_t* deserialize_ArrayPack() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_array;
        m_array = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_mask;
        m_mask = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_vector;
        if (self().read_bool()) {
        m_vector = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_vector = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayPack_t(al, loc, m_array, m_mask, m_vector, m_type, m_value);
    }
    asr_t* deserialize_ArrayReshape() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_array;
        m_array = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_shape;
        m_shape = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayReshape_t(al, loc, m_array, m_shape, m_type, m_value);
    }
    asr_t* deserialize_ArrayBroadcast() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_array;
        m_array = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_shape;
        m_shape = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayBroadcast_t(al, loc, m_array, m_shape, m_type, m_value);
    }
    asr_t* deserialize_BitCast() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_source;
        m_source = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_mold;
        m_mold = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_size;
        if (self().read_bool()) {
        m_size = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_size = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_BitCast_t(al, loc, m_source, m_mold, m_size, m_type, m_value);
    }
    asr_t* deserialize_StructInstanceMember() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::symbol_t *m_m;
        m_m = self().read_symbol();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StructInstanceMember_t(al, loc, m_v, m_m, m_type, m_value);
    }
    asr_t* deserialize_StructStaticMember() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::symbol_t *m_m;
        m_m = self().read_symbol();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_StructStaticMember_t(al, loc, m_v, m_m, m_type, m_value);
    }
    asr_t* deserialize_EnumStaticMember() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::symbol_t *m_m;
        m_m = self().read_symbol();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_EnumStaticMember_t(al, loc, m_v, m_m, m_type, m_value);
    }
    asr_t* deserialize_UnionInstanceMember() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::symbol_t *m_m;
        m_m = self().read_symbol();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_UnionInstanceMember_t(al, loc, m_v, m_m, m_type, m_value);
    }
    asr_t* deserialize_EnumName() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_enum_type;
        m_enum_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_EnumName_t(al, loc, m_v, m_enum_type, m_type, m_value);
    }
    asr_t* deserialize_EnumValue() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_v;
        m_v = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_enum_type;
        m_enum_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_EnumValue_t(al, loc, m_v, m_enum_type, m_type, m_value);
    }
    asr_t* deserialize_OverloadedCompare() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cmpopType m_op = self().deserialize_cmpop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::expr_t *m_overloaded;
        m_overloaded = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_OverloadedCompare_t(al, loc, m_left, m_op, m_right, m_type, m_value, m_overloaded);
    }
    asr_t* deserialize_OverloadedBinOp() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::binopType m_op = self().deserialize_binop();
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::expr_t *m_overloaded;
        m_overloaded = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_OverloadedBinOp_t(al, loc, m_left, m_op, m_right, m_type, m_value, m_overloaded);
    }
    asr_t* deserialize_OverloadedUnaryMinus() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::expr_t *m_overloaded;
        m_overloaded = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_OverloadedUnaryMinus_t(al, loc, m_arg, m_type, m_value, m_overloaded);
    }
    asr_t* deserialize_OverloadedStringConcat() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        ASR::expr_t *m_overloaded;
        m_overloaded = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        return ASR::make_OverloadedStringConcat_t(al, loc, m_left, m_right, m_type, m_value, m_overloaded);
    }
    asr_t* deserialize_Cast() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::cast_kindType m_kind = self().deserialize_cast_kind();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_Cast_t(al, loc, m_arg, m_kind, m_type, m_value);
    }
    asr_t* deserialize_ArrayPhysicalCast() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::array_physical_typeType m_old = self().deserialize_array_physical_type();
        ASR::array_physical_typeType m_new = self().deserialize_array_physical_type();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ArrayPhysicalCast_t(al, loc, m_arg, m_old, m_new, m_type, m_value);
    }
    asr_t* deserialize_ComplexRe() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ComplexRe_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_ComplexIm() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ComplexIm_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_DictItem() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_key;
        m_key = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_default;
        if (self().read_bool()) {
        m_default = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_default = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_DictItem_t(al, loc, m_a, m_key, m_default, m_type, m_value);
    }
    asr_t* deserialize_CLoc() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_CLoc_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_PointerToCPtr() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_PointerToCPtr_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_GetPointer() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_GetPointer_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_ListItem() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_pos;
        m_pos = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListItem_t(al, loc, m_a, m_pos, m_type, m_value);
    }
    asr_t* deserialize_TupleItem() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_pos;
        m_pos = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_TupleItem_t(al, loc, m_a, m_pos, m_type, m_value);
    }
    asr_t* deserialize_ListSection() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::array_index_t m_section = self().deserialize_array_index();
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListSection_t(al, loc, m_a, m_section, m_type, m_value);
    }
    asr_t* deserialize_ListRepeat() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_left;
        m_left = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_right;
        m_right = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_ListRepeat_t(al, loc, m_left, m_right, m_type, m_value);
    }
    asr_t* deserialize_DictPop() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_key;
        m_key = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_DictPop_t(al, loc, m_a, m_key, m_type, m_value);
    }
    asr_t* deserialize_SetPop() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_SetPop_t(al, loc, m_a, m_type, m_value);
    }
    asr_t* deserialize_IntegerBitLen() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_a;
        m_a = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_IntegerBitLen_t(al, loc, m_a, m_type, m_value);
    }
    asr_t* deserialize_Ichar() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_Ichar_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_Iachar() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_Iachar_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_SizeOfType() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_arg;
        m_arg = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_SizeOfType_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_PointerNullConstant() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_PointerNullConstant_t(al, loc, m_type);
    }
    asr_t* deserialize_PointerAssociated() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_ptr;
        m_ptr = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::expr_t *m_tgt;
        if (self().read_bool()) {
        m_tgt = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_tgt = nullptr;
        }
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_PointerAssociated_t(al, loc, m_ptr, m_tgt, m_type, m_value);
    }
    asr_t* deserialize_RealSqrt() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_arg;
        m_arg = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::expr_t *m_value;
        if (self().read_bool()) {
        m_value = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_value = nullptr;
        }
        return ASR::make_RealSqrt_t(al, loc, m_arg, m_type, m_value);
    }
    asr_t* deserialize_expr() {
        uint8_t t = self().read_int8();
        ASR::exprType ty = static_cast<ASR::exprType>(t);
        switch (ty) {
            case (ASR::exprType::IfExp) : return self().deserialize_IfExp();
            case (ASR::exprType::ComplexConstructor) : return self().deserialize_ComplexConstructor();
            case (ASR::exprType::NamedExpr) : return self().deserialize_NamedExpr();
            case (ASR::exprType::FunctionCall) : return self().deserialize_FunctionCall();
            case (ASR::exprType::IntrinsicElementalFunction) : return self().deserialize_IntrinsicElementalFunction();
            case (ASR::exprType::IntrinsicArrayFunction) : return self().deserialize_IntrinsicArrayFunction();
            case (ASR::exprType::IntrinsicImpureFunction) : return self().deserialize_IntrinsicImpureFunction();
            case (ASR::exprType::TypeInquiry) : return self().deserialize_TypeInquiry();
            case (ASR::exprType::StructConstructor) : return self().deserialize_StructConstructor();
            case (ASR::exprType::EnumConstructor) : return self().deserialize_EnumConstructor();
            case (ASR::exprType::UnionTypeConstructor) : return self().deserialize_UnionTypeConstructor();
            case (ASR::exprType::ImpliedDoLoop) : return self().deserialize_ImpliedDoLoop();
            case (ASR::exprType::IntegerConstant) : return self().deserialize_IntegerConstant();
            case (ASR::exprType::IntegerBOZ) : return self().deserialize_IntegerBOZ();
            case (ASR::exprType::IntegerBitNot) : return self().deserialize_IntegerBitNot();
            case (ASR::exprType::IntegerUnaryMinus) : return self().deserialize_IntegerUnaryMinus();
            case (ASR::exprType::IntegerCompare) : return self().deserialize_IntegerCompare();
            case (ASR::exprType::IntegerBinOp) : return self().deserialize_IntegerBinOp();
            case (ASR::exprType::UnsignedIntegerConstant) : return self().deserialize_UnsignedIntegerConstant();
            case (ASR::exprType::UnsignedIntegerUnaryMinus) : return self().deserialize_UnsignedIntegerUnaryMinus();
            case (ASR::exprType::UnsignedIntegerBitNot) : return self().deserialize_UnsignedIntegerBitNot();
            case (ASR::exprType::UnsignedIntegerCompare) : return self().deserialize_UnsignedIntegerCompare();
            case (ASR::exprType::UnsignedIntegerBinOp) : return self().deserialize_UnsignedIntegerBinOp();
            case (ASR::exprType::RealConstant) : return self().deserialize_RealConstant();
            case (ASR::exprType::RealUnaryMinus) : return self().deserialize_RealUnaryMinus();
            case (ASR::exprType::RealCompare) : return self().deserialize_RealCompare();
            case (ASR::exprType::RealBinOp) : return self().deserialize_RealBinOp();
            case (ASR::exprType::RealCopySign) : return self().deserialize_RealCopySign();
            case (ASR::exprType::ComplexConstant) : return self().deserialize_ComplexConstant();
            case (ASR::exprType::ComplexUnaryMinus) : return self().deserialize_ComplexUnaryMinus();
            case (ASR::exprType::ComplexCompare) : return self().deserialize_ComplexCompare();
            case (ASR::exprType::ComplexBinOp) : return self().deserialize_ComplexBinOp();
            case (ASR::exprType::LogicalConstant) : return self().deserialize_LogicalConstant();
            case (ASR::exprType::LogicalNot) : return self().deserialize_LogicalNot();
            case (ASR::exprType::LogicalCompare) : return self().deserialize_LogicalCompare();
            case (ASR::exprType::LogicalBinOp) : return self().deserialize_LogicalBinOp();
            case (ASR::exprType::ListConstant) : return self().deserialize_ListConstant();
            case (ASR::exprType::ListLen) : return self().deserialize_ListLen();
            case (ASR::exprType::ListConcat) : return self().deserialize_ListConcat();
            case (ASR::exprType::ListCompare) : return self().deserialize_ListCompare();
            case (ASR::exprType::ListCount) : return self().deserialize_ListCount();
            case (ASR::exprType::SetConstant) : return self().deserialize_SetConstant();
            case (ASR::exprType::SetLen) : return self().deserialize_SetLen();
            case (ASR::exprType::TupleConstant) : return self().deserialize_TupleConstant();
            case (ASR::exprType::TupleLen) : return self().deserialize_TupleLen();
            case (ASR::exprType::TupleCompare) : return self().deserialize_TupleCompare();
            case (ASR::exprType::TupleConcat) : return self().deserialize_TupleConcat();
            case (ASR::exprType::StringConstant) : return self().deserialize_StringConstant();
            case (ASR::exprType::StringConcat) : return self().deserialize_StringConcat();
            case (ASR::exprType::StringRepeat) : return self().deserialize_StringRepeat();
            case (ASR::exprType::StringLen) : return self().deserialize_StringLen();
            case (ASR::exprType::StringItem) : return self().deserialize_StringItem();
            case (ASR::exprType::StringSection) : return self().deserialize_StringSection();
            case (ASR::exprType::StringCompare) : return self().deserialize_StringCompare();
            case (ASR::exprType::StringOrd) : return self().deserialize_StringOrd();
            case (ASR::exprType::StringChr) : return self().deserialize_StringChr();
            case (ASR::exprType::StringFormat) : return self().deserialize_StringFormat();
            case (ASR::exprType::CPtrCompare) : return self().deserialize_CPtrCompare();
            case (ASR::exprType::SymbolicCompare) : return self().deserialize_SymbolicCompare();
            case (ASR::exprType::DictConstant) : return self().deserialize_DictConstant();
            case (ASR::exprType::DictLen) : return self().deserialize_DictLen();
            case (ASR::exprType::Var) : return self().deserialize_Var();
            case (ASR::exprType::FunctionParam) : return self().deserialize_FunctionParam();
            case (ASR::exprType::ArrayConstructor) : return self().deserialize_ArrayConstructor();
            case (ASR::exprType::ArrayConstant) : return self().deserialize_ArrayConstant();
            case (ASR::exprType::ArrayItem) : return self().deserialize_ArrayItem();
            case (ASR::exprType::ArraySection) : return self().deserialize_ArraySection();
            case (ASR::exprType::ArraySize) : return self().deserialize_ArraySize();
            case (ASR::exprType::ArrayBound) : return self().deserialize_ArrayBound();
            case (ASR::exprType::ArrayTranspose) : return self().deserialize_ArrayTranspose();
            case (ASR::exprType::ArrayPack) : return self().deserialize_ArrayPack();
            case (ASR::exprType::ArrayReshape) : return self().deserialize_ArrayReshape();
            case (ASR::exprType::ArrayBroadcast) : return self().deserialize_ArrayBroadcast();
            case (ASR::exprType::BitCast) : return self().deserialize_BitCast();
            case (ASR::exprType::StructInstanceMember) : return self().deserialize_StructInstanceMember();
            case (ASR::exprType::StructStaticMember) : return self().deserialize_StructStaticMember();
            case (ASR::exprType::EnumStaticMember) : return self().deserialize_EnumStaticMember();
            case (ASR::exprType::UnionInstanceMember) : return self().deserialize_UnionInstanceMember();
            case (ASR::exprType::EnumName) : return self().deserialize_EnumName();
            case (ASR::exprType::EnumValue) : return self().deserialize_EnumValue();
            case (ASR::exprType::OverloadedCompare) : return self().deserialize_OverloadedCompare();
            case (ASR::exprType::OverloadedBinOp) : return self().deserialize_OverloadedBinOp();
            case (ASR::exprType::OverloadedUnaryMinus) : return self().deserialize_OverloadedUnaryMinus();
            case (ASR::exprType::OverloadedStringConcat) : return self().deserialize_OverloadedStringConcat();
            case (ASR::exprType::Cast) : return self().deserialize_Cast();
            case (ASR::exprType::ArrayPhysicalCast) : return self().deserialize_ArrayPhysicalCast();
            case (ASR::exprType::ComplexRe) : return self().deserialize_ComplexRe();
            case (ASR::exprType::ComplexIm) : return self().deserialize_ComplexIm();
            case (ASR::exprType::DictItem) : return self().deserialize_DictItem();
            case (ASR::exprType::CLoc) : return self().deserialize_CLoc();
            case (ASR::exprType::PointerToCPtr) : return self().deserialize_PointerToCPtr();
            case (ASR::exprType::GetPointer) : return self().deserialize_GetPointer();
            case (ASR::exprType::ListItem) : return self().deserialize_ListItem();
            case (ASR::exprType::TupleItem) : return self().deserialize_TupleItem();
            case (ASR::exprType::ListSection) : return self().deserialize_ListSection();
            case (ASR::exprType::ListRepeat) : return self().deserialize_ListRepeat();
            case (ASR::exprType::DictPop) : return self().deserialize_DictPop();
            case (ASR::exprType::SetPop) : return self().deserialize_SetPop();
            case (ASR::exprType::IntegerBitLen) : return self().deserialize_IntegerBitLen();
            case (ASR::exprType::Ichar) : return self().deserialize_Ichar();
            case (ASR::exprType::Iachar) : return self().deserialize_Iachar();
            case (ASR::exprType::SizeOfType) : return self().deserialize_SizeOfType();
            case (ASR::exprType::PointerNullConstant) : return self().deserialize_PointerNullConstant();
            case (ASR::exprType::PointerAssociated) : return self().deserialize_PointerAssociated();
            case (ASR::exprType::RealSqrt) : return self().deserialize_RealSqrt();
            default : throw LCompilersException("Unknown type in deserialize_expr()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    asr_t* deserialize_Integer() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_kind = self().read_int64();
        return ASR::make_Integer_t(al, loc, m_kind);
    }
    asr_t* deserialize_UnsignedInteger() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_kind = self().read_int64();
        return ASR::make_UnsignedInteger_t(al, loc, m_kind);
    }
    asr_t* deserialize_Real() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_kind = self().read_int64();
        return ASR::make_Real_t(al, loc, m_kind);
    }
    asr_t* deserialize_Complex() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_kind = self().read_int64();
        return ASR::make_Complex_t(al, loc, m_kind);
    }
    asr_t* deserialize_Character() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_kind = self().read_int64();
        int64_t m_len = self().read_int64();
        ASR::expr_t *m_len_expr;
        if (self().read_bool()) {
        m_len_expr = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_len_expr = nullptr;
        }
        return ASR::make_Character_t(al, loc, m_kind, m_len, m_len_expr);
    }
    asr_t* deserialize_Logical() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        int64_t m_kind = self().read_int64();
        return ASR::make_Logical_t(al, loc, m_kind);
    }
    asr_t* deserialize_Set() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_Set_t(al, loc, m_type);
    }
    asr_t* deserialize_List() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_List_t(al, loc, m_type);
    }
    asr_t* deserialize_Tuple() {
        size_t n_type; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_type = self().read_int64();
        Vec<ttype_t*> v_type;
        v_type.reserve(al, n_type);
        for (size_t i=0; i<n_type; i++) {
            v_type.push_back(al, ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype()));
        }
        return ASR::make_Tuple_t(al, loc, v_type.p, v_type.n);
    }
    asr_t* deserialize_StructType() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_derived_type;
        m_derived_type = self().read_symbol();
        return ASR::make_StructType_t(al, loc, m_derived_type);
    }
    asr_t* deserialize_EnumType() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_enum_type;
        m_enum_type = self().read_symbol();
        return ASR::make_EnumType_t(al, loc, m_enum_type);
    }
    asr_t* deserialize_Union() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_union_type;
        m_union_type = self().read_symbol();
        return ASR::make_Union_t(al, loc, m_union_type);
    }
    asr_t* deserialize_ClassType() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_class_type;
        m_class_type = self().read_symbol();
        return ASR::make_ClassType_t(al, loc, m_class_type);
    }
    asr_t* deserialize_Dict() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_key_type;
        m_key_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        ASR::ttype_t *m_value_type;
        m_value_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_Dict_t(al, loc, m_key_type, m_value_type);
    }
    asr_t* deserialize_Pointer() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_Pointer_t(al, loc, m_type);
    }
    asr_t* deserialize_Allocatable() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        return ASR::make_Allocatable_t(al, loc, m_type);
    }
    asr_t* deserialize_CPtr() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        return ASR::make_CPtr_t(al, loc);
    }
    asr_t* deserialize_SymbolicExpression() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        return ASR::make_SymbolicExpression_t(al, loc);
    }
    asr_t* deserialize_TypeParameter() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_param;
        m_param = self().read_cstring();
        return ASR::make_TypeParameter_t(al, loc, m_param);
    }
    asr_t* deserialize_Array() {
        size_t n_dims; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        n_dims = self().read_int64();
        Vec<dimension_t> v_dims;
        v_dims.reserve(al, n_dims);
        for (size_t i=0; i<n_dims; i++) {
            v_dims.push_back(al, self().deserialize_dimension());
        }
        ASR::array_physical_typeType m_physical_type = self().deserialize_array_physical_type();
        return ASR::make_Array_t(al, loc, m_type, v_dims.p, v_dims.n, m_physical_type);
    }
    asr_t* deserialize_FunctionType() {
        size_t n_arg_types; // Sequence
        size_t n_restrictions; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_arg_types = self().read_int64();
        Vec<ttype_t*> v_arg_types;
        v_arg_types.reserve(al, n_arg_types);
        for (size_t i=0; i<n_arg_types; i++) {
            v_arg_types.push_back(al, ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype()));
        }
        ASR::ttype_t *m_return_var_type;
        if (self().read_bool()) {
        m_return_var_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        } else {
        m_return_var_type = nullptr;
        }
        ASR::abiType m_abi = self().deserialize_abi();
        ASR::deftypeType m_deftype = self().deserialize_deftype();
        char *m_bindc_name;
        bool m_bindc_name_present = self().read_bool();
        if (m_bindc_name_present) {
        m_bindc_name = self().read_cstring();
        } else {
        m_bindc_name = nullptr;
        }
        bool m_elemental = self().read_bool();
        bool m_pure = self().read_bool();
        bool m_module = self().read_bool();
        bool m_inline = self().read_bool();
        bool m_static = self().read_bool();
        n_restrictions = self().read_int64();
        Vec<symbol_t*> v_restrictions;
        v_restrictions.reserve(al, n_restrictions);
        for (size_t i=0; i<n_restrictions; i++) {
            v_restrictions.push_back(al, self().read_symbol());
        }
        bool m_is_restriction = self().read_bool();
        return ASR::make_FunctionType_t(al, loc, v_arg_types.p, v_arg_types.n, m_return_var_type, m_abi, m_deftype, m_bindc_name, m_elemental, m_pure, m_module, m_inline, m_static, v_restrictions.p, v_restrictions.n, m_is_restriction);
    }
    asr_t* deserialize_ttype() {
        uint8_t t = self().read_int8();
        ASR::ttypeType ty = static_cast<ASR::ttypeType>(t);
        switch (ty) {
            case (ASR::ttypeType::Integer) : return self().deserialize_Integer();
            case (ASR::ttypeType::UnsignedInteger) : return self().deserialize_UnsignedInteger();
            case (ASR::ttypeType::Real) : return self().deserialize_Real();
            case (ASR::ttypeType::Complex) : return self().deserialize_Complex();
            case (ASR::ttypeType::Character) : return self().deserialize_Character();
            case (ASR::ttypeType::Logical) : return self().deserialize_Logical();
            case (ASR::ttypeType::Set) : return self().deserialize_Set();
            case (ASR::ttypeType::List) : return self().deserialize_List();
            case (ASR::ttypeType::Tuple) : return self().deserialize_Tuple();
            case (ASR::ttypeType::StructType) : return self().deserialize_StructType();
            case (ASR::ttypeType::EnumType) : return self().deserialize_EnumType();
            case (ASR::ttypeType::Union) : return self().deserialize_Union();
            case (ASR::ttypeType::ClassType) : return self().deserialize_ClassType();
            case (ASR::ttypeType::Dict) : return self().deserialize_Dict();
            case (ASR::ttypeType::Pointer) : return self().deserialize_Pointer();
            case (ASR::ttypeType::Allocatable) : return self().deserialize_Allocatable();
            case (ASR::ttypeType::CPtr) : return self().deserialize_CPtr();
            case (ASR::ttypeType::SymbolicExpression) : return self().deserialize_SymbolicExpression();
            case (ASR::ttypeType::TypeParameter) : return self().deserialize_TypeParameter();
            case (ASR::ttypeType::Array) : return self().deserialize_Array();
            case (ASR::ttypeType::FunctionType) : return self().deserialize_FunctionType();
            default : throw LCompilersException("Unknown type in deserialize_ttype()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    cast_kindType deserialize_cast_kind() {
        uint8_t t = self().read_int8();
        cast_kindType ty = static_cast<cast_kindType>(t);
        return ty;
    }
    storage_typeType deserialize_storage_type() {
        uint8_t t = self().read_int8();
        storage_typeType ty = static_cast<storage_typeType>(t);
        return ty;
    }
    accessType deserialize_access() {
        uint8_t t = self().read_int8();
        accessType ty = static_cast<accessType>(t);
        return ty;
    }
    intentType deserialize_intent() {
        uint8_t t = self().read_int8();
        intentType ty = static_cast<intentType>(t);
        return ty;
    }
    deftypeType deserialize_deftype() {
        uint8_t t = self().read_int8();
        deftypeType ty = static_cast<deftypeType>(t);
        return ty;
    }
    presenceType deserialize_presence() {
        uint8_t t = self().read_int8();
        presenceType ty = static_cast<presenceType>(t);
        return ty;
    }
    abiType deserialize_abi() {
        uint8_t t = self().read_int8();
        abiType ty = static_cast<abiType>(t);
        return ty;
    }
    dimension_t deserialize_dimension() {
        dimension_t x;
        {
            bool present=self().read_bool();
            if (present) {
                x.m_start = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_start = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_length = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_length = nullptr;
            }
        }
        return x;
    }
    alloc_arg_t deserialize_alloc_arg() {
        alloc_arg_t x;
        {
                x.m_a = down_cast<expr_t>(deserialize_expr());
        }
        {
            uint64_t n = self().read_int64();
            Vec<dimension_t> v;
            v.reserve(al, n);
            for (uint64_t i=0; i<n; i++) {
                v.push_back(al, self().deserialize_dimension());
            }
            x.m_dims = v.p;
            x.n_dims = v.n;
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_len_expr = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_len_expr = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_type = down_cast<ttype_t>(deserialize_ttype());
            } else {
                x.m_type = nullptr;
            }
        }
        return x;
    }
    asr_t* deserialize_Attribute() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_name;
        m_name = self().read_cstring();
        n_args = self().read_int64();
        Vec<attribute_arg_t> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().deserialize_attribute_arg());
        }
        return ASR::make_Attribute_t(al, loc, m_name, v_args.p, v_args.n);
    }
    asr_t* deserialize_attribute() {
        uint8_t t = self().read_int8();
        ASR::attributeType ty = static_cast<ASR::attributeType>(t);
        switch (ty) {
            case (ASR::attributeType::Attribute) : return self().deserialize_Attribute();
            default : throw LCompilersException("Unknown type in deserialize_attribute()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    attribute_arg_t deserialize_attribute_arg() {
        attribute_arg_t x;
        {
                x.m_arg = self().read_cstring();
        }
        return x;
    }
    call_arg_t deserialize_call_arg() {
        call_arg_t x;
        {
            bool present=self().read_bool();
            if (present) {
                x.m_value = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_value = nullptr;
            }
        }
        return x;
    }
    reduction_expr_t deserialize_reduction_expr() {
        reduction_expr_t x;
        {
                x.m_op = deserialize_reduction_op();
        }
        {
                x.m_arg = down_cast<expr_t>(deserialize_expr());
        }
        return x;
    }
    asr_t* deserialize_Bind() {
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_lang;
        m_lang = self().read_cstring();
        char *m_name;
        m_name = self().read_cstring();
        return ASR::make_Bind_t(al, loc, m_lang, m_name);
    }
    asr_t* deserialize_tbind() {
        uint8_t t = self().read_int8();
        ASR::tbindType ty = static_cast<ASR::tbindType>(t);
        switch (ty) {
            case (ASR::tbindType::Bind) : return self().deserialize_Bind();
            default : throw LCompilersException("Unknown type in deserialize_tbind()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    array_index_t deserialize_array_index() {
        array_index_t x;
        {
            bool present=self().read_bool();
            if (present) {
                x.m_left = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_left = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_right = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_right = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_step = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_step = nullptr;
            }
        }
        return x;
    }
    do_loop_head_t deserialize_do_loop_head() {
        do_loop_head_t x;
        {
            bool present=self().read_bool();
            if (present) {
                x.m_v = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_v = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_start = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_start = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_end = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_end = nullptr;
            }
        }
        {
            bool present=self().read_bool();
            if (present) {
                x.m_increment = down_cast<expr_t>(deserialize_expr());
            } else {
                x.m_increment = nullptr;
            }
        }
        return x;
    }
    asr_t* deserialize_CaseStmt() {
        size_t n_test; // Sequence
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        n_test = self().read_int64();
        Vec<expr_t*> v_test;
        v_test.reserve(al, n_test);
        for (size_t i=0; i<n_test; i++) {
            v_test.push_back(al, ASR::down_cast<ASR::expr_t>(self().deserialize_expr()));
        }
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        bool m_fall_through = self().read_bool();
        return ASR::make_CaseStmt_t(al, loc, v_test.p, v_test.n, v_body.p, v_body.n, m_fall_through);
    }
    asr_t* deserialize_CaseStmt_Range() {
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::expr_t *m_start;
        if (self().read_bool()) {
        m_start = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_start = nullptr;
        }
        ASR::expr_t *m_end;
        if (self().read_bool()) {
        m_end = ASR::down_cast<ASR::expr_t>(self().deserialize_expr());
        } else {
        m_end = nullptr;
        }
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_CaseStmt_Range_t(al, loc, m_start, m_end, v_body.p, v_body.n);
    }
    asr_t* deserialize_case_stmt() {
        uint8_t t = self().read_int8();
        ASR::case_stmtType ty = static_cast<ASR::case_stmtType>(t);
        switch (ty) {
            case (ASR::case_stmtType::CaseStmt) : return self().deserialize_CaseStmt();
            case (ASR::case_stmtType::CaseStmt_Range) : return self().deserialize_CaseStmt_Range();
            default : throw LCompilersException("Unknown type in deserialize_case_stmt()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    asr_t* deserialize_TypeStmtName() {
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_sym;
        m_sym = self().read_symbol();
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_TypeStmtName_t(al, loc, m_sym, v_body.p, v_body.n);
    }
    asr_t* deserialize_ClassStmt() {
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::symbol_t *m_sym;
        m_sym = self().read_symbol();
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_ClassStmt_t(al, loc, m_sym, v_body.p, v_body.n);
    }
    asr_t* deserialize_TypeStmtType() {
        size_t n_body; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        ASR::ttype_t *m_type;
        m_type = ASR::down_cast<ASR::ttype_t>(self().deserialize_ttype());
        n_body = self().read_int64();
        Vec<stmt_t*> v_body;
        v_body.reserve(al, n_body);
        for (size_t i=0; i<n_body; i++) {
            v_body.push_back(al, ASR::down_cast<ASR::stmt_t>(self().deserialize_stmt()));
        }
        return ASR::make_TypeStmtType_t(al, loc, m_type, v_body.p, v_body.n);
    }
    asr_t* deserialize_type_stmt() {
        uint8_t t = self().read_int8();
        ASR::type_stmtType ty = static_cast<ASR::type_stmtType>(t);
        switch (ty) {
            case (ASR::type_stmtType::TypeStmtName) : return self().deserialize_TypeStmtName();
            case (ASR::type_stmtType::ClassStmt) : return self().deserialize_ClassStmt();
            case (ASR::type_stmtType::TypeStmtType) : return self().deserialize_TypeStmtType();
            default : throw LCompilersException("Unknown type in deserialize_type_stmt()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    enumtypeType deserialize_enumtype() {
        uint8_t t = self().read_int8();
        enumtypeType ty = static_cast<enumtypeType>(t);
        return ty;
    }
    asr_t* deserialize_Require() {
        size_t n_args; // Sequence
        Location loc;
        loc.first = self().read_int64();
        loc.last = self().read_int64();
        loc.first = 0;
        loc.last = 0;
        char *m_name;
        m_name = self().read_cstring();
        n_args = self().read_int64();
        Vec<char*> v_args;
        v_args.reserve(al, n_args);
        for (size_t i=0; i<n_args; i++) {
            v_args.push_back(al, self().read_cstring());
        }
        return ASR::make_Require_t(al, loc, m_name, v_args.p, v_args.n);
    }
    asr_t* deserialize_require_instantiation() {
        uint8_t t = self().read_int8();
        ASR::require_instantiationType ty = static_cast<ASR::require_instantiationType>(t);
        switch (ty) {
            case (ASR::require_instantiationType::Require) : return self().deserialize_Require();
            default : throw LCompilersException("Unknown type in deserialize_require_instantiation()");
        }
        throw LCompilersException("Switch statement above was not exhaustive.");
    }
    array_physical_typeType deserialize_array_physical_type() {
        uint8_t t = self().read_int8();
        array_physical_typeType ty = static_cast<array_physical_typeType>(t);
        return ty;
    }
    binopType deserialize_binop() {
        uint8_t t = self().read_int8();
        binopType ty = static_cast<binopType>(t);
        return ty;
    }
    reduction_opType deserialize_reduction_op() {
        uint8_t t = self().read_int8();
        reduction_opType ty = static_cast<reduction_opType>(t);
        return ty;
    }
    logicalbinopType deserialize_logicalbinop() {
        uint8_t t = self().read_int8();
        logicalbinopType ty = static_cast<logicalbinopType>(t);
        return ty;
    }
    cmpopType deserialize_cmpop() {
        uint8_t t = self().read_int8();
        cmpopType ty = static_cast<cmpopType>(t);
        return ty;
    }
    integerbozType deserialize_integerboz() {
        uint8_t t = self().read_int8();
        integerbozType ty = static_cast<integerbozType>(t);
        return ty;
    }
    arrayboundType deserialize_arraybound() {
        uint8_t t = self().read_int8();
        arrayboundType ty = static_cast<arrayboundType>(t);
        return ty;
    }
    arraystorageType deserialize_arraystorage() {
        uint8_t t = self().read_int8();
        arraystorageType ty = static_cast<arraystorageType>(t);
        return ty;
    }
    string_format_kindType deserialize_string_format_kind() {
        uint8_t t = self().read_int8();
        string_format_kindType ty = static_cast<string_format_kindType>(t);
        return ty;
    }
};


/******************************************************************************/
// Walk Visitor base class

template <class StructType>
class ASRPassBaseWalkVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    SymbolTable* current_scope=nullptr;
    void transform_stmts(ASR::stmt_t **&m_body, size_t &n_body) {
        for (size_t i = 0; i < n_body; i++) {
            self().visit_stmt(*m_body[i]);
        }
    }
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        current_scope = current_scope_copy;
    }
    void visit_Program(const Program_t &x) {
        Program_t& xx = const_cast<Program_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        current_scope = current_scope_copy;
    }
    void visit_Module(const Module_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        current_scope = current_scope_copy;
    }
    void visit_Function(const Function_t &x) {
        Function_t& xx = const_cast<Function_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().visit_ttype(*x.m_function_signature);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        if (x.m_return_var)
            self().visit_expr(*x.m_return_var);
        current_scope = current_scope_copy;
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        for (size_t i=0; i<x.n_procs; i++) {
        }
        current_scope = current_scope_copy;
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        for (size_t i=0; i<x.n_procs; i++) {
        }
        current_scope = current_scope_copy;
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        if ((bool&)x) { } // Suppress unused warning
        current_scope = current_scope_copy;
    }
    void visit_Struct(const Struct_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        if (x.m_alignment)
            self().visit_expr(*x.m_alignment);
        current_scope = current_scope_copy;
    }
    void visit_Enum(const Enum_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().visit_ttype(*x.m_type);
        current_scope = current_scope_copy;
    }
    void visit_UnionType(const UnionType_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        current_scope = current_scope_copy;
    }
    void visit_Variable(const Variable_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        if (x.m_symbolic_value)
            self().visit_expr(*x.m_symbolic_value);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_ttype(*x.m_type);
        current_scope = current_scope_copy;
    }
    void visit_Class(const Class_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        current_scope = current_scope_copy;
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        if ((bool&)x) { } // Suppress unused warning
        current_scope = current_scope_copy;
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        AssociateBlock_t& xx = const_cast<AssociateBlock_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        current_scope = current_scope_copy;
    }
    void visit_Block(const Block_t &x) {
        Block_t& xx = const_cast<Block_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        current_scope = current_scope_copy;
    }
    void visit_Requirement(const Requirement_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
        current_scope = current_scope_copy;
    }
    void visit_Template(const Template_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
        current_scope = current_scope_copy;
    }
    void visit_Allocate(const Allocate_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
        if (x.m_stat)
            self().visit_expr(*x.m_stat);
        if (x.m_errmsg)
            self().visit_expr(*x.m_errmsg);
        if (x.m_source)
            self().visit_expr(*x.m_source);
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
    }
    void visit_Assign(const Assign_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
        if (x.m_overloaded)
            self().visit_stmt(*x.m_overloaded);
    }
    void visit_Associate(const Associate_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
    }
    void visit_Cycle(const Cycle_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        DoConcurrentLoop_t& xx = const_cast<DoConcurrentLoop_t&>(x);
        self().visit_do_loop_head(x.m_head);
        for (size_t i=0; i<x.n_shared; i++) {
            self().visit_expr(*x.m_shared[i]);
        }
        for (size_t i=0; i<x.n_local; i++) {
            self().visit_expr(*x.m_local[i]);
        }
        for (size_t i=0; i<x.n_reduction; i++) {
            self().visit_reduction_expr(x.m_reduction[i]);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_DoLoop(const DoLoop_t &x) {
        DoLoop_t& xx = const_cast<DoLoop_t&>(x);
        self().visit_do_loop_head(x.m_head);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        if (x.m_code)
            self().visit_expr(*x.m_code);
    }
    void visit_Exit(const Exit_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        self().visit_do_loop_head(x.m_head);
        self().visit_stmt(*x.m_assign_stmt);
    }
    void visit_GoTo(const GoTo_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        If_t& xx = const_cast<If_t&>(x);
        self().visit_expr(*x.m_test);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        self().visit_expr(*x.m_test);
    }
    void visit_Print(const Print_t &x) {
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator)
            self().visit_expr(*x.m_separator);
        if (x.m_end)
            self().visit_expr(*x.m_end);
    }
    void visit_FileOpen(const FileOpen_t &x) {
        if (x.m_newunit)
            self().visit_expr(*x.m_newunit);
        if (x.m_filename)
            self().visit_expr(*x.m_filename);
        if (x.m_status)
            self().visit_expr(*x.m_status);
        if (x.m_form)
            self().visit_expr(*x.m_form);
    }
    void visit_FileClose(const FileClose_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_err)
            self().visit_expr(*x.m_err);
        if (x.m_status)
            self().visit_expr(*x.m_status);
    }
    void visit_FileRead(const FileRead_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_fmt)
            self().visit_expr(*x.m_fmt);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_size)
            self().visit_expr(*x.m_size);
        if (x.m_id)
            self().visit_expr(*x.m_id);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_overloaded)
            self().visit_stmt(*x.m_overloaded);
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_err)
            self().visit_expr(*x.m_err);
    }
    void visit_FileRewind(const FileRewind_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_err)
            self().visit_expr(*x.m_err);
    }
    void visit_FileInquire(const FileInquire_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_file)
            self().visit_expr(*x.m_file);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_err)
            self().visit_expr(*x.m_err);
        if (x.m_exist)
            self().visit_expr(*x.m_exist);
        if (x.m_opened)
            self().visit_expr(*x.m_opened);
        if (x.m_number)
            self().visit_expr(*x.m_number);
        if (x.m_named)
            self().visit_expr(*x.m_named);
        if (x.m_name)
            self().visit_expr(*x.m_name);
        if (x.m_access)
            self().visit_expr(*x.m_access);
        if (x.m_sequential)
            self().visit_expr(*x.m_sequential);
        if (x.m_direct)
            self().visit_expr(*x.m_direct);
        if (x.m_form)
            self().visit_expr(*x.m_form);
        if (x.m_formatted)
            self().visit_expr(*x.m_formatted);
        if (x.m_unformatted)
            self().visit_expr(*x.m_unformatted);
        if (x.m_recl)
            self().visit_expr(*x.m_recl);
        if (x.m_nextrec)
            self().visit_expr(*x.m_nextrec);
        if (x.m_blank)
            self().visit_expr(*x.m_blank);
        if (x.m_position)
            self().visit_expr(*x.m_position);
        if (x.m_action)
            self().visit_expr(*x.m_action);
        if (x.m_read)
            self().visit_expr(*x.m_read);
        if (x.m_write)
            self().visit_expr(*x.m_write);
        if (x.m_readwrite)
            self().visit_expr(*x.m_readwrite);
        if (x.m_delim)
            self().visit_expr(*x.m_delim);
        if (x.m_pad)
            self().visit_expr(*x.m_pad);
        if (x.m_flen)
            self().visit_expr(*x.m_flen);
        if (x.m_blocksize)
            self().visit_expr(*x.m_blocksize);
        if (x.m_convert)
            self().visit_expr(*x.m_convert);
        if (x.m_carriagecontrol)
            self().visit_expr(*x.m_carriagecontrol);
        if (x.m_iolength)
            self().visit_expr(*x.m_iolength);
    }
    void visit_FileWrite(const FileWrite_t &x) {
        if (x.m_unit)
            self().visit_expr(*x.m_unit);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
        if (x.m_id)
            self().visit_expr(*x.m_id);
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator)
            self().visit_expr(*x.m_separator);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        if (x.m_overloaded)
            self().visit_stmt(*x.m_overloaded);
    }
    void visit_Return(const Return_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        Select_t& xx = const_cast<Select_t&>(x);
        self().visit_expr(*x.m_test);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_case_stmt(*x.m_body[i]);
        }
        self().transform_stmts(xx.m_default, xx.n_default);
    }
    void visit_Stop(const Stop_t &x) {
        if (x.m_code)
            self().visit_expr(*x.m_code);
    }
    void visit_Assert(const Assert_t &x) {
        self().visit_expr(*x.m_test);
        if (x.m_msg)
            self().visit_expr(*x.m_msg);
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        if (x.m_dt)
            self().visit_expr(*x.m_dt);
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
    }
    void visit_Where(const Where_t &x) {
        Where_t& xx = const_cast<Where_t&>(x);
        self().visit_expr(*x.m_test);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        WhileLoop_t& xx = const_cast<WhileLoop_t&>(x);
        self().visit_expr(*x.m_test);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_Nullify(const Nullify_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
        }
    }
    void visit_Flush(const Flush_t &x) {
        self().visit_expr(*x.m_unit);
        if (x.m_err)
            self().visit_expr(*x.m_err);
        if (x.m_iomsg)
            self().visit_expr(*x.m_iomsg);
        if (x.m_iostat)
            self().visit_expr(*x.m_iostat);
    }
    void visit_ListAppend(const ListAppend_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SelectType(const SelectType_t &x) {
        SelectType_t& xx = const_cast<SelectType_t&>(x);
        self().visit_expr(*x.m_selector);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_type_stmt(*x.m_body[i]);
        }
        self().transform_stmts(xx.m_default, xx.n_default);
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        self().visit_expr(*x.m_cptr);
        self().visit_expr(*x.m_ptr);
        if (x.m_shape)
            self().visit_expr(*x.m_shape);
        if (x.m_lower_bounds)
            self().visit_expr(*x.m_lower_bounds);
    }
    void visit_BlockCall(const BlockCall_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetInsert(const SetInsert_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_SetRemove(const SetRemove_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListInsert(const ListInsert_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListRemove(const ListRemove_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_ele);
    }
    void visit_ListClear(const ListClear_t &x) {
        self().visit_expr(*x.m_a);
    }
    void visit_DictInsert(const DictInsert_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        self().visit_expr(*x.m_value);
    }
    void visit_Expr(const Expr_t &x) {
        self().visit_expr(*x.m_expression);
    }
    void visit_IfExp(const IfExp_t &x) {
        self().visit_expr(*x.m_test);
        self().visit_expr(*x.m_body);
        self().visit_expr(*x.m_orelse);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        self().visit_expr(*x.m_re);
        self().visit_expr(*x.m_im);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_value);
        self().visit_ttype(*x.m_type);
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        if (x.m_dt)
            self().visit_expr(*x.m_dt);
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type)
            self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type)
            self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type)
            self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        self().visit_ttype(*x.m_arg_type);
        if (x.m_arg)
            self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        self().visit_expr(*x.m_value);
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_expr(*x.m_var);
        self().visit_expr(*x.m_start);
        self().visit_expr(*x.m_end);
        if (x.m_increment)
            self().visit_expr(*x.m_increment);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        if (x.m_type)
            self().visit_ttype(*x.m_type);
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealConstant(const RealConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealCompare(const RealCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        self().visit_expr(*x.m_target);
        self().visit_expr(*x.m_source);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListConstant(const ListConstant_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_ListLen(const ListLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListConcat(const ListConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListCompare(const ListCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListCount(const ListCount_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_expr(*x.m_ele);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SetConstant(const SetConstant_t &x) {
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_SetLen(const SetLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        for (size_t i=0; i<x.n_elements; i++) {
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_TupleLen(const TupleLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        self().visit_expr(*x.m_value);
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringConstant(const StringConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_StringConcat(const StringConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringLen(const StringLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringItem(const StringItem_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_expr(*x.m_idx);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringSection(const StringSection_t &x) {
        self().visit_expr(*x.m_arg);
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        if (x.m_step)
            self().visit_expr(*x.m_step);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringCompare(const StringCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringOrd(const StringOrd_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringChr(const StringChr_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StringFormat(const StringFormat_t &x) {
        self().visit_expr(*x.m_fmt);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_DictConstant(const DictConstant_t &x) {
        for (size_t i=0; i<x.n_keys; i++) {
            self().visit_expr(*x.m_keys[i]);
        }
        for (size_t i=0; i<x.n_values; i++) {
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_DictLen(const DictLen_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Var(const Var_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        self().visit_expr(*x.m_v);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArraySection(const ArraySection_t &x) {
        self().visit_expr(*x.m_v);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArraySize(const ArraySize_t &x) {
        self().visit_expr(*x.m_v);
        if (x.m_dim)
            self().visit_expr(*x.m_dim);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        self().visit_expr(*x.m_v);
        if (x.m_dim)
            self().visit_expr(*x.m_dim);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        self().visit_expr(*x.m_matrix);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_mask);
        if (x.m_vector)
            self().visit_expr(*x.m_vector);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        self().visit_expr(*x.m_array);
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_BitCast(const BitCast_t &x) {
        self().visit_expr(*x.m_source);
        self().visit_expr(*x.m_mold);
        if (x.m_size)
            self().visit_expr(*x.m_size);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumName(const EnumName_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_EnumValue(const EnumValue_t &x) {
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
        self().visit_expr(*x.m_overloaded);
    }
    void visit_Cast(const Cast_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_DictItem(const DictItem_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        if (x.m_default)
            self().visit_expr(*x.m_default);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_CLoc(const CLoc_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_GetPointer(const GetPointer_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListItem(const ListItem_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_TupleItem(const TupleItem_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListSection(const ListSection_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_array_index(x.m_section);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        self().visit_expr(*x.m_left);
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_DictPop(const DictPop_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_expr(*x.m_key);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SetPop(const SetPop_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Ichar(const Ichar_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Iachar(const Iachar_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        self().visit_ttype(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        self().visit_expr(*x.m_ptr);
        if (x.m_tgt)
            self().visit_expr(*x.m_tgt);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_Integer(const Integer_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        if (x.m_len_expr)
            self().visit_expr(*x.m_len_expr);
    }
    void visit_Logical(const Logical_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_List(const List_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_Tuple(const Tuple_t &x) {
        for (size_t i=0; i<x.n_type; i++) {
            self().visit_ttype(*x.m_type[i]);
        }
    }
    void visit_StructType(const StructType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumType(const EnumType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Union(const Union_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassType(const ClassType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Dict(const Dict_t &x) {
        self().visit_ttype(*x.m_key_type);
        self().visit_ttype(*x.m_value_type);
    }
    void visit_Pointer(const Pointer_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_Allocatable(const Allocatable_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_CPtr(const CPtr_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        self().visit_ttype(*x.m_type);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
    }
    void visit_FunctionType(const FunctionType_t &x) {
        for (size_t i=0; i<x.n_arg_types; i++) {
            self().visit_ttype(*x.m_arg_types[i]);
        }
        if (x.m_return_var_type)
            self().visit_ttype(*x.m_return_var_type);
        for (size_t i=0; i<x.n_restrictions; i++) {
        }
    }
    void visit_dimension(const dimension_t &x) {
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_length)
            self().visit_expr(*x.m_length);
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        self().visit_expr(*x.m_a);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
        if (x.m_len_expr)
            self().visit_expr(*x.m_len_expr);
        if (x.m_type)
            self().visit_ttype(*x.m_type);
    }
    void visit_Attribute(const Attribute_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_attribute_arg(x.m_args[i]);
        }
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        if (x.m_value)
            self().visit_expr(*x.m_value);
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        self().visit_expr(*x.m_arg);
    }
    void visit_Bind(const Bind_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        if (x.m_left)
            self().visit_expr(*x.m_left);
        if (x.m_right)
            self().visit_expr(*x.m_right);
        if (x.m_step)
            self().visit_expr(*x.m_step);
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        if (x.m_v)
            self().visit_expr(*x.m_v);
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        if (x.m_increment)
            self().visit_expr(*x.m_increment);
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        CaseStmt_t& xx = const_cast<CaseStmt_t&>(x);
        for (size_t i=0; i<x.n_test; i++) {
            self().visit_expr(*x.m_test[i]);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        CaseStmt_Range_t& xx = const_cast<CaseStmt_Range_t&>(x);
        if (x.m_start)
            self().visit_expr(*x.m_start);
        if (x.m_end)
            self().visit_expr(*x.m_end);
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        TypeStmtName_t& xx = const_cast<TypeStmtName_t&>(x);
        self().transform_stmts(xx.m_body, xx.n_body);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        ClassStmt_t& xx = const_cast<ClassStmt_t&>(x);
        self().transform_stmts(xx.m_body, xx.n_body);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        TypeStmtType_t& xx = const_cast<TypeStmtType_t&>(x);
        self().visit_ttype(*x.m_type);
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_Require(const Require_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
};


/******************************************************************************/
// Expression and statement Duplicator class

template <class StructType>
class BaseExprStmtDuplicator {
public:
    StructType& self() { return static_cast<StructType&>(*this); }

    Allocator &al;
    bool success;
    bool allow_procedure_calls;
    bool allow_reshape;

    BaseExprStmtDuplicator(Allocator& al_) : al(al_), success(false), allow_procedure_calls(true), allow_reshape(true) {}


    ASR::asr_t* duplicate_Allocate(Allocate_t* x) {
        Vec<alloc_arg_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::alloc_arg_t alloc_arg_copy;
            alloc_arg_copy.loc = x->m_args[i].loc;
            alloc_arg_copy.m_a = self().duplicate_expr(x->m_args[i].m_a);
            alloc_arg_copy.m_len_expr = self().duplicate_expr(x->m_args[i].m_len_expr);
            alloc_arg_copy.m_type = self().duplicate_ttype(x->m_args[i].m_type);
            alloc_arg_copy.n_dims = x->m_args[i].n_dims;
            Vec<ASR::dimension_t> dims_copy;
            dims_copy.reserve(al, alloc_arg_copy.n_dims);
            for (size_t j = 0; j < alloc_arg_copy.n_dims; j++) {
                ASR::dimension_t dim_copy;
                dim_copy.loc = x->m_args[i].m_dims[j].loc;
                dim_copy.m_start = self().duplicate_expr(x->m_args[i].m_dims[j].m_start);
                dim_copy.m_length = self().duplicate_expr(x->m_args[i].m_dims[j].m_length);
                dims_copy.push_back(al, dim_copy);
            }
            alloc_arg_copy.m_dims = dims_copy.p;
            m_args.push_back(al, alloc_arg_copy);
        }
        expr_t* m_stat = self().duplicate_expr(x->m_stat);
        expr_t* m_errmsg = self().duplicate_expr(x->m_errmsg);
        expr_t* m_source = self().duplicate_expr(x->m_source);
        return make_Allocate_t(al, x->base.base.loc, m_args.p, x->n_args, m_stat, m_errmsg, m_source);
    }


    ASR::asr_t* duplicate_ReAlloc(ReAlloc_t* x) {
        Vec<alloc_arg_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::alloc_arg_t alloc_arg_copy;
            alloc_arg_copy.loc = x->m_args[i].loc;
            alloc_arg_copy.m_a = self().duplicate_expr(x->m_args[i].m_a);
            alloc_arg_copy.m_len_expr = self().duplicate_expr(x->m_args[i].m_len_expr);
            alloc_arg_copy.m_type = self().duplicate_ttype(x->m_args[i].m_type);
            alloc_arg_copy.n_dims = x->m_args[i].n_dims;
            Vec<ASR::dimension_t> dims_copy;
            dims_copy.reserve(al, alloc_arg_copy.n_dims);
            for (size_t j = 0; j < alloc_arg_copy.n_dims; j++) {
                ASR::dimension_t dim_copy;
                dim_copy.loc = x->m_args[i].m_dims[j].loc;
                dim_copy.m_start = self().duplicate_expr(x->m_args[i].m_dims[j].m_start);
                dim_copy.m_length = self().duplicate_expr(x->m_args[i].m_dims[j].m_length);
                dims_copy.push_back(al, dim_copy);
            }
            alloc_arg_copy.m_dims = dims_copy.p;
            m_args.push_back(al, alloc_arg_copy);
        }
        return make_ReAlloc_t(al, x->base.base.loc, m_args.p, x->n_args);
    }


    ASR::asr_t* duplicate_Assign(Assign_t* x) {
        return make_Assign_t(al, x->base.base.loc, x->m_label, x->m_variable);
    }


    ASR::asr_t* duplicate_Assignment(Assignment_t* x) {
        expr_t* m_target = self().duplicate_expr(x->m_target);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        stmt_t* m_overloaded = self().duplicate_stmt(x->m_overloaded);
        return make_Assignment_t(al, x->base.base.loc, m_target, m_value, m_overloaded);
    }


    ASR::asr_t* duplicate_Associate(Associate_t* x) {
        expr_t* m_target = self().duplicate_expr(x->m_target);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_Associate_t(al, x->base.base.loc, m_target, m_value);
    }


    ASR::asr_t* duplicate_Cycle(Cycle_t* x) {
        return make_Cycle_t(al, x->base.base.loc, x->m_stmt_name);
    }


    ASR::asr_t* duplicate_ExplicitDeallocate(ExplicitDeallocate_t* x) {
        Vec<expr_t*> m_vars;
        m_vars.reserve(al, x->n_vars);
        for (size_t i = 0; i < x->n_vars; i++) {
            m_vars.push_back(al, self().duplicate_expr(x->m_vars[i]));
        }
        return make_ExplicitDeallocate_t(al, x->base.base.loc, m_vars.p, x->n_vars);
    }


    ASR::asr_t* duplicate_ImplicitDeallocate(ImplicitDeallocate_t* x) {
        Vec<expr_t*> m_vars;
        m_vars.reserve(al, x->n_vars);
        for (size_t i = 0; i < x->n_vars; i++) {
            m_vars.push_back(al, self().duplicate_expr(x->m_vars[i]));
        }
        return make_ImplicitDeallocate_t(al, x->base.base.loc, m_vars.p, x->n_vars);
    }


    ASR::asr_t* duplicate_DoConcurrentLoop(DoConcurrentLoop_t* x) {
        ASR::do_loop_head_t m_head;
        m_head.loc = x->m_head.loc;
        m_head.m_v = duplicate_expr(x->m_head.m_v);
        m_head.m_start = duplicate_expr(x->m_head.m_start);
        m_head.m_end = duplicate_expr(x->m_head.m_end);
        m_head.m_increment = duplicate_expr(x->m_head.m_increment);
        Vec<expr_t*> m_shared;
        m_shared.reserve(al, x->n_shared);
        for (size_t i = 0; i < x->n_shared; i++) {
            m_shared.push_back(al, self().duplicate_expr(x->m_shared[i]));
        }
        Vec<expr_t*> m_local;
        m_local.reserve(al, x->n_local);
        for (size_t i = 0; i < x->n_local; i++) {
            m_local.push_back(al, self().duplicate_expr(x->m_local[i]));
        }
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        return make_DoConcurrentLoop_t(al, x->base.base.loc, m_head, m_shared.p, x->n_shared, m_local.p, x->n_local, x->m_reduction, x->n_reduction, m_body.p, x->n_body);
    }


    ASR::asr_t* duplicate_DoLoop(DoLoop_t* x) {
        ASR::do_loop_head_t m_head;
        m_head.loc = x->m_head.loc;
        m_head.m_v = duplicate_expr(x->m_head.m_v);
        m_head.m_start = duplicate_expr(x->m_head.m_start);
        m_head.m_end = duplicate_expr(x->m_head.m_end);
        m_head.m_increment = duplicate_expr(x->m_head.m_increment);
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        Vec<stmt_t*> m_orelse;
        m_orelse.reserve(al, x->n_orelse);
        for (size_t i = 0; i < x->n_orelse; i++) {
            m_orelse.push_back(al, self().duplicate_stmt(x->m_orelse[i]));
        }
        return make_DoLoop_t(al, x->base.base.loc, x->m_name, m_head, m_body.p, x->n_body, m_orelse.p, x->n_orelse);
    }


    ASR::asr_t* duplicate_ErrorStop(ErrorStop_t* x) {
        expr_t* m_code = self().duplicate_expr(x->m_code);
        return make_ErrorStop_t(al, x->base.base.loc, m_code);
    }


    ASR::asr_t* duplicate_Exit(Exit_t* x) {
        return make_Exit_t(al, x->base.base.loc, x->m_stmt_name);
    }


    ASR::asr_t* duplicate_ForAllSingle(ForAllSingle_t* x) {
        ASR::do_loop_head_t m_head;
        m_head.loc = x->m_head.loc;
        m_head.m_v = duplicate_expr(x->m_head.m_v);
        m_head.m_start = duplicate_expr(x->m_head.m_start);
        m_head.m_end = duplicate_expr(x->m_head.m_end);
        m_head.m_increment = duplicate_expr(x->m_head.m_increment);
        stmt_t* m_assign_stmt = self().duplicate_stmt(x->m_assign_stmt);
        return make_ForAllSingle_t(al, x->base.base.loc, m_head, m_assign_stmt);
    }


    ASR::asr_t* duplicate_GoTo(GoTo_t* x) {
        return make_GoTo_t(al, x->base.base.loc, x->m_target_id, x->m_name);
    }


    ASR::asr_t* duplicate_GoToTarget(GoToTarget_t* x) {
        return make_GoToTarget_t(al, x->base.base.loc, x->m_id, x->m_name);
    }


    ASR::asr_t* duplicate_If(If_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        Vec<stmt_t*> m_orelse;
        m_orelse.reserve(al, x->n_orelse);
        for (size_t i = 0; i < x->n_orelse; i++) {
            m_orelse.push_back(al, self().duplicate_stmt(x->m_orelse[i]));
        }
        return make_If_t(al, x->base.base.loc, m_test, m_body.p, x->n_body, m_orelse.p, x->n_orelse);
    }


    ASR::asr_t* duplicate_IfArithmetic(IfArithmetic_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        return make_IfArithmetic_t(al, x->base.base.loc, m_test, x->m_lt_label, x->m_eq_label, x->m_gt_label);
    }


    ASR::asr_t* duplicate_Print(Print_t* x) {
        Vec<expr_t*> m_values;
        m_values.reserve(al, x->n_values);
        for (size_t i = 0; i < x->n_values; i++) {
            m_values.push_back(al, self().duplicate_expr(x->m_values[i]));
        }
        expr_t* m_separator = self().duplicate_expr(x->m_separator);
        expr_t* m_end = self().duplicate_expr(x->m_end);
        return make_Print_t(al, x->base.base.loc, m_values.p, x->n_values, m_separator, m_end);
    }


    ASR::asr_t* duplicate_FileOpen(FileOpen_t* x) {
        expr_t* m_newunit = self().duplicate_expr(x->m_newunit);
        expr_t* m_filename = self().duplicate_expr(x->m_filename);
        expr_t* m_status = self().duplicate_expr(x->m_status);
        expr_t* m_form = self().duplicate_expr(x->m_form);
        return make_FileOpen_t(al, x->base.base.loc, x->m_label, m_newunit, m_filename, m_status, m_form);
    }


    ASR::asr_t* duplicate_FileClose(FileClose_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        expr_t* m_iomsg = self().duplicate_expr(x->m_iomsg);
        expr_t* m_err = self().duplicate_expr(x->m_err);
        expr_t* m_status = self().duplicate_expr(x->m_status);
        return make_FileClose_t(al, x->base.base.loc, x->m_label, m_unit, m_iostat, m_iomsg, m_err, m_status);
    }


    ASR::asr_t* duplicate_FileRead(FileRead_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_fmt = self().duplicate_expr(x->m_fmt);
        expr_t* m_iomsg = self().duplicate_expr(x->m_iomsg);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        expr_t* m_size = self().duplicate_expr(x->m_size);
        expr_t* m_id = self().duplicate_expr(x->m_id);
        Vec<expr_t*> m_values;
        m_values.reserve(al, x->n_values);
        for (size_t i = 0; i < x->n_values; i++) {
            m_values.push_back(al, self().duplicate_expr(x->m_values[i]));
        }
        stmt_t* m_overloaded = self().duplicate_stmt(x->m_overloaded);
        return make_FileRead_t(al, x->base.base.loc, x->m_label, m_unit, m_fmt, m_iomsg, m_iostat, m_size, m_id, m_values.p, x->n_values, m_overloaded);
    }


    ASR::asr_t* duplicate_FileBackspace(FileBackspace_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        expr_t* m_err = self().duplicate_expr(x->m_err);
        return make_FileBackspace_t(al, x->base.base.loc, x->m_label, m_unit, m_iostat, m_err);
    }


    ASR::asr_t* duplicate_FileRewind(FileRewind_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        expr_t* m_err = self().duplicate_expr(x->m_err);
        return make_FileRewind_t(al, x->base.base.loc, x->m_label, m_unit, m_iostat, m_err);
    }


    ASR::asr_t* duplicate_FileInquire(FileInquire_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_file = self().duplicate_expr(x->m_file);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        expr_t* m_err = self().duplicate_expr(x->m_err);
        expr_t* m_exist = self().duplicate_expr(x->m_exist);
        expr_t* m_opened = self().duplicate_expr(x->m_opened);
        expr_t* m_number = self().duplicate_expr(x->m_number);
        expr_t* m_named = self().duplicate_expr(x->m_named);
        expr_t* m_name = self().duplicate_expr(x->m_name);
        expr_t* m_access = self().duplicate_expr(x->m_access);
        expr_t* m_sequential = self().duplicate_expr(x->m_sequential);
        expr_t* m_direct = self().duplicate_expr(x->m_direct);
        expr_t* m_form = self().duplicate_expr(x->m_form);
        expr_t* m_formatted = self().duplicate_expr(x->m_formatted);
        expr_t* m_unformatted = self().duplicate_expr(x->m_unformatted);
        expr_t* m_recl = self().duplicate_expr(x->m_recl);
        expr_t* m_nextrec = self().duplicate_expr(x->m_nextrec);
        expr_t* m_blank = self().duplicate_expr(x->m_blank);
        expr_t* m_position = self().duplicate_expr(x->m_position);
        expr_t* m_action = self().duplicate_expr(x->m_action);
        expr_t* m_read = self().duplicate_expr(x->m_read);
        expr_t* m_write = self().duplicate_expr(x->m_write);
        expr_t* m_readwrite = self().duplicate_expr(x->m_readwrite);
        expr_t* m_delim = self().duplicate_expr(x->m_delim);
        expr_t* m_pad = self().duplicate_expr(x->m_pad);
        expr_t* m_flen = self().duplicate_expr(x->m_flen);
        expr_t* m_blocksize = self().duplicate_expr(x->m_blocksize);
        expr_t* m_convert = self().duplicate_expr(x->m_convert);
        expr_t* m_carriagecontrol = self().duplicate_expr(x->m_carriagecontrol);
        expr_t* m_iolength = self().duplicate_expr(x->m_iolength);
        return make_FileInquire_t(al, x->base.base.loc, x->m_label, m_unit, m_file, m_iostat, m_err, m_exist, m_opened, m_number, m_named, m_name, m_access, m_sequential, m_direct, m_form, m_formatted, m_unformatted, m_recl, m_nextrec, m_blank, m_position, m_action, m_read, m_write, m_readwrite, m_delim, m_pad, m_flen, m_blocksize, m_convert, m_carriagecontrol, m_iolength);
    }


    ASR::asr_t* duplicate_FileWrite(FileWrite_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_iomsg = self().duplicate_expr(x->m_iomsg);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        expr_t* m_id = self().duplicate_expr(x->m_id);
        Vec<expr_t*> m_values;
        m_values.reserve(al, x->n_values);
        for (size_t i = 0; i < x->n_values; i++) {
            m_values.push_back(al, self().duplicate_expr(x->m_values[i]));
        }
        expr_t* m_separator = self().duplicate_expr(x->m_separator);
        expr_t* m_end = self().duplicate_expr(x->m_end);
        stmt_t* m_overloaded = self().duplicate_stmt(x->m_overloaded);
        return make_FileWrite_t(al, x->base.base.loc, x->m_label, m_unit, m_iomsg, m_iostat, m_id, m_values.p, x->n_values, m_separator, m_end, m_overloaded);
    }


    ASR::asr_t* duplicate_Return(Return_t* x) {
        return make_Return_t(al, x->base.base.loc);
    }


    ASR::asr_t* duplicate_Select(Select_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        Vec<case_stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_case_stmt(x->m_body[i]));
        }
        Vec<stmt_t*> m_default;
        m_default.reserve(al, x->n_default);
        for (size_t i = 0; i < x->n_default; i++) {
            m_default.push_back(al, self().duplicate_stmt(x->m_default[i]));
        }
        return make_Select_t(al, x->base.base.loc, m_test, m_body.p, x->n_body, m_default.p, x->n_default, x->m_enable_fall_through);
    }


    ASR::asr_t* duplicate_Stop(Stop_t* x) {
        expr_t* m_code = self().duplicate_expr(x->m_code);
        return make_Stop_t(al, x->base.base.loc, m_code);
    }


    ASR::asr_t* duplicate_Assert(Assert_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        expr_t* m_msg = self().duplicate_expr(x->m_msg);
        return make_Assert_t(al, x->base.base.loc, m_test, m_msg);
    }


    ASR::asr_t* duplicate_SubroutineCall(SubroutineCall_t* x) {
        symbol_t* m_name = x->m_name;
        symbol_t* m_original_name = x->m_original_name;
        Vec<call_arg_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::call_arg_t call_arg_copy;
            call_arg_copy.loc = x->m_args[i].loc;
            call_arg_copy.m_value = self().duplicate_expr(x->m_args[i].m_value);
            m_args.push_back(al, call_arg_copy);
        }
        expr_t* m_dt = self().duplicate_expr(x->m_dt);
        return make_SubroutineCall_t(al, x->base.base.loc, m_name, m_original_name, m_args.p, x->n_args, m_dt);
    }


    ASR::asr_t* duplicate_IntrinsicImpureSubroutine(IntrinsicImpureSubroutine_t* x) {
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        return make_IntrinsicImpureSubroutine_t(al, x->base.base.loc, x->m_intrinsic_id, m_args.p, x->n_args, x->m_overload_id);
    }


    ASR::asr_t* duplicate_Where(Where_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        Vec<stmt_t*> m_orelse;
        m_orelse.reserve(al, x->n_orelse);
        for (size_t i = 0; i < x->n_orelse; i++) {
            m_orelse.push_back(al, self().duplicate_stmt(x->m_orelse[i]));
        }
        return make_Where_t(al, x->base.base.loc, m_test, m_body.p, x->n_body, m_orelse.p, x->n_orelse);
    }


    ASR::asr_t* duplicate_WhileLoop(WhileLoop_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        Vec<stmt_t*> m_orelse;
        m_orelse.reserve(al, x->n_orelse);
        for (size_t i = 0; i < x->n_orelse; i++) {
            m_orelse.push_back(al, self().duplicate_stmt(x->m_orelse[i]));
        }
        return make_WhileLoop_t(al, x->base.base.loc, x->m_name, m_test, m_body.p, x->n_body, m_orelse.p, x->n_orelse);
    }


    ASR::asr_t* duplicate_Nullify(Nullify_t* x) {
        Vec<symbol_t*> m_vars;
        m_vars.reserve(al, x->n_vars);
        for (size_t i = 0; i < x->n_vars; i++) {
            m_vars.push_back(al, x->m_vars[i]);
        }
        return make_Nullify_t(al, x->base.base.loc, m_vars.p, x->n_vars);
    }


    ASR::asr_t* duplicate_Flush(Flush_t* x) {
        expr_t* m_unit = self().duplicate_expr(x->m_unit);
        expr_t* m_err = self().duplicate_expr(x->m_err);
        expr_t* m_iomsg = self().duplicate_expr(x->m_iomsg);
        expr_t* m_iostat = self().duplicate_expr(x->m_iostat);
        return make_Flush_t(al, x->base.base.loc, x->m_label, m_unit, m_err, m_iomsg, m_iostat);
    }


    ASR::asr_t* duplicate_ListAppend(ListAppend_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_ele = self().duplicate_expr(x->m_ele);
        return make_ListAppend_t(al, x->base.base.loc, m_a, m_ele);
    }


    ASR::asr_t* duplicate_AssociateBlockCall(AssociateBlockCall_t* x) {
        symbol_t* m_m = x->m_m;
        return make_AssociateBlockCall_t(al, x->base.base.loc, m_m);
    }


    ASR::asr_t* duplicate_SelectType(SelectType_t* x) {
        expr_t* m_selector = self().duplicate_expr(x->m_selector);
        Vec<stmt_t*> m_default;
        m_default.reserve(al, x->n_default);
        for (size_t i = 0; i < x->n_default; i++) {
            m_default.push_back(al, self().duplicate_stmt(x->m_default[i]));
        }
        return make_SelectType_t(al, x->base.base.loc, m_selector, x->m_body, x->n_body, m_default.p, x->n_default);
    }


    ASR::asr_t* duplicate_CPtrToPointer(CPtrToPointer_t* x) {
        expr_t* m_cptr = self().duplicate_expr(x->m_cptr);
        expr_t* m_ptr = self().duplicate_expr(x->m_ptr);
        expr_t* m_shape = self().duplicate_expr(x->m_shape);
        expr_t* m_lower_bounds = self().duplicate_expr(x->m_lower_bounds);
        return make_CPtrToPointer_t(al, x->base.base.loc, m_cptr, m_ptr, m_shape, m_lower_bounds);
    }


    ASR::asr_t* duplicate_BlockCall(BlockCall_t* x) {
        symbol_t* m_m = x->m_m;
        return make_BlockCall_t(al, x->base.base.loc, x->m_label, m_m);
    }


    ASR::asr_t* duplicate_SetInsert(SetInsert_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_ele = self().duplicate_expr(x->m_ele);
        return make_SetInsert_t(al, x->base.base.loc, m_a, m_ele);
    }


    ASR::asr_t* duplicate_SetRemove(SetRemove_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_ele = self().duplicate_expr(x->m_ele);
        return make_SetRemove_t(al, x->base.base.loc, m_a, m_ele);
    }


    ASR::asr_t* duplicate_ListInsert(ListInsert_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_pos = self().duplicate_expr(x->m_pos);
        expr_t* m_ele = self().duplicate_expr(x->m_ele);
        return make_ListInsert_t(al, x->base.base.loc, m_a, m_pos, m_ele);
    }


    ASR::asr_t* duplicate_ListRemove(ListRemove_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_ele = self().duplicate_expr(x->m_ele);
        return make_ListRemove_t(al, x->base.base.loc, m_a, m_ele);
    }


    ASR::asr_t* duplicate_ListClear(ListClear_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        return make_ListClear_t(al, x->base.base.loc, m_a);
    }


    ASR::asr_t* duplicate_DictInsert(DictInsert_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_key = self().duplicate_expr(x->m_key);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_DictInsert_t(al, x->base.base.loc, m_a, m_key, m_value);
    }


    ASR::asr_t* duplicate_Expr(Expr_t* x) {
        expr_t* m_expression = self().duplicate_expr(x->m_expression);
        return make_Expr_t(al, x->base.base.loc, m_expression);
    }


    ASR::asr_t* duplicate_IfExp(IfExp_t* x) {
        expr_t* m_test = self().duplicate_expr(x->m_test);
        expr_t* m_body = self().duplicate_expr(x->m_body);
        expr_t* m_orelse = self().duplicate_expr(x->m_orelse);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IfExp_t(al, x->base.base.loc, m_test, m_body, m_orelse, m_type, m_value);
    }


    ASR::asr_t* duplicate_ComplexConstructor(ComplexConstructor_t* x) {
        expr_t* m_re = self().duplicate_expr(x->m_re);
        expr_t* m_im = self().duplicate_expr(x->m_im);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ComplexConstructor_t(al, x->base.base.loc, m_re, m_im, m_type, m_value);
    }


    ASR::asr_t* duplicate_NamedExpr(NamedExpr_t* x) {
        expr_t* m_target = self().duplicate_expr(x->m_target);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_NamedExpr_t(al, x->base.base.loc, m_target, m_value, m_type);
    }


    ASR::asr_t* duplicate_FunctionCall(FunctionCall_t* x) {
        symbol_t* m_name = x->m_name;
        symbol_t* m_original_name = x->m_original_name;
        Vec<call_arg_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::call_arg_t call_arg_copy;
            call_arg_copy.loc = x->m_args[i].loc;
            call_arg_copy.m_value = self().duplicate_expr(x->m_args[i].m_value);
            m_args.push_back(al, call_arg_copy);
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        expr_t* m_dt = self().duplicate_expr(x->m_dt);
        return make_FunctionCall_t(al, x->base.base.loc, m_name, m_original_name, m_args.p, x->n_args, m_type, m_value, m_dt);
    }


    ASR::asr_t* duplicate_IntrinsicElementalFunction(IntrinsicElementalFunction_t* x) {
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntrinsicElementalFunction_t(al, x->base.base.loc, x->m_intrinsic_id, m_args.p, x->n_args, x->m_overload_id, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntrinsicArrayFunction(IntrinsicArrayFunction_t* x) {
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntrinsicArrayFunction_t(al, x->base.base.loc, x->m_arr_intrinsic_id, m_args.p, x->n_args, x->m_overload_id, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntrinsicImpureFunction(IntrinsicImpureFunction_t* x) {
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntrinsicImpureFunction_t(al, x->base.base.loc, x->m_impure_intrinsic_id, m_args.p, x->n_args, x->m_overload_id, m_type, m_value);
    }


    ASR::asr_t* duplicate_TypeInquiry(TypeInquiry_t* x) {
        ttype_t* m_arg_type = self().duplicate_ttype(x->m_arg_type);
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_TypeInquiry_t(al, x->base.base.loc, x->m_inquiry_id, m_arg_type, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_StructConstructor(StructConstructor_t* x) {
        symbol_t* m_dt_sym = x->m_dt_sym;
        Vec<call_arg_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::call_arg_t call_arg_copy;
            call_arg_copy.loc = x->m_args[i].loc;
            call_arg_copy.m_value = self().duplicate_expr(x->m_args[i].m_value);
            m_args.push_back(al, call_arg_copy);
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StructConstructor_t(al, x->base.base.loc, m_dt_sym, m_args.p, x->n_args, m_type, m_value);
    }


    ASR::asr_t* duplicate_EnumConstructor(EnumConstructor_t* x) {
        symbol_t* m_dt_sym = x->m_dt_sym;
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_EnumConstructor_t(al, x->base.base.loc, m_dt_sym, m_args.p, x->n_args, m_type, m_value);
    }


    ASR::asr_t* duplicate_UnionTypeConstructor(UnionTypeConstructor_t* x) {
        symbol_t* m_dt_sym = x->m_dt_sym;
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_UnionTypeConstructor_t(al, x->base.base.loc, m_dt_sym, m_args.p, x->n_args, m_type, m_value);
    }


    ASR::asr_t* duplicate_ImpliedDoLoop(ImpliedDoLoop_t* x) {
        Vec<expr_t*> m_values;
        m_values.reserve(al, x->n_values);
        for (size_t i = 0; i < x->n_values; i++) {
            m_values.push_back(al, self().duplicate_expr(x->m_values[i]));
        }
        expr_t* m_var = self().duplicate_expr(x->m_var);
        expr_t* m_start = self().duplicate_expr(x->m_start);
        expr_t* m_end = self().duplicate_expr(x->m_end);
        expr_t* m_increment = self().duplicate_expr(x->m_increment);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ImpliedDoLoop_t(al, x->base.base.loc, m_values.p, x->n_values, m_var, m_start, m_end, m_increment, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntegerConstant(IntegerConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_IntegerConstant_t(al, x->base.base.loc, x->m_n, m_type);
    }


    ASR::asr_t* duplicate_IntegerBOZ(IntegerBOZ_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_IntegerBOZ_t(al, x->base.base.loc, x->m_v, x->m_intboz_type, m_type);
    }


    ASR::asr_t* duplicate_IntegerBitNot(IntegerBitNot_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntegerBitNot_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntegerUnaryMinus(IntegerUnaryMinus_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntegerUnaryMinus_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntegerCompare(IntegerCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntegerCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntegerBinOp(IntegerBinOp_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntegerBinOp_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_UnsignedIntegerConstant(UnsignedIntegerConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_UnsignedIntegerConstant_t(al, x->base.base.loc, x->m_n, m_type);
    }


    ASR::asr_t* duplicate_UnsignedIntegerUnaryMinus(UnsignedIntegerUnaryMinus_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_UnsignedIntegerUnaryMinus_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_UnsignedIntegerBitNot(UnsignedIntegerBitNot_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_UnsignedIntegerBitNot_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_UnsignedIntegerCompare(UnsignedIntegerCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_UnsignedIntegerCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_UnsignedIntegerBinOp(UnsignedIntegerBinOp_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_UnsignedIntegerBinOp_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_RealConstant(RealConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_RealConstant_t(al, x->base.base.loc, x->m_r, m_type);
    }


    ASR::asr_t* duplicate_RealUnaryMinus(RealUnaryMinus_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_RealUnaryMinus_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_RealCompare(RealCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_RealCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_RealBinOp(RealBinOp_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_RealBinOp_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_RealCopySign(RealCopySign_t* x) {
        expr_t* m_target = self().duplicate_expr(x->m_target);
        expr_t* m_source = self().duplicate_expr(x->m_source);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_RealCopySign_t(al, x->base.base.loc, m_target, m_source, m_type, m_value);
    }


    ASR::asr_t* duplicate_ComplexConstant(ComplexConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_ComplexConstant_t(al, x->base.base.loc, x->m_re, x->m_im, m_type);
    }


    ASR::asr_t* duplicate_ComplexUnaryMinus(ComplexUnaryMinus_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ComplexUnaryMinus_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_ComplexCompare(ComplexCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ComplexCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_ComplexBinOp(ComplexBinOp_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ComplexBinOp_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_LogicalConstant(LogicalConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_LogicalConstant_t(al, x->base.base.loc, x->m_value, m_type);
    }


    ASR::asr_t* duplicate_LogicalNot(LogicalNot_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_LogicalNot_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_LogicalCompare(LogicalCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_LogicalCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_LogicalBinOp(LogicalBinOp_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_LogicalBinOp_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListConstant(ListConstant_t* x) {
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_ListConstant_t(al, x->base.base.loc, m_args.p, x->n_args, m_type);
    }


    ASR::asr_t* duplicate_ListLen(ListLen_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListLen_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListConcat(ListConcat_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListConcat_t(al, x->base.base.loc, m_left, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListCompare(ListCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListCount(ListCount_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        expr_t* m_ele = self().duplicate_expr(x->m_ele);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListCount_t(al, x->base.base.loc, m_arg, m_ele, m_type, m_value);
    }


    ASR::asr_t* duplicate_SetConstant(SetConstant_t* x) {
        Vec<expr_t*> m_elements;
        m_elements.reserve(al, x->n_elements);
        for (size_t i = 0; i < x->n_elements; i++) {
            m_elements.push_back(al, self().duplicate_expr(x->m_elements[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_SetConstant_t(al, x->base.base.loc, m_elements.p, x->n_elements, m_type);
    }


    ASR::asr_t* duplicate_SetLen(SetLen_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_SetLen_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_TupleConstant(TupleConstant_t* x) {
        Vec<expr_t*> m_elements;
        m_elements.reserve(al, x->n_elements);
        for (size_t i = 0; i < x->n_elements; i++) {
            m_elements.push_back(al, self().duplicate_expr(x->m_elements[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_TupleConstant_t(al, x->base.base.loc, m_elements.p, x->n_elements, m_type);
    }


    ASR::asr_t* duplicate_TupleLen(TupleLen_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_TupleLen_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_TupleCompare(TupleCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_TupleCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_TupleConcat(TupleConcat_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_TupleConcat_t(al, x->base.base.loc, m_left, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringConstant(StringConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_StringConstant_t(al, x->base.base.loc, x->m_s, m_type);
    }


    ASR::asr_t* duplicate_StringConcat(StringConcat_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringConcat_t(al, x->base.base.loc, m_left, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringRepeat(StringRepeat_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringRepeat_t(al, x->base.base.loc, m_left, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringLen(StringLen_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringLen_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringItem(StringItem_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        expr_t* m_idx = self().duplicate_expr(x->m_idx);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringItem_t(al, x->base.base.loc, m_arg, m_idx, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringSection(StringSection_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        expr_t* m_start = self().duplicate_expr(x->m_start);
        expr_t* m_end = self().duplicate_expr(x->m_end);
        expr_t* m_step = self().duplicate_expr(x->m_step);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringSection_t(al, x->base.base.loc, m_arg, m_start, m_end, m_step, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringCompare(StringCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringOrd(StringOrd_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringOrd_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringChr(StringChr_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringChr_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_StringFormat(StringFormat_t* x) {
        expr_t* m_fmt = self().duplicate_expr(x->m_fmt);
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StringFormat_t(al, x->base.base.loc, m_fmt, m_args.p, x->n_args, x->m_kind, m_type, m_value);
    }


    ASR::asr_t* duplicate_CPtrCompare(CPtrCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_CPtrCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_SymbolicCompare(SymbolicCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_SymbolicCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_DictConstant(DictConstant_t* x) {
        Vec<expr_t*> m_keys;
        m_keys.reserve(al, x->n_keys);
        for (size_t i = 0; i < x->n_keys; i++) {
            m_keys.push_back(al, self().duplicate_expr(x->m_keys[i]));
        }
        Vec<expr_t*> m_values;
        m_values.reserve(al, x->n_values);
        for (size_t i = 0; i < x->n_values; i++) {
            m_values.push_back(al, self().duplicate_expr(x->m_values[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_DictConstant_t(al, x->base.base.loc, m_keys.p, x->n_keys, m_values.p, x->n_values, m_type);
    }


    ASR::asr_t* duplicate_DictLen(DictLen_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_DictLen_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_Var(Var_t* x) {
        symbol_t* m_v = x->m_v;
        return make_Var_t(al, x->base.base.loc, m_v);
    }


    ASR::asr_t* duplicate_FunctionParam(FunctionParam_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_FunctionParam_t(al, x->base.base.loc, x->m_param_number, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArrayConstructor(ArrayConstructor_t* x) {
        Vec<expr_t*> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            m_args.push_back(al, self().duplicate_expr(x->m_args[i]));
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayConstructor_t(al, x->base.base.loc, m_args.p, x->n_args, m_type, m_value, x->m_storage_format);
    }


    ASR::asr_t* duplicate_ArrayConstant(ArrayConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_ArrayConstant_t(al, x->base.base.loc, x->m_n_data, x->m_data, m_type, x->m_storage_format);
    }


    ASR::asr_t* duplicate_ArrayItem(ArrayItem_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        Vec<array_index_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::array_index_t array_index_copy;
            array_index_copy.loc = x->m_args[i].loc;
            array_index_copy.m_left = duplicate_expr(x->m_args[i].m_left);
            array_index_copy.m_right = duplicate_expr(x->m_args[i].m_right);
            array_index_copy.m_step = duplicate_expr(x->m_args[i].m_step);
            m_args.push_back(al, array_index_copy);
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayItem_t(al, x->base.base.loc, m_v, m_args.p, x->n_args, m_type, x->m_storage_format, m_value);
    }


    ASR::asr_t* duplicate_ArraySection(ArraySection_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        Vec<array_index_t> m_args;
        m_args.reserve(al, x->n_args);
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::array_index_t array_index_copy;
            array_index_copy.loc = x->m_args[i].loc;
            array_index_copy.m_left = duplicate_expr(x->m_args[i].m_left);
            array_index_copy.m_right = duplicate_expr(x->m_args[i].m_right);
            array_index_copy.m_step = duplicate_expr(x->m_args[i].m_step);
            m_args.push_back(al, array_index_copy);
        }
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArraySection_t(al, x->base.base.loc, m_v, m_args.p, x->n_args, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArraySize(ArraySize_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        expr_t* m_dim = self().duplicate_expr(x->m_dim);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArraySize_t(al, x->base.base.loc, m_v, m_dim, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArrayBound(ArrayBound_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        expr_t* m_dim = self().duplicate_expr(x->m_dim);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayBound_t(al, x->base.base.loc, m_v, m_dim, m_type, x->m_bound, m_value);
    }


    ASR::asr_t* duplicate_ArrayTranspose(ArrayTranspose_t* x) {
        expr_t* m_matrix = self().duplicate_expr(x->m_matrix);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayTranspose_t(al, x->base.base.loc, m_matrix, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArrayPack(ArrayPack_t* x) {
        expr_t* m_array = self().duplicate_expr(x->m_array);
        expr_t* m_mask = self().duplicate_expr(x->m_mask);
        expr_t* m_vector = self().duplicate_expr(x->m_vector);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayPack_t(al, x->base.base.loc, m_array, m_mask, m_vector, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArrayReshape(ArrayReshape_t* x) {
        expr_t* m_array = self().duplicate_expr(x->m_array);
        expr_t* m_shape = self().duplicate_expr(x->m_shape);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayReshape_t(al, x->base.base.loc, m_array, m_shape, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArrayBroadcast(ArrayBroadcast_t* x) {
        expr_t* m_array = self().duplicate_expr(x->m_array);
        expr_t* m_shape = self().duplicate_expr(x->m_shape);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayBroadcast_t(al, x->base.base.loc, m_array, m_shape, m_type, m_value);
    }


    ASR::asr_t* duplicate_BitCast(BitCast_t* x) {
        expr_t* m_source = self().duplicate_expr(x->m_source);
        expr_t* m_mold = self().duplicate_expr(x->m_mold);
        expr_t* m_size = self().duplicate_expr(x->m_size);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_BitCast_t(al, x->base.base.loc, m_source, m_mold, m_size, m_type, m_value);
    }


    ASR::asr_t* duplicate_StructInstanceMember(StructInstanceMember_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        symbol_t* m_m = x->m_m;
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StructInstanceMember_t(al, x->base.base.loc, m_v, m_m, m_type, m_value);
    }


    ASR::asr_t* duplicate_StructStaticMember(StructStaticMember_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        symbol_t* m_m = x->m_m;
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_StructStaticMember_t(al, x->base.base.loc, m_v, m_m, m_type, m_value);
    }


    ASR::asr_t* duplicate_EnumStaticMember(EnumStaticMember_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        symbol_t* m_m = x->m_m;
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_EnumStaticMember_t(al, x->base.base.loc, m_v, m_m, m_type, m_value);
    }


    ASR::asr_t* duplicate_UnionInstanceMember(UnionInstanceMember_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        symbol_t* m_m = x->m_m;
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_UnionInstanceMember_t(al, x->base.base.loc, m_v, m_m, m_type, m_value);
    }


    ASR::asr_t* duplicate_EnumName(EnumName_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        ttype_t* m_enum_type = self().duplicate_ttype(x->m_enum_type);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_EnumName_t(al, x->base.base.loc, m_v, m_enum_type, m_type, m_value);
    }


    ASR::asr_t* duplicate_EnumValue(EnumValue_t* x) {
        expr_t* m_v = self().duplicate_expr(x->m_v);
        ttype_t* m_enum_type = self().duplicate_ttype(x->m_enum_type);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_EnumValue_t(al, x->base.base.loc, m_v, m_enum_type, m_type, m_value);
    }


    ASR::asr_t* duplicate_OverloadedCompare(OverloadedCompare_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        expr_t* m_overloaded = self().duplicate_expr(x->m_overloaded);
        return make_OverloadedCompare_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value, m_overloaded);
    }


    ASR::asr_t* duplicate_OverloadedBinOp(OverloadedBinOp_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        expr_t* m_overloaded = self().duplicate_expr(x->m_overloaded);
        return make_OverloadedBinOp_t(al, x->base.base.loc, m_left, x->m_op, m_right, m_type, m_value, m_overloaded);
    }


    ASR::asr_t* duplicate_OverloadedUnaryMinus(OverloadedUnaryMinus_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        expr_t* m_overloaded = self().duplicate_expr(x->m_overloaded);
        return make_OverloadedUnaryMinus_t(al, x->base.base.loc, m_arg, m_type, m_value, m_overloaded);
    }


    ASR::asr_t* duplicate_OverloadedStringConcat(OverloadedStringConcat_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        expr_t* m_overloaded = self().duplicate_expr(x->m_overloaded);
        return make_OverloadedStringConcat_t(al, x->base.base.loc, m_left, m_right, m_type, m_value, m_overloaded);
    }


    ASR::asr_t* duplicate_Cast(Cast_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_Cast_t(al, x->base.base.loc, m_arg, x->m_kind, m_type, m_value);
    }


    ASR::asr_t* duplicate_ArrayPhysicalCast(ArrayPhysicalCast_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ArrayPhysicalCast_t(al, x->base.base.loc, m_arg, x->m_old, x->m_new, m_type, m_value);
    }


    ASR::asr_t* duplicate_ComplexRe(ComplexRe_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ComplexRe_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_ComplexIm(ComplexIm_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ComplexIm_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_DictItem(DictItem_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_key = self().duplicate_expr(x->m_key);
        expr_t* m_default = self().duplicate_expr(x->m_default);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_DictItem_t(al, x->base.base.loc, m_a, m_key, m_default, m_type, m_value);
    }


    ASR::asr_t* duplicate_CLoc(CLoc_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_CLoc_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_PointerToCPtr(PointerToCPtr_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_PointerToCPtr_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_GetPointer(GetPointer_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_GetPointer_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListItem(ListItem_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_pos = self().duplicate_expr(x->m_pos);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListItem_t(al, x->base.base.loc, m_a, m_pos, m_type, m_value);
    }


    ASR::asr_t* duplicate_TupleItem(TupleItem_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_pos = self().duplicate_expr(x->m_pos);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_TupleItem_t(al, x->base.base.loc, m_a, m_pos, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListSection(ListSection_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        ASR::array_index_t m_section;
        m_section.loc = x->m_section.loc;
        m_section.m_left = duplicate_expr(x->m_section.m_left);
        m_section.m_right = duplicate_expr(x->m_section.m_right);
        m_section.m_step = duplicate_expr(x->m_section.m_step);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListSection_t(al, x->base.base.loc, m_a, m_section, m_type, m_value);
    }


    ASR::asr_t* duplicate_ListRepeat(ListRepeat_t* x) {
        expr_t* m_left = self().duplicate_expr(x->m_left);
        expr_t* m_right = self().duplicate_expr(x->m_right);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_ListRepeat_t(al, x->base.base.loc, m_left, m_right, m_type, m_value);
    }


    ASR::asr_t* duplicate_DictPop(DictPop_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        expr_t* m_key = self().duplicate_expr(x->m_key);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_DictPop_t(al, x->base.base.loc, m_a, m_key, m_type, m_value);
    }


    ASR::asr_t* duplicate_SetPop(SetPop_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_SetPop_t(al, x->base.base.loc, m_a, m_type, m_value);
    }


    ASR::asr_t* duplicate_IntegerBitLen(IntegerBitLen_t* x) {
        expr_t* m_a = self().duplicate_expr(x->m_a);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_IntegerBitLen_t(al, x->base.base.loc, m_a, m_type, m_value);
    }


    ASR::asr_t* duplicate_Ichar(Ichar_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_Ichar_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_Iachar(Iachar_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_Iachar_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_SizeOfType(SizeOfType_t* x) {
        ttype_t* m_arg = self().duplicate_ttype(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_SizeOfType_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_PointerNullConstant(PointerNullConstant_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_PointerNullConstant_t(al, x->base.base.loc, m_type);
    }


    ASR::asr_t* duplicate_PointerAssociated(PointerAssociated_t* x) {
        expr_t* m_ptr = self().duplicate_expr(x->m_ptr);
        expr_t* m_tgt = self().duplicate_expr(x->m_tgt);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_PointerAssociated_t(al, x->base.base.loc, m_ptr, m_tgt, m_type, m_value);
    }


    ASR::asr_t* duplicate_RealSqrt(RealSqrt_t* x) {
        expr_t* m_arg = self().duplicate_expr(x->m_arg);
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        expr_t* m_value = self().duplicate_expr(x->m_value);
        return make_RealSqrt_t(al, x->base.base.loc, m_arg, m_type, m_value);
    }


    ASR::asr_t* duplicate_Integer(Integer_t* x) {
        return make_Integer_t(al, x->base.base.loc, x->m_kind);
    }


    ASR::asr_t* duplicate_UnsignedInteger(UnsignedInteger_t* x) {
        return make_UnsignedInteger_t(al, x->base.base.loc, x->m_kind);
    }


    ASR::asr_t* duplicate_Real(Real_t* x) {
        return make_Real_t(al, x->base.base.loc, x->m_kind);
    }


    ASR::asr_t* duplicate_Complex(Complex_t* x) {
        return make_Complex_t(al, x->base.base.loc, x->m_kind);
    }


    ASR::asr_t* duplicate_Character(Character_t* x) {
        expr_t* m_len_expr = self().duplicate_expr(x->m_len_expr);
        return make_Character_t(al, x->base.base.loc, x->m_kind, x->m_len, m_len_expr);
    }


    ASR::asr_t* duplicate_Logical(Logical_t* x) {
        return make_Logical_t(al, x->base.base.loc, x->m_kind);
    }


    ASR::asr_t* duplicate_Set(Set_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_Set_t(al, x->base.base.loc, m_type);
    }


    ASR::asr_t* duplicate_List(List_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_List_t(al, x->base.base.loc, m_type);
    }


    ASR::asr_t* duplicate_Tuple(Tuple_t* x) {
        Vec<ttype_t*> m_type;
        m_type.reserve(al, x->n_type);
        for (size_t i = 0; i < x->n_type; i++) {
            m_type.push_back(al, self().duplicate_ttype(x->m_type[i]));
        }
        return make_Tuple_t(al, x->base.base.loc, m_type.p, x->n_type);
    }


    ASR::asr_t* duplicate_StructType(StructType_t* x) {
        symbol_t* m_derived_type = x->m_derived_type;
        return make_StructType_t(al, x->base.base.loc, m_derived_type);
    }


    ASR::asr_t* duplicate_EnumType(EnumType_t* x) {
        symbol_t* m_enum_type = x->m_enum_type;
        return make_EnumType_t(al, x->base.base.loc, m_enum_type);
    }


    ASR::asr_t* duplicate_Union(Union_t* x) {
        symbol_t* m_union_type = x->m_union_type;
        return make_Union_t(al, x->base.base.loc, m_union_type);
    }


    ASR::asr_t* duplicate_ClassType(ClassType_t* x) {
        symbol_t* m_class_type = x->m_class_type;
        return make_ClassType_t(al, x->base.base.loc, m_class_type);
    }


    ASR::asr_t* duplicate_Dict(Dict_t* x) {
        ttype_t* m_key_type = self().duplicate_ttype(x->m_key_type);
        ttype_t* m_value_type = self().duplicate_ttype(x->m_value_type);
        return make_Dict_t(al, x->base.base.loc, m_key_type, m_value_type);
    }


    ASR::asr_t* duplicate_Pointer(Pointer_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_Pointer_t(al, x->base.base.loc, m_type);
    }


    ASR::asr_t* duplicate_Allocatable(Allocatable_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        return make_Allocatable_t(al, x->base.base.loc, m_type);
    }


    ASR::asr_t* duplicate_CPtr(CPtr_t* x) {
        return make_CPtr_t(al, x->base.base.loc);
    }


    ASR::asr_t* duplicate_SymbolicExpression(SymbolicExpression_t* x) {
        return make_SymbolicExpression_t(al, x->base.base.loc);
    }


    ASR::asr_t* duplicate_TypeParameter(TypeParameter_t* x) {
        return make_TypeParameter_t(al, x->base.base.loc, x->m_param);
    }


    ASR::asr_t* duplicate_Array(Array_t* x) {
        ttype_t* m_type = self().duplicate_ttype(x->m_type);
        Vec<dimension_t> m_dims;
        m_dims.reserve(al, x->n_dims);
        for (size_t i = 0; i < x->n_dims; i++) {
            ASR::dimension_t dim_copy;
            dim_copy.loc = x->m_dims[i].loc;
            dim_copy.m_start = self().duplicate_expr(x->m_dims[i].m_start);
            dim_copy.m_length = self().duplicate_expr(x->m_dims[i].m_length);
            m_dims.push_back(al, dim_copy);
        }
        return make_Array_t(al, x->base.base.loc, m_type, m_dims.p, x->n_dims, x->m_physical_type);
    }


    ASR::asr_t* duplicate_FunctionType(FunctionType_t* x) {
        Vec<ttype_t*> m_arg_types;
        m_arg_types.reserve(al, x->n_arg_types);
        for (size_t i = 0; i < x->n_arg_types; i++) {
            m_arg_types.push_back(al, self().duplicate_ttype(x->m_arg_types[i]));
        }
        ttype_t* m_return_var_type = self().duplicate_ttype(x->m_return_var_type);
        Vec<symbol_t*> m_restrictions;
        m_restrictions.reserve(al, x->n_restrictions);
        for (size_t i = 0; i < x->n_restrictions; i++) {
            m_restrictions.push_back(al, x->m_restrictions[i]);
        }
        return make_FunctionType_t(al, x->base.base.loc, m_arg_types.p, x->n_arg_types, m_return_var_type, x->m_abi, x->m_deftype, x->m_bindc_name, x->m_elemental, x->m_pure, x->m_module, x->m_inline, x->m_static, m_restrictions.p, x->n_restrictions, x->m_is_restriction);
    }


    ASR::asr_t* duplicate_CaseStmt(CaseStmt_t* x) {
        Vec<expr_t*> m_test;
        m_test.reserve(al, x->n_test);
        for (size_t i = 0; i < x->n_test; i++) {
            m_test.push_back(al, self().duplicate_expr(x->m_test[i]));
        }
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        return make_CaseStmt_t(al, x->base.base.loc, m_test.p, x->n_test, m_body.p, x->n_body, x->m_fall_through);
    }


    ASR::asr_t* duplicate_CaseStmt_Range(CaseStmt_Range_t* x) {
        expr_t* m_start = self().duplicate_expr(x->m_start);
        expr_t* m_end = self().duplicate_expr(x->m_end);
        Vec<stmt_t*> m_body;
        m_body.reserve(al, x->n_body);
        for (size_t i = 0; i < x->n_body; i++) {
            m_body.push_back(al, self().duplicate_stmt(x->m_body[i]));
        }
        return make_CaseStmt_Range_t(al, x->base.base.loc, m_start, m_end, m_body.p, x->n_body);
    }

    ASR::stmt_t* duplicate_stmt(ASR::stmt_t* x) {
        if( !x ) {
            return nullptr;
        }

        switch(x->type) {
            case ASR::stmtType::Allocate: {
                return down_cast<ASR::stmt_t>(self().duplicate_Allocate(down_cast<ASR::Allocate_t>(x)));
            }
            case ASR::stmtType::ReAlloc: {
                return down_cast<ASR::stmt_t>(self().duplicate_ReAlloc(down_cast<ASR::ReAlloc_t>(x)));
            }
            case ASR::stmtType::Assign: {
                return down_cast<ASR::stmt_t>(self().duplicate_Assign(down_cast<ASR::Assign_t>(x)));
            }
            case ASR::stmtType::Assignment: {
                return down_cast<ASR::stmt_t>(self().duplicate_Assignment(down_cast<ASR::Assignment_t>(x)));
            }
            case ASR::stmtType::Associate: {
                return down_cast<ASR::stmt_t>(self().duplicate_Associate(down_cast<ASR::Associate_t>(x)));
            }
            case ASR::stmtType::Cycle: {
                return down_cast<ASR::stmt_t>(self().duplicate_Cycle(down_cast<ASR::Cycle_t>(x)));
            }
            case ASR::stmtType::ExplicitDeallocate: {
                return down_cast<ASR::stmt_t>(self().duplicate_ExplicitDeallocate(down_cast<ASR::ExplicitDeallocate_t>(x)));
            }
            case ASR::stmtType::ImplicitDeallocate: {
                return down_cast<ASR::stmt_t>(self().duplicate_ImplicitDeallocate(down_cast<ASR::ImplicitDeallocate_t>(x)));
            }
            case ASR::stmtType::DoConcurrentLoop: {
                return down_cast<ASR::stmt_t>(self().duplicate_DoConcurrentLoop(down_cast<ASR::DoConcurrentLoop_t>(x)));
            }
            case ASR::stmtType::DoLoop: {
                return down_cast<ASR::stmt_t>(self().duplicate_DoLoop(down_cast<ASR::DoLoop_t>(x)));
            }
            case ASR::stmtType::ErrorStop: {
                return down_cast<ASR::stmt_t>(self().duplicate_ErrorStop(down_cast<ASR::ErrorStop_t>(x)));
            }
            case ASR::stmtType::Exit: {
                return down_cast<ASR::stmt_t>(self().duplicate_Exit(down_cast<ASR::Exit_t>(x)));
            }
            case ASR::stmtType::ForAllSingle: {
                return down_cast<ASR::stmt_t>(self().duplicate_ForAllSingle(down_cast<ASR::ForAllSingle_t>(x)));
            }
            case ASR::stmtType::GoTo: {
                return down_cast<ASR::stmt_t>(self().duplicate_GoTo(down_cast<ASR::GoTo_t>(x)));
            }
            case ASR::stmtType::GoToTarget: {
                return down_cast<ASR::stmt_t>(self().duplicate_GoToTarget(down_cast<ASR::GoToTarget_t>(x)));
            }
            case ASR::stmtType::If: {
                return down_cast<ASR::stmt_t>(self().duplicate_If(down_cast<ASR::If_t>(x)));
            }
            case ASR::stmtType::IfArithmetic: {
                return down_cast<ASR::stmt_t>(self().duplicate_IfArithmetic(down_cast<ASR::IfArithmetic_t>(x)));
            }
            case ASR::stmtType::Print: {
                return down_cast<ASR::stmt_t>(self().duplicate_Print(down_cast<ASR::Print_t>(x)));
            }
            case ASR::stmtType::FileOpen: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileOpen(down_cast<ASR::FileOpen_t>(x)));
            }
            case ASR::stmtType::FileClose: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileClose(down_cast<ASR::FileClose_t>(x)));
            }
            case ASR::stmtType::FileRead: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileRead(down_cast<ASR::FileRead_t>(x)));
            }
            case ASR::stmtType::FileBackspace: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileBackspace(down_cast<ASR::FileBackspace_t>(x)));
            }
            case ASR::stmtType::FileRewind: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileRewind(down_cast<ASR::FileRewind_t>(x)));
            }
            case ASR::stmtType::FileInquire: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileInquire(down_cast<ASR::FileInquire_t>(x)));
            }
            case ASR::stmtType::FileWrite: {
                return down_cast<ASR::stmt_t>(self().duplicate_FileWrite(down_cast<ASR::FileWrite_t>(x)));
            }
            case ASR::stmtType::Return: {
                return down_cast<ASR::stmt_t>(self().duplicate_Return(down_cast<ASR::Return_t>(x)));
            }
            case ASR::stmtType::Select: {
                return down_cast<ASR::stmt_t>(self().duplicate_Select(down_cast<ASR::Select_t>(x)));
            }
            case ASR::stmtType::Stop: {
                return down_cast<ASR::stmt_t>(self().duplicate_Stop(down_cast<ASR::Stop_t>(x)));
            }
            case ASR::stmtType::Assert: {
                return down_cast<ASR::stmt_t>(self().duplicate_Assert(down_cast<ASR::Assert_t>(x)));
            }
            case ASR::stmtType::SubroutineCall: {
                if( !allow_procedure_calls ) {
                    success = false;
                    return nullptr;
                }
                return down_cast<ASR::stmt_t>(self().duplicate_SubroutineCall(down_cast<ASR::SubroutineCall_t>(x)));
            }
            case ASR::stmtType::IntrinsicImpureSubroutine: {
                return down_cast<ASR::stmt_t>(self().duplicate_IntrinsicImpureSubroutine(down_cast<ASR::IntrinsicImpureSubroutine_t>(x)));
            }
            case ASR::stmtType::Where: {
                return down_cast<ASR::stmt_t>(self().duplicate_Where(down_cast<ASR::Where_t>(x)));
            }
            case ASR::stmtType::WhileLoop: {
                return down_cast<ASR::stmt_t>(self().duplicate_WhileLoop(down_cast<ASR::WhileLoop_t>(x)));
            }
            case ASR::stmtType::Nullify: {
                return down_cast<ASR::stmt_t>(self().duplicate_Nullify(down_cast<ASR::Nullify_t>(x)));
            }
            case ASR::stmtType::Flush: {
                return down_cast<ASR::stmt_t>(self().duplicate_Flush(down_cast<ASR::Flush_t>(x)));
            }
            case ASR::stmtType::ListAppend: {
                return down_cast<ASR::stmt_t>(self().duplicate_ListAppend(down_cast<ASR::ListAppend_t>(x)));
            }
            case ASR::stmtType::AssociateBlockCall: {
                return down_cast<ASR::stmt_t>(self().duplicate_AssociateBlockCall(down_cast<ASR::AssociateBlockCall_t>(x)));
            }
            case ASR::stmtType::SelectType: {
                return down_cast<ASR::stmt_t>(self().duplicate_SelectType(down_cast<ASR::SelectType_t>(x)));
            }
            case ASR::stmtType::CPtrToPointer: {
                return down_cast<ASR::stmt_t>(self().duplicate_CPtrToPointer(down_cast<ASR::CPtrToPointer_t>(x)));
            }
            case ASR::stmtType::BlockCall: {
                return down_cast<ASR::stmt_t>(self().duplicate_BlockCall(down_cast<ASR::BlockCall_t>(x)));
            }
            case ASR::stmtType::SetInsert: {
                return down_cast<ASR::stmt_t>(self().duplicate_SetInsert(down_cast<ASR::SetInsert_t>(x)));
            }
            case ASR::stmtType::SetRemove: {
                return down_cast<ASR::stmt_t>(self().duplicate_SetRemove(down_cast<ASR::SetRemove_t>(x)));
            }
            case ASR::stmtType::ListInsert: {
                return down_cast<ASR::stmt_t>(self().duplicate_ListInsert(down_cast<ASR::ListInsert_t>(x)));
            }
            case ASR::stmtType::ListRemove: {
                return down_cast<ASR::stmt_t>(self().duplicate_ListRemove(down_cast<ASR::ListRemove_t>(x)));
            }
            case ASR::stmtType::ListClear: {
                return down_cast<ASR::stmt_t>(self().duplicate_ListClear(down_cast<ASR::ListClear_t>(x)));
            }
            case ASR::stmtType::DictInsert: {
                return down_cast<ASR::stmt_t>(self().duplicate_DictInsert(down_cast<ASR::DictInsert_t>(x)));
            }
            case ASR::stmtType::Expr: {
                return down_cast<ASR::stmt_t>(self().duplicate_Expr(down_cast<ASR::Expr_t>(x)));
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Duplication of " + std::to_string(x->type) + " statement is not supported yet.");
            }
        }

        return nullptr;
    }

    ASR::expr_t* duplicate_expr(ASR::expr_t* x) {
        if( !x ) {
            return nullptr;
        }

        switch(x->type) {
            case ASR::exprType::IfExp: {
                return down_cast<ASR::expr_t>(self().duplicate_IfExp(down_cast<ASR::IfExp_t>(x)));
            }
            case ASR::exprType::ComplexConstructor: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexConstructor(down_cast<ASR::ComplexConstructor_t>(x)));
            }
            case ASR::exprType::NamedExpr: {
                return down_cast<ASR::expr_t>(self().duplicate_NamedExpr(down_cast<ASR::NamedExpr_t>(x)));
            }
            case ASR::exprType::FunctionCall: {
                if( !allow_procedure_calls ) {
                    success = false;
                    return nullptr;
                }
                return down_cast<ASR::expr_t>(self().duplicate_FunctionCall(down_cast<ASR::FunctionCall_t>(x)));
            }
            case ASR::exprType::IntrinsicElementalFunction: {
                return down_cast<ASR::expr_t>(self().duplicate_IntrinsicElementalFunction(down_cast<ASR::IntrinsicElementalFunction_t>(x)));
            }
            case ASR::exprType::IntrinsicArrayFunction: {
                return down_cast<ASR::expr_t>(self().duplicate_IntrinsicArrayFunction(down_cast<ASR::IntrinsicArrayFunction_t>(x)));
            }
            case ASR::exprType::IntrinsicImpureFunction: {
                return down_cast<ASR::expr_t>(self().duplicate_IntrinsicImpureFunction(down_cast<ASR::IntrinsicImpureFunction_t>(x)));
            }
            case ASR::exprType::TypeInquiry: {
                return down_cast<ASR::expr_t>(self().duplicate_TypeInquiry(down_cast<ASR::TypeInquiry_t>(x)));
            }
            case ASR::exprType::StructConstructor: {
                return down_cast<ASR::expr_t>(self().duplicate_StructConstructor(down_cast<ASR::StructConstructor_t>(x)));
            }
            case ASR::exprType::EnumConstructor: {
                return down_cast<ASR::expr_t>(self().duplicate_EnumConstructor(down_cast<ASR::EnumConstructor_t>(x)));
            }
            case ASR::exprType::UnionTypeConstructor: {
                return down_cast<ASR::expr_t>(self().duplicate_UnionTypeConstructor(down_cast<ASR::UnionTypeConstructor_t>(x)));
            }
            case ASR::exprType::ImpliedDoLoop: {
                return down_cast<ASR::expr_t>(self().duplicate_ImpliedDoLoop(down_cast<ASR::ImpliedDoLoop_t>(x)));
            }
            case ASR::exprType::IntegerConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerConstant(down_cast<ASR::IntegerConstant_t>(x)));
            }
            case ASR::exprType::IntegerBOZ: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerBOZ(down_cast<ASR::IntegerBOZ_t>(x)));
            }
            case ASR::exprType::IntegerBitNot: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerBitNot(down_cast<ASR::IntegerBitNot_t>(x)));
            }
            case ASR::exprType::IntegerUnaryMinus: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerUnaryMinus(down_cast<ASR::IntegerUnaryMinus_t>(x)));
            }
            case ASR::exprType::IntegerCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerCompare(down_cast<ASR::IntegerCompare_t>(x)));
            }
            case ASR::exprType::IntegerBinOp: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerBinOp(down_cast<ASR::IntegerBinOp_t>(x)));
            }
            case ASR::exprType::UnsignedIntegerConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_UnsignedIntegerConstant(down_cast<ASR::UnsignedIntegerConstant_t>(x)));
            }
            case ASR::exprType::UnsignedIntegerUnaryMinus: {
                return down_cast<ASR::expr_t>(self().duplicate_UnsignedIntegerUnaryMinus(down_cast<ASR::UnsignedIntegerUnaryMinus_t>(x)));
            }
            case ASR::exprType::UnsignedIntegerBitNot: {
                return down_cast<ASR::expr_t>(self().duplicate_UnsignedIntegerBitNot(down_cast<ASR::UnsignedIntegerBitNot_t>(x)));
            }
            case ASR::exprType::UnsignedIntegerCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_UnsignedIntegerCompare(down_cast<ASR::UnsignedIntegerCompare_t>(x)));
            }
            case ASR::exprType::UnsignedIntegerBinOp: {
                return down_cast<ASR::expr_t>(self().duplicate_UnsignedIntegerBinOp(down_cast<ASR::UnsignedIntegerBinOp_t>(x)));
            }
            case ASR::exprType::RealConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_RealConstant(down_cast<ASR::RealConstant_t>(x)));
            }
            case ASR::exprType::RealUnaryMinus: {
                return down_cast<ASR::expr_t>(self().duplicate_RealUnaryMinus(down_cast<ASR::RealUnaryMinus_t>(x)));
            }
            case ASR::exprType::RealCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_RealCompare(down_cast<ASR::RealCompare_t>(x)));
            }
            case ASR::exprType::RealBinOp: {
                return down_cast<ASR::expr_t>(self().duplicate_RealBinOp(down_cast<ASR::RealBinOp_t>(x)));
            }
            case ASR::exprType::RealCopySign: {
                return down_cast<ASR::expr_t>(self().duplicate_RealCopySign(down_cast<ASR::RealCopySign_t>(x)));
            }
            case ASR::exprType::ComplexConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexConstant(down_cast<ASR::ComplexConstant_t>(x)));
            }
            case ASR::exprType::ComplexUnaryMinus: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexUnaryMinus(down_cast<ASR::ComplexUnaryMinus_t>(x)));
            }
            case ASR::exprType::ComplexCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexCompare(down_cast<ASR::ComplexCompare_t>(x)));
            }
            case ASR::exprType::ComplexBinOp: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexBinOp(down_cast<ASR::ComplexBinOp_t>(x)));
            }
            case ASR::exprType::LogicalConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_LogicalConstant(down_cast<ASR::LogicalConstant_t>(x)));
            }
            case ASR::exprType::LogicalNot: {
                return down_cast<ASR::expr_t>(self().duplicate_LogicalNot(down_cast<ASR::LogicalNot_t>(x)));
            }
            case ASR::exprType::LogicalCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_LogicalCompare(down_cast<ASR::LogicalCompare_t>(x)));
            }
            case ASR::exprType::LogicalBinOp: {
                return down_cast<ASR::expr_t>(self().duplicate_LogicalBinOp(down_cast<ASR::LogicalBinOp_t>(x)));
            }
            case ASR::exprType::ListConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_ListConstant(down_cast<ASR::ListConstant_t>(x)));
            }
            case ASR::exprType::ListLen: {
                return down_cast<ASR::expr_t>(self().duplicate_ListLen(down_cast<ASR::ListLen_t>(x)));
            }
            case ASR::exprType::ListConcat: {
                return down_cast<ASR::expr_t>(self().duplicate_ListConcat(down_cast<ASR::ListConcat_t>(x)));
            }
            case ASR::exprType::ListCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_ListCompare(down_cast<ASR::ListCompare_t>(x)));
            }
            case ASR::exprType::ListCount: {
                return down_cast<ASR::expr_t>(self().duplicate_ListCount(down_cast<ASR::ListCount_t>(x)));
            }
            case ASR::exprType::SetConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_SetConstant(down_cast<ASR::SetConstant_t>(x)));
            }
            case ASR::exprType::SetLen: {
                return down_cast<ASR::expr_t>(self().duplicate_SetLen(down_cast<ASR::SetLen_t>(x)));
            }
            case ASR::exprType::TupleConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_TupleConstant(down_cast<ASR::TupleConstant_t>(x)));
            }
            case ASR::exprType::TupleLen: {
                return down_cast<ASR::expr_t>(self().duplicate_TupleLen(down_cast<ASR::TupleLen_t>(x)));
            }
            case ASR::exprType::TupleCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_TupleCompare(down_cast<ASR::TupleCompare_t>(x)));
            }
            case ASR::exprType::TupleConcat: {
                return down_cast<ASR::expr_t>(self().duplicate_TupleConcat(down_cast<ASR::TupleConcat_t>(x)));
            }
            case ASR::exprType::StringConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_StringConstant(down_cast<ASR::StringConstant_t>(x)));
            }
            case ASR::exprType::StringConcat: {
                return down_cast<ASR::expr_t>(self().duplicate_StringConcat(down_cast<ASR::StringConcat_t>(x)));
            }
            case ASR::exprType::StringRepeat: {
                return down_cast<ASR::expr_t>(self().duplicate_StringRepeat(down_cast<ASR::StringRepeat_t>(x)));
            }
            case ASR::exprType::StringLen: {
                return down_cast<ASR::expr_t>(self().duplicate_StringLen(down_cast<ASR::StringLen_t>(x)));
            }
            case ASR::exprType::StringItem: {
                return down_cast<ASR::expr_t>(self().duplicate_StringItem(down_cast<ASR::StringItem_t>(x)));
            }
            case ASR::exprType::StringSection: {
                return down_cast<ASR::expr_t>(self().duplicate_StringSection(down_cast<ASR::StringSection_t>(x)));
            }
            case ASR::exprType::StringCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_StringCompare(down_cast<ASR::StringCompare_t>(x)));
            }
            case ASR::exprType::StringOrd: {
                return down_cast<ASR::expr_t>(self().duplicate_StringOrd(down_cast<ASR::StringOrd_t>(x)));
            }
            case ASR::exprType::StringChr: {
                return down_cast<ASR::expr_t>(self().duplicate_StringChr(down_cast<ASR::StringChr_t>(x)));
            }
            case ASR::exprType::StringFormat: {
                return down_cast<ASR::expr_t>(self().duplicate_StringFormat(down_cast<ASR::StringFormat_t>(x)));
            }
            case ASR::exprType::CPtrCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_CPtrCompare(down_cast<ASR::CPtrCompare_t>(x)));
            }
            case ASR::exprType::SymbolicCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_SymbolicCompare(down_cast<ASR::SymbolicCompare_t>(x)));
            }
            case ASR::exprType::DictConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_DictConstant(down_cast<ASR::DictConstant_t>(x)));
            }
            case ASR::exprType::DictLen: {
                return down_cast<ASR::expr_t>(self().duplicate_DictLen(down_cast<ASR::DictLen_t>(x)));
            }
            case ASR::exprType::Var: {
                return down_cast<ASR::expr_t>(self().duplicate_Var(down_cast<ASR::Var_t>(x)));
            }
            case ASR::exprType::FunctionParam: {
                return down_cast<ASR::expr_t>(self().duplicate_FunctionParam(down_cast<ASR::FunctionParam_t>(x)));
            }
            case ASR::exprType::ArrayConstructor: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayConstructor(down_cast<ASR::ArrayConstructor_t>(x)));
            }
            case ASR::exprType::ArrayConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayConstant(down_cast<ASR::ArrayConstant_t>(x)));
            }
            case ASR::exprType::ArrayItem: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayItem(down_cast<ASR::ArrayItem_t>(x)));
            }
            case ASR::exprType::ArraySection: {
                return down_cast<ASR::expr_t>(self().duplicate_ArraySection(down_cast<ASR::ArraySection_t>(x)));
            }
            case ASR::exprType::ArraySize: {
                return down_cast<ASR::expr_t>(self().duplicate_ArraySize(down_cast<ASR::ArraySize_t>(x)));
            }
            case ASR::exprType::ArrayBound: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayBound(down_cast<ASR::ArrayBound_t>(x)));
            }
            case ASR::exprType::ArrayTranspose: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayTranspose(down_cast<ASR::ArrayTranspose_t>(x)));
            }
            case ASR::exprType::ArrayPack: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayPack(down_cast<ASR::ArrayPack_t>(x)));
            }
            case ASR::exprType::ArrayReshape: {
                if( !allow_reshape ) {
                    success = false;
                    return nullptr;
                }
                return down_cast<ASR::expr_t>(self().duplicate_ArrayReshape(down_cast<ASR::ArrayReshape_t>(x)));
            }
            case ASR::exprType::ArrayBroadcast: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayBroadcast(down_cast<ASR::ArrayBroadcast_t>(x)));
            }
            case ASR::exprType::BitCast: {
                return down_cast<ASR::expr_t>(self().duplicate_BitCast(down_cast<ASR::BitCast_t>(x)));
            }
            case ASR::exprType::StructInstanceMember: {
                return down_cast<ASR::expr_t>(self().duplicate_StructInstanceMember(down_cast<ASR::StructInstanceMember_t>(x)));
            }
            case ASR::exprType::StructStaticMember: {
                return down_cast<ASR::expr_t>(self().duplicate_StructStaticMember(down_cast<ASR::StructStaticMember_t>(x)));
            }
            case ASR::exprType::EnumStaticMember: {
                return down_cast<ASR::expr_t>(self().duplicate_EnumStaticMember(down_cast<ASR::EnumStaticMember_t>(x)));
            }
            case ASR::exprType::UnionInstanceMember: {
                return down_cast<ASR::expr_t>(self().duplicate_UnionInstanceMember(down_cast<ASR::UnionInstanceMember_t>(x)));
            }
            case ASR::exprType::EnumName: {
                return down_cast<ASR::expr_t>(self().duplicate_EnumName(down_cast<ASR::EnumName_t>(x)));
            }
            case ASR::exprType::EnumValue: {
                return down_cast<ASR::expr_t>(self().duplicate_EnumValue(down_cast<ASR::EnumValue_t>(x)));
            }
            case ASR::exprType::OverloadedCompare: {
                return down_cast<ASR::expr_t>(self().duplicate_OverloadedCompare(down_cast<ASR::OverloadedCompare_t>(x)));
            }
            case ASR::exprType::OverloadedBinOp: {
                return down_cast<ASR::expr_t>(self().duplicate_OverloadedBinOp(down_cast<ASR::OverloadedBinOp_t>(x)));
            }
            case ASR::exprType::OverloadedUnaryMinus: {
                return down_cast<ASR::expr_t>(self().duplicate_OverloadedUnaryMinus(down_cast<ASR::OverloadedUnaryMinus_t>(x)));
            }
            case ASR::exprType::OverloadedStringConcat: {
                return down_cast<ASR::expr_t>(self().duplicate_OverloadedStringConcat(down_cast<ASR::OverloadedStringConcat_t>(x)));
            }
            case ASR::exprType::Cast: {
                return down_cast<ASR::expr_t>(self().duplicate_Cast(down_cast<ASR::Cast_t>(x)));
            }
            case ASR::exprType::ArrayPhysicalCast: {
                return down_cast<ASR::expr_t>(self().duplicate_ArrayPhysicalCast(down_cast<ASR::ArrayPhysicalCast_t>(x)));
            }
            case ASR::exprType::ComplexRe: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexRe(down_cast<ASR::ComplexRe_t>(x)));
            }
            case ASR::exprType::ComplexIm: {
                return down_cast<ASR::expr_t>(self().duplicate_ComplexIm(down_cast<ASR::ComplexIm_t>(x)));
            }
            case ASR::exprType::DictItem: {
                return down_cast<ASR::expr_t>(self().duplicate_DictItem(down_cast<ASR::DictItem_t>(x)));
            }
            case ASR::exprType::CLoc: {
                return down_cast<ASR::expr_t>(self().duplicate_CLoc(down_cast<ASR::CLoc_t>(x)));
            }
            case ASR::exprType::PointerToCPtr: {
                return down_cast<ASR::expr_t>(self().duplicate_PointerToCPtr(down_cast<ASR::PointerToCPtr_t>(x)));
            }
            case ASR::exprType::GetPointer: {
                return down_cast<ASR::expr_t>(self().duplicate_GetPointer(down_cast<ASR::GetPointer_t>(x)));
            }
            case ASR::exprType::ListItem: {
                return down_cast<ASR::expr_t>(self().duplicate_ListItem(down_cast<ASR::ListItem_t>(x)));
            }
            case ASR::exprType::TupleItem: {
                return down_cast<ASR::expr_t>(self().duplicate_TupleItem(down_cast<ASR::TupleItem_t>(x)));
            }
            case ASR::exprType::ListSection: {
                return down_cast<ASR::expr_t>(self().duplicate_ListSection(down_cast<ASR::ListSection_t>(x)));
            }
            case ASR::exprType::ListRepeat: {
                return down_cast<ASR::expr_t>(self().duplicate_ListRepeat(down_cast<ASR::ListRepeat_t>(x)));
            }
            case ASR::exprType::DictPop: {
                return down_cast<ASR::expr_t>(self().duplicate_DictPop(down_cast<ASR::DictPop_t>(x)));
            }
            case ASR::exprType::SetPop: {
                return down_cast<ASR::expr_t>(self().duplicate_SetPop(down_cast<ASR::SetPop_t>(x)));
            }
            case ASR::exprType::IntegerBitLen: {
                return down_cast<ASR::expr_t>(self().duplicate_IntegerBitLen(down_cast<ASR::IntegerBitLen_t>(x)));
            }
            case ASR::exprType::Ichar: {
                return down_cast<ASR::expr_t>(self().duplicate_Ichar(down_cast<ASR::Ichar_t>(x)));
            }
            case ASR::exprType::Iachar: {
                return down_cast<ASR::expr_t>(self().duplicate_Iachar(down_cast<ASR::Iachar_t>(x)));
            }
            case ASR::exprType::SizeOfType: {
                return down_cast<ASR::expr_t>(self().duplicate_SizeOfType(down_cast<ASR::SizeOfType_t>(x)));
            }
            case ASR::exprType::PointerNullConstant: {
                return down_cast<ASR::expr_t>(self().duplicate_PointerNullConstant(down_cast<ASR::PointerNullConstant_t>(x)));
            }
            case ASR::exprType::PointerAssociated: {
                return down_cast<ASR::expr_t>(self().duplicate_PointerAssociated(down_cast<ASR::PointerAssociated_t>(x)));
            }
            case ASR::exprType::RealSqrt: {
                return down_cast<ASR::expr_t>(self().duplicate_RealSqrt(down_cast<ASR::RealSqrt_t>(x)));
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Duplication of " + std::to_string(x->type) + " expression is not supported yet.");
            }
        }

        return nullptr;
    }

    ASR::ttype_t* duplicate_ttype(ASR::ttype_t* x) {
        if( !x ) {
            return nullptr;
        }

        switch(x->type) {
            case ASR::ttypeType::Integer: {
                return down_cast<ASR::ttype_t>(self().duplicate_Integer(down_cast<ASR::Integer_t>(x)));
            }
            case ASR::ttypeType::UnsignedInteger: {
                return down_cast<ASR::ttype_t>(self().duplicate_UnsignedInteger(down_cast<ASR::UnsignedInteger_t>(x)));
            }
            case ASR::ttypeType::Real: {
                return down_cast<ASR::ttype_t>(self().duplicate_Real(down_cast<ASR::Real_t>(x)));
            }
            case ASR::ttypeType::Complex: {
                return down_cast<ASR::ttype_t>(self().duplicate_Complex(down_cast<ASR::Complex_t>(x)));
            }
            case ASR::ttypeType::Character: {
                return down_cast<ASR::ttype_t>(self().duplicate_Character(down_cast<ASR::Character_t>(x)));
            }
            case ASR::ttypeType::Logical: {
                return down_cast<ASR::ttype_t>(self().duplicate_Logical(down_cast<ASR::Logical_t>(x)));
            }
            case ASR::ttypeType::Set: {
                return down_cast<ASR::ttype_t>(self().duplicate_Set(down_cast<ASR::Set_t>(x)));
            }
            case ASR::ttypeType::List: {
                return down_cast<ASR::ttype_t>(self().duplicate_List(down_cast<ASR::List_t>(x)));
            }
            case ASR::ttypeType::Tuple: {
                return down_cast<ASR::ttype_t>(self().duplicate_Tuple(down_cast<ASR::Tuple_t>(x)));
            }
            case ASR::ttypeType::StructType: {
                return down_cast<ASR::ttype_t>(self().duplicate_StructType(down_cast<ASR::StructType_t>(x)));
            }
            case ASR::ttypeType::EnumType: {
                return down_cast<ASR::ttype_t>(self().duplicate_EnumType(down_cast<ASR::EnumType_t>(x)));
            }
            case ASR::ttypeType::Union: {
                return down_cast<ASR::ttype_t>(self().duplicate_Union(down_cast<ASR::Union_t>(x)));
            }
            case ASR::ttypeType::ClassType: {
                return down_cast<ASR::ttype_t>(self().duplicate_ClassType(down_cast<ASR::ClassType_t>(x)));
            }
            case ASR::ttypeType::Dict: {
                return down_cast<ASR::ttype_t>(self().duplicate_Dict(down_cast<ASR::Dict_t>(x)));
            }
            case ASR::ttypeType::Pointer: {
                return down_cast<ASR::ttype_t>(self().duplicate_Pointer(down_cast<ASR::Pointer_t>(x)));
            }
            case ASR::ttypeType::Allocatable: {
                return down_cast<ASR::ttype_t>(self().duplicate_Allocatable(down_cast<ASR::Allocatable_t>(x)));
            }
            case ASR::ttypeType::CPtr: {
                return down_cast<ASR::ttype_t>(self().duplicate_CPtr(down_cast<ASR::CPtr_t>(x)));
            }
            case ASR::ttypeType::SymbolicExpression: {
                return down_cast<ASR::ttype_t>(self().duplicate_SymbolicExpression(down_cast<ASR::SymbolicExpression_t>(x)));
            }
            case ASR::ttypeType::TypeParameter: {
                return down_cast<ASR::ttype_t>(self().duplicate_TypeParameter(down_cast<ASR::TypeParameter_t>(x)));
            }
            case ASR::ttypeType::Array: {
                return down_cast<ASR::ttype_t>(self().duplicate_Array(down_cast<ASR::Array_t>(x)));
            }
            case ASR::ttypeType::FunctionType: {
                return down_cast<ASR::ttype_t>(self().duplicate_FunctionType(down_cast<ASR::FunctionType_t>(x)));
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Duplication of " + std::to_string(x->type) + " type is not supported yet.");
            }
        }

        return nullptr;
    }

    ASR::case_stmt_t* duplicate_case_stmt(ASR::case_stmt_t* x) {
        if( !x ) {
            return nullptr;
        }

        switch(x->type) {
            case ASR::case_stmtType::CaseStmt: {
                return down_cast<ASR::case_stmt_t>(self().duplicate_CaseStmt(down_cast<ASR::CaseStmt_t>(x)));
            }
            case ASR::case_stmtType::CaseStmt_Range: {
                return down_cast<ASR::case_stmt_t>(self().duplicate_CaseStmt_Range(down_cast<ASR::CaseStmt_Range_t>(x)));
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Duplication of " + std::to_string(x->type) + " case statement is not supported yet.");
            }
        }

        return nullptr;
    }

};


/******************************************************************************/
// Expression Replacer Base class

template <class StructType>
class BaseExprReplacer {
public:
    StructType& self() { return static_cast<StructType&>(*this); }

    ASR::expr_t** current_expr;

    BaseExprReplacer() : current_expr(nullptr) {}


    void replace_IfExp(IfExp_t* x) {
        ASR::expr_t** current_expr_copy_0 = current_expr;
        current_expr = &(x->m_test);
        self().replace_expr(x->m_test);
        current_expr = current_expr_copy_0;
        ASR::expr_t** current_expr_copy_1 = current_expr;
        current_expr = &(x->m_body);
        self().replace_expr(x->m_body);
        current_expr = current_expr_copy_1;
        ASR::expr_t** current_expr_copy_2 = current_expr;
        current_expr = &(x->m_orelse);
        self().replace_expr(x->m_orelse);
        current_expr = current_expr_copy_2;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_3 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_3;
    }


    void replace_ComplexConstructor(ComplexConstructor_t* x) {
        ASR::expr_t** current_expr_copy_4 = current_expr;
        current_expr = &(x->m_re);
        self().replace_expr(x->m_re);
        current_expr = current_expr_copy_4;
        ASR::expr_t** current_expr_copy_5 = current_expr;
        current_expr = &(x->m_im);
        self().replace_expr(x->m_im);
        current_expr = current_expr_copy_5;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_6 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_6;
    }


    void replace_NamedExpr(NamedExpr_t* x) {
        ASR::expr_t** current_expr_copy_7 = current_expr;
        current_expr = &(x->m_target);
        self().replace_expr(x->m_target);
        current_expr = current_expr_copy_7;
        ASR::expr_t** current_expr_copy_8 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_8;
        self().replace_ttype(x->m_type);
    }


    void replace_FunctionCall(FunctionCall_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            if (x->m_args[i].m_value != nullptr) {
                ASR::expr_t** current_expr_copy_9 = current_expr;
                current_expr = &(x->m_args[i].m_value);
                self().replace_expr(x->m_args[i].m_value);
                current_expr = current_expr_copy_9;
            }
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_10 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_10;
        ASR::expr_t** current_expr_copy_11 = current_expr;
        current_expr = &(x->m_dt);
        self().replace_expr(x->m_dt);
        current_expr = current_expr_copy_11;
    }


    void replace_IntrinsicElementalFunction(IntrinsicElementalFunction_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_12 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_12;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_13 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_13;
    }


    void replace_IntrinsicArrayFunction(IntrinsicArrayFunction_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_14 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_14;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_15 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_15;
    }


    void replace_IntrinsicImpureFunction(IntrinsicImpureFunction_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_16 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_16;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_17 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_17;
    }


    void replace_TypeInquiry(TypeInquiry_t* x) {
        self().replace_ttype(x->m_arg_type);
        ASR::expr_t** current_expr_copy_18 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_18;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_19 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_19;
    }


    void replace_StructConstructor(StructConstructor_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            if (x->m_args[i].m_value != nullptr) {
                ASR::expr_t** current_expr_copy_20 = current_expr;
                current_expr = &(x->m_args[i].m_value);
                self().replace_expr(x->m_args[i].m_value);
                current_expr = current_expr_copy_20;
            }
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_21 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_21;
    }


    void replace_EnumConstructor(EnumConstructor_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_22 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_22;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_23 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_23;
    }


    void replace_UnionTypeConstructor(UnionTypeConstructor_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_24 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_24;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_25 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_25;
    }


    void replace_ImpliedDoLoop(ImpliedDoLoop_t* x) {
        for (size_t i = 0; i < x->n_values; i++) {
            ASR::expr_t** current_expr_copy_26 = current_expr;
            current_expr = &(x->m_values[i]);
            self().replace_expr(x->m_values[i]);
            current_expr = current_expr_copy_26;
        }
        ASR::expr_t** current_expr_copy_27 = current_expr;
        current_expr = &(x->m_var);
        self().replace_expr(x->m_var);
        current_expr = current_expr_copy_27;
        ASR::expr_t** current_expr_copy_28 = current_expr;
        current_expr = &(x->m_start);
        self().replace_expr(x->m_start);
        current_expr = current_expr_copy_28;
        ASR::expr_t** current_expr_copy_29 = current_expr;
        current_expr = &(x->m_end);
        self().replace_expr(x->m_end);
        current_expr = current_expr_copy_29;
        ASR::expr_t** current_expr_copy_30 = current_expr;
        current_expr = &(x->m_increment);
        self().replace_expr(x->m_increment);
        current_expr = current_expr_copy_30;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_31 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_31;
    }


    void replace_IntegerConstant(IntegerConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_IntegerBOZ(IntegerBOZ_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_IntegerBitNot(IntegerBitNot_t* x) {
        ASR::expr_t** current_expr_copy_32 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_32;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_33 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_33;
    }


    void replace_IntegerUnaryMinus(IntegerUnaryMinus_t* x) {
        ASR::expr_t** current_expr_copy_34 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_34;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_35 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_35;
    }


    void replace_IntegerCompare(IntegerCompare_t* x) {
        ASR::expr_t** current_expr_copy_36 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_36;
        ASR::expr_t** current_expr_copy_37 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_37;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_38 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_38;
    }


    void replace_IntegerBinOp(IntegerBinOp_t* x) {
        ASR::expr_t** current_expr_copy_39 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_39;
        ASR::expr_t** current_expr_copy_40 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_40;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_41 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_41;
    }


    void replace_UnsignedIntegerConstant(UnsignedIntegerConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_UnsignedIntegerUnaryMinus(UnsignedIntegerUnaryMinus_t* x) {
        ASR::expr_t** current_expr_copy_42 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_42;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_43 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_43;
    }


    void replace_UnsignedIntegerBitNot(UnsignedIntegerBitNot_t* x) {
        ASR::expr_t** current_expr_copy_44 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_44;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_45 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_45;
    }


    void replace_UnsignedIntegerCompare(UnsignedIntegerCompare_t* x) {
        ASR::expr_t** current_expr_copy_46 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_46;
        ASR::expr_t** current_expr_copy_47 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_47;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_48 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_48;
    }


    void replace_UnsignedIntegerBinOp(UnsignedIntegerBinOp_t* x) {
        ASR::expr_t** current_expr_copy_49 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_49;
        ASR::expr_t** current_expr_copy_50 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_50;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_51 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_51;
    }


    void replace_RealConstant(RealConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_RealUnaryMinus(RealUnaryMinus_t* x) {
        ASR::expr_t** current_expr_copy_52 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_52;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_53 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_53;
    }


    void replace_RealCompare(RealCompare_t* x) {
        ASR::expr_t** current_expr_copy_54 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_54;
        ASR::expr_t** current_expr_copy_55 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_55;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_56 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_56;
    }


    void replace_RealBinOp(RealBinOp_t* x) {
        ASR::expr_t** current_expr_copy_57 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_57;
        ASR::expr_t** current_expr_copy_58 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_58;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_59 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_59;
    }


    void replace_RealCopySign(RealCopySign_t* x) {
        ASR::expr_t** current_expr_copy_60 = current_expr;
        current_expr = &(x->m_target);
        self().replace_expr(x->m_target);
        current_expr = current_expr_copy_60;
        ASR::expr_t** current_expr_copy_61 = current_expr;
        current_expr = &(x->m_source);
        self().replace_expr(x->m_source);
        current_expr = current_expr_copy_61;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_62 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_62;
    }


    void replace_ComplexConstant(ComplexConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_ComplexUnaryMinus(ComplexUnaryMinus_t* x) {
        ASR::expr_t** current_expr_copy_63 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_63;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_64 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_64;
    }


    void replace_ComplexCompare(ComplexCompare_t* x) {
        ASR::expr_t** current_expr_copy_65 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_65;
        ASR::expr_t** current_expr_copy_66 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_66;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_67 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_67;
    }


    void replace_ComplexBinOp(ComplexBinOp_t* x) {
        ASR::expr_t** current_expr_copy_68 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_68;
        ASR::expr_t** current_expr_copy_69 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_69;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_70 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_70;
    }


    void replace_LogicalConstant(LogicalConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_LogicalNot(LogicalNot_t* x) {
        ASR::expr_t** current_expr_copy_71 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_71;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_72 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_72;
    }


    void replace_LogicalCompare(LogicalCompare_t* x) {
        ASR::expr_t** current_expr_copy_73 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_73;
        ASR::expr_t** current_expr_copy_74 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_74;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_75 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_75;
    }


    void replace_LogicalBinOp(LogicalBinOp_t* x) {
        ASR::expr_t** current_expr_copy_76 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_76;
        ASR::expr_t** current_expr_copy_77 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_77;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_78 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_78;
    }


    void replace_ListConstant(ListConstant_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_79 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_79;
        }
        self().replace_ttype(x->m_type);
    }


    void replace_ListLen(ListLen_t* x) {
        ASR::expr_t** current_expr_copy_80 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_80;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_81 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_81;
    }


    void replace_ListConcat(ListConcat_t* x) {
        ASR::expr_t** current_expr_copy_82 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_82;
        ASR::expr_t** current_expr_copy_83 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_83;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_84 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_84;
    }


    void replace_ListCompare(ListCompare_t* x) {
        ASR::expr_t** current_expr_copy_85 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_85;
        ASR::expr_t** current_expr_copy_86 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_86;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_87 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_87;
    }


    void replace_ListCount(ListCount_t* x) {
        ASR::expr_t** current_expr_copy_88 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_88;
        ASR::expr_t** current_expr_copy_89 = current_expr;
        current_expr = &(x->m_ele);
        self().replace_expr(x->m_ele);
        current_expr = current_expr_copy_89;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_90 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_90;
    }


    void replace_SetConstant(SetConstant_t* x) {
        for (size_t i = 0; i < x->n_elements; i++) {
            ASR::expr_t** current_expr_copy_91 = current_expr;
            current_expr = &(x->m_elements[i]);
            self().replace_expr(x->m_elements[i]);
            current_expr = current_expr_copy_91;
        }
        self().replace_ttype(x->m_type);
    }


    void replace_SetLen(SetLen_t* x) {
        ASR::expr_t** current_expr_copy_92 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_92;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_93 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_93;
    }


    void replace_TupleConstant(TupleConstant_t* x) {
        for (size_t i = 0; i < x->n_elements; i++) {
            ASR::expr_t** current_expr_copy_94 = current_expr;
            current_expr = &(x->m_elements[i]);
            self().replace_expr(x->m_elements[i]);
            current_expr = current_expr_copy_94;
        }
        self().replace_ttype(x->m_type);
    }


    void replace_TupleLen(TupleLen_t* x) {
        ASR::expr_t** current_expr_copy_95 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_95;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_96 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_96;
    }


    void replace_TupleCompare(TupleCompare_t* x) {
        ASR::expr_t** current_expr_copy_97 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_97;
        ASR::expr_t** current_expr_copy_98 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_98;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_99 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_99;
    }


    void replace_TupleConcat(TupleConcat_t* x) {
        ASR::expr_t** current_expr_copy_100 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_100;
        ASR::expr_t** current_expr_copy_101 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_101;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_102 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_102;
    }


    void replace_StringConstant(StringConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_StringConcat(StringConcat_t* x) {
        ASR::expr_t** current_expr_copy_103 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_103;
        ASR::expr_t** current_expr_copy_104 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_104;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_105 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_105;
    }


    void replace_StringRepeat(StringRepeat_t* x) {
        ASR::expr_t** current_expr_copy_106 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_106;
        ASR::expr_t** current_expr_copy_107 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_107;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_108 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_108;
    }


    void replace_StringLen(StringLen_t* x) {
        ASR::expr_t** current_expr_copy_109 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_109;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_110 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_110;
    }


    void replace_StringItem(StringItem_t* x) {
        ASR::expr_t** current_expr_copy_111 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_111;
        ASR::expr_t** current_expr_copy_112 = current_expr;
        current_expr = &(x->m_idx);
        self().replace_expr(x->m_idx);
        current_expr = current_expr_copy_112;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_113 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_113;
    }


    void replace_StringSection(StringSection_t* x) {
        ASR::expr_t** current_expr_copy_114 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_114;
        ASR::expr_t** current_expr_copy_115 = current_expr;
        current_expr = &(x->m_start);
        self().replace_expr(x->m_start);
        current_expr = current_expr_copy_115;
        ASR::expr_t** current_expr_copy_116 = current_expr;
        current_expr = &(x->m_end);
        self().replace_expr(x->m_end);
        current_expr = current_expr_copy_116;
        ASR::expr_t** current_expr_copy_117 = current_expr;
        current_expr = &(x->m_step);
        self().replace_expr(x->m_step);
        current_expr = current_expr_copy_117;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_118 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_118;
    }


    void replace_StringCompare(StringCompare_t* x) {
        ASR::expr_t** current_expr_copy_119 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_119;
        ASR::expr_t** current_expr_copy_120 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_120;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_121 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_121;
    }


    void replace_StringOrd(StringOrd_t* x) {
        ASR::expr_t** current_expr_copy_122 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_122;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_123 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_123;
    }


    void replace_StringChr(StringChr_t* x) {
        ASR::expr_t** current_expr_copy_124 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_124;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_125 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_125;
    }


    void replace_StringFormat(StringFormat_t* x) {
        ASR::expr_t** current_expr_copy_126 = current_expr;
        current_expr = &(x->m_fmt);
        self().replace_expr(x->m_fmt);
        current_expr = current_expr_copy_126;
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_127 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_127;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_128 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_128;
    }


    void replace_CPtrCompare(CPtrCompare_t* x) {
        ASR::expr_t** current_expr_copy_129 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_129;
        ASR::expr_t** current_expr_copy_130 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_130;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_131 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_131;
    }


    void replace_SymbolicCompare(SymbolicCompare_t* x) {
        ASR::expr_t** current_expr_copy_132 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_132;
        ASR::expr_t** current_expr_copy_133 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_133;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_134 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_134;
    }


    void replace_DictConstant(DictConstant_t* x) {
        for (size_t i = 0; i < x->n_keys; i++) {
            ASR::expr_t** current_expr_copy_135 = current_expr;
            current_expr = &(x->m_keys[i]);
            self().replace_expr(x->m_keys[i]);
            current_expr = current_expr_copy_135;
        }
        for (size_t i = 0; i < x->n_values; i++) {
            ASR::expr_t** current_expr_copy_136 = current_expr;
            current_expr = &(x->m_values[i]);
            self().replace_expr(x->m_values[i]);
            current_expr = current_expr_copy_136;
        }
        self().replace_ttype(x->m_type);
    }


    void replace_DictLen(DictLen_t* x) {
        ASR::expr_t** current_expr_copy_137 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_137;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_138 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_138;
    }


    void replace_Var(Var_t* x) {
        if (x) { }
    }


    void replace_FunctionParam(FunctionParam_t* x) {
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_139 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_139;
    }


    void replace_ArrayConstructor(ArrayConstructor_t* x) {
        for (size_t i = 0; i < x->n_args; i++) {
            ASR::expr_t** current_expr_copy_140 = current_expr;
            current_expr = &(x->m_args[i]);
            self().replace_expr(x->m_args[i]);
            current_expr = current_expr_copy_140;
        }
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_141 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_141;
    }


    void replace_ArrayConstant(ArrayConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_ArrayItem(ArrayItem_t* x) {
        ASR::expr_t** current_expr_copy_142 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_142;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_143 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_143;
    }


    void replace_ArraySection(ArraySection_t* x) {
        ASR::expr_t** current_expr_copy_144 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_144;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_145 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_145;
    }


    void replace_ArraySize(ArraySize_t* x) {
        ASR::expr_t** current_expr_copy_146 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_146;
        ASR::expr_t** current_expr_copy_147 = current_expr;
        current_expr = &(x->m_dim);
        self().replace_expr(x->m_dim);
        current_expr = current_expr_copy_147;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_148 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_148;
    }


    void replace_ArrayBound(ArrayBound_t* x) {
        ASR::expr_t** current_expr_copy_149 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_149;
        ASR::expr_t** current_expr_copy_150 = current_expr;
        current_expr = &(x->m_dim);
        self().replace_expr(x->m_dim);
        current_expr = current_expr_copy_150;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_151 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_151;
    }


    void replace_ArrayTranspose(ArrayTranspose_t* x) {
        ASR::expr_t** current_expr_copy_152 = current_expr;
        current_expr = &(x->m_matrix);
        self().replace_expr(x->m_matrix);
        current_expr = current_expr_copy_152;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_153 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_153;
    }


    void replace_ArrayPack(ArrayPack_t* x) {
        ASR::expr_t** current_expr_copy_154 = current_expr;
        current_expr = &(x->m_array);
        self().replace_expr(x->m_array);
        current_expr = current_expr_copy_154;
        ASR::expr_t** current_expr_copy_155 = current_expr;
        current_expr = &(x->m_mask);
        self().replace_expr(x->m_mask);
        current_expr = current_expr_copy_155;
        ASR::expr_t** current_expr_copy_156 = current_expr;
        current_expr = &(x->m_vector);
        self().replace_expr(x->m_vector);
        current_expr = current_expr_copy_156;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_157 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_157;
    }


    void replace_ArrayReshape(ArrayReshape_t* x) {
        ASR::expr_t** current_expr_copy_158 = current_expr;
        current_expr = &(x->m_array);
        self().replace_expr(x->m_array);
        current_expr = current_expr_copy_158;
        ASR::expr_t** current_expr_copy_159 = current_expr;
        current_expr = &(x->m_shape);
        self().replace_expr(x->m_shape);
        current_expr = current_expr_copy_159;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_160 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_160;
    }


    void replace_ArrayBroadcast(ArrayBroadcast_t* x) {
        ASR::expr_t** current_expr_copy_161 = current_expr;
        current_expr = &(x->m_array);
        self().replace_expr(x->m_array);
        current_expr = current_expr_copy_161;
        ASR::expr_t** current_expr_copy_162 = current_expr;
        current_expr = &(x->m_shape);
        self().replace_expr(x->m_shape);
        current_expr = current_expr_copy_162;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_163 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_163;
    }


    void replace_BitCast(BitCast_t* x) {
        ASR::expr_t** current_expr_copy_164 = current_expr;
        current_expr = &(x->m_source);
        self().replace_expr(x->m_source);
        current_expr = current_expr_copy_164;
        ASR::expr_t** current_expr_copy_165 = current_expr;
        current_expr = &(x->m_mold);
        self().replace_expr(x->m_mold);
        current_expr = current_expr_copy_165;
        ASR::expr_t** current_expr_copy_166 = current_expr;
        current_expr = &(x->m_size);
        self().replace_expr(x->m_size);
        current_expr = current_expr_copy_166;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_167 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_167;
    }


    void replace_StructInstanceMember(StructInstanceMember_t* x) {
        ASR::expr_t** current_expr_copy_168 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_168;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_169 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_169;
    }


    void replace_StructStaticMember(StructStaticMember_t* x) {
        ASR::expr_t** current_expr_copy_170 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_170;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_171 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_171;
    }


    void replace_EnumStaticMember(EnumStaticMember_t* x) {
        ASR::expr_t** current_expr_copy_172 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_172;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_173 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_173;
    }


    void replace_UnionInstanceMember(UnionInstanceMember_t* x) {
        ASR::expr_t** current_expr_copy_174 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_174;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_175 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_175;
    }


    void replace_EnumName(EnumName_t* x) {
        ASR::expr_t** current_expr_copy_176 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_176;
        self().replace_ttype(x->m_enum_type);
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_177 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_177;
    }


    void replace_EnumValue(EnumValue_t* x) {
        ASR::expr_t** current_expr_copy_178 = current_expr;
        current_expr = &(x->m_v);
        self().replace_expr(x->m_v);
        current_expr = current_expr_copy_178;
        self().replace_ttype(x->m_enum_type);
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_179 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_179;
    }


    void replace_OverloadedCompare(OverloadedCompare_t* x) {
        ASR::expr_t** current_expr_copy_180 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_180;
        ASR::expr_t** current_expr_copy_181 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_181;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_182 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_182;
        ASR::expr_t** current_expr_copy_183 = current_expr;
        current_expr = &(x->m_overloaded);
        self().replace_expr(x->m_overloaded);
        current_expr = current_expr_copy_183;
    }


    void replace_OverloadedBinOp(OverloadedBinOp_t* x) {
        ASR::expr_t** current_expr_copy_184 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_184;
        ASR::expr_t** current_expr_copy_185 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_185;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_186 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_186;
        ASR::expr_t** current_expr_copy_187 = current_expr;
        current_expr = &(x->m_overloaded);
        self().replace_expr(x->m_overloaded);
        current_expr = current_expr_copy_187;
    }


    void replace_OverloadedUnaryMinus(OverloadedUnaryMinus_t* x) {
        ASR::expr_t** current_expr_copy_188 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_188;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_189 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_189;
        ASR::expr_t** current_expr_copy_190 = current_expr;
        current_expr = &(x->m_overloaded);
        self().replace_expr(x->m_overloaded);
        current_expr = current_expr_copy_190;
    }


    void replace_OverloadedStringConcat(OverloadedStringConcat_t* x) {
        ASR::expr_t** current_expr_copy_191 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_191;
        ASR::expr_t** current_expr_copy_192 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_192;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_193 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_193;
        ASR::expr_t** current_expr_copy_194 = current_expr;
        current_expr = &(x->m_overloaded);
        self().replace_expr(x->m_overloaded);
        current_expr = current_expr_copy_194;
    }


    void replace_Cast(Cast_t* x) {
        ASR::expr_t** current_expr_copy_195 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_195;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_196 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_196;
    }


    void replace_ArrayPhysicalCast(ArrayPhysicalCast_t* x) {
        ASR::expr_t** current_expr_copy_197 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_197;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_198 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_198;
    }


    void replace_ComplexRe(ComplexRe_t* x) {
        ASR::expr_t** current_expr_copy_199 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_199;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_200 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_200;
    }


    void replace_ComplexIm(ComplexIm_t* x) {
        ASR::expr_t** current_expr_copy_201 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_201;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_202 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_202;
    }


    void replace_DictItem(DictItem_t* x) {
        ASR::expr_t** current_expr_copy_203 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_203;
        ASR::expr_t** current_expr_copy_204 = current_expr;
        current_expr = &(x->m_key);
        self().replace_expr(x->m_key);
        current_expr = current_expr_copy_204;
        ASR::expr_t** current_expr_copy_205 = current_expr;
        current_expr = &(x->m_default);
        self().replace_expr(x->m_default);
        current_expr = current_expr_copy_205;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_206 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_206;
    }


    void replace_CLoc(CLoc_t* x) {
        ASR::expr_t** current_expr_copy_207 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_207;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_208 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_208;
    }


    void replace_PointerToCPtr(PointerToCPtr_t* x) {
        ASR::expr_t** current_expr_copy_209 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_209;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_210 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_210;
    }


    void replace_GetPointer(GetPointer_t* x) {
        ASR::expr_t** current_expr_copy_211 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_211;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_212 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_212;
    }


    void replace_ListItem(ListItem_t* x) {
        ASR::expr_t** current_expr_copy_213 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_213;
        ASR::expr_t** current_expr_copy_214 = current_expr;
        current_expr = &(x->m_pos);
        self().replace_expr(x->m_pos);
        current_expr = current_expr_copy_214;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_215 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_215;
    }


    void replace_TupleItem(TupleItem_t* x) {
        ASR::expr_t** current_expr_copy_216 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_216;
        ASR::expr_t** current_expr_copy_217 = current_expr;
        current_expr = &(x->m_pos);
        self().replace_expr(x->m_pos);
        current_expr = current_expr_copy_217;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_218 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_218;
    }


    void replace_ListSection(ListSection_t* x) {
        ASR::expr_t** current_expr_copy_219 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_219;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_220 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_220;
    }


    void replace_ListRepeat(ListRepeat_t* x) {
        ASR::expr_t** current_expr_copy_221 = current_expr;
        current_expr = &(x->m_left);
        self().replace_expr(x->m_left);
        current_expr = current_expr_copy_221;
        ASR::expr_t** current_expr_copy_222 = current_expr;
        current_expr = &(x->m_right);
        self().replace_expr(x->m_right);
        current_expr = current_expr_copy_222;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_223 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_223;
    }


    void replace_DictPop(DictPop_t* x) {
        ASR::expr_t** current_expr_copy_224 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_224;
        ASR::expr_t** current_expr_copy_225 = current_expr;
        current_expr = &(x->m_key);
        self().replace_expr(x->m_key);
        current_expr = current_expr_copy_225;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_226 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_226;
    }


    void replace_SetPop(SetPop_t* x) {
        ASR::expr_t** current_expr_copy_227 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_227;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_228 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_228;
    }


    void replace_IntegerBitLen(IntegerBitLen_t* x) {
        ASR::expr_t** current_expr_copy_229 = current_expr;
        current_expr = &(x->m_a);
        self().replace_expr(x->m_a);
        current_expr = current_expr_copy_229;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_230 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_230;
    }


    void replace_Ichar(Ichar_t* x) {
        ASR::expr_t** current_expr_copy_231 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_231;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_232 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_232;
    }


    void replace_Iachar(Iachar_t* x) {
        ASR::expr_t** current_expr_copy_233 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_233;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_234 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_234;
    }


    void replace_SizeOfType(SizeOfType_t* x) {
        self().replace_ttype(x->m_arg);
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_235 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_235;
    }


    void replace_PointerNullConstant(PointerNullConstant_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_PointerAssociated(PointerAssociated_t* x) {
        ASR::expr_t** current_expr_copy_236 = current_expr;
        current_expr = &(x->m_ptr);
        self().replace_expr(x->m_ptr);
        current_expr = current_expr_copy_236;
        ASR::expr_t** current_expr_copy_237 = current_expr;
        current_expr = &(x->m_tgt);
        self().replace_expr(x->m_tgt);
        current_expr = current_expr_copy_237;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_238 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_238;
    }


    void replace_RealSqrt(RealSqrt_t* x) {
        ASR::expr_t** current_expr_copy_239 = current_expr;
        current_expr = &(x->m_arg);
        self().replace_expr(x->m_arg);
        current_expr = current_expr_copy_239;
        self().replace_ttype(x->m_type);
        ASR::expr_t** current_expr_copy_240 = current_expr;
        current_expr = &(x->m_value);
        self().replace_expr(x->m_value);
        current_expr = current_expr_copy_240;
    }


    void replace_Integer(Integer_t* x) {
        if (x) { }
    }


    void replace_UnsignedInteger(UnsignedInteger_t* x) {
        if (x) { }
    }


    void replace_Real(Real_t* x) {
        if (x) { }
    }


    void replace_Complex(Complex_t* x) {
        if (x) { }
    }


    void replace_Character(Character_t* x) {
        ASR::expr_t** current_expr_copy_241 = current_expr;
        current_expr = &(x->m_len_expr);
        self().replace_expr(x->m_len_expr);
        current_expr = current_expr_copy_241;
    }


    void replace_Logical(Logical_t* x) {
        if (x) { }
    }


    void replace_Set(Set_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_List(List_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_Tuple(Tuple_t* x) {
        for (size_t i = 0; i < x->n_type; i++) {
            self().replace_ttype(x->m_type[i]);
        }
    }


    void replace_StructType(StructType_t* x) {
        if (x) { }
    }


    void replace_EnumType(EnumType_t* x) {
        if (x) { }
    }


    void replace_Union(Union_t* x) {
        if (x) { }
    }


    void replace_ClassType(ClassType_t* x) {
        if (x) { }
    }


    void replace_Dict(Dict_t* x) {
        self().replace_ttype(x->m_key_type);
        self().replace_ttype(x->m_value_type);
    }


    void replace_Pointer(Pointer_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_Allocatable(Allocatable_t* x) {
        self().replace_ttype(x->m_type);
    }


    void replace_CPtr(CPtr_t* x) {
        if (x) { }
    }


    void replace_SymbolicExpression(SymbolicExpression_t* x) {
        if (x) { }
    }


    void replace_TypeParameter(TypeParameter_t* x) {
        if (x) { }
    }


    void replace_Array(Array_t* x) {
        self().replace_ttype(x->m_type);
        for (size_t i = 0; i < x->n_dims; i++) {
            ASR::expr_t** current_expr_copy_242 = current_expr;
            current_expr = &(x->m_dims[i].m_length);
            self().replace_expr(x->m_dims[i].m_length);
            current_expr = current_expr_copy_242;
            ASR::expr_t** current_expr_copy_243 = current_expr;
            current_expr = &(x->m_dims[i].m_start);
            self().replace_expr(x->m_dims[i].m_start);
            current_expr = current_expr_copy_243;
        }
    }


    void replace_FunctionType(FunctionType_t* x) {
        for (size_t i = 0; i < x->n_arg_types; i++) {
            self().replace_ttype(x->m_arg_types[i]);
        }
        self().replace_ttype(x->m_return_var_type);
        for (size_t i = 0; i < x->n_restrictions; i++) {
        }
    }

    void replace_expr(ASR::expr_t* x) {
        if( !x ) {
            return ;
        }

        switch(x->type) {
            case ASR::exprType::IfExp: {
                self().replace_IfExp(down_cast<ASR::IfExp_t>(x));
                break;
            }
            case ASR::exprType::ComplexConstructor: {
                self().replace_ComplexConstructor(down_cast<ASR::ComplexConstructor_t>(x));
                break;
            }
            case ASR::exprType::NamedExpr: {
                self().replace_NamedExpr(down_cast<ASR::NamedExpr_t>(x));
                break;
            }
            case ASR::exprType::FunctionCall: {
                self().replace_FunctionCall(down_cast<ASR::FunctionCall_t>(x));
                break;
            }
            case ASR::exprType::IntrinsicElementalFunction: {
                self().replace_IntrinsicElementalFunction(down_cast<ASR::IntrinsicElementalFunction_t>(x));
                break;
            }
            case ASR::exprType::IntrinsicArrayFunction: {
                self().replace_IntrinsicArrayFunction(down_cast<ASR::IntrinsicArrayFunction_t>(x));
                break;
            }
            case ASR::exprType::IntrinsicImpureFunction: {
                self().replace_IntrinsicImpureFunction(down_cast<ASR::IntrinsicImpureFunction_t>(x));
                break;
            }
            case ASR::exprType::TypeInquiry: {
                self().replace_TypeInquiry(down_cast<ASR::TypeInquiry_t>(x));
                break;
            }
            case ASR::exprType::StructConstructor: {
                self().replace_StructConstructor(down_cast<ASR::StructConstructor_t>(x));
                break;
            }
            case ASR::exprType::EnumConstructor: {
                self().replace_EnumConstructor(down_cast<ASR::EnumConstructor_t>(x));
                break;
            }
            case ASR::exprType::UnionTypeConstructor: {
                self().replace_UnionTypeConstructor(down_cast<ASR::UnionTypeConstructor_t>(x));
                break;
            }
            case ASR::exprType::ImpliedDoLoop: {
                self().replace_ImpliedDoLoop(down_cast<ASR::ImpliedDoLoop_t>(x));
                break;
            }
            case ASR::exprType::IntegerConstant: {
                self().replace_IntegerConstant(down_cast<ASR::IntegerConstant_t>(x));
                break;
            }
            case ASR::exprType::IntegerBOZ: {
                self().replace_IntegerBOZ(down_cast<ASR::IntegerBOZ_t>(x));
                break;
            }
            case ASR::exprType::IntegerBitNot: {
                self().replace_IntegerBitNot(down_cast<ASR::IntegerBitNot_t>(x));
                break;
            }
            case ASR::exprType::IntegerUnaryMinus: {
                self().replace_IntegerUnaryMinus(down_cast<ASR::IntegerUnaryMinus_t>(x));
                break;
            }
            case ASR::exprType::IntegerCompare: {
                self().replace_IntegerCompare(down_cast<ASR::IntegerCompare_t>(x));
                break;
            }
            case ASR::exprType::IntegerBinOp: {
                self().replace_IntegerBinOp(down_cast<ASR::IntegerBinOp_t>(x));
                break;
            }
            case ASR::exprType::UnsignedIntegerConstant: {
                self().replace_UnsignedIntegerConstant(down_cast<ASR::UnsignedIntegerConstant_t>(x));
                break;
            }
            case ASR::exprType::UnsignedIntegerUnaryMinus: {
                self().replace_UnsignedIntegerUnaryMinus(down_cast<ASR::UnsignedIntegerUnaryMinus_t>(x));
                break;
            }
            case ASR::exprType::UnsignedIntegerBitNot: {
                self().replace_UnsignedIntegerBitNot(down_cast<ASR::UnsignedIntegerBitNot_t>(x));
                break;
            }
            case ASR::exprType::UnsignedIntegerCompare: {
                self().replace_UnsignedIntegerCompare(down_cast<ASR::UnsignedIntegerCompare_t>(x));
                break;
            }
            case ASR::exprType::UnsignedIntegerBinOp: {
                self().replace_UnsignedIntegerBinOp(down_cast<ASR::UnsignedIntegerBinOp_t>(x));
                break;
            }
            case ASR::exprType::RealConstant: {
                self().replace_RealConstant(down_cast<ASR::RealConstant_t>(x));
                break;
            }
            case ASR::exprType::RealUnaryMinus: {
                self().replace_RealUnaryMinus(down_cast<ASR::RealUnaryMinus_t>(x));
                break;
            }
            case ASR::exprType::RealCompare: {
                self().replace_RealCompare(down_cast<ASR::RealCompare_t>(x));
                break;
            }
            case ASR::exprType::RealBinOp: {
                self().replace_RealBinOp(down_cast<ASR::RealBinOp_t>(x));
                break;
            }
            case ASR::exprType::RealCopySign: {
                self().replace_RealCopySign(down_cast<ASR::RealCopySign_t>(x));
                break;
            }
            case ASR::exprType::ComplexConstant: {
                self().replace_ComplexConstant(down_cast<ASR::ComplexConstant_t>(x));
                break;
            }
            case ASR::exprType::ComplexUnaryMinus: {
                self().replace_ComplexUnaryMinus(down_cast<ASR::ComplexUnaryMinus_t>(x));
                break;
            }
            case ASR::exprType::ComplexCompare: {
                self().replace_ComplexCompare(down_cast<ASR::ComplexCompare_t>(x));
                break;
            }
            case ASR::exprType::ComplexBinOp: {
                self().replace_ComplexBinOp(down_cast<ASR::ComplexBinOp_t>(x));
                break;
            }
            case ASR::exprType::LogicalConstant: {
                self().replace_LogicalConstant(down_cast<ASR::LogicalConstant_t>(x));
                break;
            }
            case ASR::exprType::LogicalNot: {
                self().replace_LogicalNot(down_cast<ASR::LogicalNot_t>(x));
                break;
            }
            case ASR::exprType::LogicalCompare: {
                self().replace_LogicalCompare(down_cast<ASR::LogicalCompare_t>(x));
                break;
            }
            case ASR::exprType::LogicalBinOp: {
                self().replace_LogicalBinOp(down_cast<ASR::LogicalBinOp_t>(x));
                break;
            }
            case ASR::exprType::ListConstant: {
                self().replace_ListConstant(down_cast<ASR::ListConstant_t>(x));
                break;
            }
            case ASR::exprType::ListLen: {
                self().replace_ListLen(down_cast<ASR::ListLen_t>(x));
                break;
            }
            case ASR::exprType::ListConcat: {
                self().replace_ListConcat(down_cast<ASR::ListConcat_t>(x));
                break;
            }
            case ASR::exprType::ListCompare: {
                self().replace_ListCompare(down_cast<ASR::ListCompare_t>(x));
                break;
            }
            case ASR::exprType::ListCount: {
                self().replace_ListCount(down_cast<ASR::ListCount_t>(x));
                break;
            }
            case ASR::exprType::SetConstant: {
                self().replace_SetConstant(down_cast<ASR::SetConstant_t>(x));
                break;
            }
            case ASR::exprType::SetLen: {
                self().replace_SetLen(down_cast<ASR::SetLen_t>(x));
                break;
            }
            case ASR::exprType::TupleConstant: {
                self().replace_TupleConstant(down_cast<ASR::TupleConstant_t>(x));
                break;
            }
            case ASR::exprType::TupleLen: {
                self().replace_TupleLen(down_cast<ASR::TupleLen_t>(x));
                break;
            }
            case ASR::exprType::TupleCompare: {
                self().replace_TupleCompare(down_cast<ASR::TupleCompare_t>(x));
                break;
            }
            case ASR::exprType::TupleConcat: {
                self().replace_TupleConcat(down_cast<ASR::TupleConcat_t>(x));
                break;
            }
            case ASR::exprType::StringConstant: {
                self().replace_StringConstant(down_cast<ASR::StringConstant_t>(x));
                break;
            }
            case ASR::exprType::StringConcat: {
                self().replace_StringConcat(down_cast<ASR::StringConcat_t>(x));
                break;
            }
            case ASR::exprType::StringRepeat: {
                self().replace_StringRepeat(down_cast<ASR::StringRepeat_t>(x));
                break;
            }
            case ASR::exprType::StringLen: {
                self().replace_StringLen(down_cast<ASR::StringLen_t>(x));
                break;
            }
            case ASR::exprType::StringItem: {
                self().replace_StringItem(down_cast<ASR::StringItem_t>(x));
                break;
            }
            case ASR::exprType::StringSection: {
                self().replace_StringSection(down_cast<ASR::StringSection_t>(x));
                break;
            }
            case ASR::exprType::StringCompare: {
                self().replace_StringCompare(down_cast<ASR::StringCompare_t>(x));
                break;
            }
            case ASR::exprType::StringOrd: {
                self().replace_StringOrd(down_cast<ASR::StringOrd_t>(x));
                break;
            }
            case ASR::exprType::StringChr: {
                self().replace_StringChr(down_cast<ASR::StringChr_t>(x));
                break;
            }
            case ASR::exprType::StringFormat: {
                self().replace_StringFormat(down_cast<ASR::StringFormat_t>(x));
                break;
            }
            case ASR::exprType::CPtrCompare: {
                self().replace_CPtrCompare(down_cast<ASR::CPtrCompare_t>(x));
                break;
            }
            case ASR::exprType::SymbolicCompare: {
                self().replace_SymbolicCompare(down_cast<ASR::SymbolicCompare_t>(x));
                break;
            }
            case ASR::exprType::DictConstant: {
                self().replace_DictConstant(down_cast<ASR::DictConstant_t>(x));
                break;
            }
            case ASR::exprType::DictLen: {
                self().replace_DictLen(down_cast<ASR::DictLen_t>(x));
                break;
            }
            case ASR::exprType::Var: {
                self().replace_Var(down_cast<ASR::Var_t>(x));
                break;
            }
            case ASR::exprType::FunctionParam: {
                self().replace_FunctionParam(down_cast<ASR::FunctionParam_t>(x));
                break;
            }
            case ASR::exprType::ArrayConstructor: {
                self().replace_ArrayConstructor(down_cast<ASR::ArrayConstructor_t>(x));
                break;
            }
            case ASR::exprType::ArrayConstant: {
                self().replace_ArrayConstant(down_cast<ASR::ArrayConstant_t>(x));
                break;
            }
            case ASR::exprType::ArrayItem: {
                self().replace_ArrayItem(down_cast<ASR::ArrayItem_t>(x));
                break;
            }
            case ASR::exprType::ArraySection: {
                self().replace_ArraySection(down_cast<ASR::ArraySection_t>(x));
                break;
            }
            case ASR::exprType::ArraySize: {
                self().replace_ArraySize(down_cast<ASR::ArraySize_t>(x));
                break;
            }
            case ASR::exprType::ArrayBound: {
                self().replace_ArrayBound(down_cast<ASR::ArrayBound_t>(x));
                break;
            }
            case ASR::exprType::ArrayTranspose: {
                self().replace_ArrayTranspose(down_cast<ASR::ArrayTranspose_t>(x));
                break;
            }
            case ASR::exprType::ArrayPack: {
                self().replace_ArrayPack(down_cast<ASR::ArrayPack_t>(x));
                break;
            }
            case ASR::exprType::ArrayReshape: {
                self().replace_ArrayReshape(down_cast<ASR::ArrayReshape_t>(x));
                break;
            }
            case ASR::exprType::ArrayBroadcast: {
                self().replace_ArrayBroadcast(down_cast<ASR::ArrayBroadcast_t>(x));
                break;
            }
            case ASR::exprType::BitCast: {
                self().replace_BitCast(down_cast<ASR::BitCast_t>(x));
                break;
            }
            case ASR::exprType::StructInstanceMember: {
                self().replace_StructInstanceMember(down_cast<ASR::StructInstanceMember_t>(x));
                break;
            }
            case ASR::exprType::StructStaticMember: {
                self().replace_StructStaticMember(down_cast<ASR::StructStaticMember_t>(x));
                break;
            }
            case ASR::exprType::EnumStaticMember: {
                self().replace_EnumStaticMember(down_cast<ASR::EnumStaticMember_t>(x));
                break;
            }
            case ASR::exprType::UnionInstanceMember: {
                self().replace_UnionInstanceMember(down_cast<ASR::UnionInstanceMember_t>(x));
                break;
            }
            case ASR::exprType::EnumName: {
                self().replace_EnumName(down_cast<ASR::EnumName_t>(x));
                break;
            }
            case ASR::exprType::EnumValue: {
                self().replace_EnumValue(down_cast<ASR::EnumValue_t>(x));
                break;
            }
            case ASR::exprType::OverloadedCompare: {
                self().replace_OverloadedCompare(down_cast<ASR::OverloadedCompare_t>(x));
                break;
            }
            case ASR::exprType::OverloadedBinOp: {
                self().replace_OverloadedBinOp(down_cast<ASR::OverloadedBinOp_t>(x));
                break;
            }
            case ASR::exprType::OverloadedUnaryMinus: {
                self().replace_OverloadedUnaryMinus(down_cast<ASR::OverloadedUnaryMinus_t>(x));
                break;
            }
            case ASR::exprType::OverloadedStringConcat: {
                self().replace_OverloadedStringConcat(down_cast<ASR::OverloadedStringConcat_t>(x));
                break;
            }
            case ASR::exprType::Cast: {
                self().replace_Cast(down_cast<ASR::Cast_t>(x));
                break;
            }
            case ASR::exprType::ArrayPhysicalCast: {
                self().replace_ArrayPhysicalCast(down_cast<ASR::ArrayPhysicalCast_t>(x));
                break;
            }
            case ASR::exprType::ComplexRe: {
                self().replace_ComplexRe(down_cast<ASR::ComplexRe_t>(x));
                break;
            }
            case ASR::exprType::ComplexIm: {
                self().replace_ComplexIm(down_cast<ASR::ComplexIm_t>(x));
                break;
            }
            case ASR::exprType::DictItem: {
                self().replace_DictItem(down_cast<ASR::DictItem_t>(x));
                break;
            }
            case ASR::exprType::CLoc: {
                self().replace_CLoc(down_cast<ASR::CLoc_t>(x));
                break;
            }
            case ASR::exprType::PointerToCPtr: {
                self().replace_PointerToCPtr(down_cast<ASR::PointerToCPtr_t>(x));
                break;
            }
            case ASR::exprType::GetPointer: {
                self().replace_GetPointer(down_cast<ASR::GetPointer_t>(x));
                break;
            }
            case ASR::exprType::ListItem: {
                self().replace_ListItem(down_cast<ASR::ListItem_t>(x));
                break;
            }
            case ASR::exprType::TupleItem: {
                self().replace_TupleItem(down_cast<ASR::TupleItem_t>(x));
                break;
            }
            case ASR::exprType::ListSection: {
                self().replace_ListSection(down_cast<ASR::ListSection_t>(x));
                break;
            }
            case ASR::exprType::ListRepeat: {
                self().replace_ListRepeat(down_cast<ASR::ListRepeat_t>(x));
                break;
            }
            case ASR::exprType::DictPop: {
                self().replace_DictPop(down_cast<ASR::DictPop_t>(x));
                break;
            }
            case ASR::exprType::SetPop: {
                self().replace_SetPop(down_cast<ASR::SetPop_t>(x));
                break;
            }
            case ASR::exprType::IntegerBitLen: {
                self().replace_IntegerBitLen(down_cast<ASR::IntegerBitLen_t>(x));
                break;
            }
            case ASR::exprType::Ichar: {
                self().replace_Ichar(down_cast<ASR::Ichar_t>(x));
                break;
            }
            case ASR::exprType::Iachar: {
                self().replace_Iachar(down_cast<ASR::Iachar_t>(x));
                break;
            }
            case ASR::exprType::SizeOfType: {
                self().replace_SizeOfType(down_cast<ASR::SizeOfType_t>(x));
                break;
            }
            case ASR::exprType::PointerNullConstant: {
                self().replace_PointerNullConstant(down_cast<ASR::PointerNullConstant_t>(x));
                break;
            }
            case ASR::exprType::PointerAssociated: {
                self().replace_PointerAssociated(down_cast<ASR::PointerAssociated_t>(x));
                break;
            }
            case ASR::exprType::RealSqrt: {
                self().replace_RealSqrt(down_cast<ASR::RealSqrt_t>(x));
                break;
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Replacement in  " + std::to_string(x->type) + " expression is not supported yet.");
            }
        }

    }
    void replace_ttype(ASR::ttype_t* x) {
        if( !x ) {
            return ;
        }

        switch(x->type) {
            case ASR::ttypeType::Integer: {
                self().replace_Integer(down_cast<ASR::Integer_t>(x));
                break;
            }
            case ASR::ttypeType::UnsignedInteger: {
                self().replace_UnsignedInteger(down_cast<ASR::UnsignedInteger_t>(x));
                break;
            }
            case ASR::ttypeType::Real: {
                self().replace_Real(down_cast<ASR::Real_t>(x));
                break;
            }
            case ASR::ttypeType::Complex: {
                self().replace_Complex(down_cast<ASR::Complex_t>(x));
                break;
            }
            case ASR::ttypeType::Character: {
                self().replace_Character(down_cast<ASR::Character_t>(x));
                break;
            }
            case ASR::ttypeType::Logical: {
                self().replace_Logical(down_cast<ASR::Logical_t>(x));
                break;
            }
            case ASR::ttypeType::Set: {
                self().replace_Set(down_cast<ASR::Set_t>(x));
                break;
            }
            case ASR::ttypeType::List: {
                self().replace_List(down_cast<ASR::List_t>(x));
                break;
            }
            case ASR::ttypeType::Tuple: {
                self().replace_Tuple(down_cast<ASR::Tuple_t>(x));
                break;
            }
            case ASR::ttypeType::StructType: {
                self().replace_StructType(down_cast<ASR::StructType_t>(x));
                break;
            }
            case ASR::ttypeType::EnumType: {
                self().replace_EnumType(down_cast<ASR::EnumType_t>(x));
                break;
            }
            case ASR::ttypeType::Union: {
                self().replace_Union(down_cast<ASR::Union_t>(x));
                break;
            }
            case ASR::ttypeType::ClassType: {
                self().replace_ClassType(down_cast<ASR::ClassType_t>(x));
                break;
            }
            case ASR::ttypeType::Dict: {
                self().replace_Dict(down_cast<ASR::Dict_t>(x));
                break;
            }
            case ASR::ttypeType::Pointer: {
                self().replace_Pointer(down_cast<ASR::Pointer_t>(x));
                break;
            }
            case ASR::ttypeType::Allocatable: {
                self().replace_Allocatable(down_cast<ASR::Allocatable_t>(x));
                break;
            }
            case ASR::ttypeType::CPtr: {
                self().replace_CPtr(down_cast<ASR::CPtr_t>(x));
                break;
            }
            case ASR::ttypeType::SymbolicExpression: {
                self().replace_SymbolicExpression(down_cast<ASR::SymbolicExpression_t>(x));
                break;
            }
            case ASR::ttypeType::TypeParameter: {
                self().replace_TypeParameter(down_cast<ASR::TypeParameter_t>(x));
                break;
            }
            case ASR::ttypeType::Array: {
                self().replace_Array(down_cast<ASR::Array_t>(x));
                break;
            }
            case ASR::ttypeType::FunctionType: {
                self().replace_FunctionType(down_cast<ASR::FunctionType_t>(x));
                break;
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Replacement in " + std::to_string(x->type) + " type is not supported yet.");
            }
        }

    }

};


/******************************************************************************/
// Statement Replacer Base class

template <class StructType>
class BaseStmtReplacer {
public:
    StructType& self() { return static_cast<StructType&>(*this); }

    ASR::stmt_t** current_stmt;
    ASR::stmt_t** current_stmt_copy;
    bool has_replacement_happened;

    BaseStmtReplacer() : current_stmt(nullptr), has_replacement_happened(false) {}


    void replace_Allocate(Allocate_t* x) {
        if (x) { }
    }


    void replace_ReAlloc(ReAlloc_t* x) {
        if (x) { }
    }


    void replace_Assign(Assign_t* x) {
        if (x) { }
    }


    void replace_Assignment(Assignment_t* x) {
        if (x) { }
    }


    void replace_Associate(Associate_t* x) {
        if (x) { }
    }


    void replace_Cycle(Cycle_t* x) {
        if (x) { }
    }


    void replace_ExplicitDeallocate(ExplicitDeallocate_t* x) {
        if (x) { }
    }


    void replace_ImplicitDeallocate(ImplicitDeallocate_t* x) {
        if (x) { }
    }


    void replace_DoConcurrentLoop(DoConcurrentLoop_t* x) {
        for (size_t i = 0; i < x->n_body; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_body[i]);
            self().replace_stmt(x->m_body[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_DoLoop(DoLoop_t* x) {
        for (size_t i = 0; i < x->n_body; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_body[i]);
            self().replace_stmt(x->m_body[i]);
            current_stmt = current_stmt_copy;
        }
        for (size_t i = 0; i < x->n_orelse; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_orelse[i]);
            self().replace_stmt(x->m_orelse[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_ErrorStop(ErrorStop_t* x) {
        if (x) { }
    }


    void replace_Exit(Exit_t* x) {
        if (x) { }
    }


    void replace_ForAllSingle(ForAllSingle_t* x) {
        if (x) { }
    }


    void replace_GoTo(GoTo_t* x) {
        if (x) { }
    }


    void replace_GoToTarget(GoToTarget_t* x) {
        if (x) { }
    }


    void replace_If(If_t* x) {
        for (size_t i = 0; i < x->n_body; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_body[i]);
            self().replace_stmt(x->m_body[i]);
            current_stmt = current_stmt_copy;
        }
        for (size_t i = 0; i < x->n_orelse; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_orelse[i]);
            self().replace_stmt(x->m_orelse[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_IfArithmetic(IfArithmetic_t* x) {
        if (x) { }
    }


    void replace_Print(Print_t* x) {
        if (x) { }
    }


    void replace_FileOpen(FileOpen_t* x) {
        if (x) { }
    }


    void replace_FileClose(FileClose_t* x) {
        if (x) { }
    }


    void replace_FileRead(FileRead_t* x) {
        if (x) { }
    }


    void replace_FileBackspace(FileBackspace_t* x) {
        if (x) { }
    }


    void replace_FileRewind(FileRewind_t* x) {
        if (x) { }
    }


    void replace_FileInquire(FileInquire_t* x) {
        if (x) { }
    }


    void replace_FileWrite(FileWrite_t* x) {
        if (x) { }
    }


    void replace_Return(Return_t* x) {
        if (x) { }
    }


    void replace_Select(Select_t* x) {
        for (size_t i = 0; i < x->n_default; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_default[i]);
            self().replace_stmt(x->m_default[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_Stop(Stop_t* x) {
        if (x) { }
    }


    void replace_Assert(Assert_t* x) {
        if (x) { }
    }


    void replace_SubroutineCall(SubroutineCall_t* x) {
        if (x) { }
    }


    void replace_IntrinsicImpureSubroutine(IntrinsicImpureSubroutine_t* x) {
        if (x) { }
    }


    void replace_Where(Where_t* x) {
        for (size_t i = 0; i < x->n_body; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_body[i]);
            self().replace_stmt(x->m_body[i]);
            current_stmt = current_stmt_copy;
        }
        for (size_t i = 0; i < x->n_orelse; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_orelse[i]);
            self().replace_stmt(x->m_orelse[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_WhileLoop(WhileLoop_t* x) {
        for (size_t i = 0; i < x->n_body; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_body[i]);
            self().replace_stmt(x->m_body[i]);
            current_stmt = current_stmt_copy;
        }
        for (size_t i = 0; i < x->n_orelse; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_orelse[i]);
            self().replace_stmt(x->m_orelse[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_Nullify(Nullify_t* x) {
        if (x) { }
    }


    void replace_Flush(Flush_t* x) {
        if (x) { }
    }


    void replace_ListAppend(ListAppend_t* x) {
        if (x) { }
    }


    void replace_AssociateBlockCall(AssociateBlockCall_t* x) {
        if (x) { }
    }


    void replace_SelectType(SelectType_t* x) {
        for (size_t i = 0; i < x->n_default; i++) {
            current_stmt_copy = current_stmt;
            current_stmt = &(x->m_default[i]);
            self().replace_stmt(x->m_default[i]);
            current_stmt = current_stmt_copy;
        }
    }


    void replace_CPtrToPointer(CPtrToPointer_t* x) {
        if (x) { }
    }


    void replace_BlockCall(BlockCall_t* x) {
        if (x) { }
    }


    void replace_SetInsert(SetInsert_t* x) {
        if (x) { }
    }


    void replace_SetRemove(SetRemove_t* x) {
        if (x) { }
    }


    void replace_ListInsert(ListInsert_t* x) {
        if (x) { }
    }


    void replace_ListRemove(ListRemove_t* x) {
        if (x) { }
    }


    void replace_ListClear(ListClear_t* x) {
        if (x) { }
    }


    void replace_DictInsert(DictInsert_t* x) {
        if (x) { }
    }


    void replace_Expr(Expr_t* x) {
        if (x) { }
    }

    void replace_stmt(ASR::stmt_t* x) {
        if( !x ) {
            return ;
        }

        switch(x->type) {
            case ASR::stmtType::Allocate: {
                self().replace_Allocate(down_cast<ASR::Allocate_t>(x));
                break;
            }
            case ASR::stmtType::ReAlloc: {
                self().replace_ReAlloc(down_cast<ASR::ReAlloc_t>(x));
                break;
            }
            case ASR::stmtType::Assign: {
                self().replace_Assign(down_cast<ASR::Assign_t>(x));
                break;
            }
            case ASR::stmtType::Assignment: {
                self().replace_Assignment(down_cast<ASR::Assignment_t>(x));
                break;
            }
            case ASR::stmtType::Associate: {
                self().replace_Associate(down_cast<ASR::Associate_t>(x));
                break;
            }
            case ASR::stmtType::Cycle: {
                self().replace_Cycle(down_cast<ASR::Cycle_t>(x));
                break;
            }
            case ASR::stmtType::ExplicitDeallocate: {
                self().replace_ExplicitDeallocate(down_cast<ASR::ExplicitDeallocate_t>(x));
                break;
            }
            case ASR::stmtType::ImplicitDeallocate: {
                self().replace_ImplicitDeallocate(down_cast<ASR::ImplicitDeallocate_t>(x));
                break;
            }
            case ASR::stmtType::DoConcurrentLoop: {
                self().replace_DoConcurrentLoop(down_cast<ASR::DoConcurrentLoop_t>(x));
                break;
            }
            case ASR::stmtType::DoLoop: {
                self().replace_DoLoop(down_cast<ASR::DoLoop_t>(x));
                break;
            }
            case ASR::stmtType::ErrorStop: {
                self().replace_ErrorStop(down_cast<ASR::ErrorStop_t>(x));
                break;
            }
            case ASR::stmtType::Exit: {
                self().replace_Exit(down_cast<ASR::Exit_t>(x));
                break;
            }
            case ASR::stmtType::ForAllSingle: {
                self().replace_ForAllSingle(down_cast<ASR::ForAllSingle_t>(x));
                break;
            }
            case ASR::stmtType::GoTo: {
                self().replace_GoTo(down_cast<ASR::GoTo_t>(x));
                break;
            }
            case ASR::stmtType::GoToTarget: {
                self().replace_GoToTarget(down_cast<ASR::GoToTarget_t>(x));
                break;
            }
            case ASR::stmtType::If: {
                self().replace_If(down_cast<ASR::If_t>(x));
                break;
            }
            case ASR::stmtType::IfArithmetic: {
                self().replace_IfArithmetic(down_cast<ASR::IfArithmetic_t>(x));
                break;
            }
            case ASR::stmtType::Print: {
                self().replace_Print(down_cast<ASR::Print_t>(x));
                break;
            }
            case ASR::stmtType::FileOpen: {
                self().replace_FileOpen(down_cast<ASR::FileOpen_t>(x));
                break;
            }
            case ASR::stmtType::FileClose: {
                self().replace_FileClose(down_cast<ASR::FileClose_t>(x));
                break;
            }
            case ASR::stmtType::FileRead: {
                self().replace_FileRead(down_cast<ASR::FileRead_t>(x));
                break;
            }
            case ASR::stmtType::FileBackspace: {
                self().replace_FileBackspace(down_cast<ASR::FileBackspace_t>(x));
                break;
            }
            case ASR::stmtType::FileRewind: {
                self().replace_FileRewind(down_cast<ASR::FileRewind_t>(x));
                break;
            }
            case ASR::stmtType::FileInquire: {
                self().replace_FileInquire(down_cast<ASR::FileInquire_t>(x));
                break;
            }
            case ASR::stmtType::FileWrite: {
                self().replace_FileWrite(down_cast<ASR::FileWrite_t>(x));
                break;
            }
            case ASR::stmtType::Return: {
                self().replace_Return(down_cast<ASR::Return_t>(x));
                break;
            }
            case ASR::stmtType::Select: {
                self().replace_Select(down_cast<ASR::Select_t>(x));
                break;
            }
            case ASR::stmtType::Stop: {
                self().replace_Stop(down_cast<ASR::Stop_t>(x));
                break;
            }
            case ASR::stmtType::Assert: {
                self().replace_Assert(down_cast<ASR::Assert_t>(x));
                break;
            }
            case ASR::stmtType::SubroutineCall: {
                self().replace_SubroutineCall(down_cast<ASR::SubroutineCall_t>(x));
                break;
            }
            case ASR::stmtType::IntrinsicImpureSubroutine: {
                self().replace_IntrinsicImpureSubroutine(down_cast<ASR::IntrinsicImpureSubroutine_t>(x));
                break;
            }
            case ASR::stmtType::Where: {
                self().replace_Where(down_cast<ASR::Where_t>(x));
                break;
            }
            case ASR::stmtType::WhileLoop: {
                self().replace_WhileLoop(down_cast<ASR::WhileLoop_t>(x));
                break;
            }
            case ASR::stmtType::Nullify: {
                self().replace_Nullify(down_cast<ASR::Nullify_t>(x));
                break;
            }
            case ASR::stmtType::Flush: {
                self().replace_Flush(down_cast<ASR::Flush_t>(x));
                break;
            }
            case ASR::stmtType::ListAppend: {
                self().replace_ListAppend(down_cast<ASR::ListAppend_t>(x));
                break;
            }
            case ASR::stmtType::AssociateBlockCall: {
                self().replace_AssociateBlockCall(down_cast<ASR::AssociateBlockCall_t>(x));
                break;
            }
            case ASR::stmtType::SelectType: {
                self().replace_SelectType(down_cast<ASR::SelectType_t>(x));
                break;
            }
            case ASR::stmtType::CPtrToPointer: {
                self().replace_CPtrToPointer(down_cast<ASR::CPtrToPointer_t>(x));
                break;
            }
            case ASR::stmtType::BlockCall: {
                self().replace_BlockCall(down_cast<ASR::BlockCall_t>(x));
                break;
            }
            case ASR::stmtType::SetInsert: {
                self().replace_SetInsert(down_cast<ASR::SetInsert_t>(x));
                break;
            }
            case ASR::stmtType::SetRemove: {
                self().replace_SetRemove(down_cast<ASR::SetRemove_t>(x));
                break;
            }
            case ASR::stmtType::ListInsert: {
                self().replace_ListInsert(down_cast<ASR::ListInsert_t>(x));
                break;
            }
            case ASR::stmtType::ListRemove: {
                self().replace_ListRemove(down_cast<ASR::ListRemove_t>(x));
                break;
            }
            case ASR::stmtType::ListClear: {
                self().replace_ListClear(down_cast<ASR::ListClear_t>(x));
                break;
            }
            case ASR::stmtType::DictInsert: {
                self().replace_DictInsert(down_cast<ASR::DictInsert_t>(x));
                break;
            }
            case ASR::stmtType::Expr: {
                self().replace_Expr(down_cast<ASR::Expr_t>(x));
                break;
            }
            default: {
                LCOMPILERS_ASSERT_MSG(false, "Replacement of " + std::to_string(x->type) + " statement is not supported yet.");
            }
        }

    }

};


/******************************************************************************/
// Walk Visitor base class

template <class StructType>
class CallReplacerOnExpressionsVisitor : public BaseVisitor<StructType>
{
private:
    StructType& self() { return static_cast<StructType&>(*this); }
public:
    ASR::expr_t** current_expr;
    SymbolTable* current_scope=nullptr;

    void call_replacer() {}
    void transform_stmts(ASR::stmt_t **&m_body, size_t &n_body) {
        for (size_t i = 0; i < n_body; i++) {
            self().visit_stmt(*m_body[i]);
        }
    }
    void visit_TranslationUnit(const TranslationUnit_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        current_scope = current_scope_copy;
    }
    void visit_Program(const Program_t &x) {
        Program_t& xx = const_cast<Program_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        current_scope = current_scope_copy;
    }
    void visit_Module(const Module_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        current_scope = current_scope_copy;
    }
    void visit_Function(const Function_t &x) {
        Function_t& xx = const_cast<Function_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().visit_ttype(*x.m_function_signature);
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_0 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_0;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        if (x.m_return_var) {
            ASR::expr_t** current_expr_copy_1 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_return_var));
            self().call_replacer();
            current_expr = current_expr_copy_1;
            if( x.m_return_var )
            self().visit_expr(*x.m_return_var);
        }
        current_scope = current_scope_copy;
    }
    void visit_GenericProcedure(const GenericProcedure_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        for (size_t i=0; i<x.n_procs; i++) {
        }
        current_scope = current_scope_copy;
    }
    void visit_CustomOperator(const CustomOperator_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        for (size_t i=0; i<x.n_procs; i++) {
        }
        current_scope = current_scope_copy;
    }
    void visit_ExternalSymbol(const ExternalSymbol_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        if ((bool&)x) { } // Suppress unused warning
        current_scope = current_scope_copy;
    }
    void visit_Struct(const Struct_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        if (x.m_alignment) {
            ASR::expr_t** current_expr_copy_2 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_alignment));
            self().call_replacer();
            current_expr = current_expr_copy_2;
            if( x.m_alignment )
            self().visit_expr(*x.m_alignment);
        }
        current_scope = current_scope_copy;
    }
    void visit_Enum(const Enum_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().visit_ttype(*x.m_type);
        current_scope = current_scope_copy;
    }
    void visit_UnionType(const UnionType_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_initializers; i++) {
            self().visit_call_arg(x.m_initializers[i]);
        }
        current_scope = current_scope_copy;
    }
    void visit_Variable(const Variable_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        if (x.m_symbolic_value) {
            ASR::expr_t** current_expr_copy_3 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_symbolic_value));
            self().call_replacer();
            current_expr = current_expr_copy_3;
            if( x.m_symbolic_value )
            self().visit_expr(*x.m_symbolic_value);
        }
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_4 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_4;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
        self().visit_ttype(*x.m_type);
        current_scope = current_scope_copy;
    }
    void visit_Class(const Class_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        current_scope = current_scope_copy;
    }
    void visit_ClassProcedure(const ClassProcedure_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_parent_symtab;
        if ((bool&)x) { } // Suppress unused warning
        current_scope = current_scope_copy;
    }
    void visit_AssociateBlock(const AssociateBlock_t &x) {
        AssociateBlock_t& xx = const_cast<AssociateBlock_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        current_scope = current_scope_copy;
    }
    void visit_Block(const Block_t &x) {
        Block_t& xx = const_cast<Block_t&>(x);
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
        current_scope = current_scope_copy;
    }
    void visit_Requirement(const Requirement_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
        current_scope = current_scope_copy;
    }
    void visit_Template(const Template_t &x) {
        SymbolTable* current_scope_copy = current_scope;
        current_scope = x.m_symtab;
        for (auto &a : x.m_symtab->get_scope()) {
            this->visit_symbol(*a.second);
        }
        for (size_t i=0; i<x.n_requires; i++) {
            self().visit_require_instantiation(*x.m_requires[i]);
        }
        current_scope = current_scope_copy;
    }
    void visit_Allocate(const Allocate_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
        if (x.m_stat) {
            ASR::expr_t** current_expr_copy_5 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_stat));
            self().call_replacer();
            current_expr = current_expr_copy_5;
            if( x.m_stat )
            self().visit_expr(*x.m_stat);
        }
        if (x.m_errmsg) {
            ASR::expr_t** current_expr_copy_6 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_errmsg));
            self().call_replacer();
            current_expr = current_expr_copy_6;
            if( x.m_errmsg )
            self().visit_expr(*x.m_errmsg);
        }
        if (x.m_source) {
            ASR::expr_t** current_expr_copy_7 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_source));
            self().call_replacer();
            current_expr = current_expr_copy_7;
            if( x.m_source )
            self().visit_expr(*x.m_source);
        }
    }
    void visit_ReAlloc(const ReAlloc_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_alloc_arg(x.m_args[i]);
        }
    }
    void visit_Assign(const Assign_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Assignment(const Assignment_t &x) {
        ASR::expr_t** current_expr_copy_8 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_target));
        self().call_replacer();
        current_expr = current_expr_copy_8;
        if( x.m_target )
        self().visit_expr(*x.m_target);
        ASR::expr_t** current_expr_copy_9 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
        self().call_replacer();
        current_expr = current_expr_copy_9;
        if( x.m_value )
        self().visit_expr(*x.m_value);
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        }
    }
    void visit_Associate(const Associate_t &x) {
        ASR::expr_t** current_expr_copy_10 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_target));
        self().call_replacer();
        current_expr = current_expr_copy_10;
        if( x.m_target )
        self().visit_expr(*x.m_target);
        ASR::expr_t** current_expr_copy_11 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
        self().call_replacer();
        current_expr = current_expr_copy_11;
        if( x.m_value )
        self().visit_expr(*x.m_value);
    }
    void visit_Cycle(const Cycle_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ExplicitDeallocate(const ExplicitDeallocate_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
            ASR::expr_t** current_expr_copy_12 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_vars[i]));
            self().call_replacer();
            current_expr = current_expr_copy_12;
            if( x.m_vars[i] )
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
            ASR::expr_t** current_expr_copy_13 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_vars[i]));
            self().call_replacer();
            current_expr = current_expr_copy_13;
            if( x.m_vars[i] )
            self().visit_expr(*x.m_vars[i]);
        }
    }
    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
        DoConcurrentLoop_t& xx = const_cast<DoConcurrentLoop_t&>(x);
        self().visit_do_loop_head(x.m_head);
        for (size_t i=0; i<x.n_shared; i++) {
            ASR::expr_t** current_expr_copy_14 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_shared[i]));
            self().call_replacer();
            current_expr = current_expr_copy_14;
            if( x.m_shared[i] )
            self().visit_expr(*x.m_shared[i]);
        }
        for (size_t i=0; i<x.n_local; i++) {
            ASR::expr_t** current_expr_copy_15 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_local[i]));
            self().call_replacer();
            current_expr = current_expr_copy_15;
            if( x.m_local[i] )
            self().visit_expr(*x.m_local[i]);
        }
        for (size_t i=0; i<x.n_reduction; i++) {
            self().visit_reduction_expr(x.m_reduction[i]);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_DoLoop(const DoLoop_t &x) {
        DoLoop_t& xx = const_cast<DoLoop_t&>(x);
        self().visit_do_loop_head(x.m_head);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_ErrorStop(const ErrorStop_t &x) {
        if (x.m_code) {
            ASR::expr_t** current_expr_copy_16 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_code));
            self().call_replacer();
            current_expr = current_expr_copy_16;
            if( x.m_code )
            self().visit_expr(*x.m_code);
        }
    }
    void visit_Exit(const Exit_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ForAllSingle(const ForAllSingle_t &x) {
        self().visit_do_loop_head(x.m_head);
        self().visit_stmt(*x.m_assign_stmt);
    }
    void visit_GoTo(const GoTo_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_GoToTarget(const GoToTarget_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_If(const If_t &x) {
        If_t& xx = const_cast<If_t&>(x);
        ASR::expr_t** current_expr_copy_17 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_17;
        if( x.m_test )
        self().visit_expr(*x.m_test);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_IfArithmetic(const IfArithmetic_t &x) {
        ASR::expr_t** current_expr_copy_18 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_18;
        if( x.m_test )
        self().visit_expr(*x.m_test);
    }
    void visit_Print(const Print_t &x) {
        for (size_t i=0; i<x.n_values; i++) {
            ASR::expr_t** current_expr_copy_19 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_values[i]));
            self().call_replacer();
            current_expr = current_expr_copy_19;
            if( x.m_values[i] )
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator) {
            ASR::expr_t** current_expr_copy_20 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_separator));
            self().call_replacer();
            current_expr = current_expr_copy_20;
            if( x.m_separator )
            self().visit_expr(*x.m_separator);
        }
        if (x.m_end) {
            ASR::expr_t** current_expr_copy_21 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_end));
            self().call_replacer();
            current_expr = current_expr_copy_21;
            if( x.m_end )
            self().visit_expr(*x.m_end);
        }
    }
    void visit_FileOpen(const FileOpen_t &x) {
        if (x.m_newunit) {
            ASR::expr_t** current_expr_copy_22 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_newunit));
            self().call_replacer();
            current_expr = current_expr_copy_22;
            if( x.m_newunit )
            self().visit_expr(*x.m_newunit);
        }
        if (x.m_filename) {
            ASR::expr_t** current_expr_copy_23 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_filename));
            self().call_replacer();
            current_expr = current_expr_copy_23;
            if( x.m_filename )
            self().visit_expr(*x.m_filename);
        }
        if (x.m_status) {
            ASR::expr_t** current_expr_copy_24 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_status));
            self().call_replacer();
            current_expr = current_expr_copy_24;
            if( x.m_status )
            self().visit_expr(*x.m_status);
        }
        if (x.m_form) {
            ASR::expr_t** current_expr_copy_25 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_form));
            self().call_replacer();
            current_expr = current_expr_copy_25;
            if( x.m_form )
            self().visit_expr(*x.m_form);
        }
    }
    void visit_FileClose(const FileClose_t &x) {
        if (x.m_unit) {
            ASR::expr_t** current_expr_copy_26 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
            self().call_replacer();
            current_expr = current_expr_copy_26;
            if( x.m_unit )
            self().visit_expr(*x.m_unit);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_27 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_27;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
        if (x.m_iomsg) {
            ASR::expr_t** current_expr_copy_28 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iomsg));
            self().call_replacer();
            current_expr = current_expr_copy_28;
            if( x.m_iomsg )
            self().visit_expr(*x.m_iomsg);
        }
        if (x.m_err) {
            ASR::expr_t** current_expr_copy_29 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_err));
            self().call_replacer();
            current_expr = current_expr_copy_29;
            if( x.m_err )
            self().visit_expr(*x.m_err);
        }
        if (x.m_status) {
            ASR::expr_t** current_expr_copy_30 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_status));
            self().call_replacer();
            current_expr = current_expr_copy_30;
            if( x.m_status )
            self().visit_expr(*x.m_status);
        }
    }
    void visit_FileRead(const FileRead_t &x) {
        if (x.m_unit) {
            ASR::expr_t** current_expr_copy_31 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
            self().call_replacer();
            current_expr = current_expr_copy_31;
            if( x.m_unit )
            self().visit_expr(*x.m_unit);
        }
        if (x.m_fmt) {
            ASR::expr_t** current_expr_copy_32 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_fmt));
            self().call_replacer();
            current_expr = current_expr_copy_32;
            if( x.m_fmt )
            self().visit_expr(*x.m_fmt);
        }
        if (x.m_iomsg) {
            ASR::expr_t** current_expr_copy_33 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iomsg));
            self().call_replacer();
            current_expr = current_expr_copy_33;
            if( x.m_iomsg )
            self().visit_expr(*x.m_iomsg);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_34 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_34;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
        if (x.m_size) {
            ASR::expr_t** current_expr_copy_35 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_size));
            self().call_replacer();
            current_expr = current_expr_copy_35;
            if( x.m_size )
            self().visit_expr(*x.m_size);
        }
        if (x.m_id) {
            ASR::expr_t** current_expr_copy_36 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_id));
            self().call_replacer();
            current_expr = current_expr_copy_36;
            if( x.m_id )
            self().visit_expr(*x.m_id);
        }
        for (size_t i=0; i<x.n_values; i++) {
            ASR::expr_t** current_expr_copy_37 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_values[i]));
            self().call_replacer();
            current_expr = current_expr_copy_37;
            if( x.m_values[i] )
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        }
    }
    void visit_FileBackspace(const FileBackspace_t &x) {
        if (x.m_unit) {
            ASR::expr_t** current_expr_copy_38 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
            self().call_replacer();
            current_expr = current_expr_copy_38;
            if( x.m_unit )
            self().visit_expr(*x.m_unit);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_39 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_39;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
        if (x.m_err) {
            ASR::expr_t** current_expr_copy_40 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_err));
            self().call_replacer();
            current_expr = current_expr_copy_40;
            if( x.m_err )
            self().visit_expr(*x.m_err);
        }
    }
    void visit_FileRewind(const FileRewind_t &x) {
        if (x.m_unit) {
            ASR::expr_t** current_expr_copy_41 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
            self().call_replacer();
            current_expr = current_expr_copy_41;
            if( x.m_unit )
            self().visit_expr(*x.m_unit);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_42 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_42;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
        if (x.m_err) {
            ASR::expr_t** current_expr_copy_43 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_err));
            self().call_replacer();
            current_expr = current_expr_copy_43;
            if( x.m_err )
            self().visit_expr(*x.m_err);
        }
    }
    void visit_FileInquire(const FileInquire_t &x) {
        if (x.m_unit) {
            ASR::expr_t** current_expr_copy_44 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
            self().call_replacer();
            current_expr = current_expr_copy_44;
            if( x.m_unit )
            self().visit_expr(*x.m_unit);
        }
        if (x.m_file) {
            ASR::expr_t** current_expr_copy_45 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_file));
            self().call_replacer();
            current_expr = current_expr_copy_45;
            if( x.m_file )
            self().visit_expr(*x.m_file);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_46 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_46;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
        if (x.m_err) {
            ASR::expr_t** current_expr_copy_47 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_err));
            self().call_replacer();
            current_expr = current_expr_copy_47;
            if( x.m_err )
            self().visit_expr(*x.m_err);
        }
        if (x.m_exist) {
            ASR::expr_t** current_expr_copy_48 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_exist));
            self().call_replacer();
            current_expr = current_expr_copy_48;
            if( x.m_exist )
            self().visit_expr(*x.m_exist);
        }
        if (x.m_opened) {
            ASR::expr_t** current_expr_copy_49 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_opened));
            self().call_replacer();
            current_expr = current_expr_copy_49;
            if( x.m_opened )
            self().visit_expr(*x.m_opened);
        }
        if (x.m_number) {
            ASR::expr_t** current_expr_copy_50 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_number));
            self().call_replacer();
            current_expr = current_expr_copy_50;
            if( x.m_number )
            self().visit_expr(*x.m_number);
        }
        if (x.m_named) {
            ASR::expr_t** current_expr_copy_51 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_named));
            self().call_replacer();
            current_expr = current_expr_copy_51;
            if( x.m_named )
            self().visit_expr(*x.m_named);
        }
        if (x.m_name) {
            ASR::expr_t** current_expr_copy_52 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_name));
            self().call_replacer();
            current_expr = current_expr_copy_52;
            if( x.m_name )
            self().visit_expr(*x.m_name);
        }
        if (x.m_access) {
            ASR::expr_t** current_expr_copy_53 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_access));
            self().call_replacer();
            current_expr = current_expr_copy_53;
            if( x.m_access )
            self().visit_expr(*x.m_access);
        }
        if (x.m_sequential) {
            ASR::expr_t** current_expr_copy_54 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_sequential));
            self().call_replacer();
            current_expr = current_expr_copy_54;
            if( x.m_sequential )
            self().visit_expr(*x.m_sequential);
        }
        if (x.m_direct) {
            ASR::expr_t** current_expr_copy_55 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_direct));
            self().call_replacer();
            current_expr = current_expr_copy_55;
            if( x.m_direct )
            self().visit_expr(*x.m_direct);
        }
        if (x.m_form) {
            ASR::expr_t** current_expr_copy_56 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_form));
            self().call_replacer();
            current_expr = current_expr_copy_56;
            if( x.m_form )
            self().visit_expr(*x.m_form);
        }
        if (x.m_formatted) {
            ASR::expr_t** current_expr_copy_57 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_formatted));
            self().call_replacer();
            current_expr = current_expr_copy_57;
            if( x.m_formatted )
            self().visit_expr(*x.m_formatted);
        }
        if (x.m_unformatted) {
            ASR::expr_t** current_expr_copy_58 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unformatted));
            self().call_replacer();
            current_expr = current_expr_copy_58;
            if( x.m_unformatted )
            self().visit_expr(*x.m_unformatted);
        }
        if (x.m_recl) {
            ASR::expr_t** current_expr_copy_59 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_recl));
            self().call_replacer();
            current_expr = current_expr_copy_59;
            if( x.m_recl )
            self().visit_expr(*x.m_recl);
        }
        if (x.m_nextrec) {
            ASR::expr_t** current_expr_copy_60 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_nextrec));
            self().call_replacer();
            current_expr = current_expr_copy_60;
            if( x.m_nextrec )
            self().visit_expr(*x.m_nextrec);
        }
        if (x.m_blank) {
            ASR::expr_t** current_expr_copy_61 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_blank));
            self().call_replacer();
            current_expr = current_expr_copy_61;
            if( x.m_blank )
            self().visit_expr(*x.m_blank);
        }
        if (x.m_position) {
            ASR::expr_t** current_expr_copy_62 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_position));
            self().call_replacer();
            current_expr = current_expr_copy_62;
            if( x.m_position )
            self().visit_expr(*x.m_position);
        }
        if (x.m_action) {
            ASR::expr_t** current_expr_copy_63 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_action));
            self().call_replacer();
            current_expr = current_expr_copy_63;
            if( x.m_action )
            self().visit_expr(*x.m_action);
        }
        if (x.m_read) {
            ASR::expr_t** current_expr_copy_64 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_read));
            self().call_replacer();
            current_expr = current_expr_copy_64;
            if( x.m_read )
            self().visit_expr(*x.m_read);
        }
        if (x.m_write) {
            ASR::expr_t** current_expr_copy_65 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_write));
            self().call_replacer();
            current_expr = current_expr_copy_65;
            if( x.m_write )
            self().visit_expr(*x.m_write);
        }
        if (x.m_readwrite) {
            ASR::expr_t** current_expr_copy_66 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_readwrite));
            self().call_replacer();
            current_expr = current_expr_copy_66;
            if( x.m_readwrite )
            self().visit_expr(*x.m_readwrite);
        }
        if (x.m_delim) {
            ASR::expr_t** current_expr_copy_67 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_delim));
            self().call_replacer();
            current_expr = current_expr_copy_67;
            if( x.m_delim )
            self().visit_expr(*x.m_delim);
        }
        if (x.m_pad) {
            ASR::expr_t** current_expr_copy_68 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_pad));
            self().call_replacer();
            current_expr = current_expr_copy_68;
            if( x.m_pad )
            self().visit_expr(*x.m_pad);
        }
        if (x.m_flen) {
            ASR::expr_t** current_expr_copy_69 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_flen));
            self().call_replacer();
            current_expr = current_expr_copy_69;
            if( x.m_flen )
            self().visit_expr(*x.m_flen);
        }
        if (x.m_blocksize) {
            ASR::expr_t** current_expr_copy_70 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_blocksize));
            self().call_replacer();
            current_expr = current_expr_copy_70;
            if( x.m_blocksize )
            self().visit_expr(*x.m_blocksize);
        }
        if (x.m_convert) {
            ASR::expr_t** current_expr_copy_71 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_convert));
            self().call_replacer();
            current_expr = current_expr_copy_71;
            if( x.m_convert )
            self().visit_expr(*x.m_convert);
        }
        if (x.m_carriagecontrol) {
            ASR::expr_t** current_expr_copy_72 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_carriagecontrol));
            self().call_replacer();
            current_expr = current_expr_copy_72;
            if( x.m_carriagecontrol )
            self().visit_expr(*x.m_carriagecontrol);
        }
        if (x.m_iolength) {
            ASR::expr_t** current_expr_copy_73 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iolength));
            self().call_replacer();
            current_expr = current_expr_copy_73;
            if( x.m_iolength )
            self().visit_expr(*x.m_iolength);
        }
    }
    void visit_FileWrite(const FileWrite_t &x) {
        if (x.m_unit) {
            ASR::expr_t** current_expr_copy_74 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
            self().call_replacer();
            current_expr = current_expr_copy_74;
            if( x.m_unit )
            self().visit_expr(*x.m_unit);
        }
        if (x.m_iomsg) {
            ASR::expr_t** current_expr_copy_75 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iomsg));
            self().call_replacer();
            current_expr = current_expr_copy_75;
            if( x.m_iomsg )
            self().visit_expr(*x.m_iomsg);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_76 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_76;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
        if (x.m_id) {
            ASR::expr_t** current_expr_copy_77 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_id));
            self().call_replacer();
            current_expr = current_expr_copy_77;
            if( x.m_id )
            self().visit_expr(*x.m_id);
        }
        for (size_t i=0; i<x.n_values; i++) {
            ASR::expr_t** current_expr_copy_78 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_values[i]));
            self().call_replacer();
            current_expr = current_expr_copy_78;
            if( x.m_values[i] )
            self().visit_expr(*x.m_values[i]);
        }
        if (x.m_separator) {
            ASR::expr_t** current_expr_copy_79 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_separator));
            self().call_replacer();
            current_expr = current_expr_copy_79;
            if( x.m_separator )
            self().visit_expr(*x.m_separator);
        }
        if (x.m_end) {
            ASR::expr_t** current_expr_copy_80 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_end));
            self().call_replacer();
            current_expr = current_expr_copy_80;
            if( x.m_end )
            self().visit_expr(*x.m_end);
        }
        if (x.m_overloaded) {
            self().visit_stmt(*x.m_overloaded);
        }
    }
    void visit_Return(const Return_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Select(const Select_t &x) {
        Select_t& xx = const_cast<Select_t&>(x);
        ASR::expr_t** current_expr_copy_81 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_81;
        if( x.m_test )
        self().visit_expr(*x.m_test);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_case_stmt(*x.m_body[i]);
        }
        self().transform_stmts(xx.m_default, xx.n_default);
    }
    void visit_Stop(const Stop_t &x) {
        if (x.m_code) {
            ASR::expr_t** current_expr_copy_82 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_code));
            self().call_replacer();
            current_expr = current_expr_copy_82;
            if( x.m_code )
            self().visit_expr(*x.m_code);
        }
    }
    void visit_Assert(const Assert_t &x) {
        ASR::expr_t** current_expr_copy_83 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_83;
        if( x.m_test )
        self().visit_expr(*x.m_test);
        if (x.m_msg) {
            ASR::expr_t** current_expr_copy_84 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_msg));
            self().call_replacer();
            current_expr = current_expr_copy_84;
            if( x.m_msg )
            self().visit_expr(*x.m_msg);
        }
    }
    void visit_SubroutineCall(const SubroutineCall_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        if (x.m_dt) {
            ASR::expr_t** current_expr_copy_85 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_dt));
            self().call_replacer();
            current_expr = current_expr_copy_85;
            if( x.m_dt )
            self().visit_expr(*x.m_dt);
        }
    }
    void visit_IntrinsicImpureSubroutine(const IntrinsicImpureSubroutine_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_86 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_86;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
    }
    void visit_Where(const Where_t &x) {
        Where_t& xx = const_cast<Where_t&>(x);
        ASR::expr_t** current_expr_copy_87 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_87;
        if( x.m_test )
        self().visit_expr(*x.m_test);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_WhileLoop(const WhileLoop_t &x) {
        WhileLoop_t& xx = const_cast<WhileLoop_t&>(x);
        ASR::expr_t** current_expr_copy_88 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_88;
        if( x.m_test )
        self().visit_expr(*x.m_test);
        self().transform_stmts(xx.m_body, xx.n_body);
        self().transform_stmts(xx.m_orelse, xx.n_orelse);
    }
    void visit_Nullify(const Nullify_t &x) {
        for (size_t i=0; i<x.n_vars; i++) {
        }
    }
    void visit_Flush(const Flush_t &x) {
        ASR::expr_t** current_expr_copy_89 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_unit));
        self().call_replacer();
        current_expr = current_expr_copy_89;
        if( x.m_unit )
        self().visit_expr(*x.m_unit);
        if (x.m_err) {
            ASR::expr_t** current_expr_copy_90 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_err));
            self().call_replacer();
            current_expr = current_expr_copy_90;
            if( x.m_err )
            self().visit_expr(*x.m_err);
        }
        if (x.m_iomsg) {
            ASR::expr_t** current_expr_copy_91 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iomsg));
            self().call_replacer();
            current_expr = current_expr_copy_91;
            if( x.m_iomsg )
            self().visit_expr(*x.m_iomsg);
        }
        if (x.m_iostat) {
            ASR::expr_t** current_expr_copy_92 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_iostat));
            self().call_replacer();
            current_expr = current_expr_copy_92;
            if( x.m_iostat )
            self().visit_expr(*x.m_iostat);
        }
    }
    void visit_ListAppend(const ListAppend_t &x) {
        ASR::expr_t** current_expr_copy_93 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_93;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_94 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ele));
        self().call_replacer();
        current_expr = current_expr_copy_94;
        if( x.m_ele )
        self().visit_expr(*x.m_ele);
    }
    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SelectType(const SelectType_t &x) {
        SelectType_t& xx = const_cast<SelectType_t&>(x);
        ASR::expr_t** current_expr_copy_95 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_selector));
        self().call_replacer();
        current_expr = current_expr_copy_95;
        if( x.m_selector )
        self().visit_expr(*x.m_selector);
        for (size_t i=0; i<x.n_body; i++) {
            self().visit_type_stmt(*x.m_body[i]);
        }
        self().transform_stmts(xx.m_default, xx.n_default);
    }
    void visit_CPtrToPointer(const CPtrToPointer_t &x) {
        ASR::expr_t** current_expr_copy_96 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_cptr));
        self().call_replacer();
        current_expr = current_expr_copy_96;
        if( x.m_cptr )
        self().visit_expr(*x.m_cptr);
        ASR::expr_t** current_expr_copy_97 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ptr));
        self().call_replacer();
        current_expr = current_expr_copy_97;
        if( x.m_ptr )
        self().visit_expr(*x.m_ptr);
        if (x.m_shape) {
            ASR::expr_t** current_expr_copy_98 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_shape));
            self().call_replacer();
            current_expr = current_expr_copy_98;
            if( x.m_shape )
            self().visit_expr(*x.m_shape);
        }
        if (x.m_lower_bounds) {
            ASR::expr_t** current_expr_copy_99 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_lower_bounds));
            self().call_replacer();
            current_expr = current_expr_copy_99;
            if( x.m_lower_bounds )
            self().visit_expr(*x.m_lower_bounds);
        }
    }
    void visit_BlockCall(const BlockCall_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SetInsert(const SetInsert_t &x) {
        ASR::expr_t** current_expr_copy_100 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_100;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_101 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ele));
        self().call_replacer();
        current_expr = current_expr_copy_101;
        if( x.m_ele )
        self().visit_expr(*x.m_ele);
    }
    void visit_SetRemove(const SetRemove_t &x) {
        ASR::expr_t** current_expr_copy_102 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_102;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_103 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ele));
        self().call_replacer();
        current_expr = current_expr_copy_103;
        if( x.m_ele )
        self().visit_expr(*x.m_ele);
    }
    void visit_ListInsert(const ListInsert_t &x) {
        ASR::expr_t** current_expr_copy_104 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_104;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_105 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_pos));
        self().call_replacer();
        current_expr = current_expr_copy_105;
        if( x.m_pos )
        self().visit_expr(*x.m_pos);
        ASR::expr_t** current_expr_copy_106 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ele));
        self().call_replacer();
        current_expr = current_expr_copy_106;
        if( x.m_ele )
        self().visit_expr(*x.m_ele);
    }
    void visit_ListRemove(const ListRemove_t &x) {
        ASR::expr_t** current_expr_copy_107 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_107;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_108 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ele));
        self().call_replacer();
        current_expr = current_expr_copy_108;
        if( x.m_ele )
        self().visit_expr(*x.m_ele);
    }
    void visit_ListClear(const ListClear_t &x) {
        ASR::expr_t** current_expr_copy_109 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_109;
        if( x.m_a )
        self().visit_expr(*x.m_a);
    }
    void visit_DictInsert(const DictInsert_t &x) {
        ASR::expr_t** current_expr_copy_110 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_110;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_111 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_key));
        self().call_replacer();
        current_expr = current_expr_copy_111;
        if( x.m_key )
        self().visit_expr(*x.m_key);
        ASR::expr_t** current_expr_copy_112 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
        self().call_replacer();
        current_expr = current_expr_copy_112;
        if( x.m_value )
        self().visit_expr(*x.m_value);
    }
    void visit_Expr(const Expr_t &x) {
        ASR::expr_t** current_expr_copy_113 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_expression));
        self().call_replacer();
        current_expr = current_expr_copy_113;
        if( x.m_expression )
        self().visit_expr(*x.m_expression);
    }
    void visit_IfExp(const IfExp_t &x) {
        ASR::expr_t** current_expr_copy_114 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_test));
        self().call_replacer();
        current_expr = current_expr_copy_114;
        if( x.m_test )
        self().visit_expr(*x.m_test);
        ASR::expr_t** current_expr_copy_115 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_body));
        self().call_replacer();
        current_expr = current_expr_copy_115;
        if( x.m_body )
        self().visit_expr(*x.m_body);
        ASR::expr_t** current_expr_copy_116 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_orelse));
        self().call_replacer();
        current_expr = current_expr_copy_116;
        if( x.m_orelse )
        self().visit_expr(*x.m_orelse);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_117 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_117;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ComplexConstructor(const ComplexConstructor_t &x) {
        ASR::expr_t** current_expr_copy_118 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_re));
        self().call_replacer();
        current_expr = current_expr_copy_118;
        if( x.m_re )
        self().visit_expr(*x.m_re);
        ASR::expr_t** current_expr_copy_119 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_im));
        self().call_replacer();
        current_expr = current_expr_copy_119;
        if( x.m_im )
        self().visit_expr(*x.m_im);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_120 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_120;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_NamedExpr(const NamedExpr_t &x) {
        ASR::expr_t** current_expr_copy_121 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_target));
        self().call_replacer();
        current_expr = current_expr_copy_121;
        if( x.m_target )
        self().visit_expr(*x.m_target);
        ASR::expr_t** current_expr_copy_122 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
        self().call_replacer();
        current_expr = current_expr_copy_122;
        if( x.m_value )
        self().visit_expr(*x.m_value);
        self().visit_ttype(*x.m_type);
    }
    void visit_FunctionCall(const FunctionCall_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_123 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_123;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
        if (x.m_dt) {
            ASR::expr_t** current_expr_copy_124 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_dt));
            self().call_replacer();
            current_expr = current_expr_copy_124;
            if( x.m_dt )
            self().visit_expr(*x.m_dt);
        }
    }
    void visit_IntrinsicElementalFunction(const IntrinsicElementalFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_125 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_125;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        }
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_126 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_126;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntrinsicArrayFunction(const IntrinsicArrayFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_127 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_127;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        }
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_128 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_128;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntrinsicImpureFunction(const IntrinsicImpureFunction_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_129 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_129;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        }
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_130 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_130;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_TypeInquiry(const TypeInquiry_t &x) {
        self().visit_ttype(*x.m_arg_type);
        if (x.m_arg) {
            ASR::expr_t** current_expr_copy_131 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
            self().call_replacer();
            current_expr = current_expr_copy_131;
            if( x.m_arg )
            self().visit_expr(*x.m_arg);
        }
        self().visit_ttype(*x.m_type);
        ASR::expr_t** current_expr_copy_132 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
        self().call_replacer();
        current_expr = current_expr_copy_132;
        if( x.m_value )
        self().visit_expr(*x.m_value);
    }
    void visit_StructConstructor(const StructConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_call_arg(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_133 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_133;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_EnumConstructor(const EnumConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_134 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_134;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_135 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_135;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_UnionTypeConstructor(const UnionTypeConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_136 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_136;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_137 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_137;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ImpliedDoLoop(const ImpliedDoLoop_t &x) {
        for (size_t i=0; i<x.n_values; i++) {
            ASR::expr_t** current_expr_copy_138 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_values[i]));
            self().call_replacer();
            current_expr = current_expr_copy_138;
            if( x.m_values[i] )
            self().visit_expr(*x.m_values[i]);
        }
        ASR::expr_t** current_expr_copy_139 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_var));
        self().call_replacer();
        current_expr = current_expr_copy_139;
        if( x.m_var )
        self().visit_expr(*x.m_var);
        ASR::expr_t** current_expr_copy_140 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_start));
        self().call_replacer();
        current_expr = current_expr_copy_140;
        if( x.m_start )
        self().visit_expr(*x.m_start);
        ASR::expr_t** current_expr_copy_141 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_end));
        self().call_replacer();
        current_expr = current_expr_copy_141;
        if( x.m_end )
        self().visit_expr(*x.m_end);
        if (x.m_increment) {
            ASR::expr_t** current_expr_copy_142 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_increment));
            self().call_replacer();
            current_expr = current_expr_copy_142;
            if( x.m_increment )
            self().visit_expr(*x.m_increment);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_143 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_143;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntegerConstant(const IntegerConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_IntegerBOZ(const IntegerBOZ_t &x) {
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        }
    }
    void visit_IntegerBitNot(const IntegerBitNot_t &x) {
        ASR::expr_t** current_expr_copy_144 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_144;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_145 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_145;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntegerUnaryMinus(const IntegerUnaryMinus_t &x) {
        ASR::expr_t** current_expr_copy_146 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_146;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_147 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_147;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntegerCompare(const IntegerCompare_t &x) {
        ASR::expr_t** current_expr_copy_148 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_148;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_149 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_149;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_150 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_150;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntegerBinOp(const IntegerBinOp_t &x) {
        ASR::expr_t** current_expr_copy_151 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_151;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_152 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_152;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_153 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_153;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_UnsignedIntegerConstant(const UnsignedIntegerConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_UnsignedIntegerUnaryMinus(const UnsignedIntegerUnaryMinus_t &x) {
        ASR::expr_t** current_expr_copy_154 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_154;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_155 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_155;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_UnsignedIntegerBitNot(const UnsignedIntegerBitNot_t &x) {
        ASR::expr_t** current_expr_copy_156 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_156;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_157 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_157;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_UnsignedIntegerCompare(const UnsignedIntegerCompare_t &x) {
        ASR::expr_t** current_expr_copy_158 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_158;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_159 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_159;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_160 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_160;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_UnsignedIntegerBinOp(const UnsignedIntegerBinOp_t &x) {
        ASR::expr_t** current_expr_copy_161 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_161;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_162 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_162;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_163 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_163;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_RealConstant(const RealConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_RealUnaryMinus(const RealUnaryMinus_t &x) {
        ASR::expr_t** current_expr_copy_164 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_164;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_165 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_165;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_RealCompare(const RealCompare_t &x) {
        ASR::expr_t** current_expr_copy_166 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_166;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_167 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_167;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_168 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_168;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_RealBinOp(const RealBinOp_t &x) {
        ASR::expr_t** current_expr_copy_169 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_169;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_170 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_170;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_171 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_171;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_RealCopySign(const RealCopySign_t &x) {
        ASR::expr_t** current_expr_copy_172 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_target));
        self().call_replacer();
        current_expr = current_expr_copy_172;
        if( x.m_target )
        self().visit_expr(*x.m_target);
        ASR::expr_t** current_expr_copy_173 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_source));
        self().call_replacer();
        current_expr = current_expr_copy_173;
        if( x.m_source )
        self().visit_expr(*x.m_source);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_174 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_174;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ComplexConstant(const ComplexConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_ComplexUnaryMinus(const ComplexUnaryMinus_t &x) {
        ASR::expr_t** current_expr_copy_175 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_175;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_176 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_176;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ComplexCompare(const ComplexCompare_t &x) {
        ASR::expr_t** current_expr_copy_177 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_177;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_178 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_178;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_179 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_179;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ComplexBinOp(const ComplexBinOp_t &x) {
        ASR::expr_t** current_expr_copy_180 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_180;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_181 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_181;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_182 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_182;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_LogicalConstant(const LogicalConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_LogicalNot(const LogicalNot_t &x) {
        ASR::expr_t** current_expr_copy_183 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_183;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_184 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_184;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_LogicalCompare(const LogicalCompare_t &x) {
        ASR::expr_t** current_expr_copy_185 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_185;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_186 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_186;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_187 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_187;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_LogicalBinOp(const LogicalBinOp_t &x) {
        ASR::expr_t** current_expr_copy_188 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_188;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_189 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_189;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_190 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_190;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListConstant(const ListConstant_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_191 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_191;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_ListLen(const ListLen_t &x) {
        ASR::expr_t** current_expr_copy_192 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_192;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_193 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_193;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListConcat(const ListConcat_t &x) {
        ASR::expr_t** current_expr_copy_194 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_194;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_195 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_195;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_196 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_196;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListCompare(const ListCompare_t &x) {
        ASR::expr_t** current_expr_copy_197 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_197;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_198 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_198;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_199 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_199;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListCount(const ListCount_t &x) {
        ASR::expr_t** current_expr_copy_200 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_200;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        ASR::expr_t** current_expr_copy_201 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ele));
        self().call_replacer();
        current_expr = current_expr_copy_201;
        if( x.m_ele )
        self().visit_expr(*x.m_ele);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_202 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_202;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_SetConstant(const SetConstant_t &x) {
        for (size_t i=0; i<x.n_elements; i++) {
            ASR::expr_t** current_expr_copy_203 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_elements[i]));
            self().call_replacer();
            current_expr = current_expr_copy_203;
            if( x.m_elements[i] )
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_SetLen(const SetLen_t &x) {
        ASR::expr_t** current_expr_copy_204 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_204;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_205 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_205;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_TupleConstant(const TupleConstant_t &x) {
        for (size_t i=0; i<x.n_elements; i++) {
            ASR::expr_t** current_expr_copy_206 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_elements[i]));
            self().call_replacer();
            current_expr = current_expr_copy_206;
            if( x.m_elements[i] )
            self().visit_expr(*x.m_elements[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_TupleLen(const TupleLen_t &x) {
        ASR::expr_t** current_expr_copy_207 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_207;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        ASR::expr_t** current_expr_copy_208 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
        self().call_replacer();
        current_expr = current_expr_copy_208;
        if( x.m_value )
        self().visit_expr(*x.m_value);
    }
    void visit_TupleCompare(const TupleCompare_t &x) {
        ASR::expr_t** current_expr_copy_209 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_209;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_210 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_210;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_211 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_211;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_TupleConcat(const TupleConcat_t &x) {
        ASR::expr_t** current_expr_copy_212 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_212;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_213 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_213;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_214 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_214;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringConstant(const StringConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_StringConcat(const StringConcat_t &x) {
        ASR::expr_t** current_expr_copy_215 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_215;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_216 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_216;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_217 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_217;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringRepeat(const StringRepeat_t &x) {
        ASR::expr_t** current_expr_copy_218 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_218;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_219 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_219;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_220 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_220;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringLen(const StringLen_t &x) {
        ASR::expr_t** current_expr_copy_221 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_221;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_222 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_222;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringItem(const StringItem_t &x) {
        ASR::expr_t** current_expr_copy_223 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_223;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        ASR::expr_t** current_expr_copy_224 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_idx));
        self().call_replacer();
        current_expr = current_expr_copy_224;
        if( x.m_idx )
        self().visit_expr(*x.m_idx);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_225 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_225;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringSection(const StringSection_t &x) {
        ASR::expr_t** current_expr_copy_226 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_226;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        if (x.m_start) {
            ASR::expr_t** current_expr_copy_227 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_start));
            self().call_replacer();
            current_expr = current_expr_copy_227;
            if( x.m_start )
            self().visit_expr(*x.m_start);
        }
        if (x.m_end) {
            ASR::expr_t** current_expr_copy_228 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_end));
            self().call_replacer();
            current_expr = current_expr_copy_228;
            if( x.m_end )
            self().visit_expr(*x.m_end);
        }
        if (x.m_step) {
            ASR::expr_t** current_expr_copy_229 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_step));
            self().call_replacer();
            current_expr = current_expr_copy_229;
            if( x.m_step )
            self().visit_expr(*x.m_step);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_230 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_230;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringCompare(const StringCompare_t &x) {
        ASR::expr_t** current_expr_copy_231 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_231;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_232 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_232;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_233 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_233;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringOrd(const StringOrd_t &x) {
        ASR::expr_t** current_expr_copy_234 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_234;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_235 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_235;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringChr(const StringChr_t &x) {
        ASR::expr_t** current_expr_copy_236 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_236;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_237 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_237;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StringFormat(const StringFormat_t &x) {
        ASR::expr_t** current_expr_copy_238 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_fmt));
        self().call_replacer();
        current_expr = current_expr_copy_238;
        if( x.m_fmt )
        self().visit_expr(*x.m_fmt);
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_239 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_239;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_240 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_240;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_CPtrCompare(const CPtrCompare_t &x) {
        ASR::expr_t** current_expr_copy_241 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_241;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_242 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_242;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_243 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_243;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_SymbolicCompare(const SymbolicCompare_t &x) {
        ASR::expr_t** current_expr_copy_244 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_244;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_245 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_245;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_246 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_246;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_DictConstant(const DictConstant_t &x) {
        for (size_t i=0; i<x.n_keys; i++) {
            ASR::expr_t** current_expr_copy_247 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_keys[i]));
            self().call_replacer();
            current_expr = current_expr_copy_247;
            if( x.m_keys[i] )
            self().visit_expr(*x.m_keys[i]);
        }
        for (size_t i=0; i<x.n_values; i++) {
            ASR::expr_t** current_expr_copy_248 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_values[i]));
            self().call_replacer();
            current_expr = current_expr_copy_248;
            if( x.m_values[i] )
            self().visit_expr(*x.m_values[i]);
        }
        self().visit_ttype(*x.m_type);
    }
    void visit_DictLen(const DictLen_t &x) {
        ASR::expr_t** current_expr_copy_249 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_249;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_250 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_250;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_Var(const Var_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_FunctionParam(const FunctionParam_t &x) {
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_251 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_251;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayConstructor(const ArrayConstructor_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            ASR::expr_t** current_expr_copy_252 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_args[i]));
            self().call_replacer();
            current_expr = current_expr_copy_252;
            if( x.m_args[i] )
            self().visit_expr(*x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_253 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_253;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayConstant(const ArrayConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_ArrayItem(const ArrayItem_t &x) {
        ASR::expr_t** current_expr_copy_254 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_254;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_255 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_255;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArraySection(const ArraySection_t &x) {
        ASR::expr_t** current_expr_copy_256 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_256;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_array_index(x.m_args[i]);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_257 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_257;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArraySize(const ArraySize_t &x) {
        ASR::expr_t** current_expr_copy_258 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_258;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        if (x.m_dim) {
            ASR::expr_t** current_expr_copy_259 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_dim));
            self().call_replacer();
            current_expr = current_expr_copy_259;
            if( x.m_dim )
            self().visit_expr(*x.m_dim);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_260 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_260;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayBound(const ArrayBound_t &x) {
        ASR::expr_t** current_expr_copy_261 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_261;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        if (x.m_dim) {
            ASR::expr_t** current_expr_copy_262 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_dim));
            self().call_replacer();
            current_expr = current_expr_copy_262;
            if( x.m_dim )
            self().visit_expr(*x.m_dim);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_263 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_263;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayTranspose(const ArrayTranspose_t &x) {
        ASR::expr_t** current_expr_copy_264 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_matrix));
        self().call_replacer();
        current_expr = current_expr_copy_264;
        if( x.m_matrix )
        self().visit_expr(*x.m_matrix);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_265 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_265;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayPack(const ArrayPack_t &x) {
        ASR::expr_t** current_expr_copy_266 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_array));
        self().call_replacer();
        current_expr = current_expr_copy_266;
        if( x.m_array )
        self().visit_expr(*x.m_array);
        ASR::expr_t** current_expr_copy_267 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_mask));
        self().call_replacer();
        current_expr = current_expr_copy_267;
        if( x.m_mask )
        self().visit_expr(*x.m_mask);
        if (x.m_vector) {
            ASR::expr_t** current_expr_copy_268 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_vector));
            self().call_replacer();
            current_expr = current_expr_copy_268;
            if( x.m_vector )
            self().visit_expr(*x.m_vector);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_269 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_269;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayReshape(const ArrayReshape_t &x) {
        ASR::expr_t** current_expr_copy_270 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_array));
        self().call_replacer();
        current_expr = current_expr_copy_270;
        if( x.m_array )
        self().visit_expr(*x.m_array);
        ASR::expr_t** current_expr_copy_271 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_shape));
        self().call_replacer();
        current_expr = current_expr_copy_271;
        if( x.m_shape )
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_272 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_272;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayBroadcast(const ArrayBroadcast_t &x) {
        ASR::expr_t** current_expr_copy_273 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_array));
        self().call_replacer();
        current_expr = current_expr_copy_273;
        if( x.m_array )
        self().visit_expr(*x.m_array);
        ASR::expr_t** current_expr_copy_274 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_shape));
        self().call_replacer();
        current_expr = current_expr_copy_274;
        if( x.m_shape )
        self().visit_expr(*x.m_shape);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_275 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_275;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_BitCast(const BitCast_t &x) {
        ASR::expr_t** current_expr_copy_276 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_source));
        self().call_replacer();
        current_expr = current_expr_copy_276;
        if( x.m_source )
        self().visit_expr(*x.m_source);
        ASR::expr_t** current_expr_copy_277 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_mold));
        self().call_replacer();
        current_expr = current_expr_copy_277;
        if( x.m_mold )
        self().visit_expr(*x.m_mold);
        if (x.m_size) {
            ASR::expr_t** current_expr_copy_278 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_size));
            self().call_replacer();
            current_expr = current_expr_copy_278;
            if( x.m_size )
            self().visit_expr(*x.m_size);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_279 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_279;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StructInstanceMember(const StructInstanceMember_t &x) {
        ASR::expr_t** current_expr_copy_280 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_280;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_281 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_281;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_StructStaticMember(const StructStaticMember_t &x) {
        ASR::expr_t** current_expr_copy_282 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_282;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_283 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_283;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_EnumStaticMember(const EnumStaticMember_t &x) {
        ASR::expr_t** current_expr_copy_284 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_284;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_285 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_285;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_UnionInstanceMember(const UnionInstanceMember_t &x) {
        ASR::expr_t** current_expr_copy_286 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_286;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_287 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_287;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_EnumName(const EnumName_t &x) {
        ASR::expr_t** current_expr_copy_288 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_288;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_289 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_289;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_EnumValue(const EnumValue_t &x) {
        ASR::expr_t** current_expr_copy_290 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
        self().call_replacer();
        current_expr = current_expr_copy_290;
        if( x.m_v )
        self().visit_expr(*x.m_v);
        self().visit_ttype(*x.m_enum_type);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_291 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_291;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_OverloadedCompare(const OverloadedCompare_t &x) {
        ASR::expr_t** current_expr_copy_292 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_292;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_293 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_293;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_294 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_294;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
        ASR::expr_t** current_expr_copy_295 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_overloaded));
        self().call_replacer();
        current_expr = current_expr_copy_295;
        if( x.m_overloaded )
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedBinOp(const OverloadedBinOp_t &x) {
        ASR::expr_t** current_expr_copy_296 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_296;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_297 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_297;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_298 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_298;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
        ASR::expr_t** current_expr_copy_299 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_overloaded));
        self().call_replacer();
        current_expr = current_expr_copy_299;
        if( x.m_overloaded )
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedUnaryMinus(const OverloadedUnaryMinus_t &x) {
        ASR::expr_t** current_expr_copy_300 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_300;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_301 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_301;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
        ASR::expr_t** current_expr_copy_302 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_overloaded));
        self().call_replacer();
        current_expr = current_expr_copy_302;
        if( x.m_overloaded )
        self().visit_expr(*x.m_overloaded);
    }
    void visit_OverloadedStringConcat(const OverloadedStringConcat_t &x) {
        ASR::expr_t** current_expr_copy_303 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_303;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_304 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_304;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_305 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_305;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
        ASR::expr_t** current_expr_copy_306 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_overloaded));
        self().call_replacer();
        current_expr = current_expr_copy_306;
        if( x.m_overloaded )
        self().visit_expr(*x.m_overloaded);
    }
    void visit_Cast(const Cast_t &x) {
        ASR::expr_t** current_expr_copy_307 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_307;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_308 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_308;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ArrayPhysicalCast(const ArrayPhysicalCast_t &x) {
        ASR::expr_t** current_expr_copy_309 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_309;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_310 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_310;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ComplexRe(const ComplexRe_t &x) {
        ASR::expr_t** current_expr_copy_311 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_311;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_312 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_312;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ComplexIm(const ComplexIm_t &x) {
        ASR::expr_t** current_expr_copy_313 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_313;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_314 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_314;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_DictItem(const DictItem_t &x) {
        ASR::expr_t** current_expr_copy_315 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_315;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_316 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_key));
        self().call_replacer();
        current_expr = current_expr_copy_316;
        if( x.m_key )
        self().visit_expr(*x.m_key);
        if (x.m_default) {
            ASR::expr_t** current_expr_copy_317 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_default));
            self().call_replacer();
            current_expr = current_expr_copy_317;
            if( x.m_default )
            self().visit_expr(*x.m_default);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_318 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_318;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_CLoc(const CLoc_t &x) {
        ASR::expr_t** current_expr_copy_319 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_319;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_320 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_320;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_PointerToCPtr(const PointerToCPtr_t &x) {
        ASR::expr_t** current_expr_copy_321 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_321;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_322 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_322;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_GetPointer(const GetPointer_t &x) {
        ASR::expr_t** current_expr_copy_323 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_323;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_324 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_324;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListItem(const ListItem_t &x) {
        ASR::expr_t** current_expr_copy_325 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_325;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_326 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_pos));
        self().call_replacer();
        current_expr = current_expr_copy_326;
        if( x.m_pos )
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_327 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_327;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_TupleItem(const TupleItem_t &x) {
        ASR::expr_t** current_expr_copy_328 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_328;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_329 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_pos));
        self().call_replacer();
        current_expr = current_expr_copy_329;
        if( x.m_pos )
        self().visit_expr(*x.m_pos);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_330 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_330;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListSection(const ListSection_t &x) {
        ASR::expr_t** current_expr_copy_331 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_331;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        self().visit_array_index(x.m_section);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_332 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_332;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_ListRepeat(const ListRepeat_t &x) {
        ASR::expr_t** current_expr_copy_333 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
        self().call_replacer();
        current_expr = current_expr_copy_333;
        if( x.m_left )
        self().visit_expr(*x.m_left);
        ASR::expr_t** current_expr_copy_334 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
        self().call_replacer();
        current_expr = current_expr_copy_334;
        if( x.m_right )
        self().visit_expr(*x.m_right);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_335 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_335;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_DictPop(const DictPop_t &x) {
        ASR::expr_t** current_expr_copy_336 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_336;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        ASR::expr_t** current_expr_copy_337 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_key));
        self().call_replacer();
        current_expr = current_expr_copy_337;
        if( x.m_key )
        self().visit_expr(*x.m_key);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_338 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_338;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_SetPop(const SetPop_t &x) {
        ASR::expr_t** current_expr_copy_339 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_339;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_340 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_340;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_IntegerBitLen(const IntegerBitLen_t &x) {
        ASR::expr_t** current_expr_copy_341 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_341;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_342 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_342;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_Ichar(const Ichar_t &x) {
        ASR::expr_t** current_expr_copy_343 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_343;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_344 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_344;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_Iachar(const Iachar_t &x) {
        ASR::expr_t** current_expr_copy_345 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_345;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_346 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_346;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_SizeOfType(const SizeOfType_t &x) {
        self().visit_ttype(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_347 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_347;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_PointerNullConstant(const PointerNullConstant_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_PointerAssociated(const PointerAssociated_t &x) {
        ASR::expr_t** current_expr_copy_348 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_ptr));
        self().call_replacer();
        current_expr = current_expr_copy_348;
        if( x.m_ptr )
        self().visit_expr(*x.m_ptr);
        if (x.m_tgt) {
            ASR::expr_t** current_expr_copy_349 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_tgt));
            self().call_replacer();
            current_expr = current_expr_copy_349;
            if( x.m_tgt )
            self().visit_expr(*x.m_tgt);
        }
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_350 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_350;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_RealSqrt(const RealSqrt_t &x) {
        ASR::expr_t** current_expr_copy_351 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_351;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
        self().visit_ttype(*x.m_type);
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_352 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_352;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_Integer(const Integer_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_UnsignedInteger(const UnsignedInteger_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Real(const Real_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Complex(const Complex_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Character(const Character_t &x) {
        if (x.m_len_expr) {
            ASR::expr_t** current_expr_copy_353 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_len_expr));
            self().call_replacer();
            current_expr = current_expr_copy_353;
            if( x.m_len_expr )
            self().visit_expr(*x.m_len_expr);
        }
    }
    void visit_Logical(const Logical_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Set(const Set_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_List(const List_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_Tuple(const Tuple_t &x) {
        for (size_t i=0; i<x.n_type; i++) {
            self().visit_ttype(*x.m_type[i]);
        }
    }
    void visit_StructType(const StructType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_EnumType(const EnumType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Union(const Union_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassType(const ClassType_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Dict(const Dict_t &x) {
        self().visit_ttype(*x.m_key_type);
        self().visit_ttype(*x.m_value_type);
    }
    void visit_Pointer(const Pointer_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_Allocatable(const Allocatable_t &x) {
        self().visit_ttype(*x.m_type);
    }
    void visit_CPtr(const CPtr_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_SymbolicExpression(const SymbolicExpression_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeParameter(const TypeParameter_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_Array(const Array_t &x) {
        self().visit_ttype(*x.m_type);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
    }
    void visit_FunctionType(const FunctionType_t &x) {
        for (size_t i=0; i<x.n_arg_types; i++) {
            self().visit_ttype(*x.m_arg_types[i]);
        }
        if (x.m_return_var_type) {
            self().visit_ttype(*x.m_return_var_type);
        }
        for (size_t i=0; i<x.n_restrictions; i++) {
        }
    }
    void visit_dimension(const dimension_t &x) {
        if (x.m_start) {
            ASR::expr_t** current_expr_copy_354 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_start));
            self().call_replacer();
            current_expr = current_expr_copy_354;
            if( x.m_start )
            self().visit_expr(*x.m_start);
        }
        if (x.m_length) {
            ASR::expr_t** current_expr_copy_355 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_length));
            self().call_replacer();
            current_expr = current_expr_copy_355;
            if( x.m_length )
            self().visit_expr(*x.m_length);
        }
    }
    void visit_alloc_arg(const alloc_arg_t &x) {
        ASR::expr_t** current_expr_copy_356 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_a));
        self().call_replacer();
        current_expr = current_expr_copy_356;
        if( x.m_a )
        self().visit_expr(*x.m_a);
        for (size_t i=0; i<x.n_dims; i++) {
            self().visit_dimension(x.m_dims[i]);
        }
        if (x.m_len_expr) {
            ASR::expr_t** current_expr_copy_357 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_len_expr));
            self().call_replacer();
            current_expr = current_expr_copy_357;
            if( x.m_len_expr )
            self().visit_expr(*x.m_len_expr);
        }
        if (x.m_type) {
            self().visit_ttype(*x.m_type);
        }
    }
    void visit_Attribute(const Attribute_t &x) {
        for (size_t i=0; i<x.n_args; i++) {
            self().visit_attribute_arg(x.m_args[i]);
        }
    }
    void visit_attribute_arg(const attribute_arg_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_call_arg(const call_arg_t &x) {
        if (x.m_value) {
            ASR::expr_t** current_expr_copy_358 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
            self().call_replacer();
            current_expr = current_expr_copy_358;
            if( x.m_value )
            self().visit_expr(*x.m_value);
        }
    }
    void visit_reduction_expr(const reduction_expr_t &x) {
        ASR::expr_t** current_expr_copy_359 = current_expr;
        current_expr = const_cast<ASR::expr_t**>(&(x.m_arg));
        self().call_replacer();
        current_expr = current_expr_copy_359;
        if( x.m_arg )
        self().visit_expr(*x.m_arg);
    }
    void visit_Bind(const Bind_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_array_index(const array_index_t &x) {
        if (x.m_left) {
            ASR::expr_t** current_expr_copy_360 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_left));
            self().call_replacer();
            current_expr = current_expr_copy_360;
            if( x.m_left )
            self().visit_expr(*x.m_left);
        }
        if (x.m_right) {
            ASR::expr_t** current_expr_copy_361 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_right));
            self().call_replacer();
            current_expr = current_expr_copy_361;
            if( x.m_right )
            self().visit_expr(*x.m_right);
        }
        if (x.m_step) {
            ASR::expr_t** current_expr_copy_362 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_step));
            self().call_replacer();
            current_expr = current_expr_copy_362;
            if( x.m_step )
            self().visit_expr(*x.m_step);
        }
    }
    void visit_do_loop_head(const do_loop_head_t &x) {
        if (x.m_v) {
            ASR::expr_t** current_expr_copy_363 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_v));
            self().call_replacer();
            current_expr = current_expr_copy_363;
            if( x.m_v )
            self().visit_expr(*x.m_v);
        }
        if (x.m_start) {
            ASR::expr_t** current_expr_copy_364 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_start));
            self().call_replacer();
            current_expr = current_expr_copy_364;
            if( x.m_start )
            self().visit_expr(*x.m_start);
        }
        if (x.m_end) {
            ASR::expr_t** current_expr_copy_365 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_end));
            self().call_replacer();
            current_expr = current_expr_copy_365;
            if( x.m_end )
            self().visit_expr(*x.m_end);
        }
        if (x.m_increment) {
            ASR::expr_t** current_expr_copy_366 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_increment));
            self().call_replacer();
            current_expr = current_expr_copy_366;
            if( x.m_increment )
            self().visit_expr(*x.m_increment);
        }
    }
    void visit_CaseStmt(const CaseStmt_t &x) {
        CaseStmt_t& xx = const_cast<CaseStmt_t&>(x);
        for (size_t i=0; i<x.n_test; i++) {
            ASR::expr_t** current_expr_copy_367 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_test[i]));
            self().call_replacer();
            current_expr = current_expr_copy_367;
            if( x.m_test[i] )
            self().visit_expr(*x.m_test[i]);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_CaseStmt_Range(const CaseStmt_Range_t &x) {
        CaseStmt_Range_t& xx = const_cast<CaseStmt_Range_t&>(x);
        if (x.m_start) {
            ASR::expr_t** current_expr_copy_368 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_start));
            self().call_replacer();
            current_expr = current_expr_copy_368;
            if( x.m_start )
            self().visit_expr(*x.m_start);
        }
        if (x.m_end) {
            ASR::expr_t** current_expr_copy_369 = current_expr;
            current_expr = const_cast<ASR::expr_t**>(&(x.m_end));
            self().call_replacer();
            current_expr = current_expr_copy_369;
            if( x.m_end )
            self().visit_expr(*x.m_end);
        }
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_TypeStmtName(const TypeStmtName_t &x) {
        TypeStmtName_t& xx = const_cast<TypeStmtName_t&>(x);
        self().transform_stmts(xx.m_body, xx.n_body);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_ClassStmt(const ClassStmt_t &x) {
        ClassStmt_t& xx = const_cast<ClassStmt_t&>(x);
        self().transform_stmts(xx.m_body, xx.n_body);
        if ((bool&)x) { } // Suppress unused warning
    }
    void visit_TypeStmtType(const TypeStmtType_t &x) {
        TypeStmtType_t& xx = const_cast<TypeStmtType_t&>(x);
        self().visit_ttype(*x.m_type);
        self().transform_stmts(xx.m_body, xx.n_body);
    }
    void visit_Require(const Require_t &x) {
        if ((bool&)x) { } // Suppress unused warning
    }
};


/******************************************************************************/
// Expression Type (`expr_type`) visitor
static inline ASR::ttype_t* expr_type0(const ASR::expr_t *f)
{
    LCOMPILERS_ASSERT(f != nullptr);
    switch (f->type) {
        case ASR::exprType::IfExp: { return ((ASR::IfExp_t*)f)->m_type; }
        case ASR::exprType::ComplexConstructor: { return ((ASR::ComplexConstructor_t*)f)->m_type; }
        case ASR::exprType::NamedExpr: { return ((ASR::NamedExpr_t*)f)->m_type; }
        case ASR::exprType::FunctionCall: { return ((ASR::FunctionCall_t*)f)->m_type; }
        case ASR::exprType::IntrinsicElementalFunction: { return ((ASR::IntrinsicElementalFunction_t*)f)->m_type; }
        case ASR::exprType::IntrinsicArrayFunction: { return ((ASR::IntrinsicArrayFunction_t*)f)->m_type; }
        case ASR::exprType::IntrinsicImpureFunction: { return ((ASR::IntrinsicImpureFunction_t*)f)->m_type; }
        case ASR::exprType::TypeInquiry: { return ((ASR::TypeInquiry_t*)f)->m_type; }
        case ASR::exprType::StructConstructor: { return ((ASR::StructConstructor_t*)f)->m_type; }
        case ASR::exprType::EnumConstructor: { return ((ASR::EnumConstructor_t*)f)->m_type; }
        case ASR::exprType::UnionTypeConstructor: { return ((ASR::UnionTypeConstructor_t*)f)->m_type; }
        case ASR::exprType::ImpliedDoLoop: { return ((ASR::ImpliedDoLoop_t*)f)->m_type; }
        case ASR::exprType::IntegerConstant: { return ((ASR::IntegerConstant_t*)f)->m_type; }
        case ASR::exprType::IntegerBOZ: { return ((ASR::IntegerBOZ_t*)f)->m_type; }
        case ASR::exprType::IntegerBitNot: { return ((ASR::IntegerBitNot_t*)f)->m_type; }
        case ASR::exprType::IntegerUnaryMinus: { return ((ASR::IntegerUnaryMinus_t*)f)->m_type; }
        case ASR::exprType::IntegerCompare: { return ((ASR::IntegerCompare_t*)f)->m_type; }
        case ASR::exprType::IntegerBinOp: { return ((ASR::IntegerBinOp_t*)f)->m_type; }
        case ASR::exprType::UnsignedIntegerConstant: { return ((ASR::UnsignedIntegerConstant_t*)f)->m_type; }
        case ASR::exprType::UnsignedIntegerUnaryMinus: { return ((ASR::UnsignedIntegerUnaryMinus_t*)f)->m_type; }
        case ASR::exprType::UnsignedIntegerBitNot: { return ((ASR::UnsignedIntegerBitNot_t*)f)->m_type; }
        case ASR::exprType::UnsignedIntegerCompare: { return ((ASR::UnsignedIntegerCompare_t*)f)->m_type; }
        case ASR::exprType::UnsignedIntegerBinOp: { return ((ASR::UnsignedIntegerBinOp_t*)f)->m_type; }
        case ASR::exprType::RealConstant: { return ((ASR::RealConstant_t*)f)->m_type; }
        case ASR::exprType::RealUnaryMinus: { return ((ASR::RealUnaryMinus_t*)f)->m_type; }
        case ASR::exprType::RealCompare: { return ((ASR::RealCompare_t*)f)->m_type; }
        case ASR::exprType::RealBinOp: { return ((ASR::RealBinOp_t*)f)->m_type; }
        case ASR::exprType::RealCopySign: { return ((ASR::RealCopySign_t*)f)->m_type; }
        case ASR::exprType::ComplexConstant: { return ((ASR::ComplexConstant_t*)f)->m_type; }
        case ASR::exprType::ComplexUnaryMinus: { return ((ASR::ComplexUnaryMinus_t*)f)->m_type; }
        case ASR::exprType::ComplexCompare: { return ((ASR::ComplexCompare_t*)f)->m_type; }
        case ASR::exprType::ComplexBinOp: { return ((ASR::ComplexBinOp_t*)f)->m_type; }
        case ASR::exprType::LogicalConstant: { return ((ASR::LogicalConstant_t*)f)->m_type; }
        case ASR::exprType::LogicalNot: { return ((ASR::LogicalNot_t*)f)->m_type; }
        case ASR::exprType::LogicalCompare: { return ((ASR::LogicalCompare_t*)f)->m_type; }
        case ASR::exprType::LogicalBinOp: { return ((ASR::LogicalBinOp_t*)f)->m_type; }
        case ASR::exprType::ListConstant: { return ((ASR::ListConstant_t*)f)->m_type; }
        case ASR::exprType::ListLen: { return ((ASR::ListLen_t*)f)->m_type; }
        case ASR::exprType::ListConcat: { return ((ASR::ListConcat_t*)f)->m_type; }
        case ASR::exprType::ListCompare: { return ((ASR::ListCompare_t*)f)->m_type; }
        case ASR::exprType::ListCount: { return ((ASR::ListCount_t*)f)->m_type; }
        case ASR::exprType::SetConstant: { return ((ASR::SetConstant_t*)f)->m_type; }
        case ASR::exprType::SetLen: { return ((ASR::SetLen_t*)f)->m_type; }
        case ASR::exprType::TupleConstant: { return ((ASR::TupleConstant_t*)f)->m_type; }
        case ASR::exprType::TupleLen: { return ((ASR::TupleLen_t*)f)->m_type; }
        case ASR::exprType::TupleCompare: { return ((ASR::TupleCompare_t*)f)->m_type; }
        case ASR::exprType::TupleConcat: { return ((ASR::TupleConcat_t*)f)->m_type; }
        case ASR::exprType::StringConstant: { return ((ASR::StringConstant_t*)f)->m_type; }
        case ASR::exprType::StringConcat: { return ((ASR::StringConcat_t*)f)->m_type; }
        case ASR::exprType::StringRepeat: { return ((ASR::StringRepeat_t*)f)->m_type; }
        case ASR::exprType::StringLen: { return ((ASR::StringLen_t*)f)->m_type; }
        case ASR::exprType::StringItem: { return ((ASR::StringItem_t*)f)->m_type; }
        case ASR::exprType::StringSection: { return ((ASR::StringSection_t*)f)->m_type; }
        case ASR::exprType::StringCompare: { return ((ASR::StringCompare_t*)f)->m_type; }
        case ASR::exprType::StringOrd: { return ((ASR::StringOrd_t*)f)->m_type; }
        case ASR::exprType::StringChr: { return ((ASR::StringChr_t*)f)->m_type; }
        case ASR::exprType::StringFormat: { return ((ASR::StringFormat_t*)f)->m_type; }
        case ASR::exprType::CPtrCompare: { return ((ASR::CPtrCompare_t*)f)->m_type; }
        case ASR::exprType::SymbolicCompare: { return ((ASR::SymbolicCompare_t*)f)->m_type; }
        case ASR::exprType::DictConstant: { return ((ASR::DictConstant_t*)f)->m_type; }
        case ASR::exprType::DictLen: { return ((ASR::DictLen_t*)f)->m_type; }
        case ASR::exprType::Var: {
            ASR::symbol_t *s = ((ASR::Var_t*)f)->m_v;
            if (s->type == ASR::symbolType::ExternalSymbol) {
                ASR::ExternalSymbol_t *e = ASR::down_cast<ASR::ExternalSymbol_t>(s);
                LCOMPILERS_ASSERT(e->m_external);
                LCOMPILERS_ASSERT(!ASR::is_a<ASR::ExternalSymbol_t>(*e->m_external));
                s = e->m_external;
            }
            if (s->type == ASR::symbolType::Function) {
                return ASR::down_cast<ASR::Function_t>(s)->m_function_signature;
            } else if( s->type == ASR::symbolType::Variable ) {
                return ASR::down_cast<ASR::Variable_t>(s)->m_type;
            } else {
                LCOMPILERS_ASSERT_MSG(false, std::to_string(s->type));
            }
            return nullptr;
        }
        case ASR::exprType::FunctionParam: { return ((ASR::FunctionParam_t*)f)->m_type; }
        case ASR::exprType::ArrayConstructor: { return ((ASR::ArrayConstructor_t*)f)->m_type; }
        case ASR::exprType::ArrayConstant: { return ((ASR::ArrayConstant_t*)f)->m_type; }
        case ASR::exprType::ArrayItem: { return ((ASR::ArrayItem_t*)f)->m_type; }
        case ASR::exprType::ArraySection: { return ((ASR::ArraySection_t*)f)->m_type; }
        case ASR::exprType::ArraySize: { return ((ASR::ArraySize_t*)f)->m_type; }
        case ASR::exprType::ArrayBound: { return ((ASR::ArrayBound_t*)f)->m_type; }
        case ASR::exprType::ArrayTranspose: { return ((ASR::ArrayTranspose_t*)f)->m_type; }
        case ASR::exprType::ArrayPack: { return ((ASR::ArrayPack_t*)f)->m_type; }
        case ASR::exprType::ArrayReshape: { return ((ASR::ArrayReshape_t*)f)->m_type; }
        case ASR::exprType::ArrayBroadcast: { return ((ASR::ArrayBroadcast_t*)f)->m_type; }
        case ASR::exprType::BitCast: { return ((ASR::BitCast_t*)f)->m_type; }
        case ASR::exprType::StructInstanceMember: { return ((ASR::StructInstanceMember_t*)f)->m_type; }
        case ASR::exprType::StructStaticMember: { return ((ASR::StructStaticMember_t*)f)->m_type; }
        case ASR::exprType::EnumStaticMember: { return ((ASR::EnumStaticMember_t*)f)->m_type; }
        case ASR::exprType::UnionInstanceMember: { return ((ASR::UnionInstanceMember_t*)f)->m_type; }
        case ASR::exprType::EnumName: { return ((ASR::EnumName_t*)f)->m_type; }
        case ASR::exprType::EnumValue: { return ((ASR::EnumValue_t*)f)->m_type; }
        case ASR::exprType::OverloadedCompare: { return ((ASR::OverloadedCompare_t*)f)->m_type; }
        case ASR::exprType::OverloadedBinOp: { return expr_type0(((ASR::OverloadedBinOp_t*)f)->m_overloaded); }
        case ASR::exprType::OverloadedUnaryMinus: { return ((ASR::OverloadedUnaryMinus_t*)f)->m_type; }
        case ASR::exprType::OverloadedStringConcat: { return ((ASR::OverloadedStringConcat_t*)f)->m_type; }
        case ASR::exprType::Cast: { return ((ASR::Cast_t*)f)->m_type; }
        case ASR::exprType::ArrayPhysicalCast: { return ((ASR::ArrayPhysicalCast_t*)f)->m_type; }
        case ASR::exprType::ComplexRe: { return ((ASR::ComplexRe_t*)f)->m_type; }
        case ASR::exprType::ComplexIm: { return ((ASR::ComplexIm_t*)f)->m_type; }
        case ASR::exprType::DictItem: { return ((ASR::DictItem_t*)f)->m_type; }
        case ASR::exprType::CLoc: { return ((ASR::CLoc_t*)f)->m_type; }
        case ASR::exprType::PointerToCPtr: { return ((ASR::PointerToCPtr_t*)f)->m_type; }
        case ASR::exprType::GetPointer: { return ((ASR::GetPointer_t*)f)->m_type; }
        case ASR::exprType::ListItem: { return ((ASR::ListItem_t*)f)->m_type; }
        case ASR::exprType::TupleItem: { return ((ASR::TupleItem_t*)f)->m_type; }
        case ASR::exprType::ListSection: { return ((ASR::ListSection_t*)f)->m_type; }
        case ASR::exprType::ListRepeat: { return ((ASR::ListRepeat_t*)f)->m_type; }
        case ASR::exprType::DictPop: { return ((ASR::DictPop_t*)f)->m_type; }
        case ASR::exprType::SetPop: { return ((ASR::SetPop_t*)f)->m_type; }
        case ASR::exprType::IntegerBitLen: { return ((ASR::IntegerBitLen_t*)f)->m_type; }
        case ASR::exprType::Ichar: { return ((ASR::Ichar_t*)f)->m_type; }
        case ASR::exprType::Iachar: { return ((ASR::Iachar_t*)f)->m_type; }
        case ASR::exprType::SizeOfType: { return ((ASR::SizeOfType_t*)f)->m_type; }
        case ASR::exprType::PointerNullConstant: { return ((ASR::PointerNullConstant_t*)f)->m_type; }
        case ASR::exprType::PointerAssociated: { return ((ASR::PointerAssociated_t*)f)->m_type; }
        case ASR::exprType::RealSqrt: { return ((ASR::RealSqrt_t*)f)->m_type; }
        default : throw LCompilersException("Not implemented");
    }
}



/******************************************************************************/
// Expression Value (`expr_value`) visitor
static inline ASR::expr_t* expr_value0(ASR::expr_t *f)
{
    LCOMPILERS_ASSERT(f != nullptr);
    switch (f->type) {
        case ASR::exprType::IfExp: { return ((ASR::IfExp_t*)f)->m_value; }
        case ASR::exprType::ComplexConstructor: { return ((ASR::ComplexConstructor_t*)f)->m_value; }
        case ASR::exprType::NamedExpr: { return ((ASR::NamedExpr_t*)f)->m_value; }
        case ASR::exprType::FunctionCall: { return ((ASR::FunctionCall_t*)f)->m_value; }
        case ASR::exprType::IntrinsicElementalFunction: { return ((ASR::IntrinsicElementalFunction_t*)f)->m_value; }
        case ASR::exprType::IntrinsicArrayFunction: { return ((ASR::IntrinsicArrayFunction_t*)f)->m_value; }
        case ASR::exprType::IntrinsicImpureFunction: { return ((ASR::IntrinsicImpureFunction_t*)f)->m_value; }
        case ASR::exprType::TypeInquiry: { return ((ASR::TypeInquiry_t*)f)->m_value; }
        case ASR::exprType::StructConstructor: { return ((ASR::StructConstructor_t*)f)->m_value; }
        case ASR::exprType::EnumConstructor: { return ((ASR::EnumConstructor_t*)f)->m_value; }
        case ASR::exprType::UnionTypeConstructor: { return ((ASR::UnionTypeConstructor_t*)f)->m_value; }
        case ASR::exprType::ImpliedDoLoop: { return ((ASR::ImpliedDoLoop_t*)f)->m_value; }
        case ASR::exprType::IntegerConstant: { return f; }
        case ASR::exprType::IntegerBOZ: { return f; }
        case ASR::exprType::IntegerBitNot: { return ((ASR::IntegerBitNot_t*)f)->m_value; }
        case ASR::exprType::IntegerUnaryMinus: { return ((ASR::IntegerUnaryMinus_t*)f)->m_value; }
        case ASR::exprType::IntegerCompare: { return ((ASR::IntegerCompare_t*)f)->m_value; }
        case ASR::exprType::IntegerBinOp: { return ((ASR::IntegerBinOp_t*)f)->m_value; }
        case ASR::exprType::UnsignedIntegerConstant: { return f; }
        case ASR::exprType::UnsignedIntegerUnaryMinus: { return ((ASR::UnsignedIntegerUnaryMinus_t*)f)->m_value; }
        case ASR::exprType::UnsignedIntegerBitNot: { return ((ASR::UnsignedIntegerBitNot_t*)f)->m_value; }
        case ASR::exprType::UnsignedIntegerCompare: { return ((ASR::UnsignedIntegerCompare_t*)f)->m_value; }
        case ASR::exprType::UnsignedIntegerBinOp: { return ((ASR::UnsignedIntegerBinOp_t*)f)->m_value; }
        case ASR::exprType::RealConstant: { return f; }
        case ASR::exprType::RealUnaryMinus: { return ((ASR::RealUnaryMinus_t*)f)->m_value; }
        case ASR::exprType::RealCompare: { return ((ASR::RealCompare_t*)f)->m_value; }
        case ASR::exprType::RealBinOp: { return ((ASR::RealBinOp_t*)f)->m_value; }
        case ASR::exprType::RealCopySign: { return ((ASR::RealCopySign_t*)f)->m_value; }
        case ASR::exprType::ComplexConstant: { return f; }
        case ASR::exprType::ComplexUnaryMinus: { return ((ASR::ComplexUnaryMinus_t*)f)->m_value; }
        case ASR::exprType::ComplexCompare: { return ((ASR::ComplexCompare_t*)f)->m_value; }
        case ASR::exprType::ComplexBinOp: { return ((ASR::ComplexBinOp_t*)f)->m_value; }
        case ASR::exprType::LogicalConstant: { return f; }
        case ASR::exprType::LogicalNot: { return ((ASR::LogicalNot_t*)f)->m_value; }
        case ASR::exprType::LogicalCompare: { return ((ASR::LogicalCompare_t*)f)->m_value; }
        case ASR::exprType::LogicalBinOp: { return ((ASR::LogicalBinOp_t*)f)->m_value; }
        case ASR::exprType::ListConstant: { return f; }
        case ASR::exprType::ListLen: { return ((ASR::ListLen_t*)f)->m_value; }
        case ASR::exprType::ListConcat: { return ((ASR::ListConcat_t*)f)->m_value; }
        case ASR::exprType::ListCompare: { return ((ASR::ListCompare_t*)f)->m_value; }
        case ASR::exprType::ListCount: { return ((ASR::ListCount_t*)f)->m_value; }
        case ASR::exprType::SetConstant: { return f; }
        case ASR::exprType::SetLen: { return ((ASR::SetLen_t*)f)->m_value; }
        case ASR::exprType::TupleConstant: { return f; }
        case ASR::exprType::TupleLen: { return ((ASR::TupleLen_t*)f)->m_value; }
        case ASR::exprType::TupleCompare: { return ((ASR::TupleCompare_t*)f)->m_value; }
        case ASR::exprType::TupleConcat: { return ((ASR::TupleConcat_t*)f)->m_value; }
        case ASR::exprType::StringConstant: { return f; }
        case ASR::exprType::StringConcat: { return ((ASR::StringConcat_t*)f)->m_value; }
        case ASR::exprType::StringRepeat: { return ((ASR::StringRepeat_t*)f)->m_value; }
        case ASR::exprType::StringLen: { return ((ASR::StringLen_t*)f)->m_value; }
        case ASR::exprType::StringItem: { return ((ASR::StringItem_t*)f)->m_value; }
        case ASR::exprType::StringSection: { return ((ASR::StringSection_t*)f)->m_value; }
        case ASR::exprType::StringCompare: { return ((ASR::StringCompare_t*)f)->m_value; }
        case ASR::exprType::StringOrd: { return ((ASR::StringOrd_t*)f)->m_value; }
        case ASR::exprType::StringChr: { return ((ASR::StringChr_t*)f)->m_value; }
        case ASR::exprType::StringFormat: { return ((ASR::StringFormat_t*)f)->m_value; }
        case ASR::exprType::CPtrCompare: { return ((ASR::CPtrCompare_t*)f)->m_value; }
        case ASR::exprType::SymbolicCompare: { return ((ASR::SymbolicCompare_t*)f)->m_value; }
        case ASR::exprType::DictConstant: { return f; }
        case ASR::exprType::DictLen: { return ((ASR::DictLen_t*)f)->m_value; }
        case ASR::exprType::Var: {
            ASR::symbol_t *s = ((ASR::Var_t*)f)->m_v;
            if (s->type == ASR::symbolType::ExternalSymbol) {
                ASR::ExternalSymbol_t *e = ASR::down_cast<ASR::ExternalSymbol_t>(s);
                LCOMPILERS_ASSERT(!ASR::is_a<ASR::ExternalSymbol_t>(*e->m_external));
                s = e->m_external;
            }
            if( ASR::is_a<ASR::Function_t>(*s) ||
                ASR::down_cast<ASR::Variable_t>(s)->m_storage !=
                ASR::storage_typeType::Parameter ) {
                return nullptr;
            }
            return ASR::down_cast<ASR::Variable_t>(s)->m_value;
        }
        case ASR::exprType::FunctionParam: { return ((ASR::FunctionParam_t*)f)->m_value; }
        case ASR::exprType::ArrayConstructor: { return ((ASR::ArrayConstructor_t*)f)->m_value; }
        case ASR::exprType::ArrayConstant: { return f; }
        case ASR::exprType::ArrayItem: { return ((ASR::ArrayItem_t*)f)->m_value; }
        case ASR::exprType::ArraySection: { return ((ASR::ArraySection_t*)f)->m_value; }
        case ASR::exprType::ArraySize: { return ((ASR::ArraySize_t*)f)->m_value; }
        case ASR::exprType::ArrayBound: { return ((ASR::ArrayBound_t*)f)->m_value; }
        case ASR::exprType::ArrayTranspose: { return ((ASR::ArrayTranspose_t*)f)->m_value; }
        case ASR::exprType::ArrayPack: { return ((ASR::ArrayPack_t*)f)->m_value; }
        case ASR::exprType::ArrayReshape: { return ((ASR::ArrayReshape_t*)f)->m_value; }
        case ASR::exprType::ArrayBroadcast: { return ((ASR::ArrayBroadcast_t*)f)->m_value; }
        case ASR::exprType::BitCast: { return ((ASR::BitCast_t*)f)->m_value; }
        case ASR::exprType::StructInstanceMember: { return ((ASR::StructInstanceMember_t*)f)->m_value; }
        case ASR::exprType::StructStaticMember: { return ((ASR::StructStaticMember_t*)f)->m_value; }
        case ASR::exprType::EnumStaticMember: { return ((ASR::EnumStaticMember_t*)f)->m_value; }
        case ASR::exprType::UnionInstanceMember: { return ((ASR::UnionInstanceMember_t*)f)->m_value; }
        case ASR::exprType::EnumName: { return ((ASR::EnumName_t*)f)->m_value; }
        case ASR::exprType::EnumValue: { return ((ASR::EnumValue_t*)f)->m_value; }
        case ASR::exprType::OverloadedCompare: { return ((ASR::OverloadedCompare_t*)f)->m_value; }
        case ASR::exprType::OverloadedBinOp: { return ((ASR::OverloadedBinOp_t*)f)->m_value; }
        case ASR::exprType::OverloadedUnaryMinus: { return ((ASR::OverloadedUnaryMinus_t*)f)->m_value; }
        case ASR::exprType::OverloadedStringConcat: { return ((ASR::OverloadedStringConcat_t*)f)->m_value; }
        case ASR::exprType::Cast: { return ((ASR::Cast_t*)f)->m_value; }
        case ASR::exprType::ArrayPhysicalCast: { return ((ASR::ArrayPhysicalCast_t*)f)->m_value; }
        case ASR::exprType::ComplexRe: { return ((ASR::ComplexRe_t*)f)->m_value; }
        case ASR::exprType::ComplexIm: { return ((ASR::ComplexIm_t*)f)->m_value; }
        case ASR::exprType::DictItem: { return ((ASR::DictItem_t*)f)->m_value; }
        case ASR::exprType::CLoc: { return ((ASR::CLoc_t*)f)->m_value; }
        case ASR::exprType::PointerToCPtr: { return ((ASR::PointerToCPtr_t*)f)->m_value; }
        case ASR::exprType::GetPointer: { return ((ASR::GetPointer_t*)f)->m_value; }
        case ASR::exprType::ListItem: { return ((ASR::ListItem_t*)f)->m_value; }
        case ASR::exprType::TupleItem: { return ((ASR::TupleItem_t*)f)->m_value; }
        case ASR::exprType::ListSection: { return ((ASR::ListSection_t*)f)->m_value; }
        case ASR::exprType::ListRepeat: { return ((ASR::ListRepeat_t*)f)->m_value; }
        case ASR::exprType::DictPop: { return ((ASR::DictPop_t*)f)->m_value; }
        case ASR::exprType::SetPop: { return ((ASR::SetPop_t*)f)->m_value; }
        case ASR::exprType::IntegerBitLen: { return ((ASR::IntegerBitLen_t*)f)->m_value; }
        case ASR::exprType::Ichar: { return ((ASR::Ichar_t*)f)->m_value; }
        case ASR::exprType::Iachar: { return ((ASR::Iachar_t*)f)->m_value; }
        case ASR::exprType::SizeOfType: { return ((ASR::SizeOfType_t*)f)->m_value; }
        case ASR::exprType::PointerNullConstant: { return f; }
        case ASR::exprType::PointerAssociated: { return ((ASR::PointerAssociated_t*)f)->m_value; }
        case ASR::exprType::RealSqrt: { return ((ASR::RealSqrt_t*)f)->m_value; }
        default : throw LCompilersException("Not implemented");
    }
}



} // namespace LCompilers::ASR

#endif // LFORTRAN_ASR_H
